( function( blocks, editor, element, components ) {
    const __ = wp.i18n.__; // The __() for internationalization.
    const el = element.createElement; // The wp.element.createElement() function to create elements.
    const registerBlockType = blocks.registerBlockType; // The registerBlockType() to register blocks.
    const { SelectControl } = components;
    const withSelect = wp.data.withSelect;

    function Icon() {
	  return el(
	    'svg',
	    {
	      xmlns: 'http://www.w3.org/2000/svg',
	      width: '27',
	      height: '27',
	      viewBox: '0 0 27 27'
	    },
	    el(
	      'g',
	      { fill: 'none', fillRule: 'evenodd', stroke: 'none', strokeWidth: '1' },
	      el(
	        'g',
	        { fill: '#000', fillRule: 'nonzero' },
	        el('path', { d: 'M10.688 27a.563.563 0 01-.563-.563V15.073c0-.483-.207-.942-.567-1.261L.943 6.158A2.816 2.816 0 010 4.05V1.688C0 .757.757 0 1.688 0h23.625C26.242 0 27 .757 27 1.688V4.05c0 .809-.344 1.578-.944 2.11l-8.614 7.653c-.36.32-.567.778-.567 1.26v7.352c0 .588-.313 1.143-.819 1.446l-5.08 3.048a.55.55 0 01-.289.081zm-9-25.875a.563.563 0 00-.563.563V4.05c0 .487.206.95.564 1.266l8.616 7.654c.6.533.945 1.3.945 2.103v10.371l4.228-2.536a.568.568 0 00.272-.482v-7.352c0-.803.345-1.57.945-2.103l8.615-7.653c.36-.318.565-.78.565-1.268V1.688a.563.563 0 00-.563-.563H1.688z' })
	      )
	    )
	  );
	}

    registerBlockType( 'piotnetgrid/facet', { // Block name; must be a string that contains a namespace prefix followed by a forward slash.
        title: __( 'Facet', 'piotnetgrid' ),
        icon: el(Icon, null),
        category: 'piotnetgrid', // Group blocks together based on common traits, can be 'common', 'formatting', 'layout', 'widgets' or 'embed'.
        keywords: [ 'facet', 'piotnet' ], // Keywords for custom block, limited to three.
        attributes: { // Placeholders to store custom block setting values.
            grid_id: {
                type: 'string',
            },
            facet_id: {
                type: 'string',
            },
        },
        edit: withSelect( function( select, props ) {
        		var facet_ids = select('core').getEntityRecords('postType', 'piotnetgrid-facet', { per_page: -1 });
	            var grid_ids = select('core').getEntityRecords('postType', 'piotnetgrid', { per_page: -1 });

	            return {
	                grid_ids: grid_ids,
	                facet_ids: facet_ids
	            };
	        } )
        	( function( props ) {

        		var options_grid = [];
		
				if( props.grid_ids ) {
					options_grid.push( { value: '0', label: 'Apply for Grid' } );
					props.grid_ids.forEach((post) => {
						options_grid.push({value:post.id, label:post.title.rendered});
					});
				} else {
					options_grid.push( { value: '', label: 'Loading...' } )
				}

				var options_facet = [];
		
				if( props.facet_ids ) {
					options_facet.push( { value: '', label: 'Select a Facet' } );
					props.facet_ids.forEach((post) => {
						options_facet.push({value:post.id, label:post.title.rendered});
					});
				} else {
					options_facet.push( { value: '', label: 'Loading...' } )
				}

	            return [
	                el( 'div',
	                    {
	                        key: 'piotnetgrid-gutenberg-facet', // React requires each top-level item in the returned array to have a unique key.
	                        className: 'piotnetgrid-gutenberg', // className property is automatically enabled for each block.
	                    },
	                    el( SelectControl,
					        {
					            label: __( 'Facet - Piotnetgrid', 'piotnetgrid' ),
					            options: options_facet,
					            onChange: ( value ) => {
					                props.setAttributes(
					                    {
					                        facet_id: value
					                    }
					                );
					            },
					            value: props.attributes.facet_id
					        }
					    ),
	                    el( SelectControl,
					        {
					            options: options_grid,
					            onChange: ( value ) => {
					                props.setAttributes(
					                    {
					                        grid_id: value
					                    }
					                );
					            },
					            value: props.attributes.grid_id
					        }
					    )
	                )
	            ];
	        } )
    });
}(
    window.wp.blocks,
    window.wp.blockEditor,
    window.wp.element,
    window.wp.components
) );