( function( blocks, editor, element, components ) {
    const __ = wp.i18n.__; // The __() for internationalization.
    const el = element.createElement; // The wp.element.createElement() function to create elements.
    const registerBlockType = blocks.registerBlockType; // The registerBlockType() to register blocks.
    const { SelectControl } = components;
    const withSelect = wp.data.withSelect;

    function Icon() {
	  return el(
	    'svg',
	    {
	      xmlns: 'http://www.w3.org/2000/svg',
	      width: '29',
	      height: '29',
	      viewBox: '0 0 29 29'
	    },
	    el(
	      'g',
	      { fill: 'none', fillRule: 'evenodd', stroke: 'none', strokeWidth: '1' },
	      el(
	        'g',
	        { transform: 'translate(1 1)' },
	        el(
	          'g',
	          null,
	          el('path', { d: 'M0 0H11V11H0z' }),
	          el('path', { stroke: '#000', d: 'M0 0H11V11H0z' })
	        ),
	        el(
	          'g',
	          { transform: 'translate(0 16)' },
	          el('path', { d: 'M0 0H11V11H0z' }),
	          el('path', { stroke: '#000', d: 'M0 0H11V11H0z' })
	        ),
	        el(
	          'g',
	          { transform: 'translate(16)' },
	          el('path', { d: 'M0 0H11V11H0z' }),
	          el('path', { stroke: '#000', d: 'M0 0H11V11H0z' })
	        ),
	        el(
	          'g',
	          { transform: 'translate(16 16)' },
	          el('path', { d: 'M0 0H11V11H0z' }),
	          el('path', { stroke: '#000', d: 'M0 0H11V11H0z' })
	        )
	      )
	    )
	  );
	}

    registerBlockType( 'piotnetgrid/grid', { // Block name; must be a string that contains a namespace prefix followed by a forward slash.
        title: __( 'Grid', 'piotnetgrid' ),
        icon: el(Icon, null),
        category: 'piotnetgrid', // Group blocks together based on common traits, can be 'common', 'formatting', 'layout', 'widgets' or 'embed'.
        keywords: [ 'grid', 'piotnet' ], // Keywords for custom block, limited to three.
        attributes: { // Placeholders to store custom block setting values.
            grid_id: {
                type: 'string',
            },
        },
        edit: withSelect( function( select, props ) {
	            var grid_ids = select('core').getEntityRecords('postType', 'piotnetgrid', { per_page: -1 });

	            return {
	                grid_ids: grid_ids
	            };
	        } )
        	( function( props ) {

        		var options = [];
		
				if( props.grid_ids ) {
					options.push( { value: '', label: 'Select a grid' } );
					props.grid_ids.forEach((post) => {
						options.push({value:post.id, label:post.title.rendered});
					});
				} else {
					options.push( { value: '', label: 'Loading...' } )
				}

	            return [
	                el( 'div',
	                    {
	                        key: 'piotnetgrid-gutenberg', // React requires each top-level item in the returned array to have a unique key.
	                        className: 'piotnetgrid-gutenberg', // className property is automatically enabled for each block.
	                    },
	                    el( SelectControl,
					        {
					            label: __( 'Grid - Piotnetgrid', 'piotnetgrid' ),
					            options: options,
					            onChange: ( value ) => {
					                props.setAttributes(
					                    {
					                        grid_id: value
					                    }
					                );
					            },
					            value: props.attributes.grid_id
					        }
					    )
	                )
	            ];
	        } )
    });
}(
    window.wp.blocks,
    window.wp.blockEditor,
    window.wp.element,
    window.wp.components
) );