( function( blocks, editor, element, components ) {
    const __ = wp.i18n.__; // The __() for internationalization.
    const el = element.createElement; // The wp.element.createElement() function to create elements.
    const registerBlockType = blocks.registerBlockType; // The registerBlockType() to register blocks.
    const { SelectControl } = components;
    const withSelect = wp.data.withSelect;

    function Icon() {
	  return el(
	    'svg',
	    {
	      xmlns: 'http://www.w3.org/2000/svg',
	      width: '35',
	      height: '28',
	      viewBox: '0 0 35 28'
	    },
	    el(
	      'g',
	      { fill: 'none', fillRule: 'evenodd', stroke: 'none', strokeWidth: '1' },
	      el(
	        'g',
	        { fill: '#000', fillRule: 'nonzero' },
	        el(
	          'g',
	          null,
	          el('path', { d: 'M34.486 0H.514A.515.515 0 000 .516v26.968c0 .285.23.516.514.516h33.972c.284 0 .514-.23.514-.516V.516A.515.515 0 0034.486 0zM1.028 26.968V4.334h5.547a.515.515 0 000-1.032H1.028v-2.27h32.944v2.27H9.042a.515.515 0 000 1.032h24.93v22.634H1.028z' }),
	          el('path', { d: 'M30.732 2.121a.426.426 0 01-.427.424.426.426 0 01-.427-.424c0-.234.191-.424.427-.424s.427.19.427.424zM29.024 2.121a.426.426 0 01-.426.424.426.426 0 01-.427-.424c0-.234.191-.424.427-.424.235 0 .426.19.426.424zM32.439 2.121a.426.426 0 01-.427.424.426.426 0 01-.427-.424c0-.234.192-.424.427-.424.236 0 .427.19.427.424z' }),
	          el('path', { d: 'M17.401 6.788H3.941a.517.517 0 00-.526.508v16.802c0 .28.235.508.526.508H17.4c.29 0 .526-.227.526-.508V7.296a.517.517 0 00-.526-.508zm-.526 16.802H4.466V7.804h12.41V23.59zM31.06 6.788H19.306c-.29 0-.525.221-.525.494v5.8c0 .272.236.494.525.494h11.756c.29 0 .524-.222.524-.495V7.282c0-.273-.235-.494-.524-.494zm-.524 5.8H19.83V7.776h10.706v4.81zM31.06 15.273H19.306a.522.522 0 00-.525.518v8.296c0 .287.236.52.525.52h11.756c.29 0 .524-.233.524-.52v-8.296a.522.522 0 00-.524-.518zm-.524 8.296H19.83v-7.26h10.706v7.26z' })	          
	        )
	      )
	    )
	  );
	}

    registerBlockType( 'piotnetgrid/template', { // Block name; must be a string that contains a namespace prefix followed by a forward slash.
        title: __( 'Template', 'piotnetgrid' ),
        icon: el(Icon, null),
        category: 'piotnetgrid', // Group blocks together based on common traits, can be 'common', 'formatting', 'layout', 'widgets' or 'embed'.
        keywords: [ 'template', 'grid', 'piotnet' ], // Keywords for custom block, limited to three.
        attributes: { // Placeholders to store custom block setting values.
            template_id: {
                type: 'string',
            },
        },
        edit: withSelect( function( select, props ) {
	            var template_ids = select('core').getEntityRecords('postType', 'piotnetgrid-template', { per_page: -1 });

	            return {
	                template_ids: template_ids
	            };
	        } )
        	( function( props ) {

        		var options = [];
		
				if( props.template_ids ) {
					options.push( { value: '', label: 'Select a template' } );
					props.template_ids.forEach((post) => {
						options.push({value:post.id, label:post.title.rendered});
					});
				} else {
					options.push( { value: '', label: 'Loading...' } )
				}

	            return [
	                el( 'div',
	                    {
	                        key: 'piotnetgrid-gutenberg', // React requires each top-level item in the returned array to have a unique key.
	                        className: 'piotnetgrid-gutenberg', // className property is automatically enabled for each block.
	                    },
	                    el( SelectControl,
					        {
					            label: __( 'Template - Piotnetgrid', 'piotnetgrid' ),
					            options: options,
					            onChange: ( value ) => {
					                props.setAttributes(
					                    {
					                        template_id: value
					                    }
					                );
					            },
					            value: props.attributes.template_id
					        }
					    )
	                )
	            ];
	        } )
    });
}(
    window.wp.blocks,
    window.wp.blockEditor,
    window.wp.element,
    window.wp.components
) );