import './lib/jquery-throttle.js';
import Macy from './lib/masonry.js';
import LazyLoad from './lib/lazyload.js';
import noUiSlider from 'nouislider';

jQuery(document).ready(function($) {

	var pluginURL = $('[data-piotnetgrid-plugin-url]').attr('data-piotnetgrid-plugin-url');
	var $facetField = $('[data-piotnetgrid-facet-field]');

	// Fix Oxygen not full width
	$('#piotnetgrid').closest('span.ct-span').css({'display':'block'});

	window.piotnetgridGetUrlParameter = function( name, url ) {
		if (!url) url = location.href;
		name = name.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
		var regexS = "[\\?&]"+name+"=([^&#]*)";
		var regex = new RegExp( regexS );
		var results = regex.exec( url );
		return results == null ? null : decodeURIComponent(results[1]);
	}

	function piotnetgridFormatNumberBy3(num, decpoint, sep) {
		var integerPart = Number(num.toFixed(0)).toLocaleString('en-US');
		var floatPart = "";
		if (!Number.isInteger(num)) {
			const numStr = num.toString();
			floatPart = numStr.substr(numStr.indexOf('.'));
		}
		num = integerPart + floatPart;
		num = num.replace(/\./g, '|');
		num = num.replace(/\,/g, sep);
		num = num.replace(/\|/g, decpoint);
		return num;
	}

	function piotnetgridIsElementInViewport(el) {
	    if (typeof jQuery === "function" && el instanceof jQuery) {
	        el = el[0];
	    }

	    var rect = el.getBoundingClientRect();

	    return (
	        rect.top >= 0 &&
	        rect.left >= 0 &&
	        rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
	        rect.right <= (window.innerWidth || document.documentElement.clientWidth)
	    );
	}

	$(window).on('DOMContentLoaded load resize scroll', $.debounce( 250, function(e) {
			$(document).find('[data-piotnetgrid-facet-load-more-infinite-scroll]:not(.piotnetgrid-loading)').each(function(){
				if (piotnetgridIsElementInViewport($(this))) {
					$(this).find('[data-piotnetgrid-facet-load-more]').trigger('click');
				}
			})

		})
	);

	function piotnetgridRound(value, decimals, decimalsSymbol, seperatorsSymbol, decimalsShow) {
		var afterRound = Number(Math.round(value+'e'+decimals)+'e-'+decimals);
		if (decimalsShow == '') {
			return piotnetgridFormatNumberBy3( afterRound, decimalsSymbol, seperatorsSymbol );
		} else {
			var afterFixed = Number(afterRound.toFixed(decimals));
			var formattedNumber = piotnetgridFormatNumberBy3( afterFixed, decimalsSymbol, seperatorsSymbol );
			if (decimals > 0) {
				if (Number.isInteger(afterFixed)) {
					formattedNumber += decimalsSymbol + '0'.repeat(decimals);
				} else if (typeof(afterFixed) === 'number') {
					const decimalsCurrent = formattedNumber.length - formattedNumber.indexOf(decimalsSymbol) - 1;
					formattedNumber += '0'.repeat(decimals -decimalsCurrent);
				}
			}
			return formattedNumber;
		}
	}

	window.piotnetgridFacetSlider = function($selector, $reset) {
		$selector.each(function(){
			var sliderOptionsJson = $(this).attr('data-piotnetgrid-facet-slider');
			if (sliderOptionsJson) {
				var sliderOptions = JSON.parse(sliderOptionsJson),
					$slider = $(this).find('[data-piotnetgrid-facet-slider-bar]'),
					$facetField = $(this).find('[data-piotnetgrid-facet-field]'),
					$facetFieldValue = $(this).find('[data-piotnetgrid-facet-slider-value]'),
					facetOptions = JSON.parse($(this).attr('data-piotnetgrid-facet')),
					sliderValue = window.piotnetgridGetUrlParameter( facetOptions.name );

				if (!sliderOptions.from && sliderValue && !$reset) {
					sliderValue = sliderValue.split(',');
					sliderOptions.from = sliderValue[0];
					sliderOptions.to = sliderValue[1];
				}

				if (!sliderOptions.from) {
					sliderOptions.from = sliderOptions.min;
					sliderOptions.to = sliderOptions.max;
				}

				var	facetSliderOptions = {
					start: !$reset ? [parseFloat(sliderOptions.from), parseFloat(sliderOptions.to)] : [parseFloat(sliderOptions.min), parseFloat(sliderOptions.max)],
					connect: true,
					step: parseFloat(sliderOptions.step) ? parseFloat(sliderOptions.step) : 10,
					range: {
						'min': parseFloat(sliderOptions.min),
						'max': parseFloat(sliderOptions.max)
					}
				};

				var prefix = sliderOptions.prefix ? sliderOptions.prefix : '',
					postfix = sliderOptions.postfix ? sliderOptions.postfix : '';

				if (sliderOptions.min !== sliderOptions.max) {
					var facetSlider = noUiSlider.create($slider[0], facetSliderOptions),
						facetSliderValue = facetSlider.get();

					if (facetSliderOptions.start[0] == sliderOptions.min && facetSliderOptions.start[1] == sliderOptions.max) {
						$facetField.val('');
					}

					if (facetSliderValue.length == 2) {
						$facetFieldValue.html( prefix + piotnetgridRound(facetSliderValue[0], sliderOptions.decimals_places, sliderOptions.decimals_separator, sliderOptions.thousands_separator, '') + postfix + ' - ' + prefix + piotnetgridRound(facetSliderValue[1], sliderOptions.decimals_places, sliderOptions.decimals_separator, sliderOptions.thousands_separator, '') + postfix );
					}

					facetSlider.on('change', function() {
						var facetSliderValue = facetSlider.get();

						$facetField.val(facetSliderValue).change();
						$facetFieldValue.html( prefix + piotnetgridRound(facetSliderValue[0], sliderOptions.decimals_places, sliderOptions.decimals_separator, sliderOptions.thousands_separator, '') + postfix + ' - ' + prefix + piotnetgridRound(facetSliderValue[1], sliderOptions.decimals_places, sliderOptions.decimals_separator, sliderOptions.thousands_separator, '') + postfix );
					});
				} else {
					$facetFieldValue.html( prefix + sliderOptions.min + postfix );
				}
			}
		});
	}

	window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-slider]'), false);

	window.piotnetgridPhotoswipe = function () {
		$('[data-piotnetgrid-grid-id]').each(function (){
			var container = [];
			$(this).find('.piotnetgrid-card').each(function() {
				var title = $(this).find('.piotnetgrid-pswp__text').html();
				var $link = $(this).find('.piotnetgrid-photoswipe'),
					detail = {
						src: $link.attr('data-href'),
						w: $link.attr('data-width'),
						h: $link.attr('data-height'),
						title: title,
					};
				container.push(detail);
			});
			var $photoswipeItem = $(this).find('.piotnetgrid-photoswipe:not(.piotnetgrid-photoswipe--item)');
			$photoswipeItem.click(function(event) {
				event.preventDefault();
				var bgOpacity = $(this).attr('data-grid-pswp-bg-opacity');
				var $pswp = $('.pswp')[0];
				var bgColor = $photoswipeItem.attr('data-grid-pswp-bg-color');
				$('.pswp__bg').css({'background-color':bgColor});

				var options = {
					index: $(this).closest('.piotnetgrid-card').index(),
					bgOpacity: bgOpacity,
					showHideOpacity: true,
					tapToClose: true,
					shareEl: false,
					fullscreenEl: false,
				};
				// Initialize PhotoSwipe
				var gallery = new PhotoSwipe($pswp, PhotoSwipeUI_Default, container, options);
				gallery.init();
			});
			$photoswipeItem.addClass("piotnetgrid-photoswipe--item");
		});
	}

	piotnetgridPhotoswipe();

	window.piotnetgridFacetDate = function($selector) {
		$selector.each(function(){
			var options = $(this).attr('data-piotnetgrid-facet-date');
			if (options) {
				$(this).flatpickr(JSON.parse(options));
			}
		});
	}

	window.piotnetgridFacetDate($(document).find('[data-piotnetgrid-facet-date]'));

	function piotnetgridReplaceQueryParam(param, newval, search) {
		var regex = new RegExp("([?;&])" + param + "[^&;]*[;&]?");
		var query = search.replace(regex, "$1").replace(/&$/, '');

		return (query.length > 2 ? query + "&" : "?") + (newval ? param + "=" + encodeURIComponent(newval) : '');
	}

	//Gallery Carousel
	window.piotnetgridGalleryCarousel = function($selector) {
		$selector.each(function() {
			if ($(this).attr('data-piotnetgrid-gallery-carousel') !== 'undefined') {
				let slick_options = JSON.parse($(this).attr('data-piotnetgrid-gallery-carousel'));
				$(this).find('[data-piotnetgrid-gallery]').slick(slick_options);
			}
		});
	}
	var $piotnetgridGallerySlick = $(document).find('[data-piotnetgrid-gallery-carousel]');
	window.piotnetgridGalleryCarousel($piotnetgridGallerySlick);

	window.piotnetgridCarousel = function($selector) {
		$selector.each(function() {
			if ($(this).attr('data-piotnetgrid-grid-carousel')) {
				var slick_options = JSON.parse($(this).attr('data-piotnetgrid-grid-carousel'));
				var disableDesktop = $(this).attr('data-piotnetgrid-grid-carousel-disable-on-desktop');
				var disableTablet = $(this).attr('data-piotnetgrid-grid-carousel-disable-on-tablet');
				var disableMobile = $(this).attr('data-piotnetgrid-grid-carousel-disable-on-mobile');
				let windowWidth = window.innerWidth;
				var unslick = disableDesktop === 'yes' && windowWidth >= 992 || disableTablet === 'yes' && windowWidth < 992 && windowWidth > 767 || disableMobile === 'yes' && windowWidth < 768 ? true : false;

				if (!unslick) {
					$(this).find('[data-piotnetgrid-grid-posts]').slick(slick_options);
				}
			}
		});
	}

	var $piotnetgridSlick = $(document).find('[data-piotnetgrid-grid-carousel]');
	window.piotnetgridCarousel($piotnetgridSlick);
	$piotnetgridSlick.on("grid_carousels", function() {
		let $grid = $(document).find('[data-piotnetgrid-grid-posts]'),
			$parent = $grid.closest('[data-piotnetgrid-grid-carousel]');
		$grid.slick('unslick');
		window.piotnetgridCarousel($parent);
	});
	$('[data-piotnetgrid-trigger-action-carousel]').click(function() {
		var triggerActionData = JSON.parse($(this).attr('data-piotnetgrid-trigger-action-carousel')),
			$grid = $(document).find('#' + triggerActionData.grid_custom_id + '[data-piotnetgrid-grid-carousel]');
		
		if ($grid.length > 0) {
			$grid.find('[data-piotnetgrid-grid-posts]').slick(triggerActionData.action);
		}
	});

	function md5(d){return rstr2hex(binl2rstr(binl_md5(rstr2binl(d),8*d.length)))}function rstr2hex(d){for(var _,m="0123456789ABCDEF",f="",r=0;r<d.length;r++)_=d.charCodeAt(r),f+=m.charAt(_>>>4&15)+m.charAt(15&_);return f}function rstr2binl(d){for(var _=Array(d.length>>2),m=0;m<_.length;m++)_[m]=0;for(m=0;m<8*d.length;m+=8)_[m>>5]|=(255&d.charCodeAt(m/8))<<m%32;return _}function binl2rstr(d){for(var _="",m=0;m<32*d.length;m+=8)_+=String.fromCharCode(d[m>>5]>>>m%32&255);return _}function binl_md5(d,_){d[_>>5]|=128<<_%32,d[14+(_+64>>>9<<4)]=_;for(var m=1732584193,f=-271733879,r=-1732584194,i=271733878,n=0;n<d.length;n+=16){var h=m,t=f,g=r,e=i;f=md5_ii(f=md5_ii(f=md5_ii(f=md5_ii(f=md5_hh(f=md5_hh(f=md5_hh(f=md5_hh(f=md5_gg(f=md5_gg(f=md5_gg(f=md5_gg(f=md5_ff(f=md5_ff(f=md5_ff(f=md5_ff(f,r=md5_ff(r,i=md5_ff(i,m=md5_ff(m,f,r,i,d[n+0],7,-680876936),f,r,d[n+1],12,-389564586),m,f,d[n+2],17,606105819),i,m,d[n+3],22,-1044525330),r=md5_ff(r,i=md5_ff(i,m=md5_ff(m,f,r,i,d[n+4],7,-176418897),f,r,d[n+5],12,1200080426),m,f,d[n+6],17,-1473231341),i,m,d[n+7],22,-45705983),r=md5_ff(r,i=md5_ff(i,m=md5_ff(m,f,r,i,d[n+8],7,1770035416),f,r,d[n+9],12,-1958414417),m,f,d[n+10],17,-42063),i,m,d[n+11],22,-1990404162),r=md5_ff(r,i=md5_ff(i,m=md5_ff(m,f,r,i,d[n+12],7,1804603682),f,r,d[n+13],12,-40341101),m,f,d[n+14],17,-1502002290),i,m,d[n+15],22,1236535329),r=md5_gg(r,i=md5_gg(i,m=md5_gg(m,f,r,i,d[n+1],5,-165796510),f,r,d[n+6],9,-1069501632),m,f,d[n+11],14,643717713),i,m,d[n+0],20,-373897302),r=md5_gg(r,i=md5_gg(i,m=md5_gg(m,f,r,i,d[n+5],5,-701558691),f,r,d[n+10],9,38016083),m,f,d[n+15],14,-660478335),i,m,d[n+4],20,-405537848),r=md5_gg(r,i=md5_gg(i,m=md5_gg(m,f,r,i,d[n+9],5,568446438),f,r,d[n+14],9,-1019803690),m,f,d[n+3],14,-187363961),i,m,d[n+8],20,1163531501),r=md5_gg(r,i=md5_gg(i,m=md5_gg(m,f,r,i,d[n+13],5,-1444681467),f,r,d[n+2],9,-51403784),m,f,d[n+7],14,1735328473),i,m,d[n+12],20,-1926607734),r=md5_hh(r,i=md5_hh(i,m=md5_hh(m,f,r,i,d[n+5],4,-378558),f,r,d[n+8],11,-2022574463),m,f,d[n+11],16,1839030562),i,m,d[n+14],23,-35309556),r=md5_hh(r,i=md5_hh(i,m=md5_hh(m,f,r,i,d[n+1],4,-1530992060),f,r,d[n+4],11,1272893353),m,f,d[n+7],16,-155497632),i,m,d[n+10],23,-1094730640),r=md5_hh(r,i=md5_hh(i,m=md5_hh(m,f,r,i,d[n+13],4,681279174),f,r,d[n+0],11,-358537222),m,f,d[n+3],16,-722521979),i,m,d[n+6],23,76029189),r=md5_hh(r,i=md5_hh(i,m=md5_hh(m,f,r,i,d[n+9],4,-640364487),f,r,d[n+12],11,-421815835),m,f,d[n+15],16,530742520),i,m,d[n+2],23,-995338651),r=md5_ii(r,i=md5_ii(i,m=md5_ii(m,f,r,i,d[n+0],6,-198630844),f,r,d[n+7],10,1126891415),m,f,d[n+14],15,-1416354905),i,m,d[n+5],21,-57434055),r=md5_ii(r,i=md5_ii(i,m=md5_ii(m,f,r,i,d[n+12],6,1700485571),f,r,d[n+3],10,-1894986606),m,f,d[n+10],15,-1051523),i,m,d[n+1],21,-2054922799),r=md5_ii(r,i=md5_ii(i,m=md5_ii(m,f,r,i,d[n+8],6,1873313359),f,r,d[n+15],10,-30611744),m,f,d[n+6],15,-1560198380),i,m,d[n+13],21,1309151649),r=md5_ii(r,i=md5_ii(i,m=md5_ii(m,f,r,i,d[n+4],6,-145523070),f,r,d[n+11],10,-1120210379),m,f,d[n+2],15,718787259),i,m,d[n+9],21,-343485551),m=safe_add(m,h),f=safe_add(f,t),r=safe_add(r,g),i=safe_add(i,e)}return Array(m,f,r,i)}function md5_cmn(d,_,m,f,r,i){return safe_add(bit_rol(safe_add(safe_add(_,d),safe_add(f,i)),r),m)}function md5_ff(d,_,m,f,r,i,n){return md5_cmn(_&m|~_&f,d,_,r,i,n)}function md5_gg(d,_,m,f,r,i,n){return md5_cmn(_&f|m&~f,d,_,r,i,n)}function md5_hh(d,_,m,f,r,i,n){return md5_cmn(_^m^f,d,_,r,i,n)}function md5_ii(d,_,m,f,r,i,n){return md5_cmn(m^(_|~f),d,_,r,i,n)}function safe_add(d,_){var m=(65535&d)+(65535&_);return(d>>16)+(_>>16)+(m>>16)<<16|65535&m}function bit_rol(d,_){return d<<_|d>>>32-_}

	function replaceAll(str, find, replace) {
    	return str.replace(new RegExp(find, 'g'), replace);
    }

	function piotnetgrid_check_wishlist() {
		var wishlist_list = sessionStorage.getItem('piotnetgrid_wishlist') ? sessionStorage.getItem('piotnetgrid_wishlist').split(',') : [];

		if (wishlist_list.length > 0) {
			for (var i = 0; i < wishlist_list.length; i++) {
				$(document).find('[data-piotnetgrid-add-to-wishlist="' + wishlist_list[i] + '"]').addClass('piotnetgrid-add-to-wishlist--active');
			}
			$(document).find('[data-piotnetgrid-add-to-wishlist]').each(function(){
				var wishlist_id = $(this).attr('data-piotnetgrid-add-to-wishlist');
				if (wishlist_list.indexOf(wishlist_id) > -1) {
					$(this).addClass('piotnetgrid-add-to-wishlist--active');
				} else {
					$(this).removeClass('piotnetgrid-add-to-wishlist--active');
				}
			});
		}
	}

	window.piotnetgridMasonryLayout= function($grid,dom) {
		var macyInstances = [];
		for (var i = 0; i < $grid.length; i++) {
			if ($($grid[i]).attr('data-piotnetgrid-masonry') !== undefined) {
				var masonry_options = JSON.parse($($grid[i]).attr('data-piotnetgrid-masonry'));
				var newId = 'data-piotnetgrid-masonry-id-' + i;
				masonry_options.container = '[data-piotnetgrid-masonry-id-' + i + ']';
				$($grid[i]).find('[data-piotnetgrid-grid-posts]').attr(newId,'');
				if (dom) {
					masonry_options.piotDocument = dom;
				}
				macyInstances.push(Macy(masonry_options));
			}
		}
		let newMacyInstances = macyInstances.map(
			x => x.runOnImageLoad(function () {
				x.recalculate(true);
			}, true)
		);
	}

	function piotnetgridRemoveTheID() {
		$(document).find('[id="piotnetgrid"]').each(function(){
			if ($(this).find('[id="piotnetgrid"]').length > 0 && $(this).closest('[data-piotnetgrid-widget-preview]').length == 0) {
				$(this).find('[id="piotnetgrid"]').each(function(){
					$(this).removeAttr('id');
				});
			}
		});
	}

	piotnetgridRemoveTheID();

	window.piotnetgridFacet = function($this, $firstload) {
		var $parent = $this.closest('[data-piotnetgrid-facet]'),
			facet_current_data = JSON.parse($parent.attr('data-piotnetgrid-facet')),
			grid_post_id = facet_current_data.grid ? facet_current_data.grid : 0;

		var $apply_facets = $(document).find('[data-piotnetgrid-apply-facets=' + grid_post_id + ']');

		var	$grids = $(document).find('[data-piotnetgrid-shortcode-id=' + grid_post_id + '] [data-piotnetgrid-grid-id], .' + grid_post_id),
			$facets = $(document).find('[data-piotnetgrid-facet-grid=' + grid_post_id + ']'),
			facet_current_id = $parent.attr('data-piotnetgrid-facet-id'),
			facet_current_value = $this.val(),
			facet_current_name = facet_current_data.name ? facet_current_data.name : '',
			facet_action = facet_current_data.action ? facet_current_data.action : '',
			prev_request_url = window.location.pathname + window.location.search,
			cache_enable = false,
			cache_boost_loading = window[grid_post_id + '_cache_boost_loading'] !== undefined ? window[grid_post_id + '_cache_boost_loading'] : false,
			cache_expiration = '',
			lang = '',
			piotnetgrid = true,
			widgets = [];

		if ($this.closest('[data-piotnetgrid-facet-pagination]').length === 0) {
			if (grid_post_id !== 0) {
				$(document).find('[data-piotnetgrid-facet-grid="' + grid_post_id + '"]').not($parent).not('[data-piotnetgrid-facet-sort]').not('[data-piotnetgrid-facet-posts-per-page]').not('[data-piotnetgrid-facet-search]').addClass('piotnetgrid-loading');
			}
		}

		switch(facet_action) {
			case 'filter':
				var facet_current_type = facet_current_data.filter_type ? facet_current_data.filter_type : '';
				break;
			case 'load' :
				var facet_current_type = facet_current_data.loading_type ? facet_current_data.loading_type : '';
				break;
		}

		if ($apply_facets.length > 0) {
			var disable_facets_action = !$firstload && $apply_facets.attr('data-piotnetgrid-disable-facets-action') !== undefined && $apply_facets.attr('data-piotnetgrid-disable-facets-action-deactive') == undefined && $this.closest('[data-piotnetgrid-facet-pagination]').length === 0 ? true : false;
			if (facet_current_type !== 'pagination' && facet_current_type !== 'load_more') {
				$apply_facets.attr('data-piotnetgrid-apply-facets-current-facet', facet_current_id);
				var apply_facets_redirect_url = $apply_facets.attr('data-piotnetgrid-apply-facets-redirect');
			}
		}

		if (disable_facets_action) {
			$facets.removeClass('piotnetgrid-facet-loading').removeClass('piotnetgrid-loading');
		}

		if (facet_current_type == 'radio') {
			facet_current_value = '';
			$parent.find('[data-piotnetgrid-facet-field]').each(function(){
				if ($(this).is(":checked")) {
					facet_current_value = $(this).val();
				}
			});
		}

		if (facet_current_type == 'checkboxes') {
			facet_current_value = [];
			$parent.find('[data-piotnetgrid-facet-field]:checked').each(function(){
				facet_current_value.push($(this).val());
			});
			facet_current_value = facet_current_value.join();
		}

		if (facet_current_data['cache'] !== undefined) {
			cache_enable = true;
			if (window[grid_post_id + '_cache_boost_loading'] == undefined) {
				cache_boost_loading = true;
				window[grid_post_id + '_cache_boost_loading'] = cache_boost_loading;
			}
			cache_expiration = facet_current_data['cache_expiration'];
		}

		var url_query = window.location.search;

		if (facet_current_type !== 'load_more' || $apply_facets.length > 0 && apply_facets_redirect_url !== undefined) {

			if (apply_facets_redirect_url !== undefined) {
				var apply_facets_redirect_url_array = apply_facets_redirect_url.split('?');
				url_query = apply_facets_redirect_url_array.length > 1 ? '?' + apply_facets_redirect_url_array[1] : '';
			}

			if (facet_current_value) {
				url_query = piotnetgridReplaceQueryParam(facet_current_name, facet_current_value, url_query);
			} else {
				url_query = piotnetgridReplaceQueryParam(facet_current_name, false, url_query);
			}

			var url_query_last_char = url_query.charAt(url_query.length-1);

			if (url_query_last_char == '?' || url_query_last_char == '&') {
				url_query = url_query.substring(0, url_query.length - 1);
			}

			var url_query_first_char = url_query.charAt(0);

			if (url_query.indexOf('&') !== -1) {
				var url_query_array = url_query.substring(1, url_query.length);
				url_query_array = url_query_array.split('&').sort();
				url_query = '?' + url_query_array.join('&');
			}

			var request_url = window.location.pathname + url_query;

			if (!apply_facets_redirect_url) {
				if ($parent.attr('data-piotnetgrid-facet-disable-url') == undefined) {
					window.history.pushState({}, '', request_url);
				}
			} else {
				if (facet_current_type !== 'pagination') {
					$apply_facets.attr('data-piotnetgrid-apply-facets-redirect', apply_facets_redirect_url_array[0] + url_query);
				}
			}
		}

		if (facet_current_type == 'load_more') {
			var request_url = window.location.pathname + url_query;
		}

		if (grid_post_id !== 0 && $grids.length > 0 && !disable_facets_action) {
			if (facet_current_type != 'load_more') {
				$grids.css({'opacity': '0.5'});
			}

			var grid_ids = [],
				facets = {},
				current_query = undefined,
				main_query = undefined;

			$grids.each(function() {
				if ($(this).attr('data-piotnetgrid-grid-id') !== undefined) {
					grid_ids.push($(this).attr('data-piotnetgrid-grid-id'));
					widgets.push($(this).attr('data-piotnetgrid-grid-id'));
					current_query = $(this).attr('data-piotnetgrid-grid-current-query');
					lang = $(this).attr('data-piotnetgrid-grid-lang') != undefined ? $(this).attr('data-piotnetgrid-grid-lang') : '';
					if (!cache_enable && $(this).attr('data-piotnetgrid-grid-cache') !== undefined) {
						cache_enable = true;
						var cache_settings = JSON.parse( $(this).attr('data-piotnetgrid-grid-cache') );
						if (window[grid_post_id + '_cache_boost_loading'] == undefined) {
							cache_boost_loading = cache_settings['boost_loading'];
							window[grid_post_id + '_cache_boost_loading'] = cache_boost_loading;
						}
						cache_expiration = cache_settings['expiration'];
					}
				} else {
					if (window[grid_post_id] !== undefined) {
						grid_ids.push(grid_post_id);
						widgets.push(grid_post_id);
						current_query = window[grid_post_id]['query'];
						main_query = window[grid_post_id]['main_query'];
						piotnetgrid = false;
					}
				}
			});

			$facets.each(function() {
				var facet_post_id = $(this).closest('[data-piotnetgrid-shortcode-id]').attr('data-piotnetgrid-shortcode-id'),
					facet_data = JSON.parse($(this).attr('data-piotnetgrid-facet')),
					facet_action = facet_data.action,
					facet_value = $(this).find('[data-piotnetgrid-facet-field]').val(),
					facet_value_label = '',
					facet_label = $(this).find('[data-piotnetgrid-facet-label]').length > 0 ? $(this).find('[data-piotnetgrid-facet-label]').text() : '',
					facet_name = facet_data.name;

				switch(facet_action) {
					case 'filter':
						var facet_type = facet_data.filter_type ? facet_data.filter_type : '';
						break;
					case 'load' :
						var facet_type = facet_data.loading_type ? facet_data.loading_type : '';
						break;
				}

				if ($(this).find('select[data-piotnetgrid-facet-field]').length > 0 && facet_value != '') {
					facet_value_label = $(this).find('option[value="' + facet_value + '"]').attr('data-piotnetgrid-option-label');
				}

				if (facet_type == 'radio' || facet_type == 'az') {
					facet_value = '';
					$(this).find('[data-piotnetgrid-facet-field]').each(function(){
						if ($(this).is(":checked")) {
							facet_value = $(this).val();
							facet_value_label = $(this).attr('data-piotnetgrid-option-label');
						}
					});
				}

				if (facet_type == 'checkboxes') {
					facet_value = [];
					facet_value_label = [];
					$(this).find('[data-piotnetgrid-facet-field]:checked').each(function(){
						facet_value.push($(this).val());
						facet_value_label.push($(this).attr('data-piotnetgrid-option-label'));
					});
				}

				if (facet_type == 'slider') {
					var facet_value_from_url = window.piotnetgridGetUrlParameter( facet_name );
					if (facet_value_from_url != '') {
						facet_value = facet_value_from_url;
					}
					if (facet_value && facet_value.includes(',')) {
						facet_value = facet_value.split(',');
					}
				}

				var	facet = {
					id: $(this).attr('data-piotnetgrid-facet-id'),
					name: facet_data.name.substring(1),
					value: facet_value,
					value_label: facet_value_label !== '' || facet_value_label !== undefined ? facet_value_label : facet_value,
					action: facet_action,
					label: facet_label,
				};

				widgets.push($(this).attr('data-piotnetgrid-facet-id'));

				if ( facet_data.az_list !== undefined ) {
					facet['az_list'] = facet_data['az_list'];
				}

				if (!facets[facet_post_id]) {
					facets[facet_post_id] = [];
				}

				switch(facet_action) {
					case 'filter':
						facet['filter_type'] = facet_data.filter_type ? facet_data.filter_type : '';
						facet['filter_source'] = facet_data.filter_source ? facet_data.filter_source : '';
						facet['filter_taxonomy'] = facet_data.filter_taxonomy ? facet_data.filter_taxonomy : '';
						facet['filter_custom_field'] = facet_data.filter_custom_field ? facet_data.filter_custom_field : '';
						facet['filter_logic'] = facet_data.filter_logic ? facet_data.filter_logic : '';
						facet['exclude_terms'] = facet_data.exclude_terms ? facet_data.exclude_terms : '';
						break;
					case 'load' :
						facet['loading_type'] = facet_data.loading_type ? facet_data.loading_type : '';
						break;
				}

				facets[facet_post_id].push(facet);

				// remove pagination from url
				if (facet_type == 'pagination' && facet_current_type !== 'pagination' && !$firstload) {
					var url_query = url_query !== undefined ? url_query : window.location.search;

					if (request_url !== undefined) {
						if (request_url.includes('?')) {
							url_query = '?' + request_url.split('?')[1];
						}
					}

					url_query = piotnetgridReplaceQueryParam(facet_name, false, url_query);

					var url_query_last_char = url_query.charAt(url_query.length-1);

					if (url_query_last_char == '?' || url_query_last_char == '&') {
						url_query = url_query.substring(0, url_query.length - 1);
					}

					request_url = window.location.pathname + url_query;

					if ($parent.attr('data-piotnetgrid-facet-disable-url') == undefined) {
						window.history.pushState({}, '', window.location.pathname + url_query);
					}
				}
			});

			var data = {
				'facets': facets,
				'facet_current_id': facet_current_id,
				'facet_current_type': facet_current_type,
				'grid_post_id': grid_post_id,
				'grid_ids': grid_ids,
				'firstload': $firstload,
				'request_url': request_url !== undefined ? request_url : '',
				'current_url_path': window.location.href.replace(window.location.search,''),
				'cache_enable': cache_enable,
				'cache_expiration': cache_expiration,
				'lang': lang,
				'piotnetgrid': piotnetgrid
			};

			if (current_query) {
				data.current_query = current_query;
			}

			if (request_url !== undefined) {
				var request_url_md5 = md5(request_url + grid_post_id).toLowerCase();
			}

			if (window['cache_files'] !== undefined) {
				var cache_files_exist = window['cache_files'].filter(item => item.indexOf(request_url_md5 + '_') > -1);
			} else {
				var cache_files_exist = [];
			}
			
			if (request_url_md5 !== undefined && cache_files_exist.length > 0 && cache_boost_loading) {
				$.getJSON( piotnetgridWpcontentURL + '/cache/piotnetgrid/json/' + cache_files_exist[0] + '.json', function(response){

					if (response.html == undefined) {
			        	for(var k in response) {
			        		if (facet_current_type == 'load_more') {
			        			$(document).find('[data-piotnetgrid-grid-id=' + k + '] [data-piotnetgrid-grid-posts]').append($(replaceAll(response[k].outputHTML,'piotnetgrid-fadein', '')).find('[data-piotnetgrid-grid-posts]').html());
			        		} else {
			        			$(document).find('[data-piotnetgrid-grid-id=' + k + ']').replaceWith($(replaceAll(response[k].outputHTML,'piotnetgrid-fadein', '')));
			        		}
						    $(document).find('[data-piotnetgrid-facet-id=' + k + ']').replaceWith(replaceAll(response[k].outputHTML,'piotnetgrid-fadein', ''));
						    if ($firstload) {
						    	window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-id=' + k + ']'), true);
						    } else {
						    	window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-id=' + k + ']'));
						    }

							piotnetgridPhotoswipe();
							piotnetgridLazyload($(document).find('[data-piotnetgrid-grid-id=' + k + ']'));
							piotnetgridMasonryLayout($(document).find('[data-piotnetgrid-grid-id=' + k + ']'));
							piotnetgridCardAnimation($(document).find('[data-piotnetgrid-grid-id=' + k + ']'));
						    window.piotnetgridFacetDate($(document).find('[data-piotnetgrid-facet-id=' + k + '] [data-piotnetgrid-facet-date]'));
						    window.piotnetgridCarousel($(document).find('[data-piotnetgrid-grid-id=' + k + '][data-piotnetgrid-grid-carousel]'));
						    window.piotnetgridGalleryCarousel($(document).find('[data-piotnetgrid-grid-id=' + k + '] [data-piotnetgrid-gallery-carousel]'));
						}
					} else {

			        	for(var $k = 0; $k < widgets.length; $k++) {
			        		var $widget = $(replaceAll(response['html'],'piotnetgrid-fadein', '')).find('.' + widgets[$k]);

			        		if (facet_current_type == 'load_more') {
			        			$(document).find('.' + grid_post_id).append($widget.html());
			        		} else {
			        			if (piotnetgrid) {
			        			 	$(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + ']').replaceWith($widget);
		        				} else {
		        					let html = '<div>' + response['html'] + '</div>';
		        					if ($(html).find('.' + grid_post_id).length > 0) {
		        						var gridHTML = $(html).find('.' + grid_post_id).html();
	        						} else if ($(html).hasClass(grid_post_id)) {
	        							var gridHTML = $(html).html();
	        						}
	        						if (gridHTML !== undefined) {
	        							$(document).find('.' + grid_post_id).html( gridHTML );
        							}
		        				}
			        		}

			        		$(document).find('[data-piotnetgrid-facet-id=' + widgets[$k] + ']').replaceWith($widget);
						    if ($firstload) {
						    	window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-id=' + widgets[$k] + ']'), true);
						    } else {
						    	window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-id=' + widgets[$k] + ']'));
						    }

						    // macyInstance.recalculate();
							piotnetgridPhotoswipe();
							facetLimitChoicesOptions($(document).find('[data-piotnetgrid-facet-id=' + k + ']'));
							piotnetgridLazyload($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + ']'));
						    piotnetgridMasonryLayout($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + ']'));
						    piotnetgridCardAnimation($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + ']'));
						    window.piotnetgridFacetDate($(document).find('[data-piotnetgrid-facet-id=' + widgets[$k] + '] [data-piotnetgrid-facet-date]'));
						    window.piotnetgridCarousel($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + '][data-piotnetgrid-grid-carousel]'));
						    window.piotnetgridGalleryCarousel($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + '] [data-piotnetgrid-gallery-carousel]'));
		        		}
					}

					piotnetgrid_check_wishlist();

					$grids.css({'opacity': '1'});

					var $searchResultsSection = $('[data-piotnetgrid-search-results-section="' + facet_current_name.substring(1) + '"]');

					if ($searchResultsSection.length > 0) {
						if (facet_current_value.trim() !== '') {
							$searchResultsSection.show();
						} else {
							$searchResultsSection.hide();
						}
					}

					if ($apply_facets.length > 0) {
						$apply_facets.removeAttr('data-piotnetgrid-disable-facets-action-deactive');
					}

					piotnetgridRemoveTheID();

					$(document).find('.piotnetgrid-loading, .piotnetgrid-facet-loading').removeClass('piotnetgrid-loading').removeClass('piotnetgrid-facet-loading');
				});
			} else {
				let piotnetgridRestURLAjax = piotnetgridRestURL;
				if (!piotnetgrid) {
					piotnetgridRestURLAjax = request_url;
					data = {
						'get_html': true,
						'grid_post_id': grid_post_id,
						'current_query': current_query,
						'main_query': main_query,
						'request_url': request_url !== undefined ? request_url : '',
						'current_url_path': window.location.href.replace(window.location.search,'')
					};

					if (facet_current_data['cache'] !== undefined) {
						data['cache'] = true;
						data['cache_expiration'] = facet_current_data['cache_expiration'];
					}

					if (facet_current_type == 'load_more' && facet_current_value !== '') {
						data['load_more'] = facet_current_value;
					}
				}

				var dataType = piotnetgrid ? 'json' : 'text';

		        $.post(piotnetgridRestURLAjax, data, function(response) {
		        	if (piotnetgrid) {
		        		response = JSON.parse( response );
			        	for(var k in response['widgets']) {
			        		if (facet_current_type == 'load_more') {
			        			$(document).find('[data-piotnetgrid-grid-id=' + k + '] [data-piotnetgrid-grid-posts]').append($(replaceAll(response['widgets'][k].outputHTML,'piotnetgrid-fadein', '')).find('[data-piotnetgrid-grid-posts]').html());
			        		} else {
			        			if (piotnetgrid) {
			        				$(document).find('[data-piotnetgrid-grid-id=' + k + ']').replaceWith($(replaceAll(response['widgets'][k].outputHTML,'piotnetgrid-fadein', '')));
		        				} else {
		        					if (piotnetgridIsJsonString(response['widgets'][k].outputHTML)) {
		        						var gridHTML = JSON.parse(response['widgets'][k].outputHTML);
		        						$(document).find('.' + grid_post_id).html( $(gridHTML['html']).find('.' + grid_post_id).html() );
	        						}

		        				}
			        		}
						    $(document).find('[data-piotnetgrid-facet-id=' + k + ']').replaceWith(replaceAll(response['widgets'][k].outputHTML,'piotnetgrid-fadein', ''));
						    if ($firstload) {
						    	window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-id=' + k + ']'), true);
							} else {
						    	window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-id=' + k + ']'));
						    }

						    // macyInstance.recalculate();
							piotnetgridPhotoswipe();
							facetLimitChoicesOptions($(document).find('[data-piotnetgrid-facet-id=' + k + ']'));
							piotnetgridLazyload($(document).find('[data-piotnetgrid-grid-id=' + k + ']'));
						    piotnetgridMasonryLayout($(document).find('[data-piotnetgrid-grid-id=' + k + ']'));
						    piotnetgridCardAnimation($(document).find('[data-piotnetgrid-grid-id=' + k + ']'));
						    window.piotnetgridFacetDate($(document).find('[data-piotnetgrid-facet-id=' + k + '] [data-piotnetgrid-facet-date]'));
						    window.piotnetgridCarousel($(document).find('[data-piotnetgrid-grid-id=' + k + '][data-piotnetgrid-grid-carousel]'));
						    window.piotnetgridGalleryCarousel($(document).find('[data-piotnetgrid-grid-id=' + k + '] [data-piotnetgrid-gallery-carousel]'));
						}
	        		} else {
	        			var pos = response.indexOf('{"html');
                        if (-1 < pos) {
                            var response = JSON.parse(response.substr(pos));
                        }

                        if (facet_current_type == 'load_more' && facet_current_value !== '') {
		        			let html = '<div>' + response['html'] + '</div>';
        					if ($(html).find('.' + grid_post_id).length > 0) {
        						var $gridSelector = $(html).find('.' + grid_post_id);
        						if ($gridSelector.hasClass('oxy-woo-products')) {
        							var gridHTML = $gridSelector.find('.products').prop('outerHTML');
        						} else {
        							var gridHTML = $gridSelector.html();
        						}
    						} else if ($(html).hasClass(grid_post_id)) {
    							var $gridSelector = $(html);
        						if ($gridSelector.hasClass('oxy-woo-products')) {
        							var gridHTML = $gridSelector.find('.products').prop('outerHTML');
        						} else {
        							var gridHTML = $gridSelector.html();
        						}
    						}

    						if (gridHTML !== undefined) {
    							$(document).find('.' + grid_post_id).append( gridHTML );
							}
		        		}

			        	for(var $k = 0; $k < widgets.length; $k++) {
			        		var $widget = $(replaceAll(response['html'],'piotnetgrid-fadein', '')).find('.' + widgets[$k]);

			        		if (facet_current_type != 'load_more') {
			        			if (piotnetgrid) {
			        			 	$(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + ']').replaceWith($widget);
		        				} else {
		        					let html = '<div>' + response['html'] + '</div>';
		        					if ($(html).find('.' + grid_post_id).length > 0) {
		        						var gridHTML = $(html).find('.' + grid_post_id).html();
	        						} else if ($(html).hasClass(grid_post_id)) {
	        							var gridHTML = $(html).html();
	        						}
	        						if (gridHTML !== undefined) {
	        							$(document).find('.' + grid_post_id).html( gridHTML );
        							}
		        				}
			        		}

			        		$(document).find('[data-piotnetgrid-facet-id=' + widgets[$k] + ']').replaceWith($widget);
						    // if ($firstload) {
						    // 	window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-id=' + widgets[$k] + ']'), true);
						    // } else {
						    // 	window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-id=' + widgets[$k] + ']'));
						    // }

						    // macyInstance.recalculate();
							piotnetgridPhotoswipe();
							facetLimitChoicesOptions($(document).find('[data-piotnetgrid-facet-grid]'));
							piotnetgridLazyload($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + ']'));
							piotnetgridMasonryLayout($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + ']'));
						    piotnetgridCardAnimation($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + ']'));
						    window.piotnetgridFacetDate($(document).find('[data-piotnetgrid-facet-id=' + widgets[$k] + '] [data-piotnetgrid-facet-date]'));
						    window.piotnetgridCarousel($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + '][data-piotnetgrid-grid-carousel]'));
						    window.piotnetgridGalleryCarousel($(document).find('[data-piotnetgrid-grid-id=' + widgets[$k] + '] [data-piotnetgrid-gallery-carousel]'));
						    window.piotnetgridFacetSlider($(document).find('[data-piotnetgrid-facet-id=' + widgets[$k] + ']'));
		        			
		        			if (typeof oxygenVSBInitToggleState === 'function') {
		        				oxygenVSBInitToggleState();
		        			}
		        		}
	        		}

					if (response['cache_file'] !== undefined) {
						window['cache_files'].push(response['cache_file']);
					}

					piotnetgrid_check_wishlist();

					$grids.css({'opacity': '1'});

					var $searchResultsSection = $('[data-piotnetgrid-search-results-section="' + facet_current_name.substring(1) + '"]');

					if ($searchResultsSection.length > 0) {
						if (facet_current_value.trim() !== '') {
							$searchResultsSection.show();
						} else {
							$searchResultsSection.hide();
						}
					}

					if ($apply_facets.length > 0) {
						$apply_facets.removeAttr('data-piotnetgrid-disable-facets-action-deactive');
					}

					piotnetgridRemoveTheID();

					$(document).find('.piotnetgrid-loading, .piotnetgrid-facet-loading').removeClass('piotnetgrid-loading').removeClass('piotnetgrid-facet-loading');
					
				}, dataType);

				$('[data-piotnetgrid-query]').remove();
			}
		}
	}

	function piotnetgridIsJsonString(str) {
	    try {
	        JSON.parse(str);
	    } catch (e) {
	        return false;
	    }
	    return true;
	}

	$('[data-piotnetgrid-query]').each(function(){
		let query = $(this).attr('data-piotnetgrid-query');
		if (piotnetgridIsJsonString(query) && $(this).closest('[class*="piotnetgrid-id"]').length > 0) {
			let queryData = JSON.parse(query);
			if (queryData['piotnetgrid_id'] !== undefined) {
				window[queryData['piotnetgrid_id']] = queryData;
				let $facets = $(document).find('[data-piotnetgrid-facet-grid="' + queryData['piotnetgrid_id'] + '"]');
				if ($facets.length > 0) {
					$(this).closest('[class*="piotnetgrid-id"]').find('.oxy-repeater-pages-wrap, .oxy-easy-posts-pages').remove();
					window.piotnetgridFacet($facets.eq(0).find('[data-piotnetgrid-facet-field]').eq(0),true);
				}
			}
		}
		$(this).remove();
	});

	function piotnetgridCardAnimation($grids){
		var animationEffect = $grids.attr('data-piotnetgrid-grid-animation');

		if (animationEffect !== undefined) {
			var animationSpeed = $grids.attr('data-piotnetgrid-grid-animation-speed');
			var $items = $grids.find('.piotnetgrid-card');
			var itemArr = [];

			if ($items.length > 0) {
				for ( var i = 0; i < $items.length; i++ ){
					itemArr.push($items.eq(i));
				}
				$.each(itemArr, function(key,value){
					$(this).each(function () {
						$(this).delay(key * 100).queue(function () {
							$(this).addClass("animate__animated " + animationEffect + " " + animationSpeed).css('opacity', '1');
						});
					});
				});
			}
		}
	}

	function piotnetgridScrollToTop($this) {
		var $parent = $this.closest('[data-facet-pagination-scroll-to-top]');
		if ($parent.length > 0) {
			var facet_data = JSON.parse($parent.attr('data-piotnetgrid-facet')),
				grid_id = facet_data.grid ? facet_data.grid : '',
				$grids = $(document).find('[data-piotnetgrid-shortcode-id=' + grid_id + '] [data-piotnetgrid-grid-id], .' + grid_id),
				offset_desktop = $parent.attr('data-facet-pagination-scroll-to-top-offset-desktop'),
				offset_tablet = $parent.attr('data-facet-pagination-scroll-to-top-offset-tablet'),
				offset_mobile = $parent.attr('data-facet-pagination-scroll-to-top-offset-mobile'),
				height = $grids.offset().top;
			if ($grids.length > 0) {
				let windowWidth = window.innerWidth;
				if( windowWidth >= 1025 ) {
					$('html, body').animate({scrollTop:  height - offset_desktop },300);
				}
				if (windowWidth >= 768 && windowWidth < 1025) {
					$('html, body').animate({scrollTop:  height - offset_tablet },300);
				}
				if (windowWidth < 768) {
					$('html, body').animate({scrollTop:  height - offset_mobile },300);
				}
			}
		}
	}

	$(document).on( 'click','[data-piotnetgrid-facet-pagination-page-number]', function(){
		var page_number = $(this).attr('data-piotnetgrid-facet-pagination-page-number');
		$(this).closest('[data-piotnetgrid-facet-id]').find('[data-piotnetgrid-facet-field]').val(page_number).change();
		piotnetgridScrollToTop($(this));
	});

	$(document).on( 'click','[data-piotnetgrid-facet-load-more]', function(){
		var $parent = $(this).closest('[data-piotnetgrid-facet]'),
			$facetField = $parent.find('[data-piotnetgrid-facet-field]'),
			load_more_data = JSON.parse($(this).attr('data-piotnetgrid-facet-load-more')),
			page_max = load_more_data.max,
			paged = $facetField.val();

		$(this).addClass('piotnetgrid-button--loading');

		if (paged == '') {
			paged = 1;
		}

		if (page_max > paged) {
			$facetField.val(parseFloat(paged) + 1).change();
		}
	});

	var $gridAnimation = $(document).find('[data-piotnetgrid-grid-animation]');

	if ($gridAnimation.length === 1){
		piotnetgridCardAnimation($gridAnimation);
	} else if ($gridAnimation.length > 1) {
		$gridAnimation.each(function() {
			var animationDelay = $(this).attr('data-piotnetgrid-grid-animation-delay');
			setTimeout(function(element) {
				piotnetgridCardAnimation(element);
			},animationDelay, $(this));
		});
	}

	$(document).on( 'keyup','[data-piotnetgrid-facet-search] [data-piotnetgrid-facet-field]', $.debounce( 300, function(e){
			window.piotnetgridFacet($(this),false);
	    })
	);

	$(document).on( 'change','[data-piotnetgrid-facet-field]', $.debounce( 250, function(e){
			if ($(this).closest('[data-piotnetgrid-facet-search]').length == 0) {
				window.piotnetgridFacet($(this),false);
			}
	    })
	);

	// $(document).find('[data-piotnetgrid-facet-field]').each(function() {
	// 	if (($(this).attr('type') == 'checkbox' || $(this).attr('type') == 'radio')) {
	// 		if ($(this).is(":checked") && $(this).val() !== '') {
	// 			window.piotnetgridFacet($(this),false);
	// 		}
	// 	}
	// });

	$(document).on( 'change','[data-piotnetgrid-facet-field]', function(e){
		if ($(this).closest('[data-piotnetgrid-facet-search]').length == 0) {
			$(this).closest('[data-piotnetgrid-facet-grid]').addClass('piotnetgrid-facet-loading');
		}
    });

	$(document).on( 'click','[data-piotnetgrid-apply-facets-redirect]:not([data-piotnetgrid-apply-facets-redirect-url])', function() {
		window.location.href = $(this).attr('data-piotnetgrid-apply-facets-redirect');
	});

	$(document).on( 'click','[data-piotnetgrid-apply-facets-redirect-url]', function() {
		window.location.href = $(this).attr('data-piotnetgrid-apply-facets-redirect-url');
	});

	$(document).on( 'click','[data-piotnetgrid-disable-facets-action]', function() {
		$(this).attr('data-piotnetgrid-disable-facets-action-deactive','');
		var current_facet_id = $(this).attr('data-piotnetgrid-apply-facets-current-facet');
		if (current_facet_id) {
			var $checkedField = $(document).find('[data-piotnetgrid-facet-id="' + current_facet_id + '"] [data-piotnetgrid-facet-field][type="checkbox"]:checked');
			if ($checkedField.length > 0) {
				$checkedField.change();
			} else {
				var $currentField = $(document).find('[data-piotnetgrid-facet-id="' + current_facet_id + '"] [data-piotnetgrid-facet-field]');
				if ($currentField.closest('[data-piotnetgrid-facet-search]').length == 0) {
					$currentField.change()
				} else {
					$currentField.trigger('keyup');
				}
			}
		}
	});

	$(document).on( 'change','[data-piotnetgrid-facet-radio] [data-piotnetgrid-facet-field]', function() {
		$(this).closest('[data-piotnetgrid-facet-radio]').find('[data-piotnetgrid-facet-field]').not(this).prop('checked',false);
	});

	//Button remove facet date
	$(document).on('click', '.piotnetgrid-remove-date-field', function(){
		let $parent = $(this).closest('.piotnetgrid-facet__field-wrapper');
		let facetField = $parent.find('[data-piotnetgrid-facet-field]');
		if(facetField.val()){
			facetField.val('');
			facetField.change();
		}
	});

	$('.piotnetgrid-button--add-to-cart').click(function(){
		$(this).addClass('piotnetgrid-loading');
	});

	$(document).on('click', '[data-piotnetgrid-collapse-section-button]', function(){
		$('body').addClass('piotnetgrid-modal-open');
		$(this).closest('.piotnetgrid-button-wrapper').siblings('[data-piotnetgrid-collapse-section]').addClass('piotnetgrid-collapse-section--open');
		$(this).closest('.piotnetgrid-button-wrapper').siblings('[data-piotnetgrid-collapse-section-bg]').addClass('piotnetgrid-collapse-section-bg-open');
	});

	$(document).on('mousedown touchstart',function (e) {
		if (!$('[data-piotnetgrid-collapse-section], [data-piotnetgrid-collapse-section] *').is(e.target) && !$('[data-piotnetgrid-modal-content], [data-piotnetgrid-modal-content] *').is(e.target)) {
			$(document).find('.piotnetgrid-collapse-section--open').removeClass('piotnetgrid-collapse-section--open');
			$(document).find('.piotnetgrid-collapse-section-bg-open').removeClass('piotnetgrid-collapse-section-bg-open');
			$('body').removeClass('piotnetgrid-modal-open');
		}
	});

	function piotnetgridLazyload($selector) {
		if ($selector.find('.piotnetgrid-image__tag--lazy').length > 0) {
			var myLazyLoad = new LazyLoad({
				elements_selector: ".piotnetgrid-image__tag--lazy",
			});
		}
	}
	piotnetgridLazyload($(document));

	$('[data-piotnetgrid-shortcode-title]').each(function(){
		var shortcodeID = $(this).attr('data-piotnetgrid-shortcode-id');
		if ($('#wp-admin-bar-piotnetgrid-default').find('[data-piotnetgrid-edit-template="' + shortcodeID + '"]').length == 0) {
			$('#wp-admin-bar-piotnetgrid-default').append('<li data-piotnetgrid-edit-template="' + shortcodeID + '"><a class="ab-item" href="' + piotnetgridAdminURL + 'admin.php?page=piotnetgrid&post=' + shortcodeID + '">Edit ' + $(this).attr('data-piotnetgrid-shortcode-title') + '</a></li>');
		}
	});

	setTimeout(function() {
		let $selector = $(document).find('[data-piotnetgrid-masonry]');
			piotnetgridMasonryLayout($selector);

	}, 500);

	setTimeout(function() {
		$('.piotnetgrid-fadein').removeClass('piotnetgrid-fadein');
	}, 1000);

	$('.piotnetgrid:not(.piotnetgrid-widget-preview) [data-piotnetgrid-auto-scrolling]').each(function() {

		var $parent = $(this).closest('.piotnetgrid-section');
		$parent.css({'overflow':'hidden'});

		var $clone = $(this).children().clone(true).addClass('piotnetgrid-auto-scrolling-item-clone');
		var i = 1;
		while(i <= 2){
			$clone.clone(true).appendTo($(this));
			i++;
		}

		var $scrolling = $parent.find('[data-piotnetgrid-auto-scrolling]');
		var $scrollingClone = $scrolling.clone(true).addClass('piotnetgrid-auto-scrolling-clone').removeAttr('data-piotnetgrid-auto-scrolling');
		$scrollingClone.clone(true).appendTo($parent);

		var mosaicWidth = $(this).outerWidth(),
		mosaicMaxLeft = (mosaicWidth/3) * 2;

		$(this).css({'left' : '-' + mosaicWidth/3 + 'px'});

		function mosaicLoop(mosaic, direction, speed, leftCurrent) {
			if(leftCurrent == undefined) {
				var leftCurrent = parseInt(mosaic.css('left').replace("px", ""));
			} else {
				leftCurrent = 0-leftCurrent;
			}

			if(-leftCurrent > mosaicMaxLeft) {
				leftCurrent = - (-leftCurrent - mosaicMaxLeft + mosaicWidth/3);
			}

			mosaic.css({'left':leftCurrent+'px'}); 

			if (direction == 'next') {
				var time = parseInt(Math.round(0 - ( 0 - mosaicMaxLeft - leftCurrent ) / speed));
				mosaic.animate({'left' : '-' + mosaicMaxLeft + 'px'}, time, 'linear', function() {
			    	mosaic.css({'left' : '-' + mosaicWidth/3 + 'px'});
			    	mosaicLoop($(this), direction, speed);
			    });
			} else {
				var time = parseInt(Math.round(0 - leftCurrent ) / speed);
				mosaic.animate({'left' : '0px'}, time, 'linear', function() {
			    	mosaic.css({'left' : '-' + mosaicWidth/3 + 'px'});
			    	mosaicLoop($(this), direction, speed);
			    });
			}
		}

		mosaicLoop($(this),'next',0.04);

		$(this).mousemove(function(event){
			$(this).stop();
			var mouseX = event.clientX,
				windowWidth = $(window).width();
			if( windowWidth>=992 ) {
				if(mouseX < windowWidth*0.2) {
					mosaicLoop($('[data-piotnetgrid-auto-scrolling]'),'prev',0.2);
				}
				if(mouseX >= windowWidth*0.2 && mouseX <= windowWidth*0.8) {
					$('[data-piotnetgrid-auto-scrolling]').stop();
				}
				if(mouseX > windowWidth*0.8) {
					mosaicLoop($('[data-piotnetgrid-auto-scrolling]'),'next',0.2); 
				}
			}
		}); 

		$(this).on('touchstart', function(){
			$(this).stop();
			var leftCurrent = 0 - parseInt($(this).css('left').replace("px", ""));
			$(this).hide();
			$(this).css({'left':0});
			$(this).parent().find('.piotnetgrid-auto-scrolling-clone').css({'display':'flex'});
			if(leftCurrent != 0) {
				$(this).closest('.piotnetgrid-section').css({'overflow':'scroll'});
				$(this).closest('.piotnetgrid-section').scrollLeft(leftCurrent);
			}
		});

		$(document).on('touchstart',function (e) {
		    if (!$('[data-piotnetgrid-auto-scrolling], .piotnetgrid-auto-scrolling-clone, [data-piotnetgrid-auto-scrolling] *, .piotnetgrid-auto-scrolling-clone *').is(e.target) ) {
		    	if ($scrolling.css('display') == 'none') {
			    	var scrollCurrent = parseInt($('[data-piotnetgrid-auto-scrolling]').closest('.piotnetgrid-section').scrollLeft());
			    	$scrolling.css({'display':'flex'});
					$scrolling.parent().find('.piotnetgrid-auto-scrolling-clone').hide();
			    	$scrolling.stop();		
					$scrolling.closest('.piotnetgrid-section').css({'overflow':'hidden'});
					$scrolling.closest('.piotnetgrid-section').scrollLeft(0);
					mosaicLoop($scrolling,'next',0.04,scrollCurrent);
				}
		    }
		});

		$(this).mouseleave(function(){
			var windowWidth = $(window).width();
			$scrolling.stop();
			mosaicLoop($(this),'next',0.04);
			if( windowWidth<992 ) {
				$scrolling.closest('.piotnetgrid-section').css({'overflow':'hidden'});
			}
		});

	});

	function piotnetGridIframeHeight() {
		$('[data-piotnetgrid-video] iframe[src*="youtube"],[data-piotnetgrid-video] iframe[src*="vimeo"]').each(function(){
			var width = $(this).width();
			var height = width * 9/16;
			$(this).css({'height' : height + 'px'});
			$(this).closest('[data-piotnetgrid-video]').css({'height' : height + 'px'});
		});
	};

	piotnetGridIframeHeight();

	$(window).resize(function(event) {
		piotnetGridIframeHeight();
	});

	let $facet = $(document).find('[data-piotnetgrid-facet-grid]');
    function facetLimitChoicesOptions($facet) {
		$facet.each(function(){
			let $facetLimitChoicesOptions = $(this).find('[data-piotnetgrid-limit-choices-option]');
			if ( $facetLimitChoicesOptions.length > 0 ) {
				let limitChoiceNumber = $facetLimitChoicesOptions.attr('data-piotnetgrid-limit-choices-option');
				let $options = $facetLimitChoicesOptions.find('.piotnetgrid-facet__field-sub');
				let $hideOptions = $options.slice(limitChoiceNumber).hide();
				let $showButton = $facetLimitChoicesOptions.find('.piotnetgrid-facet__show-more-less-btn--more');
				let $hideButton = $facetLimitChoicesOptions.find('.piotnetgrid-facet__show-more-less-btn--less');

				if ($showButton.length > 0) {
					$showButton.click(function () {
						$hideOptions.show();
						$(this).hide();
						$hideButton.show();
					});
				}

				if ($hideButton.length > 0) {
					$hideButton.hide();
					$hideButton.click(function () {
						$hideOptions.hide();
						$(this).hide();
						$showButton.show();
					});
				}
			}
		});
    }
    facetLimitChoicesOptions($facet);

});
