<?php
	if ( ! defined( 'ABSPATH' ) ) { exit; }

	require_once __DIR__ . '/../../source/import.php';

	add_action( 'wp_ajax_piotnetgrid_import_demo', 'piotnetgrid_import_demo' );
	add_action( 'wp_ajax_nopriv_piotnetgrid_import_demo', 'piotnetgrid_import_demo' );

	function piotnetgrid_import( $file_content ) {
		$data = json_decode( $file_content, true );
		if ( json_last_error() > 0 ) {
			$last_error_msg = json_last_error_msg();
			print_r( "Can't parse file: " . $last_error_msg );
		} elseif ( array_key_exists( 'error', $data ) ) {
			print_r( 'Error: ' . $data['error'] );
		} else {
			$post = [
				'post_title'  => $data['title'],
				'post_status' => 'publish',
				'post_type'   => $data['post_type'],
			];

			$post_id = wp_insert_post( $post );

			if ( !is_wp_error( $post_id ) ) {
				piotnetgrid_do_import( $post_id, $data );
				return $post_id;
			}
		}
	}

	function piotnetgrid_import_demo() {

		$arrContextOptions=array(
		    "ssl"=>array(
		        "verify_peer"=>false,
		        "verify_peer_name"=>false,
		    ),
		);  

		if (!empty($_POST['file'])) {
			$file = $_POST['file'];
			if (is_array($file)) {
				foreach ($file as $key => $file_item) {
					switch ($key) {
						case 'card':
							$card_ids = [];

							foreach ($file_item as $file_item_folder) {
								$file_url = WP_PLUGIN_DIR . '/piotnetgrid/assets/demos/templates/' . $file_item_folder . '/file.json';
								$file_content = file_get_contents( $file_url, false, stream_context_create($arrContextOptions) );
								$file_content_data = json_decode($file_content, true);
								$card_ids[$file_content_data['post_id']] = piotnetgrid_import( $file_content );
							}

							break;
						
						case 'grid':
							$grid_ids = [];

							foreach ($file_item as $file_item_folder) {
								$file_url = WP_PLUGIN_DIR . '/piotnetgrid/assets/demos/templates/' . $file_item_folder . '/file.json';
								$file_content = file_get_contents( $file_url, false, stream_context_create($arrContextOptions) );
								$file_content_data = json_decode($file_content, true);

								foreach ($file_content_data['widgets'] as $widget_id => $grid_widget) {
									if ($grid_widget['type'] == 'grid') {
										$card_template = floatval( str_replace('export', '', $grid_widget['settings']['card_template']) );
										if (!empty($card_ids[$card_template])) {
											$file_content_data['widgets'][$widget_id]['settings']['card_template'] = $card_ids[$card_template];
										}
									}
								}

								$file_content = json_encode($file_content_data);

								$grid_ids[$file_content_data['post_id']] = piotnetgrid_import( $file_content );
							}

							break;

						case 'facet':
							$facet_ids = [];

							foreach ($file_item as $file_item_folder) {
								$file_url = WP_PLUGIN_DIR . '/piotnetgrid/assets/demos/templates/' . $file_item_folder . '/file.json';
								$file_content = file_get_contents( $file_url, false, stream_context_create($arrContextOptions) );
								$file_content_data = json_decode($file_content, true);
								$facet_ids[$file_content_data['post_id']] = piotnetgrid_import( $file_content );
							}

							break;

						case 'template':

							foreach ($file_item as $file_item_folder) {
								$file_url = WP_PLUGIN_DIR . '/piotnetgrid/assets/demos/templates/' . $file_item_folder . '/file.json';
								$file_content = file_get_contents( $file_url, false, stream_context_create($arrContextOptions) );
								$file_content_data = json_decode($file_content, true);

								foreach ($file_content_data['widgets'] as $widget_id => $widget) {
									if ($widget['type'] == 'grid_embed') {
										$grid_id = floatval( str_replace('export', '', $widget['settings']['grid']) );
										if (!empty($grid_ids[$grid_id])) {
											$file_content_data['widgets'][$widget_id]['settings']['grid'] = $grid_ids[$grid_id];
										}
									}

									if ($widget['type'] == 'facet_embed') {
										$facet_id = floatval( str_replace('export', '', $widget['settings']['facet']) );
										if (!empty($facet_ids[$facet_id])) {
											$file_content_data['widgets'][$widget_id]['settings']['facet'] = $facet_ids[$facet_id];
										}

										$facet_grid_id = floatval( str_replace('export', '', $widget['settings']['facet_grid']) );
										if (!empty($grid_ids[$facet_grid_id])) {
											$file_content_data['widgets'][$widget_id]['settings']['facet_grid'] = $grid_ids[$facet_grid_id];
										}
									}
								}

								$file_content = json_encode($file_content_data);

								piotnetgrid_import( $file_content );
							}
					}
				}
			} else {
				$file_url = WP_PLUGIN_DIR . '/piotnetgrid/assets/demos/templates/' . $file;
				$file_content = file_get_contents( $file_url, false, stream_context_create($arrContextOptions) );
				piotnetgrid_import( $file_content );
			}
		}

		wp_die();
	}