<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Base extends piotnetgrid_Variables_Pro {

	public $settings;
	public $post_id;
	private $additional_attributes;
	public $widget_id;
	private $editor;

	public function __construct() {
		parent::__construct();
		$this->post_id = !empty($_GET['post']) ? $_GET['post'] : '';

		if ( method_exists($this, 'get_script') ){
            $scripts = $this->get_script();
            if ( !empty($scripts) ) {
            	foreach ($scripts as $script) {
            		wp_enqueue_script( $script );
            	}
            }
        }

        if ( method_exists($this, 'get_style') ){
            $styles = $this->get_style();
            if ( !empty($styles) ) {
            	foreach ($styles as $style) {
            		wp_enqueue_style( $style );
            	}
            }
        }
	}

	public function get_id() {
		return $this->widget_id;
	}

	public function get_settings() {
		return $this->settings;
	}

	public function widget_visibility() {
		$settings   = $this->settings;
		$visibility = true;
		if ( ! empty( $settings['conditional_visibility_enable'] ) ) {
			if ( 'yes' === $settings['conditional_visibility_enable'] ) {
				$visibility_roles = $settings['conditional_visibility_roles'];
				$condition1       = false;
				$user             = wp_get_current_user();
				$user_roles       = $user->roles;
				if ( in_array( 'all', $visibility_roles ) || in_array( 'logged_in', $visibility_roles ) && is_user_logged_in() || in_array( 'non_logged_in', $visibility_roles ) && ! is_user_logged_in() ) {
					$condition1 = true;
				}

				if ( isset( $user_roles[0] ) ) {
					if ( in_array( $user_roles[0], $visibility_roles ) ) {
						$condition1 = true;
					}
				}

				$condition2 = true;

				$show = 'show';

				if ( ! empty( $settings['conditional_visibility_by_backend'] ) ) {
					if ( array_key_exists( 'conditional_visibility_by_backend_list', $settings ) ) {
						$list = $settings['conditional_visibility_by_backend_list'];
						$show = $settings['conditional_visibility_action'];

						if ( ! empty( $list[0]['conditional_visibility_by_backend_select'] ) ) {
							$conditionals_count  = count( $list );
							$conditionals_and_or = '';
							$error               = 0;
							$condition           = false;
							foreach ( $list as $item ) {
								$conditionals_and_or = $item['conditional_visibility_and_or_operators'];
								if ( $item['conditional_visibility_by_backend_select'] === 'custom_field' && ! empty( $item['conditional_visibility_custom_field_key'] ) ) {

									$field_key        = $item['conditional_visibility_custom_field_key'];
									$field_source     = $item['conditional_visibility_custom_field_source'];
									$field_value      = '';
									$comparison       = $item['conditional_visibility_custom_field_comparison_operators'];
									$comparison_value = isset( $item['conditional_visibility_custom_field_value'] ) ? $item['conditional_visibility_custom_field_value'] : '';
									$id               = get_the_ID();

									if ( $field_source === 'post_custom_field' ) {
										$field_value = get_post_meta( $id, $field_key, true );
									} else {
										if ( function_exists( 'get_field' ) ) {
											$field_value = get_field( $field_key, $id );
										}
									}

									if ( isset( $item['conditional_visibility_custom_field_type'] ) ) {
										if ( $item['conditional_visibility_custom_field_type'] === 'number' ) {
											$field_value = floatval( $field_value );
										}
									}

									if ( is_array( $field_value ) && $comparison === 'contains' ) {
										if ( in_array( $comparison_value, $field_value ) ) {
											$condition = true;
										} else {
											$error++;
										}
									} else {
										if ( $comparison === 'not-empty' && ! empty( $field_value ) || $comparison === 'empty' && empty( $field_value ) || $comparison === 'true' && $field_value === true || $comparison === 'false' && $field_value === false || $comparison === '=' && $field_value === $comparison_value || $comparison === '!=' && $field_value !== $comparison_value || $comparison === '>' && $field_value > $comparison_value || $comparison === '>=' && $field_value >= $comparison_value || $comparison === '<' && $field_value < $comparison_value || $comparison === '<=' && $field_value <= $comparison_value ) {
											$condition = true;
										} else {
											$error++;
										}
									}
								}

								if ( $item['conditional_visibility_by_backend_select'] === 'url_parameter' && ! empty( $item['conditional_visibility_url_parameter'] ) ) {

									$url_parameter    = $item['conditional_visibility_url_parameter'];
									$comparison       = $item['conditional_visibility_custom_field_comparison_operators'];
									$comparison_value = isset( $item['conditional_visibility_custom_field_value'] ) ? $item['conditional_visibility_custom_field_value'] : '';
									$field_value      = '';

									if ( ! empty( $_GET[ $url_parameter ] ) ) {
										$field_value = $_GET[ $url_parameter ];
									}

									if ( isset( $item['conditional_visibility_custom_field_type'] ) ) {
										if ( $item['conditional_visibility_custom_field_type'] === 'number' ) {
											$field_value = floatval( $field_value );
										}
									}

									if ( $comparison === 'not-empty' && ! empty( $field_value ) || $comparison === 'empty' && empty( $field_value ) || $comparison === 'true' && $field_value === true || $comparison === 'false' && $field_value === false || $comparison === '=' && $field_value === $comparison_value || $comparison === '!=' && $field_value !== $comparison_value || $comparison === '>' && $field_value > $comparison_value || $comparison === '>=' && $field_value >= $comparison_value || $comparison === '<' && $field_value < $comparison_value || $comparison === '<=' && $field_value <= $comparison_value ) {
										$condition = true;
									} else {
										$error++;
									}
								}

								if ( $item['conditional_visibility_by_backend_select'] === 'url_contains' && ! empty( $item['conditional_visibility_custom_field_value_url_contains'] ) ) {

									$url_contains = $item['conditional_visibility_custom_field_value_url_contains'];
									$actual_link  = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http' ) . "://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
									$find         = strpos( $actual_link, $url_contains );

									if ( $find !== false ) {
										$condition = true;
									} else {
										$error++;
									}
								}
							}

							if ( $conditionals_and_or === 'or' ) {
								if ( $conditionals_count <= $error ) {
									$condition2 = false;
								}
							}

							if ( $conditionals_and_or === 'and' ) {
								if ( $error !== 0 ) {
									$condition2 = false;
								}
							}
						}
					}
				}

				if ( $condition1 === true && $condition2 === true && $show === 'show' ) {
					$visibility = true;
				} elseif ( $condition1 === true && $condition2 === false && $show === 'hide' ) {
					$visibility = true;
				} else {
					$visibility = false;
				}
			} else {
				$visibility = true;
			}
		}

		return $visibility;
	}

	public function dynamic_css() {
		$settings = $this->settings;

		$this->register_controls();
		$control_dynamic_css = $this->control_dynamic_css;
		$dynamic_css = '';
		$widget_id = $this->get_id();
		$class_id = !empty($GLOBALS['term']) && is_object($GLOBALS['term']) ? $GLOBALS['term']->term_id : get_the_ID();
		
		foreach ($control_dynamic_css as $control_name => $control_dynamic_css_item) {
			if (!empty($settings[$control_name])) {
				$control_value = piotnetgrid_dynamic_tags($settings[$control_name]);
				if ($control_value !== "") {
					foreach ($control_dynamic_css_item as $control_selectors => $control_css) {
						$control_selectors = str_replace('{{WRAPPER}}', '.piotnetgrid.piotnetgrid-' . $class_id . ' .' . $widget_id, $control_selectors);
						$control_selectors = str_replace('{{WIDGET}}', '.' . $widget_id, $control_selectors);
						$control_selectors = str_replace('{{WRAPPER_EDITOR}}', '.piotnetgrid.piotnetgrid-' . $class_id . ' [data-piotnetgrid-editor-widgets-item-id="' . $widget_id . '"]', $control_selectors);
						$control_selectors = str_replace('{{WRAPPER_CARD}}', '.piotnetgrid.piotnetgrid-' . $class_id . ' .piotnetgrid-card', $control_selectors);

						$control_css = str_replace('{{VALUE}}', $control_value, $control_css);

						$dynamic_css .= $control_selectors . '{' . $control_css . '}';
					}
				}
			}
		}

		if (!empty($dynamic_css)) {
			echo '<style>' . $dynamic_css . '</style>';
		}
	}

	public function custom_css() {
		$settings = $this->settings;
		$custom_css = '';
		$widget_id = $this->get_id();

		if (!empty($settings['advanced_custom_css'])) {
			$custom_css = $settings['advanced_custom_css'];
			$custom_css = str_replace('{{WRAPPER}}', '#piotnetgrid .' . $widget_id, $custom_css);
		}

		if (!empty($custom_css)) {
			echo '<style>' . $custom_css . '</style>';
		}
	}

	public function output_wrapper_start( $widget_id = '', $editor = false ) {
		ob_start();

		$widget_information = [
			'type'       => $this->get_type(),
			'class_name' => $this->get_class_name(),
			'title'      => $this->get_title(),
			'icon'       => $this->get_icon(),
		];

		$settings = $this->settings;

		$visibility = true;

		if ( ! $editor ) {
			$visibility = $this->widget_visibility();
		}

		if ( $editor ) {
			$this->add_render_attribute( 'widget_wrapper', 'class', 'piotnetgrid-widget' );
			$this->add_render_attribute( 'widget_wrapper', 'data-piotnetgrid-editor-widgets-item', json_encode( $widget_information ) );
			$this->add_render_attribute( 'widget_wrapper', 'data-piotnetgrid-editor-widgets-item-id', $widget_id );

			if ( $this->get_type() === 'section' ) {
				$this->add_render_attribute( 'widget_wrapper', 'data-piotnetgrid-editor-widgets-item-section', '' );
				$this->add_render_attribute( 'widget_wrapper', 'data-piotnetgrid-editor-section', '' );
				$this->add_render_attribute( 'widget_wrapper', 'draggable', 'true' );
			}

			if ( $this->get_type() === 'column' ) {
				$this->add_render_attribute( 'widget_wrapper', 'data-piotnetgrid-editor-column', '' );
			}
		}

		if ( $this->get_type() === 'section' ) {
			$this->add_render_attribute( 'widget_wrapper', 'class', 'piotnetgrid-section' );
			if ( !empty($settings['search_results_section_enable']) && !empty($settings['search_results_section_facet_name']) ) {
				$this->add_render_attribute( 'widget_wrapper', 'data-piotnetgrid-search-results-section', $settings['search_results_section_facet_name'] );
			}
		}

		if ( $this->get_type() === 'column' ) {
			$this->add_render_attribute( 'widget_wrapper', 'class', 'piotnetgrid-column' );
		}

		if ( $this->get_type() === 'section' || $this->get_type() === 'column' ) {
			$this->add_render_attribute( 'widget_wrapper', 'class', $this->widget_id );

			if ( ! empty( $settings['advanced_custom_id'] ) ) {
				$this->add_render_attribute( 'widget_wrapper', 'id', $settings['advanced_custom_id'] );
			}

			if ( ! empty( $settings['advanced_custom_classes'] ) ) {
				$this->add_render_attribute( 'widget_wrapper', 'class', $settings['advanced_custom_classes'] );
			}

			if ( ! empty( $settings['piotnetgrid_repeater_enable'] ) && ! empty( $settings['piotnetgrid_repeater_form_id'] ) && ! empty( $settings['piotnetgrid_repeater_id'] ) && ! empty( $settings['piotnetgrid_repeater_label'] ) ) {
				$this->add_render_attribute(
					'widget_wrapper',
					[
						'data-piotnetgrid-repeater-form-id' => $settings['piotnetgrid_repeater_form_id'],
						'data-piotnetgrid-repeater-id'    => $settings['piotnetgrid_repeater_id'],
						'data-piotnetgrid-repeater-label' => $settings['piotnetgrid_repeater_label'],
						'data-piotnetgrid-repeater-limit' => isset( $settings['piotnetgrid_repeater_limit'] ) ? $settings['piotnetgrid_repeater_limit'] : '',
					]
				);
				wp_enqueue_script( $this->slug . '-advanced-script' );
			}

			if ( !empty( $settings['advanced_overflownormal'] ) && $settings['advanced_overflownormal'] == 'hidden' ) {
				if (!empty( $settings['advanced_border_radius_normal_responsive_desktop'] ) || !empty( $settings['advanced_border_radius_normal_responsive_tablet'] ) || !empty( $settings['advanced_border_radius_normal_responsive_mobile'] )) {
					$this->add_render_attribute(
						'widget_wrapper',
						[
							'class' => 'piotnetgrid-overflow-radius',
						]
					);
				}
			}
		}

		if ( ! $editor ) {
			if ( $this->get_type() !== 'field' && $this->get_type() !== 'submit' && $this->get_type() !== 'multi-step-form' ) {
				if ( ! empty( $settings['piotnetgrid_conditional_logic_form_enable_new'] ) && ! empty( $settings['piotnetgrid_conditional_logic_form_form_id'] ) ) {
					if ( array_key_exists( 'piotnetgrid_conditional_logic_form_list_new', $settings ) ) {
						$list_conditional = $settings['piotnetgrid_conditional_logic_form_list_new'];
						//if ( ! empty( $list_conditional[0]['piotnetgrid_conditional_logic_form_if'] ) && ! empty( $list_conditional[0]['piotnetgrid_conditional_logic_form_comparison_operators'] ) ) {
							$this->add_render_attribute(
								'widget_wrapper',
								[
									'data-piotnetgrid-conditional-logic' => json_encode( $list_conditional ),
									'data-piotnetgrid-conditional-logic-not-field' => '',
									'data-piotnetgrid-conditional-logic-not-field-form-id' => $settings['piotnetgrid_conditional_logic_form_form_id'],
									'data-piotnetgrid-conditional-logic-speed' => $settings['piotnetgrid_conditional_logic_form_speed_new'],
									'data-piotnetgrid-conditional-logic-easing' => $settings['piotnetgrid_conditional_logic_form_easing_new'],
								]
							);
						//}

						wp_enqueue_script( $this->slug . '-advanced-script' );
					}
				}
			}
		}

		if (!empty($settings['attributes'])) {
			$attributes = [];
			foreach ($settings['attributes'] as $attribute) {
				$attributes[$attribute['attribute_name']] = $attribute['attribute_value'];
			}
			$this->add_render_attribute(
				'widget_wrapper',
				$attributes
			);
		}

		if ( ! empty( $settings['advanced_background_image_dynamic_data'] ) ) {
			$advanced_background_image_url = '';
			if ( empty( $settings['advanced_background_image']['url'] ) && $settings['advanced_background_image_dynamic_data'] == 'featured_image' ) {
				$advanced_background_image_url = get_the_post_thumbnail_url( get_the_ID(), 'post-thumbnail' );
			}
			$this->add_render_attribute( 'widget_wrapper', 'style', 'background-image: url(' . $advanced_background_image_url . ')' );
		}

		$this->before_render();

		$wrapper_tag = !empty($settings['section_link_url']) && !$editor ? 'a' : 'div';

		if (!empty($settings['section_link_url'])) {
			$this->add_render_attribute( 'widget_wrapper', 'href', $settings['section_link_url'] );
			if (!empty($settings['section_link_target'])) {
				$this->add_render_attribute( 'widget_wrapper', 'target', $settings['section_link_target'] );
			}
		}

		echo '<' . $wrapper_tag . ' ' . $this->get_render_attribute_string( 'widget_wrapper' ) . '>';

		if ( $visibility ) {
			$this->render_start( $editor );
		}

		return ob_get_clean();
	}

	public function output_wrapper_end( $widget_id = '', $editor = false ) {
		ob_start();
		$settings = $this->settings;
		$visibility = true;

		if ( ! $editor ) {
			$visibility = $this->widget_visibility();
		}

		if ( $visibility ) {
			$this->render_end( $editor );
			$this->dynamic_css();
			$this->custom_css();
			$wrapper_tag = !empty($settings['section_link_url']) && !$editor ? 'a' : 'div';
			echo '</' . $wrapper_tag . '>';
		}

		return ob_get_clean();
	}

	public function output( $widget_id = '', $editor = false, $args_additional = [], $preview_card_post_id = '' ) {
		ob_start();

		$settings = $this->settings;

		$widget_information = [
			'type'       => $this->get_type(),
			'class_name' => $this->get_class_name(),
			'title'      => $this->get_title(),
			'icon'       => $this->get_icon(),
		];

		$visibility = true;

		if ( ! $editor ) {
			$visibility = $this->widget_visibility();
		}

		if (!empty($preview_card_post_id)) {
			$GLOBALS['preview_card_post_id'] = $preview_card_post_id;
		}

		if ( $editor ) {
			$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-editor-widgets-item-root', '' );

			$this->add_render_attribute( 'widget_wrapper_editor', 'class', 'piotnetgrid-widget' );
			$this->add_render_attribute( 'widget_wrapper_editor', 'data-piotnetgrid-editor-widgets-item', json_encode( $widget_information ) );
			$this->add_render_attribute( 'widget_wrapper_editor', 'data-piotnetgrid-editor-widgets-item-id', $widget_id );
			$this->add_render_attribute( 'widget_wrapper_editor', 'draggable', 'true' );

			echo '<div ' . $this->get_render_attribute_string( 'widget_wrapper_editor' ) . '>';

			?>
				<div class="piotnetgrid-widget__controls" data-piotnetgrid-controls>
					<div class="piotnetgrid-widget__controls-item piotnetgrid-widget__controls-item--edit" title="Edit" data-piotnetgrid-control-edit>
						<i class="fas fa-th"></i>
					</div>
					<div class="piotnetgrid-widget__controls-item piotnetgrid-widget__controls-item--duplicate" title="Duplicate" data-piotnetgrid-control-duplicate>
						<i class="far fa-clone"></i>
					</div>
					<div class="piotnetgrid-widget__controls-item piotnetgrid-widget__controls-item--remove" title="Delete" data-piotnetgrid-control-remove>
						<i class="fas fa-times"></i>
					</div>
				</div>
				<div class="piotnetgrid-widget__container" data-piotnetgrid-container>
			<?php

			if (!empty($settings['advanced_position_normal']) || !empty($settings['advanced_position_hover']) || !empty($settings['advanced_position_hover_card'])) {
				$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-editor-widgets-item-absolute', '' );
			}
		}

		$this->add_render_attribute( 'wrapper', 'class', $widget_id );

		if ( ! empty( $settings['advanced_custom_id'] ) ) {
			$this->add_render_attribute( 'wrapper', 'id', $settings['advanced_custom_id'] );
		}

		if ( ! empty( $settings['advanced_custom_classes'] ) ) {
			$this->add_render_attribute( 'wrapper', 'class', $settings['advanced_custom_classes'] );
		}

		if ( ! empty( $settings['piotnetgrid_repeater_enable_trigger'] ) && ! empty( $settings['piotnetgrid_repeater_form_id_trigger'] ) && ! empty( $settings['piotnetgrid_repeater_id_trigger'] ) && ! empty( $settings['piotnetgrid_repeater_trigger_action'] ) ) {
			$this->add_render_attribute(
				'wrapper',
				[
					'data-piotnetgrid-repeater-form-id-trigger' => $settings['piotnetgrid_repeater_form_id_trigger'],
					'data-piotnetgrid-repeater-id-trigger' => $settings['piotnetgrid_repeater_id_trigger'],
					'data-piotnetgrid-repeater-trigger-action' => $settings['piotnetgrid_repeater_trigger_action'],
				]
			);
			wp_enqueue_script( $this->slug . '-advanced-script' );
		}

		if ( $this->get_type() !== 'field' && $this->get_type() !== 'submit' && $this->get_type() !== 'multi-step-form' ) {
			if ( ! empty( $settings['piotnetgrid_conditional_logic_form_enable_new'] ) && ! empty( $settings['piotnetgrid_conditional_logic_form_form_id'] ) ) {
				if ( array_key_exists( 'piotnetgrid_conditional_logic_form_list_new', $settings ) ) {
					$list_conditional = $settings['piotnetgrid_conditional_logic_form_list_new'];
					//if ( ! empty( $list_conditional[0]['piotnetgrid_conditional_logic_form_if'] ) && ! empty( $list_conditional[0]['piotnetgrid_conditional_logic_form_comparison_operators'] ) ) {
						$this->add_render_attribute(
							'wrapper',
							[
								'data-piotnetgrid-conditional-logic' => json_encode( $list_conditional ),
								'data-piotnetgrid-conditional-logic-not-field' => '',
								'data-piotnetgrid-conditional-logic-not-field-form-id' => $settings['piotnetgrid_conditional_logic_form_form_id'],
								'data-piotnetgrid-conditional-logic-speed' => $settings['piotnetgrid_conditional_logic_form_speed_new'],
								'data-piotnetgrid-conditional-logic-easing' => $settings['piotnetgrid_conditional_logic_form_easing_new'],
							]
						);
					//}

					wp_enqueue_script( $this->slug . '-advanced-script' );
				}
			}
		}

		if (!empty($settings['attributes'])) {
			$attributes = [];
			foreach ($settings['attributes'] as $attribute) {
				$attributes[$attribute['attribute_name']] = $attribute['attribute_value'];
			}
			$this->add_render_attribute(
				'wrapper',
				$attributes
			);
		}

		if (!empty($settings['trigger_action_carousel'])) {
			$trigger_action_carousel = [
				'action' => !empty($settings['trigger_action_carousel_action']) ? $settings['trigger_action_carousel_action'] : 'next',
				'grid_custom_id' => !empty($settings['trigger_action_carousel_grid_custom_id']) ? $settings['trigger_action_carousel_grid_custom_id'] : '',
			];
			
			$this->add_render_attribute(
				'wrapper',
				'data-piotnetgrid-trigger-action-carousel',
				json_encode($trigger_action_carousel)
			);
		}

		if ( ! empty( $settings['advanced_background_image_dynamic_data'] ) ) {
			$advanced_background_image_url = '';
			if ( empty( $settings['advanced_background_image']['url'] ) && $settings['advanced_background_image_dynamic_data'] == 'featured_image' ) {
				$advanced_background_image_url = get_the_post_thumbnail_url( get_the_ID(), 'post-thumbnail' );
			}

			$default_image_url = plugins_url() . '/piotnetgrid/assets/images/placeholder-image.png';
			$advanced_background_image_url = ! empty( $settings['advanced_background_image']['url'] ) ? $settings['advanced_background_image']['url'] : $default_image_url;
			
			if ( !$editor ) {
				switch ($settings['advanced_background_image_dynamic_data']) {
					case 'featured_image':
						$advanced_background_image_url = get_the_post_thumbnail_url( get_the_ID(), 'full' );
						break;

					case 'author_profile_picture':
						$image_size_author_profile_picture = !empty($settings['advanced_background_image_dynamic_data_size_author_profile_picture']) ? $settings['advanced_background_image_dynamic_data_size_author_profile_picture'] : 150;
						$advanced_background_image_url = get_avatar_url(get_the_author_meta('ID'), array('size' => $image_size_author_profile_picture));
						break;

					case 'post_custom_field':
					case 'acf_field':
					case 'metabox_field':
					case 'pods_field':
					case 'toolset_field':
					case 'jetengine_field':
						$advanced_background_image_url = $this->get_image_url_from_custom_field($settings['advanced_background_image_dynamic_data'], $settings['advanced_background_image_dynamic_data_field_name']);
						break;
				}
			}

			if ( !$advanced_background_image_url ) {
				$advanced_background_image_url = $default_image_url;
			}

			$this->add_render_attribute( 'wrapper', 'style', 'background-image: url(' . $advanced_background_image_url . ')' );
		}

		$this->before_render();

		if ( $visibility ) {
			$this->render( $editor, $args_additional );
			$this->dynamic_css();
			$this->custom_css();
		}

		if ( $editor ) {
			echo '</div></div>';
		}

		return ob_get_clean();
	}

	public function before_render() {

	}

	public $render_attributes = [];

	public function add_render_attribute( $element, $key = null, $value = null, $overwrite = false ) {
		if ( is_array( $element ) ) {
			foreach ( $element as $element_key => $attributes ) {
				$this->add_render_attribute( $element_key, $attributes, null, $overwrite );
			}

			return $this;
		}

		if ( is_array( $key ) ) {
			foreach ( $key as $attribute_key => $attributes ) {
				$this->add_render_attribute( $element, $attribute_key, $attributes, $overwrite );
			}

			return $this;
		}

		if ( empty( $this->render_attributes[ $element ][ $key ] ) ) {
			$this->render_attributes[ $element ][ $key ] = [];
		}

		settype( $value, 'array' );

		if ( $overwrite ) {
			$this->render_attributes[ $element ][ $key ] = $value;
		} else {
			$this->render_attributes[ $element ][ $key ] = array_merge( $this->render_attributes[ $element ][ $key ], $value );
		}

		return $this;
	}

	public function remove_render_attribute( $element, $key = null, $values = null ) {
		if ( $key && ! isset( $this->render_attributes[ $element ][ $key ] ) ) {
			return;
		}

		if ( $values ) {
			$values = (array) $values;

			$this->render_attributes[ $element ][ $key ] = array_diff( $this->render_attributes[ $element ][ $key ], $values );

			return;
		}

		if ( $key ) {
			unset( $this->render_attributes[ $element ][ $key ] );

			return;
		}

		if ( isset( $this->render_attributes[ $element ] ) ) {
			unset( $this->render_attributes[ $element ] );
		}
	}

	public static function render_html_attributes( array $attributes ) {
		$rendered_attributes = [];

		foreach ( $attributes as $attribute_key => $attribute_values ) {
			if ( is_array( $attribute_values ) ) {
				$attribute_values = implode( ' ', $attribute_values );
			}

			$rendered_attributes[] = sprintf( '%1$s="%2$s"', $attribute_key, esc_attr( $attribute_values ) );
		}

		return implode( ' ', $rendered_attributes );
	}

	public function get_render_attribute_string( $element ) {
		if ( empty( $this->render_attributes[ $element ] ) ) {
			return '';
		}

		return $this->render_html_attributes( $this->render_attributes[ $element ] );
	}

	public function get_image_url_with_size($advanced_background_image_url, $image_size) {
		global $wpdb;
		$attachment = $wpdb->get_col($wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE guid='%s';", $advanced_background_image_url )); 
		$image_id = $attachment[0];
		return wp_get_attachment_image_src($image_id, $image_size)[0];
	}

	public function get_image_url_from_custom_field($custom_field_type, $custom_field_name) {
		$advanced_background_image_url = '';

		$custom_field_types = array (
			'post_custom_field',
			'acf_field',
			'metabox_field',
			'pods_field',
			'toolset_field',
			'jetengine_field',
		);

		if (in_array($custom_field_type, $custom_field_types)) {
			$post_id = get_the_ID();
			switch ($custom_field_type) {
				case 'post_custom_field':
					$advanced_background_image_url = get_post_meta( $post_id, $custom_field_name, true );
					break;
				
				case 'acf_field':
					$image_id = get_field( $custom_field_name, false, false );
					$advanced_background_image_url = wp_get_attachment_url( $image_id );
					break;

				case 'metabox_field':
					$images = rwmb_get_value( $custom_field_name, array( 'limit' => 1, 'size' => 'full' ), $post_id );
                    $image = reset( $images );
                    if ( is_array( $image ) ) {
                        $advanced_background_image_url = $image['url'];
                    }
					break;

				case 'pods_field':
					$image = pods_field( get_post_type(), $post_id, $custom_field_name, true );
					if ( is_array( $image ) ) {
						$advanced_background_image_url = $image['guid'];
					}
					break;

				case 'toolset_field':
					$advanced_background_image_url = get_post_meta( $post_id, 'wpcf-' . $custom_field_name, true );
					break;

				case 'jetengine_field':
					$image_id = get_post_meta( $post_id, $custom_field_name, true );
					$advanced_background_image_url = wp_get_attachment_url( $image_id );
					break;
			}

		}

		return $advanced_background_image_url;
	}

}
