<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

function piotnetgrid_generate_widget_id($unique_widget_ids) {
    while (true) {
        $widget_id = 'p' . rand (1000000, 99999999);
        if (!in_array($widget_id, $unique_widget_ids)) {
            return $widget_id;
        }
    }
}

function piotnetgrid_replace_widget_id_in_content( $content, $id, $new_id ) {
	foreach ( $content as &$widget ) {
		$current_id = $widget['id'];
		if ( $current_id === $id ) {
			$widget['id'] = $new_id;
			break;
		}

		if ( isset( $widget['elements'] ) ) {
			$widget['elements'] = piotnetgrid_replace_widget_id_in_content( $widget['elements'], $id, $new_id );
		}
	}
	return $content;
}

function piotnetgrid_fill_settings_in_content( $content, $widgets ) {
	$length = count( $content );
	for ( $i = 0; $i <= $length - 1; $i++ ) {
		$widget = $widgets[ $content[ $i ]['id'] ];

		$content[ $i ]['settings'] = $widget['settings'];

		if ( isset( $widget['fonts'] ) ) {
			$content[ $i ]['fonts'] = $widget['fonts'];
		}

		if ( isset( $content[ $i ]['elements'] ) ) {
			$content[ $i ]['elements'] = piotnetgrid_fill_settings_in_content( $content[ $i ]['elements'], $widgets );
		}
	}
	return $content;
}

function piotnetgrid_do_import( $post_id, $data ) {
	$widgets     = $data['widgets'];
	$content     = $data['content'];
	$new_widgets = [];

    $unique_widget_ids = [];
	$widget_ids     = [];
	$new_widget_ids = [];

    foreach ( $widgets as $widget_id => $widget ) {
        $unique_widget_ids[] = $widget_id;
    }

    foreach ( $widgets as $widget_id => $widget ) {
		$new_widget_id = piotnetgrid_generate_widget_id($unique_widget_ids);

		$widget_ids[]     = $widget_id;
		$new_widget_ids[] = $new_widget_id;
        $unique_widget_ids[] = $new_widget_id;

        if (!empty($widget['settings']['facet_name'])) {
        	$facet_settings = get_option( '_piotnetgrid_facet_' . $widget['settings']['facet_name'] );
        	$facet_name = $widget['settings']['facet_name'];

			if (!empty($facet_settings)) {
				$facet_settings = json_decode( stripslashes( $facet_settings ), true );
				$check_facet_name_post_id = !empty($facet_settings['post_id']) ? $facet_settings['post_id'] : '';
				$check_facet_name_widget_id = !empty($facet_settings['widget_id']) ? $facet_settings['widget_id'] : 'piotnetgrid';
				$check_facet_name_raw_data = json_decode(get_post_meta( $check_facet_name_post_id, '_piotnetgrid_data', true ), true);

				if (isset($check_facet_name_raw_data['widgets'][$check_facet_name_widget_id])) {
					$facet_name = $facet_name . '_' . $new_widget_id;
					$widget['settings']['facet_name'] = $facet_name;
				}
			}

			$facet_settings_new = $widget['settings'];
			$facet_settings_new['post_id'] = $post_id;
			$facet_settings_new['widget_id'] = $new_widget_id;

			update_option( '_piotnetgrid_facet_' . $facet_name, wp_slash( json_encode( $facet_settings_new ) ) );
        }

		$new_widgets[ $new_widget_id ] = $widget;

		$content = piotnetgrid_replace_widget_id_in_content( $content, $widget_id, $new_widget_id );
	}

	$data['widgets'] = $new_widgets;
	$data['content'] = piotnetgrid_fill_settings_in_content( $content, $new_widgets );

	$css = str_replace( $widget_ids, $new_widget_ids, $data['css'] );
	unset( $data['css'] );

	update_post_meta( $post_id, '_piotnetgrid_data', wp_slash( json_encode( $data ) ) );

	update_post_meta( $post_id, '_piotnetgrid-revision-version', 1 );

	$upload_dir = wp_upload_dir()['basedir'] . '/piotnetgrid/css/';
	$css_path   = $upload_dir . $post_id . '.css';

	$file = fopen( $css_path, 'wb' );
	fwrite( $file, stripslashes( $css ) );
	fclose( $file );
}
