<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

require_once __DIR__ . '/../class/piotnetgrid-base.php';

abstract class Base_Widget_piotnetgrid extends piotnetgrid_Base {
	abstract public function get_type();
	abstract public function get_class_name();
	abstract public function get_title();
	abstract public function get_icon();
	abstract public function get_categories();
	abstract public function get_keywords();
	abstract public function render($editor, $ar);
	abstract public function live_preview();
	abstract public function register_controls();

	protected $is_add_conditional_logic = true;

	protected $structure       = [];
	private $current_tab       = null;
	private $current_section   = null;
	private $current_controls  = null;
	private $previous_controls = null;

	private $tab_names     = [];
	private $section_names = [];
	public $control_names = [];
	public $control_dynamic_css = [];

	protected function start_tab( string $name, string $label ) {
		if ( in_array( $name, $this->tab_names ) ) {
			echo 'Warning: Tab name "' . $name . '" has been declared.<br>';
			return;
		}
		array_push( $this->tab_names, $name );

		$tab = [
			'name'     => $name,
			'label'    => $label,
			'sections' => [],
		];

		if ( count( $this->structure ) === 0 ) {
			$tab['active'] = true;
		}

		$this->current_tab = &$tab;
		$this->structure[] = &$tab;
	}

	protected function start_section( string $name, string $label, $args = [] ) {
		if ( in_array( $name, $this->section_names ) ) {
			echo 'Warning: Section name "' . $name . '" has been declared.<br>';
			return;
		}

		$controls = [];
		$section  = [
			'name'     => $name,
			'label'    => $label,
			'controls' => &$controls,
		];

		$args = $this->conditions_old_version( $args );

		if ( isset( $args['conditions'] ) ) {
			$section['conditions'] = $args['conditions'];
		}

		if ( count( $this->current_tab['sections'] ) === 0 ) {
			$section['active'] = true;
		}

		$this->current_section           = &$section;
		$this->current_controls          = &$controls;
		$this->current_tab['sections'][] = &$section;
	}

	protected function add_control( string $name, $args, $options = [], $add_current_controls = true ) {
		$default_options = [
			'overwrite' => false,
			'index'     => null,
		];

		$options = array_merge( $default_options, $options );

		if ( ! $options['overwrite'] && ! empty( $name ) && $name !== '' && in_array( $name, $this->control_names ) ) {
			echo 'Warning: Control name "' . $name . '" has been declared.<br>';
			return;
		}
		if ( ! $options['overwrite'] && ! empty( $name ) && $name !== '' ) {
			array_push( $this->control_names, $name );

			if (stripos( $name, '__dynamic__' ) !== false) {
				$this->control_dynamic_css[$name] = $args['selectors'];
			}
		}

		$args['name'] = $name;

		// Add Condition for dynamic control

		if ( !empty($args['dynamic']) && !empty($args['dynamic_field']) && strpos( $name, '__dynamic__' ) == false ) {
			$args['conditions']['terms'][] = [
				'name'     => '__dynamic__' . $name,
				'operator' => '=',
				'value'    => '',
			];
		}

		// Fix Old Version

		$args = $this->conditions_old_version( $args );

		if ( $args['type'] == 'slider' ) {
			$args['label_block'] = true;

			if ( ! empty( $args['range'] ) ) {
				if ( ! empty( $args['size_units'] ) ) {
					unset( $args['size_units'] );
				}
				$args['size_units'] = $args['range'];
			}
		}

		if ( ! empty( $args['default'] ) ) {
			$args['value'] = $args['default'];
		}

		if ( $args['type'] == 'dimensions' ) {
			$args['label_block'] = true;
			if ( empty( $args['size_units'] ) ) {
				$args['size_units'] = [ 'px' ];
			}
			if ( empty( $args['value'] ) ) {
				$args['value'] = [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				];
			}
		}

		if ( $args['type'] == 'gallery' ) {
			$args['label_block'] = true;
		}

		// End Fix Old Version

		if ($add_current_controls) {
			$this->current_controls[] = $args;
		}

		if (!empty($args['dynamic_field'])) {
			$args_dynamic_field = $args;
			$args_dynamic_field['type'] = 'text';
			$args_dynamic_field['label_block'] = true;
			unset($args_dynamic_field['dynamic_field']);
			unset($args_dynamic_field['conditions']);
			$this->add_control('__dynamic__' . $name, $args_dynamic_field );
		}

		return $args;
	}

    public function get_additional_image_sizes() {
        global $_wp_additional_image_sizes;

        $image_sizes = array(
            'full' => 'Full',
        );

        foreach ( get_intermediate_image_sizes() as $_size ) {
            if ( in_array( $_size, array('thumbnail', 'medium', 'medium_large', 'large') ) ) {
                $image_sizes[ $_size ] = ucwords( str_replace( '_', ' ', $_size ) ) . ' - ' . get_option( "{$_size}_size_w" ) . ' x ' . get_option( "{$_size}_size_h" );
            } elseif ( isset( $_wp_additional_image_sizes[ $_size ] ) ) {
                $image_sizes[ $_size ] = ucwords( str_replace( '_', ' ', $_size ) ) . ' - ' . $_wp_additional_image_sizes[ $_size ]['width'] . ' x ' . $_wp_additional_image_sizes[ $_size ]['height'];
            }
        }
        return $image_sizes;
    }

	public function get_info() {
		return [
			'type'       => $this->get_type(),
			'class_name' => $this->get_class_name(),
			'title'      => $this->get_title(),
			'icon'       => $this->get_icon(),
			'structure'  => $this->register_controls(),
		];
	}

	protected function add_responsive_control( string $name, array $args ) {
		$responsive_device = [ 'desktop', 'tablet', 'mobile' ];
		foreach ( $responsive_device as $device ) {
			$responsive_args               = $args;
			$control_name                  = $name . '_responsive_' . $device;
			$responsive_args['responsive'] = $device;
			$this->add_control( $control_name, $responsive_args );
		}
	}

	protected function new_group_controls() {
		$this->previous_controls = $this->current_controls;
		$this->current_controls  = [];
		return $this->previous_controls;
	}

	protected function get_group_controls( $previous_controls = null ) {
		$controls               = $this->current_controls;
		$this->current_controls = $previous_controls === null ? $this->previous_controls : $previous_controls;
		return $controls;
	}

	protected function get_group_controls_self( $previous_controls = null ) {
		$controls = $this->current_controls;
		return $controls;
	}

	protected function add_text_typography_controls( string $name, $args = [], $inner_section = false ) {
		$this->new_group_controls();
		
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';

		$this->add_control(
			$name . '_font_family',
			[
				'type'           => 'select',
				'label'          => __( 'Family', 'piotnetgrid' ),
				'options_source' => 'google-fonts',
				'selectors'      => [
					$wrapper => 'font-family:{{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			$name . '_font_size',
			[
				'type'        => 'slider',
				'label'       => __( 'Size', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'px'  => [
						'min'  => 1,
						'max'  => 200,
						'step' => 1,
					],
					'em'  => [
						'min'  => 0.1,
						'max'  => 10,
						'step' => 0.1,
					],
					'rem' => [
						'min'  => 0.1,
						'max'  => 10,
						'step' => 0.1,
					],
					'vw'  => [
						'min'  => 0.1,
						'max'  => 10,
						'step' => 0.1,
					],
				],
				'selectors'   => [
					$wrapper => 'font-size:{{SIZE}}{{UNIT}}',
				],
			]
		);

		$this->add_control(
			$name . '_font_weight',
			[
				'type'      => 'select',
				'label'     => __( 'Weight', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					'' => __( 'Default', 'piotnetgrid' ),
					'100' => '100',
					'200' => '200',
					'300' => '300',
					'400' => '400 Normal',
					'500' => '500',
					'600' => '600',
					'700' => '700 Bold',
					'800' => '800',
					'900' => '900',
				],
				'selectors' => [
					$wrapper => 'font-weight:{{VALUE}}',
				],
			]
		);

		$this->add_control(
			$name . '_transform',
			[
				'type'      => 'select',
				'label'     => __( 'Transform', 'piotnetgrid' ),
				'options'   => [
					''           => 'Default',
					'uppercase'  => 'Uppercase',
					'lowercase'  => 'Lowercase',
					'capitalize' => 'Capitalize',
					'none'       => 'Normal',
				],
				'selectors' => [
					$wrapper => 'text-transform:{{VALUE}}',
				],
			]
		);

		$this->add_control(
			$name . '_font_style',
			[
				'type'      => 'select',
				'label'     => __( 'Style', 'piotnetgrid' ),
				'options'   => [
					''        => 'Default',
					'normal'  => 'Normal',
					'italic'  => 'Italic',
					'oblique' => 'Oblique',
				],
				'selectors' => [
					$wrapper => 'font-style:{{VALUE}}',
				],
			]
		);

		$this->add_control(
			$name . '_decoration',
			[
				'type'      => 'select',
				'label'     => __( 'Decoration', 'piotnetgrid' ),
				'options'   => [
					''             => 'Default',
					'underline'    => 'Underline',
					'overline'     => 'Overline',
					'line-through' => 'Line Through',
					'none'         => 'None',
				],
				'selectors' => [
					$wrapper => 'text-decoration:{{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			$name . '_line_height',
			[
				'type'        => 'slider',
				'label'       => __( 'Line-Height', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'em',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'px' => [
						'min'  => 1,
						'max'  => 200,
						'step' => 1,
					],
					'em' => [
						'min'  => 0.1,
						'max'  => 10,
						'step' => 0.1,
					],
				],
				'selectors'   => [
					$wrapper => 'line-height:{{SIZE}}{{UNIT}}',
				],
			]
		);

		$this->add_responsive_control(
			$name . '_letter_spacing',
			[
				'type'        => 'slider',
				'label'       => __( 'Letter Spacing', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'px' => [
						'min'  => -5,
						'max'  => 10,
						'step' => 1,
					],
				],
				'selectors'   => [
					$wrapper => 'letter-spacing:{{SIZE}}{{UNIT}}',
				],
			]
		);

		$typography_controls = !$inner_section ? $this->get_group_controls() : $this->get_group_controls_self();

		$typography_args = [
			'type'           => 'typography',
			'label'          => isset($args['label']) ? $args['label'] : __( 'Typography', 'piotnetgrid' ),
			'label_block'    => false,
			'controls'       => $typography_controls,
			'controls_query' => '.piotnetgrid-tooltip__body',
			'icon'           => 'fas fa-pencil-alt',
		];

		$args = $this->conditions_old_version( $args );

		if ( isset( $args['conditions'] ) ) {
			$typography_args['conditions'] = $args['conditions'];
		}

		if (!$inner_section) {
			$this->add_control(
				$name,
				$typography_args
			);
		}
	}

	protected function add_background_gradient_settings() {

		$this->add_control(
			'background_gradient',
			[
				'type'         => 'switch',
				'label'        => __( 'Background Gradient Enable', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'yes',
			]
		);

		$this->new_group_controls();

		$this->add_control(
			'background_gradient_color',
			[
				'type' => 'color',
				'label' => __( 'Color', 'piotnetgrid' ),
			]
		);

		$this->add_control(
			'background_gradient_location',
			[
				'type'        => 'slider',
				'label'       => __( 'Background Gradient Location', 'piotnetgrid' ),
				'value'       => [
					'unit' => '%',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'%'  => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
			]
		);
		
		$this->add_control(
			'repeater_id',
			[
				'type' => 'hidden',
			],
            [
                'overwrite' => 'true',
            ]
		);

		$repeater_items = $this->get_group_controls();

		$this->new_group_controls();

		$this->add_control(
			'',
			[
				'type'           => 'repeater-item',
				'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
				'controls'       => $repeater_items,
				'controls_query' => '.piotnetgrid-control-repeater-field',
			]
		);

		$repeater_list = $this->get_group_controls();

		$this->add_control(
			'background_gradient_list',
			[
				'type'           => 'repeater',
				'label'          => __( 'Background Gradient Color List', 'piotnetgrid' ),
				'value'          => 'or',
				'label_block'    => true,
				'add_label'      => __( 'Add Item', 'piotnetgrid' ),
				'controls'       => $repeater_list,
				'controls_query' => '.piotnetgrid-control-repeater-list',
				'conditions'     => [
					[
						'name'     => 'background_gradient',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);

		$this->add_control(
			'background_gradient_type',
			[
				'type'        => 'select',
				'label'       => __( 'Background Gradient Type', 'piotnetgrid' ),
				'options'       => [
					'linear' => __( 'Linear', 'piotnetgrid' ),
					'radial' => __( 'Radial', 'piotnetgrid' ),
				],
			]
		);

		$this->add_control(
			'background_gradient_angle',
			[
				'type'        => 'slider',
				'label'       => __( 'Background Gradient Angle', 'piotnetgrid' ),
				'value'       => [
					'unit' => '%',
					'size' => '100',
				],
				'label_block' => true,
				'size_units'  => [
					'ms'  => [
						'min'  => 0,
						'max'  => 360,
						'step' => 1,
					],
				],
			]
		);

		$this->add_control(
			'background_gradient_opacity',
			[
				'type'        => 'number',
				'label'       => __( 'Background Gradient Opacity', 'piotnetgrid' ),
			]
		);
	}

	protected function add_attributes_settings() {
		$this->start_section( 'attributes_section', 'Attributes' );

		$this->new_group_controls();
		$this->add_control(
			'attribute_name',
			[
				'type'        => 'text',
				'get_fields'  => true,
				'label'       => __( 'Name', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
			]
		);
		$this->add_control(
			'attribute_value',
			[
				'type'        => 'text',
				'get_fields'  => true,
				'label'       => __( 'Value', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
			]
		);
		$this->add_control(
			'repeater_id',
			[
				'type' => 'hidden',
			],
            [
                'overwrite' => 'true',
            ]
		);
		$repeater_items = $this->get_group_controls();
		$this->new_group_controls();
		$this->add_control(
			'',
			[
				'type'           => 'repeater-item',
				'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
				'controls'       => $repeater_items,
				'controls_query' => '.piotnetgrid-control-repeater-field',
			]
		);
		$repeater_list = $this->get_group_controls();
		$this->add_control(
			'attributes',
			[
				'type'           => 'repeater',
				'label'          => __( 'Attributes List', 'piotnetgrid' ),
				'value'          => 'or',
				'label_block'    => true,
				'add_label'      => __( 'Add Item', 'piotnetgrid' ),
				'controls'       => $repeater_list,
				'controls_query' => '.piotnetgrid-control-repeater-list',
			]
		);
	}

	protected function add_animation_settings() {
		$this->start_section( 'animation_section', 'Animation' );

		$this->add_control(
			'transition_timing_function',
			[
				'type'    => 'select',
				'label'   => __( 'Timing Funtion', 'piotnetgrid' ),
				'options' => [
					'' => __( 'Select', 'piotnetgrid' ),
					'ease' => __( 'Ease', 'piotnetgrid' ),
					'linear' => __( 'Linear', 'piotnetgrid' ),
					'ease-in' => __( 'Ease In', 'piotnetgrid' ),
					'ease-out' => __( 'Ease Out', 'piotnetgrid' ),
					'ease-in-out' => __( 'Ease In Out', 'piotnetgrid' ),
				],
				'selectors'    => [
					'{{WRAPPER}}' => 'transition-timing-function: {{VALUE}}',
				],
			]
		);
		$this->add_control(
			'transition_duration',
			[
				'type'        => 'slider',
				'label'       => __( 'Duration (ms)', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'ms',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'ms'  => [
						'min'  => 0,
						'max'  => 3000,
						'step' => 50,
					],
				],
				'selectors'   => [
					'{{WRAPPER}}' => 'transition-duration: {{SIZE}}{{UNIT}}',
				],
			]
		);

		$this->add_control(
			'transition_delay',
			[
				'type'        => 'slider',
				'label'       => __( 'Delay (ms)', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'ms',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'ms'  => [
						'min'  => 0,
						'max'  => 3000,
						'step' => 50,
					],
				],
				'selectors'   => [
					'{{WRAPPER}}' => 'transition-delay: {{SIZE}}{{UNIT}}',
				],
			]
		);
	}

	public function add_trigger_action_carousel() {

		$this->start_section( 'add_trigger_action_carousel_section', 'Trigger Prev/Next Action for Grid Carousel' );
		
		$this->add_control(
			'trigger_action_carousel',
			[
				'type'         => 'switch',
				'label'        => __( 'Enable', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'yes',
			]
		);

		$this->add_control(
			'trigger_action_carousel_grid_custom_id',
			[
				'type'        => 'text',
				'label'       => __( 'Grid Custom ID', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => false,
				'conditions'  => [
					[
						'name'     => 'trigger_action_carousel',
						'operator' => '=',
						'value'    => 'yes',
					],
				],
			]
		);

		$this->add_control(
			'trigger_action_carousel_action',
			[
				'type'        => 'select',
				'label'       => __( 'Action', 'piotnetgrid' ),
				'options'     => [
					'slickNext' => __( 'Next', 'piotnetgrid' ),
					'slickPrev' => __( 'Previous', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
						'name'     => 'trigger_action_carousel',
						'operator' => '=',
						'value'    => 'yes',
					],
				],
			]
		);
	}

	protected function add_advanced_style_tab() {
		$this->start_tab( 'advanced', 'Advanced' );
		$this->start_section( 'advanced_style_section', 'Advanced Style' );

		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'advanced_style_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'advanced_style_hover_tab',
						'title' => __( 'HOVER WIDGET', 'piotnetgrid' ),
					],
					[
						'name'  => 'advanced_style_hover_card_tab',
						'title' => __( 'HOVER CARD', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_advanced_style_controls(
			'normal',
			[
				'selectors' => '{{WRAPPER}}',
			]
		);

		$this->add_control(
			'advanced_style_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_advanced_style_controls(
			'hover',
			[
				'selectors' => '{{WRAPPER}}:hover',
			]
		);

		$this->add_control(
			'advanced_style_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover Widget', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_card_selectors = '{{WRAPPER_CARD}}:hover {{WIDGET}}';

		$hover_card_controls = $this->add_advanced_style_controls(
			'hover_card',
			[
				'selectors' => $hover_card_selectors,
			]
		);

		$this->add_control(
			'advanced_style_hover_card_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover Card', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_card_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	protected function add_advanced_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$wrapper_frontend = str_replace('{{WRAPPER}}', '{{WRAPPER_FRONTEND}}', $wrapper);
		$previous_controls = $this->new_group_controls();

        $this->new_group_controls();

		$this->add_responsive_control(
			'advanced_margin_' . $name,
			[
				'type'        => 'dimensions',
				'label'       => __( 'Margin', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em', 'rem' ],
				'selectors'   => [
					$wrapper => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'advanced_padding_' . $name,
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em', 'rem' ],
				'selectors'   => [
					$wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_margin_padding_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Margin, Padding', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

        $this->new_group_controls();

		$this->add_responsive_control(
			'advanced_width_' . $name,
			[
				'type'        => 'text',
				'label'       => __( 'Width', 'piotnetgrid' ),
				'label_block' => true,
				'description' => __( 'Examples: 50%, 150px, calc(50% - 30px)', 'piotnetgrid'),
				'selectors'   => [
					$wrapper_frontend => 'width: {{VALUE}};',
					'{{WRAPPER_EDITOR}}' => 'width: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => 'advanced_width_inline_' . $name,
						'operator' => '!=',
						'value'    => 'auto',
					],
				],
			]
		);

		$this->add_control(
			'advanced_width_inline_' . $name,
			[
				'type'         => 'switch',
				'label'        => __( 'Width Inline (auto)', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'auto',
				'selectors'   => [
					$wrapper => 'width: {{VALUE}}!important;',
					'{{WRAPPER_EDITOR}}' => 'width: {{VALUE}}!important;',
				],
			]
		);

		$this->add_responsive_control(
			'advanced_height_' . $name,
			[
				'type'        => 'slider',
				'label'       => __( 'Height', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'size_units'  => [
					'px' => [
						'min'  => 0,
						'max'  => 2000,
						'step' => 1,
					],
					'%'  => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'selectors'   => [
					$wrapper => 'height: {{SIZE}}{{UNIT}};',
					'{{WRAPPER_EDITOR}}' => 'height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_width_height_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Width, Height', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

        $this->new_group_controls();

		$this->add_control(
			'advanced_position_' . $name,
			[
				'type'      => 'select',
				'label'     => __( 'Position', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'Default',
					'absolute'  => 'Absolute',
				],
				'selectors' => [
					$wrapper_frontend => 'position:{{VALUE}};',
					'{{WRAPPER_EDITOR}}' => 'position:{{VALUE}};',
				],
			]
		);

		$this->add_control(
			'advanced_z_index' . $name,
			[
				'type'      => 'number',
				'label'     => __( 'Z-index', 'piotnetgrid' ),
				'value'     => '',
				'selectors' => [
					$wrapper_frontend => 'z-index:{{VALUE}};',
					'{{WRAPPER_EDITOR}}' => 'z-index:{{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'advanced_position_absolute_top' . $name,
			[
				'type'      => 'slider',
				'label'     => __( 'Top', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'size_units'  => [
					'px' => [
						'min'  => -1000,
						'max'  => 1000,
						'step' => 1,
					],
					'%'  => [
						'min'  => -200,
						'max'  => 200,
						'step' => 1,
					],
				],
				'selectors'   => [
					$wrapper_frontend => 'top: {{SIZE}}{{UNIT}};',
					'{{WRAPPER_EDITOR}}' => 'top: {{SIZE}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => 'advanced_position_' . $name,
						'operator' => '=',
						'value'    => 'absolute',
					],
				],
			]
		);

		$this->add_responsive_control(
			'advanced_position_absolute_right' . $name,
			[
				'type'      => 'slider',
				'label'     => __( 'Right', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'size_units'  => [
					'px' => [
						'min'  => -1000,
						'max'  => 1000,
						'step' => 1,
					],
					'%'  => [
						'min'  => -200,
						'max'  => 200,
						'step' => 1,
					],
				],
				'selectors'   => [
					$wrapper_frontend => 'right: {{SIZE}}{{UNIT}};',
					'{{WRAPPER_EDITOR}}' => 'right: {{SIZE}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => 'advanced_position_' . $name,
						'operator' => '=',
						'value'    => 'absolute',
					],
				],
			]
		);

		$this->add_responsive_control(
			'advanced_position_absolute_bottom' . $name,
			[
				'type'      => 'slider',
				'label'     => __( 'Bottom', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'size_units'  => [
					'px' => [
						'min'  => -1000,
						'max'  => 1000,
						'step' => 1,
					],
					'%'  => [
						'min'  => -200,
						'max'  => 200,
						'step' => 1,
					],
				],
				'selectors'   => [
					$wrapper_frontend => 'bottom: {{SIZE}}{{UNIT}};',
					'{{WRAPPER_EDITOR}}' => 'bottom: {{SIZE}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => 'advanced_position_' . $name,
						'operator' => '=',
						'value'    => 'absolute',
					],
				],
			]
		);

		$this->add_responsive_control(
			'advanced_position_absolute_left' . $name,
			[
				'type'      => 'slider',
				'label'     => __( 'Left', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'size_units'  => [
					'px' => [
						'min'  => -1000,
						'max'  => 1000,
						'step' => 1,
					],
					'%'  => [
						'min'  => -200,
						'max'  => 200,
						'step' => 1,
					],
				],
				'selectors'   => [
					$wrapper_frontend => 'left: {{SIZE}}{{UNIT}};',
					'{{WRAPPER_EDITOR}}' => 'left: {{SIZE}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => 'advanced_position_' . $name,
						'operator' => '=',
						'value'    => 'absolute',
					],
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_position_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Position', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

        $this->new_group_controls();

        $this->add_control(
			'advanced_overflow' . $name,
			[
				'type'      => 'select',
				'label'     => __( 'Overflow', 'piotnetgrid' ),
				'description' => __( 'Overflow hidden for section, column only works on the frontend', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'visible'  => 'Visible',
					'hidden' => 'Hidden',
				],
				'selectors' => [
					$wrapper => 'overflow:{{VALUE}};',
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_overflow_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Overflow', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

        $this->new_group_controls();

		$this->add_responsive_control(
			'advanced_text_align' . $name,
			[
				'type'         => 'select',
				'label'        => __( 'Text Alignment', 'piotnetgrid' ),
				'label_block'  => true,
				'options'      => [
					''   => __( 'Default', 'piotnetgrid' ),
					'left'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'right'  => __( 'Right', 'piotnetgrid' ),
				],
				'selectors'    => [
					$wrapper => 'text-align: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'advanced_text_color_' . $name,
			[
				'type'      => 'color',
				'label'     => 'Text Color',
				'dynamic'	=> true,
				'dynamic_field'	=> true,
				'selectors' => [
					$wrapper => 'color: {{VALUE}}',
					$wrapper.' .piotnetgrid-link' => 'color: {{VALUE}}',
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_text_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Text Align, Color', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

		$this->add_text_typography_controls(
			'advanced_typography_' . $name,
			[
				'selectors' => $wrapper,
			],
			true
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_typography_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Typography', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

        $this->new_group_controls();

		$this->add_control(
			'advanced_box_shadow_' . $name,
			[
				'type'        => 'box-shadow',
				'label'       => __( 'Box Shadow', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => false,
				'render_type' => 'none',
				'selectors'   => [
					$wrapper => 'box-shadow: {{VALUE}};',
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_box_shadow_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Box Shadow', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

        $this->new_group_controls();

		$this->add_control(
			'advanced_border_style_' . $name,
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'advanced_border_color_' . $name,
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => 'advanced_border_style_' . $name,
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			'advanced_border_width_' . $name,
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => 'advanced_border_style_' . $name,
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			'advanced_border_radius_' . $name,
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_border_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Border', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

        $this->new_group_controls();

		$this->add_control(
			'advanced_opacity_' . $name,
			[
				'type'        => 'number',
				'label'       => __( 'Opacity', 'piotnetgrid' ),
				'value'       => '',
				'selectors'   => [
					$wrapper => 'opacity: {{VALUE}}',
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_opacity_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Opacity', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

        $this->new_group_controls();

		$this->add_control(
			'advanced_background_color_' . $name,
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'placeholder' => '',
				'dynamic'     => true,
				'dynamic_field' => true,
				'selectors'   => [
					$wrapper => 'background-color: {{VALUE}}',
				],
			]
		);
		$this->add_control(
			'advanced_background_image_' . $name,
			[
				'type'      => 'media',
				'label'     => __( 'Background Image', 'piotnetgrid' ),
				'value'     => '',
				'dynamic'     => true,
				'dynamic_field' => true,
				'selectors' => [
					$wrapper => 'background-image: url({{VALUE}})',
				],
			]
		);
		$this->add_responsive_control(
			'advanced_background_position_' . $name,
			[
				'type'        => 'select',
				'label'       => __( 'Background Position', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					''              => 'Default',
					'center center' => 'Center Center',
					'center left'   => 'Center Left',
					'center right'  => 'Center Right',
					'top center'    => 'Top Center',
					'top left'      => 'Top Left',
					'top right'     => 'Top Right',
					'bottom center' => 'Bottom Center',
					'bottom left'   => 'Bottom Left',
					'bottom right'  => 'Bottom Right',
					'initial'       => 'Custom',
				],
				'selectors'   => [
					$wrapper => 'background-position: {{VALUE}}',
				],
			]
		);
		$this->add_responsive_control(
			'advanced_background_attachment_' . $name,
			[
				'type'        => 'select',
				'label'       => __( 'Background Attachment', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					''       => 'Default',
					'scroll' => 'scroll',
					'fixed'  => 'Fixed',
				],
				'selectors'   => [
					$wrapper => 'background-attachment: {{VALUE}}',
				],
			]
		);
		$this->add_responsive_control(
			'advanced_background_repeat_' . $name,
			[
				'type'        => 'select',
				'label'       => __( 'Background Repeat', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					''          => 'Default',
					'no-repeat' => 'No-repeat',
					'repeat'    => 'Repeat',
					'repeat-x'  => 'Repeat-x',
					'repeat-y'  => 'Repeat-y',
				],
				'selectors'   => [
					$wrapper => 'background-repeat: {{VALUE}}',
				],
			]
		);
		$this->add_responsive_control(
			'advanced_background_size_' . $name,
			[
				'type'        => 'select',
				'label'       => __( 'Background Size', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					''        => 'Default',
					'auto'    => 'Auto',
					'cover'   => 'Cover',
					'Contain' => 'Contain',
					'initial' => 'Custom',
				],
				'selectors'   => [
					$wrapper => 'background-size: {{VALUE}}',
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_background_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Background', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

		if ($this->get_type() == 'section' || $this->get_type() == 'column') {

			$this->new_group_controls();

			$gradient_controls = [];

			$gradient_controls[] = $this->add_control(
				'background_gradient_' . $name,
				[
					'type'         => 'switch',
					'label'        => __( 'Background Gradient', 'piotnetgrid' ),
					'value'        => '',
					'label_on'     => 'Yes',
					'label_off'    => 'No',
					'return_value' => 'yes',
				]
			);

			$gradient_repeater_controls = [];

			$gradient_repeater_controls[] = $this->add_control(
				'background_gradient_color_' . $name,
				[
					'type' => 'color',
					'label' => __( 'Color', 'piotnetgrid' ),
				],
				[],
				false
			);

			$gradient_repeater_controls[] = $this->add_control(
				'background_gradient_location_' . $name,
				[
					'type'        => 'slider',
					'label'       => __( 'Gradient Location', 'piotnetgrid' ),
					'value'       => [
						'unit' => '%',
						'size' => '',
					],
					'label_block' => true,
					'size_units'  => [
						'%'  => [
							'min'  => 0,
							'max'  => 100,
							'step' => 1,
						],
					],
				],
				[],
				false
			);
			
			$gradient_repeater_controls[] = $this->add_control(
				'repeater_id',
				[
					'type' => 'hidden',
				],
	            [
	                'overwrite' => 'true',
	            ],
				[],
				false
			);

			$repeater_items = $gradient_repeater_controls;

			$gradient_repeater_list_controls = [];

			$gradient_repeater_list_controls[] = $this->add_control(
				'',
				[
					'type'           => 'repeater-item',
					'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
					'controls'       => $repeater_items,
					'controls_query' => '.piotnetgrid-control-repeater-field',
				],
				[],
				false
			);

			$repeater_list = $gradient_repeater_list_controls;

			$this->add_control(
				'background_gradient_list_' . $name,
				[
					'type'           => 'repeater',
					'label'          => __( 'Gradient Color List', 'piotnetgrid' ),
					'value'          => 'or',
					'label_block'    => true,
					'add_label'      => __( 'Add Item', 'piotnetgrid' ),
					'controls'       => $repeater_list,
					'controls_query' => '.piotnetgrid-control-repeater-list',
					'conditions'     => [
						[
							'name'     => 'background_gradient_' . $name,
							'operator' => '==',
							'value'    => 'yes',
						],
					],
				]
			);

			$this->add_control(
				'background_gradient_type_' . $name,
				[
					'type'        => 'select',
					'label'       => __( 'Gradient Type', 'piotnetgrid' ),
					'options'       => [
						'linear-gradient' => __( 'Linear', 'piotnetgrid' ),
						'radial-gradient' => __( 'Radial', 'piotnetgrid' ),
					],
					'value' => 'linear-gradient',
					'conditions'     => [
						[
							'name'     => 'background_gradient_' . $name,
							'operator' => '==',
							'value'    => 'yes',
						],
					],
				]
			);

			$this->add_control(
				'background_gradient_angle_' . $name,
				[
					'type'        => 'slider',
					'label'       => __( 'Gradient Angle', 'piotnetgrid' ),
					'value'       => [
						'unit' => 'degrees',
						'size' => '180',
					],
					'label_block' => true,
					'size_units'  => [
						'degrees'  => [
							'min'  => 0,
							'max'  => 360,
							'step' => 1,
						],
					],
					'conditions'     => [
						[
							'name'     => 'background_gradient_' . $name,
							'operator' => '==',
							'value'    => 'yes',
						],
						[
							'name'     => 'background_gradient_type_' . $name,
							'operator' => '==',
							'value'    => 'linear-gradient',
						],
					],
				]
			);

			$this->add_control(
				'background_gradient_position_' . $name,
				[
					'type'        => 'select',
					'label'       => __( 'Gradient Position', 'piotnetgrid' ),
					'options'       => [
						'center center' => __( 'Center Center', 'piotnetgrid' ),
						'center left' => __( 'Center Left', 'piotnetgrid' ),
						'center right' => __( 'Center Right', 'piotnetgrid' ),
						'top center' => __( 'Top Center', 'piotnetgrid' ),
						'top left' => __( 'Top Left', 'piotnetgrid' ),
						'top right' => __( 'Top Right', 'piotnetgrid' ),
						'bottom center' => __( 'Bottom Center', 'piotnetgrid' ),
						'bottom left' => __( 'Bottom Left', 'piotnetgrid' ),
						'bottom right' => __( 'Bottom Right', 'piotnetgrid' ),
					],
					'value' => 'center center',
					'conditions'     => [
						[
							'name'     => 'background_gradient_' . $name,
							'operator' => '==',
							'value'    => 'yes',
						],
						[
							'name'     => 'background_gradient_type_' . $name,
							'operator' => '==',
							'value'    => 'radial-gradient',
						],
					],
				]
			);

			$this->add_control(
				'background_gradient_opacity_' . $name,
				[
					'type'        => 'number',
					'label'       => __( 'Gradient Opacity', 'piotnetgrid' ),
					'conditions'     => [
						[
							'name'     => 'background_gradient_' . $name,
							'operator' => '==',
							'value'    => 'yes',
						],
					],
				]
			);

			$this->add_control(
				'background_gradient_css_' . $name,
				[
					'type'        => 'hidden',
					'selectors' => [
						$wrapper . '>.piotnetgrid-background-overlay' => '{{VALUE}}',
					],
					'conditions'     => [
						[
							'name'     => 'background_gradient_' . $name,
							'operator' => '==',
							'value'    => 'yes',
						],
					],
				]
			);

			$inner_section_controls = $this->get_group_controls();
	        $this->add_control(
	            'inner_section_background_gradient_' . $name,
	            [
	                'type'           => 'inner-section',
	                'label'          => __( 'Background Gradient', 'piotnetgrid' ),
	                'active'          => false,
	                'controls'       => $inner_section_controls,
	                'controls_query' => '.piotnetgrid-controls-inner-section__body',
	            ]
	        );
		}

        $this->new_group_controls();

		$this->add_control(
			'advanced_custom_css_' . $name,
			[
				'type'      => 'textarea',
				'label'     => 'Custom CSS',
				'label_block' => true,
				'placeholder' => 'color: blue;',
				'selectors' => [
					$wrapper => '{{VALUE}}',
				],
			]
		);

		$this->add_control(
			'advanced_custom_css_before_' . $name,
			[
				'type'      => 'textarea',
				'label'     => 'Custom CSS ::before',
				'label_block' => true,
				'placeholder' => 'content: "before";',
				'selectors' => [
					$wrapper . '::before' => '{{VALUE}}',
				],
			]
		);

		$this->add_control(
			'advanced_custom_css_after_' . $name,
			[
				'type'      => 'textarea',
				'label'     => 'Custom CSS ::after',
				'label_block' => true,
				'placeholder' => 'content: "after";',
				'selectors' => [
					$wrapper . '::after' => '{{VALUE}}',
				],
			]
		);

		$inner_section_controls = $this->get_group_controls();
        $this->add_control(
            'inner_section_custom_css_' . $name,
            [
                'type'           => 'inner-section',
                'label'          => __( 'Custom CSS', 'piotnetgrid' ),
                'active'          => false,
                'controls'       => $inner_section_controls,
                'controls_query' => '.piotnetgrid-controls-inner-section__body',
            ]
        );

		return $this->get_group_controls( $previous_controls );
	}

	protected function add_advanced_tab() {
		$this->start_tab( 'advanced', 'Advanced' );
		$this->start_section( 'advanced_section', 'Advanced' );
		$this->add_advanced_controls();

		$this->start_section( 'border_section', 'Border' );
		$this->add_border_controls();

		$this->start_section( 'background_section', 'Background' );
		$this->add_background_controls();

		// $this->add_repeater_trigger_controls();

		// if ( $this->is_add_conditional_logic ) {
		// 	$this->start_section( 'section_conditional_logic_new', 'Conditional Logic' );
		// 	$this->add_conditional_logic_controls();
		// }

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		$this->start_section( 'custom_css_section', 'Custom CSS' );
		$this->add_custom_css_controls();
	}

	protected function add_advanced_controls() {
		$this->add_responsive_control(
			'advanced_margin',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Margin', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em', 'rem' ],
				'selectors'   => [
					'{{WRAPPER}}' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'advanced_padding',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em', 'rem' ],
				'selectors'   => [
					'{{WRAPPER}}' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'advanced_width',
			[
				'type'        => 'text',
				'label'       => __( 'Width', 'piotnetgrid' ),
				'label_block' => true,
				'description' => __( 'Examples: 50%, 150px, calc(50% - 30px)', 'piotnetgrid'),
				'selectors'   => [
					'{{WRAPPER_FRONTEND}}' => 'width: {{VALUE}};',
					'{{WRAPPER_EDITOR}}' => 'width: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => 'advanced_width_inline',
						'operator' => '!=',
						'value'    => 'auto',
					],
				],
			]
		);

		$this->add_control(
			'advanced_width_inline',
			[
				'type'         => 'switch',
				'label'        => __( 'Width Inline (auto)', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'auto',
				'selectors'   => [
					'{{WRAPPER}}' => 'width: {{VALUE}}!important;',
					'{{WRAPPER_EDITOR}}' => 'width: {{VALUE}}!important;',
				],
			]
		);

		$this->add_box_shadow_and_custom_id();
	}

	protected function add_box_shadow_and_custom_id() {
		$this->add_control(
			'advanced_box_shadow',
			[
				'type'        => 'box-shadow',
				'label'       => __( 'Box Shadow', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => false,
				'render_type' => 'none',
				'selectors'   => [
					'{{WRAPPER}}' => 'box-shadow: {{VALUE}};',
				],
			]
		);

		$this->add_custom_class_id();
	}

	protected function add_custom_class_id() {
		$this->add_control(
			'advanced_custom_id',
			[
				'type'        => 'text',
				'label'       => __( 'Custom ID', 'piotnetgrid' ),
				'value'       => '',
				'placeholder' => '',
			]
		);

		$this->add_control(
			'advanced_custom_classes',
			[
				'type'        => 'text',
				'label'       => __( 'Custom Classes', 'piotnetgrid' ),
				'value'       => '',
				'placeholder' => '',
			]
		);
	}

	protected function add_border_controls() {
		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'advanced_border_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'advanced_border_hover_tab',
						'title' => __( 'HOVER', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_tab_border_controls(
			'advanced_border_normal',
			[
				'selectors' => '{{WRAPPER}}',
			]
		);
		$this->add_control(
			'advanced_border_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_tab_border_controls(
			'advanced_border_hover',
			[
				'selectors' => '{{WRAPPER}}:hover',
			]
		);
		$this->add_control(
			'advanced_border_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_tab_border_controls( string $name, $args = [] ) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';

		$this->new_group_controls();
		$this->add_control(
			$name . '_style',
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			$name . '_color',
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => $name . '_style',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name . '_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => $name . '_style',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name . '_radius',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		return $this->get_group_controls();
	}

	protected function add_background_controls() {
		$this->add_control(
			'advanced_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'placeholder' => '',
				'dynamic'     => true,
				'dynamic_field' => true,
				'selectors'   => [
					'{{WRAPPER}}' => 'background-color: {{VALUE}}',
				],
			]
		);
		$this->add_control(
			'advanced_background_image',
			[
				'type'      => 'media',
				'label'     => __( 'Choose Image', 'piotnetgrid' ),
				'value'     => '',
				'selectors' => [
					'{{WRAPPER}}' => 'background-image: url({{VALUE}})',
				],
			]
		);
		$this->add_control(
			'advanced_background_image_dynamic_data',
			[
				'type'        => 'select',
				'label'       => __( 'Dynamic Data', 'piotnetgrid' ),
				'options'     => [
					'' => __( 'None', 'piotnetgrid' ),
					'featured_image' => __( 'Featured Image', 'piotnetgrid' ),
					'author_profile_picture' => __( 'Author Profile Picture', 'piotnetgrid' ),
					'post_custom_field' => __( 'Post Custom Field', 'piotnetgrid' ),
					'acf_field' => __( 'ACF Field', 'piotnetgrid' ),
					'metabox_field' => __( 'Metabox Field', 'piotnetgrid' ),
					'pods_field' => __( 'Pods Field', 'piotnetgrid' ),
					'toolset_field' => __( 'Toolset Field', 'piotnetgrid' ),
					'jetengine_field' => __( 'JetEngine Field', 'piotnetgrid' ),
				],
			]
		);
		$this->add_control(
			'advanced_background_image_dynamic_data_field_name',
			[
				'type'        => 'text',
				'label'       => __( 'Field Name', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'advanced_background_image_dynamic_data',
						'operator' => 'in',
						'value'    => [
							'post_custom_field',
							'acf_field',
							'metabox_field',
							'pods_field',
							'toolset_field',
							'jetengine_field',
						],
					],
				],
			]
		);
		$this->add_control(
			'advanced_background_image_dynamic_data_size_author_profile_picture',
			[
				'type'    => 'text',
				'label'   => __( 'Profile Picture Size (px)', 'piotnetgrid' ),
				'value'   => '150',
				'conditions'  => [
					[
						'name'     => 'advanced_background_image_dynamic_data',
						'operator' => '==',
						'value'    => 'author_profile_picture',
					],
				],
			]
		);
		$this->add_responsive_control(
			'advanced_background_position',
			[
				'type'        => 'select',
				'label'       => __( 'Position', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					''              => 'Default',
					'center center' => 'Center Center',
					'center left'   => 'Center Left',
					'center right'  => 'Center Right',
					'top center'    => 'Top Center',
					'top left'      => 'Top Left',
					'top right'     => 'Top Right',
					'bottom center' => 'Bottom Center',
					'bottom left'   => 'Bottom Left',
					'bottom right'  => 'Bottom Right',
					'initial'       => 'Custom',
				],
				'selectors'   => [
					'{{WRAPPER}}' => 'background-position: {{VALUE}}',
				],
			]
		);
		$this->add_responsive_control(
			'advanced_background_attachment',
			[
				'type'        => 'select',
				'label'       => __( 'Attachment', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					''       => 'Default',
					'scroll' => 'scroll',
					'fixed'  => 'Fixed',
				],
				'selectors'   => [
					'{{WRAPPER}}' => 'background-attachment: {{VALUE}}',
				],
			]
		);
		$this->add_responsive_control(
			'advanced_background_repeat',
			[
				'type'        => 'select',
				'label'       => __( 'Repeat', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					''          => 'Default',
					'no-repeat' => 'No-repeat',
					'repeat'    => 'Repeat',
					'repeat-x'  => 'Repeat-x',
					'repeat-y'  => 'Repeat-y',
				],
				'selectors'   => [
					'{{WRAPPER}}' => 'background-repeat: {{VALUE}}',
				],
			]
		);
		$this->add_responsive_control(
			'advanced_background_size',
			[
				'type'        => 'select',
				'label'       => __( 'Size', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					''        => 'Default',
					'auto'    => 'Auto',
					'cover'   => 'Cover',
					'Contain' => 'Contain',
					'initial' => 'Custom',
				],
				'selectors'   => [
					'{{WRAPPER}}' => 'background-size: {{VALUE}}',
				],
			]
		);
	}

	protected function add_custom_css_controls() {
		$this->add_control(
			'advanced_custom_css',
			[
				'type'        => 'textarea',
				'label'       => __( 'Add your own custom CSS here', 'piotnetgrid' ),
				'description' => 'Use "{{WRAPPER}}" to target wrapper element. Examples:<br>
					{{WRAPPER}} {color: red;} // For main element<br>
					{{WRAPPER}} .child-element {margin: 10px;} // For child element<br>
					.my-class {text-align: center;} // Or use any custom selector',
				'label_block' => true,
			]
		);
	}

	protected function add_repeater_trigger_controls() {
		$this->start_section( 'repeater_trigger_section', 'Repeater Trigger' );
		$this->add_control(
			'piotnetgrid_repeater_enable_trigger',
			[
				'type'         => 'switch',
				'label'        => __( 'Enable', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'yes',
			]
		);
		$this->add_control(
			'piotnetgrid_repeater_form_id_trigger',
			[
				'type'        => 'text',
				'label'       => __( 'Form ID* (Required)', 'piotnetgrid' ),
				'value'       => '',
				'description' => __( 'Enter the same form id for all fields in a form, with latin character and no space. E.g order_form', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'piotnetgrid_repeater_enable_trigger',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);
		$this->add_control(
			'piotnetgrid_repeater_id_trigger',
			[
				'type'        => 'text',
				'label'       => __( 'Repeater ID* (Required)', 'piotnetgrid' ),
				'value'       => '',
				'description' => __( 'Enter Repeater ID with latin character and no space. E.g products_repeater', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'piotnetgrid_repeater_enable_trigger',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);
		$this->add_control(
			'piotnetgrid_repeater_trigger_action',
			[
				'type'        => 'select',
				'label'       => __( 'Action', 'piotnetgrid' ),
				'value'       => 'add',
				'options'     => [
					'add'    => __( 'Add', 'piotnetgrid' ),
					'remove' => __( 'Remove', 'piotnetgrid' ),
				],
				'description' => __( 'Enter Repeater ID with latin character and no space. E.g products_repeater', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'piotnetgrid_repeater_enable_trigger',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);
	}

	private function add_conditional_logic_controls() {
		$this->add_control(
			'piotnetgrid_conditional_logic_form_enable_new',
			[
				'type'         => 'switch',
				'label'        => __( 'Enable', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'yes',
				'description'  => __( 'This feature only works on the frontend.', 'piotnetgrid' ),
			]
		);
		$this->add_control(
			'piotnetgrid_conditional_logic_form_form_id',
			[
				'type'       => 'text',
				'label'      => __( 'Form ID', 'piotnetgrid' ),
				'value'      => '',
				'conditions' => [
					[
						'name'     => 'piotnetgrid_conditional_logic_form_enable_new',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);
		$this->add_control(
			'piotnetgrid_conditional_logic_form_speed_new',
			[
				'type'        => 'text',
				'label'       => __( 'Speed', 'piotnetgrid' ),
				'value'       => 400,
				'conditions'  => [
					[
						'name'     => 'piotnetgrid_conditional_logic_form_enable_new',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
				'description' => __( 'E.g 100, 1000, slow, fast', 'piotnetgrid' ),
			]
		);
		$this->add_control(
			'piotnetgrid_conditional_logic_form_easing_new',
			[
				'type'        => 'text',
				'label'       => __( 'Easing', 'piotnetgrid' ),
				'value'       => 'swing',
				'conditions'  => [
					[
						'name'     => 'piotnetgrid_conditional_logic_form_enable_new',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
				'description' => __( 'E.g swing, linear', 'piotnetgrid' ),
			]
		);

		$this->add_conditional_logic_repeater_controls();
	}

	private function add_conditional_logic_repeater_controls() {
		$this->new_group_controls();
		$this->add_control(
			'piotnetgrid_conditional_logic_form_if',
			[
				'type'        => 'select',
				'get_fields'  => true,
				'label'       => __( 'If', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'placeholder' => __( 'Field ID', 'piotnetgrid' ),
			]
		);
		$this->add_control(
			'piotnetgrid_conditional_logic_form_comparison_operators',
			[
				'type'        => 'select',
				'label'       => __( 'Comparison Operators', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					'not-empty' => __( 'not empty', 'piotnetgrid' ),
					'empty'     => __( 'empty', 'piotnetgrid' ),
					'='         => __( 'equals', 'piotnetgrid' ),
					'!='        => __( 'not equals', 'piotnetgrid' ),
					'>'         => __( '>', 'piotnetgrid' ),
					'>='        => __( '>=', 'piotnetgrid' ),
					'<'         => __( '<', 'piotnetgrid' ),
					'<='        => __( '<=', 'piotnetgrid' ),
					'checked'   => __( 'checked', 'piotnetgrid' ),
					'unchecked' => __( 'unchecked', 'piotnetgrid' ),
					'contains'  => __( 'contains', 'piotnetgrid' ),
				],
			]
		);
		$this->add_control(
			'piotnetgrid_conditional_logic_form_type',
			[
				'type'        => 'select',
				'label'       => __( 'Type Value', 'piotnetgrid' ),
				'value'       => 'string',
				'label_block' => true,
				'options'     => [
					'string' => __( 'String', 'piotnetgrid' ),
					'number' => __( 'Number', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
						'name'     => 'piotnetgrid_conditional_logic_form_comparison_operators',
						'operator' => 'in',
						'value'    => [ '=', '!=', '>', '>=', '<', '<=' ],
					],
				],
			]
		);
		$this->add_control(
			'piotnetgrid_conditional_logic_form_value',
			[
				'type'        => 'text',
				'label'       => __( 'Value', 'piotnetgrid' ),
				'value'       => '50',
				'label_block' => true,
				'conditions'  => [
					[
						'name'     => 'piotnetgrid_conditional_logic_form_comparison_operators',
						'operator' => 'in',
						'value'    => [ '=', '!=', '>', '>=', '<', '<=', 'contains' ],
					],
				],
			]
		);
		$this->add_control(
			'piotnetgrid_conditional_logic_form_and_or_operators',
			[
				'type'        => 'select',
				'label'       => __( 'OR, AND Operators', 'piotnetgrid' ),
				'value'       => 'or',
				'label_block' => true,
				'options' => [
					'or'  => __( 'OR', 'piotnetgrid' ),
					'and' => __( 'AND', 'piotnetgrid' ),
				],
			]
		);
		$this->add_control(
			'repeater_id',
			[
				'type' => 'hidden',
			],
            [
                'overwrite' => 'true',
            ]
		);
		$repeater_items = $this->get_group_controls();
		$this->new_group_controls();
		$this->add_control(
			'',
			[
				'type'           => 'repeater-item',
				'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
				'controls'       => $repeater_items,
				'controls_query' => '.piotnetgrid-control-repeater-field',
			]
		);
		$repeater_list = $this->get_group_controls();
		$this->add_control(
			'piotnetgrid_conditional_logic_form_list_new',
			[
				'type'           => 'repeater',
				'label'          => __( 'Conditional List', 'piotnetgrid' ),
				'value'          => 'or',
				'label_block'    => true,
				'add_label'      => __( 'Add Item', 'piotnetgrid' ),
				'controls'       => $repeater_list,
				'controls_query' => '.piotnetgrid-control-repeater-list',
				'conditions'     => [
					[
						'name'     => 'piotnetgrid_conditional_logic_form_enable_new',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);
	}

	protected function add_conditional_visibility_controls() {
		$this->add_control(
			'conditional_visibility_enable',
			[
				'type'         => 'switch',
				'label'        => __( 'Enable', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'yes',
			]
		);

		$roles_array = [
			'all'           => 'All',
			'non_logged_in' => 'Non Logged',
			'logged_in'     => 'Logged In',
		];
		global $wp_roles;
		$roles = $wp_roles->roles;
		foreach ( $roles as $key => $value ) {
			$roles_array[ $key ] = $value['name'];
		}
		$this->add_control(
			'conditional_visibility_roles',
			[
				'type'         => 'select2',
				'label'        => __( 'Visibility For User', 'piotnetgrid' ),
				'value'        => [
					'all',
				],
				'multiple'     => true,
				'options'      => $roles_array,
				'label_block'  => true,
				'return_value' => 'yes',
				'conditions'   => [
					[
						'name'     => 'conditional_visibility_enable',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);

		$this->add_control(
			'conditional_visibility_by_backend',
			[
				'type'         => 'switch',
				'label'        => __( 'Conditional Visibility By Custom Fields and URL Parameters, URL Contains', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'yes',
				'conditions'   => [
					[
						'name'     => 'conditional_visibility_enable',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);

		$this->add_control(
			'conditional_visibility_action',
			[
				'type'         => 'select',
				'label'        => __( 'Action', 'piotnetgrid' ),
				'value'        => 'show',
				'label_block'  => true,
				'options'      => [
					'show' => __( 'Show', 'piotnetgrid' ),
					'hide' => __( 'Hide', 'piotnetgrid' ),
				],
				'multiple'     => true,
				'return_value' => 'yes',
				'conditions'   => [
					[
						'name'     => 'conditional_visibility_enable',
						'operator' => '==',
						'value'    => 'yes',
					],
					[
						'name'     => 'conditional_visibility_by_backend',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);

		$this->add_conditional_visibility_repeater_controls();
	}

	private function add_conditional_visibility_repeater_controls() {
		$this->new_group_controls();
		$this->add_control(
			'conditional_visibility_by_backend_select',
			[
				'type'        => 'select',
				'label'       => __( 'Custom Fields or URL Parameters, URL Contains', 'piotnetgrid' ),
				'value'       => 'custom_field',
				'label_block' => true,
				'options'     => [
					'custom_field'  => __( 'Custom Field', 'piotnetgrid' ),
					'url_parameter' => __( 'URL Parameter', 'piotnetgrid' ),
					'url_contains'  => __( 'URL Contains', 'piotnetgrid' ),
				],
			]
		);

		$this->add_control(
			'conditional_visibility_custom_field_source',
			[
				'type'        => 'select',
				'label'       => __( 'Custom Fields', 'piotnetgrid' ),
				'value'       => 'post_custom_field',
				'label_block' => true,
				'options'     => [
					'post_custom_field' => __( 'Post Custom Field', 'piotnetgrid' ),
					'acf_field'         => __( 'ACF Field', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
						'name'     => 'conditional_visibility_by_backend_select',
						'operator' => '==',
						'value'    => 'custom_field',
					],
				],
			]
		);
		$this->add_control(
			'conditional_visibility_url_parameter',
			[
				'type'        => 'text',
				'label'       => __( 'URL Parameter', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'description' => __( 'E.g ref, yourparam', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'conditional_visibility_by_backend_select',
						'operator' => '==',
						'value'    => 'url_parameter',
					],
				],
			]
		);
		$this->add_control(
			'conditional_visibility_custom_field_key',
			[
				'type'        => 'text',
				'label'       => __( 'Custom Field Key', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'conditions'  => [
					[
						'name'     => 'conditional_visibility_by_backend_select',
						'operator' => '==',
						'value'    => 'custom_field',
					],
				],
			]
		);
		$this->add_control(
			'conditional_visibility_custom_field_comparison_operators',
			[
				'type'        => 'select',
				'label'       => __( 'Comparison Operators', 'piotnetgrid' ),
				'value'       => 'not-empty',
				'label_block' => true,
				'options'     => [
					'not-empty' => __( 'not empty', 'piotnetgrid' ),
					'empty'     => __( 'empty', 'piotnetgrid' ),
					'='         => __( 'equals', 'piotnetgrid' ),
					'!='        => __( 'not equals', 'piotnetgrid' ),
					'>'         => __( '>', 'piotnetgrid' ),
					'>='        => __( '>=', 'piotnetgrid' ),
					'<'         => __( '<', 'piotnetgrid' ),
					'<='        => __( '<=', 'piotnetgrid' ),
					'true'      => __( 'true', 'piotnetgrid' ),
					'false'     => __( 'false', 'piotnetgrid' ),
					'contains'  => __( 'contains (ACF Checkbox)', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
						'name'     => 'conditional_visibility_by_backend_select',
						'operator' => '!=',
						'value'    => 'url_contains',
					],
				],
			]
		);
		$this->add_control(
			'conditional_visibility_custom_field_type',
			[
				'type'        => 'select',
				'label'       => __( 'Type Value', 'piotnetgrid' ),
				'value'       => 'string',
				'label_block' => true,
				'options'     => [
					'string' => __( 'String', 'piotnetgrid' ),
					'number' => __( 'Number', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
						'name'     => 'conditional_visibility_custom_field_comparison_operators',
						'operator' => 'in',
						'value'    => [ '=', '!=', '>', '>=', '<', '<=', 'contains' ],
					],
				],
			]
		);
		$this->add_control(
			'conditional_visibility_custom_field_value',
			[
				'type'        => 'text',
				'label'       => __( 'Value', 'piotnetgrid' ),
				'value'       => '50',
				'label_block' => true,
				'conditions'  => [
					[
						'name'     => 'conditional_visibility_by_backend_select',
						'operator' => '!=',
						'value'    => 'url_contains',
					],
					[
						'name'     => 'conditional_visibility_custom_field_comparison_operators',
						'operator' => 'in',
						'value'    => [ '=', '!=', '>', '>=', '<', '<=', 'contains' ],
					],
				],
			]
		);
		$this->add_control(
			'conditional_visibility_custom_field_value_url_contains',
			[
				'type'        => 'text',
				'label'       => __( 'Value', 'piotnetgrid' ),
				'value'       => '50',
				'label_block' => true,
				'placeholder' => __( '/page-slug/', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'conditional_visibility_by_backend_select',
						'operator' => '==',
						'value'    => 'url_contains',
					],
				],
			]
		);
		$this->add_control(
			'conditional_visibility_and_or_operators',
			[
				'type'        => 'select',
				'label'       => __( 'OR, AND Operators', 'piotnetgrid' ),
				'value'       => 'or',
				'label_block' => true,
				'options'     => [
					'or'  => __( 'OR', 'piotnetgrid' ),
					'and' => __( 'AND', 'piotnetgrid' ),
				],
			]
		);
		// $this->add_control(
		// 	'conditional_visibility_comparison_operators',
		// 	[
		// 		'type'        => 'select',
		// 		'label'       => __( 'Comparison Operators', 'piotnetgrid' ),
		// 		'value'       => '',
		// 		'label_block' => true,
		// 		'options'     => [
		// 			'not-empty' => __( 'not empty', 'piotnetgrid' ),
		// 			'empty'     => __( 'empty', 'piotnetgrid' ),
		// 			'='         => __( 'equals', 'piotnetgrid' ),
		// 			'!='        => __( 'not equals', 'piotnetgrid' ),
		// 			'>'         => __( '>', 'piotnetgrid' ),
		// 			'>='        => __( '>=', 'piotnetgrid' ),
		// 			'<'         => __( '<', 'piotnetgrid' ),
		// 			'<='        => __( '<=', 'piotnetgrid' ),
		// 			'checked'   => __( 'checked', 'piotnetgrid' ),
		// 			'unchecked' => __( 'unchecked', 'piotnetgrid' ),
		// 			'contains'  => __( 'contains', 'piotnetgrid' ),
		// 		],
		// 	]
		// );
		$this->add_control(
			'conditional_visibility_type',
			[
				'type'        => 'select',
				'label'       => __( 'Type Value', 'piotnetgrid' ),
				'value'       => 'string',
				'label_block' => true,
				'options'     => [
					'string' => __( 'String', 'piotnetgrid' ),
					'number' => __( 'Number', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
						'name'     => 'conditional_visibility_comparison_operators',
						'operator' => 'in',
						'value'    => [ '=', '!=', '>', '>=', '<', '<=' ],
					],
				],
			]
		);
		$this->add_control(
			'conditional_visibility_value',
			[
				'type'        => 'text',
				'label'       => __( 'Value', 'piotnetgrid' ),
				'value'       => '50',
				'label_block' => true,
				'conditions'  => [
					[
						'name'     => 'conditional_visibility_comparison_operators',
						'operator' => 'in',
						'value'    => [ '=', '!=', '>', '>=', '<', '<=', 'contains' ],
					],
				],
			]
		);

		$this->add_control(
			'repeater_id',
			[
				'type' => 'hidden',
			],
			[
				'overwrite' => 'true',
			]
		);
		$repeater_items = $this->get_group_controls();

		$this->new_group_controls();
		$this->add_control(
			'',
			[
				'type'           => 'repeater-item',
				'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
				'controls'       => $repeater_items,
				'controls_query' => '.piotnetgrid-control-repeater-field',
			]
		);
		$repeater_list = $this->get_group_controls();
		$this->add_control(
			'conditional_visibility_by_backend_list',
			[
				'type'           => 'repeater',
				'label'          => __( 'Conditional List', 'piotnetgrid' ),
				'value'          => '',
				'label_block'    => true,
				'add_label'      => __( 'Add Item', 'piotnetgrid' ),
				'controls'       => $repeater_list,
				'controls_query' => '.piotnetgrid-control-repeater-list',
				'conditions'     => [
					[
						'name'     => 'conditional_visibility_enable',
						'operator' => '==',
						'value'    => 'yes',
					],
					[
						'name'     => 'conditional_visibility_by_backend',
						'operator' => '==',
						'value'    => 'yes',
					],
				],
			]
		);
	}

	private function conditions_old_version( $args = [] ) {
		if ( ! empty( $args['condition'] ) ) {
            if ( empty( $args['conditions'] ) ) {
                $args['conditions'] = [];
            }

			foreach ( $args['condition'] as $condition_key => $condition_value ) {
				$condition_operator = '==';

				if ( stripos( $condition_key, '!' ) !== false ) {
					$condition_operator = '!=';
					$condition_key      = str_replace( '!', '', $condition_key );
				}

				if ( is_array( $condition_value ) ) {
					$condition_operator = 'in';
					if ( stripos( $condition_key, '!' ) !== false ) {
						$condition_operator = '!in';
						$condition_key      = str_replace( '!', '', $condition_key );
					}
				}

				$args['conditions'][] = [
					'name'     => $condition_key,
					'operator' => $condition_operator,
					'value'    => $condition_value,
				];
			}
            unset( $args['condition'] );
		}

		if ( ! empty( $args['conditions'] ) ) {
			if ( ! empty( $args['conditions']['terms'] ) ) {
				$args['conditions'] = $args['conditions']['terms'];
				unset( $args['conditions']['terms'] );
			}
		}

		return $args;
	}

}
