<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class Piotnetgrid_Add_To_Cart_Button extends Base_Widget_piotnetgrid {
	public function get_type() {
		return 'button-add-to-cart';
	}

	public function get_class_name() {
		return 'Piotnetgrid_Add_To_Cart_Button';
	}

	public function get_title() {
		return 'Add To Cart';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-add-to-cart.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'button' ];
	}

	public function get_script() {
		return [
			'piotnetgrid-advanced-script',
		];
	}

	public function get_style() {
		return [ 
			'piotnetgrid-fontawesome-style'
		];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'button_styles_section', 'Add To Cart Button' );
		$this->add_setting_controls();

		$this->start_section( 'button_view_cart_styles_section', 'View Cart Button' );
		$this->add_setting_controls('--view-cart');

		$this->add_advanced_style_tab();

		$this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
		$this->add_custom_class_id();

		$this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
		$this->add_custom_css_controls();

		$this->add_animation_settings();

		$this->add_attributes_settings();

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		return $this->structure;
	}
	private function add_setting_controls( $type = '' ) {
		if (empty($type)) {
			$this->add_responsive_control(
				'button_add_to_cart_align' . $type,
				[
					'type'         => 'select',
					'label'        => __( 'Alignment', 'piotnetgrid' ),
					'label_block'  => true,
					'value'        => '',
					'options'      => [
						''       => __( 'Default', 'piotnetgrid' ),
						'flex-start'   => __( 'Left', 'piotnetgrid' ),
						'center' => __( 'Center', 'piotnetgrid' ),
						'flex-end'  => __( 'Right', 'piotnetgrid' ),
						'justify'  => __( 'Justify', 'piotnetgrid' ),
					],
					'render_type'  => 'both',
					'selectors'    => [
						'{{WRAPPER}}' => 'justify-content: {{VALUE}}',
						'{{WRAPPER}} .piotnetgrid-button-atc' => 'justify-content: {{VALUE}}',
					],
				]
			);
		}

		$this->add_control(
			'button_add_to_cart_icon' . $type,
			[
				'type'           => 'icon',
				'label'          => __( 'Button Icon', 'piotnetgrid' ),
				'value'          => '',
				'label_block'    => false,
				'placeholder'    => '',
				'options_source' => 'fontawesome',
			]
		);

		$this->add_control(
			'button_add_to_cart_icon_position' . $type,
			[
				'type'       => 'select',
				'label'      => __( 'Icon Position', 'piotnetgrid' ),
				'value'      => 'left',
				'options'    => [
					'before' => __( 'Before', 'piotnetgrid' ),
					'after'  => __( 'After', 'piotnetgrid' ),
				],
				'conditions' => [
					[
						'name'     => 'button_add_to_cart_icon',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);

		$this->add_style_controls( $type );
	}

	private function add_style_controls( $type = '' ) {
		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'button_normal_tab' . $type,
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'button_hover_tab' . $type,
						'title' => __( 'HOVER WIDGET', 'piotnetgrid' ),
					],
					[
						'name'  => 'button_hover_card_tab' . $type,
						'title' => __( 'HOVER CARD', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_button_style_controls(
			'normal' . $type,
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button' . $type,
			]
		);
		$this->add_control(
			'button_normal_tab' . $type,
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_button_style_controls(
			'hover' . $type,
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button' . $type . ':hover',
			]
		);
		$this->add_control(
			'button_hover_tab' . $type,
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_card_controls = $this->add_button_style_controls(
			'hover_card' . $type,
			[
				'selectors' => '.piotnetgrid-card:hover {{WRAPPER}} .piotnetgrid-button' . $type,
			]
		);
		$this->add_control(
			'button_hover_card_tab' . $type,
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover Card', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_card_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_button_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();
		$this->add_control(
			$name.'_button_color',
			[
				'type'        => 'color',
				'label'       => __( 'Text Color', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_text_typography_controls(
			$name.'_button_text_typography',
			[
				'selectors' => $wrapper,
			]
		);
		$this->add_control(
			$name.'_button_icon_color',
			[
				'type'        => 'color',
				'label'       => __( 'Icon Color', 'piotnetgrid' ),
				'label_block' => true,
				'conditions' => [
					'terms' => [
						[
							'name'     => 'button_add_to_cart_icon',
							'operator' => '!=',
							'value'    => '',
						],
					],
				],
				'selectors'   => [
					$wrapper . ' i' => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_icon_spacing',
			[
				'label'      => __( 'Icon Spacing', 'piotnetforms' ),
				'type'       => 'slider',
				'size_units' => [
					'px' => [
						'min'  => 1,
						'max'  => 200,
						'step' => 1,
					],
				],
				'value'      => [
					'unit' => 'px',
					'size' => '',
				],
				'conditions' => [
					'terms' => [
						[
							'name'     => 'button_add_to_cart_icon',
							'operator' => '!=',
							'value'    => '',
						],
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-button--after i' => 'margin-left: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-button--before i' => 'margin-right: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			$name.'_button_icon_size',
			[
				'label'      => __( 'Icon Size', 'piotnetforms' ),
				'type'       => 'slider',
				'size_units' => [
					'px' => [
						'min'  => 1,
						'max'  => 50,
						'step' => 1,
					],
				],
				'value'      => [
					'unit' => 'px',
					'size' => '',
				],
				'conditions' => [
					'terms' => [
						[
							'name'     => 'button_add_to_cart_icon',
							'operator' => '!=',
							'value'    => '',
						],
					],
				],
				'selectors'  => [
					'{{WRAPPER}} i' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'background-color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_type',
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_radius',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_padding',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		return $this->get_group_controls( $previous_controls );
	}

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
		global $product;
		$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-button-wrapper' );

		if ( $product ) {
			$button_classes = 'piotnetgrid-button piotnetgrid-button--add-to-cart';

			$this->add_render_attribute( 'button',
				[
					'href' => $product->add_to_cart_url(),
					'data-piotnetgrid-atc-product-id' => $product->get_id(),
					'class' => $button_classes,
					'aria-label' => esc_html($product->add_to_cart_text()),
					'data-piotnetgrid-atc-product-quantity' => 1,
				]
			);

			$this->add_render_attribute( 'button-view-cart',
				[
					'href' => wc_get_cart_url(),
					'class' => 'piotnetgrid-button piotnetgrid-button--view-cart',
				]
			);
		} else {
			$this->add_render_attribute( 'button',
				[
					'class' => 'piotnetgrid-button piotnetgrid-button--add-to-cart',
				]
			);
		}

		if (!empty($settings['button_add_to_cart_align_responsive_desktop'])) {
			if ($settings['button_add_to_cart_align_responsive_desktop'] == 'justify') {
				$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--justify' );
				$this->add_render_attribute( 'button-view-cart', 'class', 'piotnetgrid-button--justify' );
			}
		}

		$button_icon = !empty( $settings['button_add_to_cart_icon'] ) ? '<i class="' . $settings['button_add_to_cart_icon'] . '"></i>' : '';
		$button_icon_before = !empty($button_icon) && $settings['button_add_to_cart_icon_position'] == 'before' ? $button_icon : '';
		$button_icon_after = !empty($button_icon) && $settings['button_add_to_cart_icon_position'] == 'after' ? $button_icon : '';

		if (!empty($button_icon_before)) {
			$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--before' );
		}

		if (!empty($button_icon_after)) {
			$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--after' );
		}

		$button_icon_view_cart = !empty( $settings['button_add_to_cart_icon--view-cart'] ) ? '<i class="' . $settings['button_add_to_cart_icon--view-cart'] . '"></i>' : '';
		$button_icon_before_view_cart = !empty($button_icon_view_cart) && $settings['button_add_to_cart_icon_position--view-cart'] == 'before' ? $button_icon_view_cart : '';
		$button_icon_after_view_cart = !empty($button_icon_view_cart) && $settings['button_add_to_cart_icon_position--view-cart'] == 'after' ? $button_icon_view_cart : '';

		if (!empty($button_icon_before_view_cart)) {
			$this->add_render_attribute( 'button-view-cart', 'class', 'piotnetgrid-button--before' );
		}

		if (!empty($button_icon_after_view_cart)) {
			$this->add_render_attribute( 'button-view-cart', 'class', 'piotnetgrid-button--after' );
		}

		if ( $product ) :
        ?>
        <div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
        	<div class="piotnetgrid-button-atc piotnetgrid-button-atc--simple active">
        		<?php if ($product->is_type('simple')) : ?>
            		<span <?php echo $this->get_render_attribute_string( 'button' ); ?>><?php echo $button_icon_before; ?><?php echo esc_html($product->add_to_cart_text()) ?><?php echo $button_icon_after; ?></span>
            	<?php else : ?>
            		<a <?php echo $this->get_render_attribute_string( 'button' ); ?>><?php echo esc_html($product->add_to_cart_text()) ?></a>
            	<?php endif; ?>
        	</div>
            <?php
            	if ( $product->is_type('variable') ) :
            		$this->add_render_attribute( 'button',
						[
							'class' => 'piotnetgrid-button piotnetgrid-button--add-to-cart',
							'data-piotnetgrid-atc-product-variation' => '',
						]
					);
        	?>
        	<div class="piotnetgrid-button-atc piotnetgrid-button-atc--variation<?php if(!empty($settings['button_add_to_cart_align_responsive_desktop'])) { echo ' piotnetgrid-button-atc--' . $settings['button_add_to_cart_align_responsive_desktop']; } ?>">
            	<span <?php echo $this->get_render_attribute_string( 'button' ); ?>><?php echo $button_icon_before; ?><?php echo esc_html__( 'Add to cart', 'woocommerce' ); ?><?php echo $button_icon_after; ?></span>
        	</div>
        	<?php endif; ?>
            <a <?php echo $this->get_render_attribute_string( 'button-view-cart' ); ?>><?php echo $button_icon_before_view_cart; ?><?php echo esc_html__( 'View cart', 'woocommerce' ); ?><?php echo $button_icon_after_view_cart; ?></a>
        </div>
        <?php
        else :
    	?>
        	<div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
	            <span <?php echo $this->get_render_attribute_string( 'button' ); ?>><?php echo $button_icon_before; ?>Add to cart<?php echo $button_icon_after; ?></span>
	        </div>
        <?php
        endif;
	}

	public function live_preview() {
	}
}
