<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class Piotnetgrid_Add_To_Wishlist extends Base_Widget_piotnetgrid {
	public function get_type() {
		return 'add-to-wishlist';
	}

	public function get_class_name() {
		return 'Piotnetgrid_Add_To_Wishlist';
	}

	public function get_title() {
		return 'Add To Wishlist';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-add-to-wishlist.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'wishlist' ];
	}

	public function get_style() {
		return [ 
			'piotnetgrid-fontawesome-style'
		];
	}

	public function get_script() {
		return [
			'piotnetgrid-advanced-script',
		];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'add_to_wishlist_settings_section', 'Settings' );
		$this->add_setting_controls();

		$this->add_advanced_style_tab();

		$this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
		$this->add_custom_class_id();

		$this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
		$this->add_custom_css_controls();

		$this->add_animation_settings();

		$this->add_attributes_settings();

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		return $this->structure;
	}
	private function add_setting_controls() {
		$this->add_responsive_control(
			'add_to_wishlist_align',
			[
				'type'         => 'select',
				'label'        => __( 'Alignment', 'piotnetgrid' ),
				'label_block'  => true,
				'value'        => '',
				'options'      => [
					''       => __( 'Default', 'piotnetgrid' ),
					'flex-start'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end'  => __( 'Right', 'piotnetgrid' ),
					'justify'  => __( 'Justify', 'piotnetgrid' ),
				],
				'render_type'  => 'both',
				'selectors'    => [
					'{{WRAPPER}}' => 'justify-content: {{VALUE}}',
				],
			]
		);

		$this->add_style_controls();
	}

	private function add_style_controls() {
		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'add_to_wishlist_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'add_to_wishlist_hover_tab',
						'title' => __( 'HOVER', 'piotnetgrid' ),
					],
					[
						'name'  => 'add_to_wishlist_active_tab',
						'title' => __( 'ACTIVE', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_button_style_controls(
			'normal',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-add-to-wishlist__icon',
			]
		);
		$this->add_control(
			'add_to_wishlist_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_button_style_controls(
			'hover',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-add-to-wishlist__icon.piotnetgrid-add-to-wishlist__icon--normal:hover',
			]
		);
		$this->add_control(
			'add_to_wishlist_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$active_controls = $this->add_button_style_controls(
			'active',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-add-to-wishlist__icon.piotnetgrid-add-to-wishlist__icon--active',
			]
		);
		$this->add_control(
			'add_to_wishlist_active_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Active', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $active_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_button_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();

		$this->add_control(
			'add_to_wishlist_color_' . $name,
			[
				'type'        => 'color',
				'label'       => __( 'Color', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					$wrapper . ' i' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'add_to_wishlist_size_' . $name,
			[
				'label'      => __( 'Icon Size', 'piotnetforms' ),
				'type'       => 'slider',
				'size_units' => [
					'px' => [
						'min'  => 1,
						'max'  => 200,
						'step' => 1,
					],
				],
				'value'      => [
					'unit' => 'px',
					'size' => '30',
				],
				'selectors'  => [
					$wrapper . ' i' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'add_to_wishlist_background_color_' . $name,
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper . ' i' => 'background-color: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'add_to_wishlist_padding_' . $name,
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper . ' i' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'add_to_wishlist_border_radius_' . $name,
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper . ' i' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		
		return $this->get_group_controls( $previous_controls );
	}

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
		$post_id = get_the_ID();
		$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-add-to-wishlist' );
		$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-add-to-wishlist', $post_id );
		if (!empty($_SESSION['piotnetgrid_wishlist']) && !$editor) {
			if (in_array($post_id, $_SESSION['piotnetgrid_wishlist'])) {
				$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-add-to-wishlist--active' );
			}
		}
    	?>
        	<div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
        		<div class="piotnetgrid-add-to-wishlist__icon piotnetgrid-add-to-wishlist__icon--normal" data-piotnetgrid-add-to-wishlist-action="add">
        			<i class="far fa-heart"></i>
        		</div>
        		<div class="piotnetgrid-add-to-wishlist__icon piotnetgrid-add-to-wishlist__icon--active" data-piotnetgrid-add-to-wishlist-action="remove">
        			<i class="fas fa-heart"></i>
        		</div>
	        </div>
        <?php
	}

	public function live_preview() {
	}
}
