<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Button extends Base_Widget_piotnetgrid {
	public function get_type() {
		return 'button';
	}

	public function get_class_name() {
		return 'piotnetgrid_Button';
	}

	public function get_title() {
		return 'Button';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-button.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'button' ];
	}

	public function get_style() {
		return [ 
			'piotnetgrid-fontawesome-style'
		];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'button_settings_section', 'Settings' );
		$this->add_setting_controls();

		$this->start_tab( 'style', 'Style' );
		$this->start_section( 'button_styles_section', 'Style' );
		$this->add_style_controls();

		$this->add_advanced_style_tab();

		$this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
		$this->add_custom_class_id();

		$this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
		$this->add_custom_css_controls();

		$this->add_animation_settings();

		$this->add_attributes_settings();

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		$this->add_trigger_action_carousel();

		return $this->structure;
	}
	private function add_setting_controls() {
		$this->add_control(
			'button_text',
			[
				'type'        => 'text',
				'label'       => __( 'Button text', 'piotnetgrid' ),
				'value'       => 'Click here',
				'label_block' => false,
				'placeholder' => '',
			]
		);
		$this->add_control(
			'button_link',
			[
				'type'        => 'text',
				'label'       => __( 'Button Link', 'piotnetgrid' ),
				'dynamic'   => true,
				'label_block' => false,
				'placeholder' => '',
			]
		);
		$this->add_control(
			'button_link_target',
			[
				'type'         => 'select',
				'label'        => __( 'Link Target', 'piotnetgrid' ),
				'value'        => 'left',
				'options'      => [
					'_self'   => __( 'Self', 'piotnetgrid' ),
					'_blank' => __( 'Blank', 'piotnetgrid' ),
				],
				'prefix_class' => 'piotnetgrid%s-align-',
				'conditions' => [
					[
						'name' => 'button_link',
						'operator' => '!=',
						'value' => '#'
					],
					[
						'name' => 'button_link',
						'operator' => '!=',
						'value' => ''
					]
				]
			]
		);
		$this->add_responsive_control(
			'button_align',
			[
				'type'         => 'select',
				'label'        => __( 'Alignment', 'piotnetgrid' ),
				'label_block'  => true,
				'value'        => '',
				'options'      => [
					''       => __( 'Default', 'piotnetgrid' ),
					'flex-start'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end'  => __( 'Right', 'piotnetgrid' ),
					'justify'  => __( 'Justify', 'piotnetgrid' ),
				],
				'prefix_class' => 'piotnetgrid%s-align-',
				'render_type'  => 'both',
				'selectors'    => [
					'{{WRAPPER}}' => 'justify-content: {{VALUE}}',
				],
			]
		);
		$this->add_control(
			'button_icon',
			[
				'type'           => 'icon',
				'label'          => __( 'Button Icon', 'piotnetgrid' ),
				'value'          => '',
				'label_block'    => false,
				'placeholder'    => '',
				'options_source' => 'fontawesome',
			]
		);
		$this->add_control(
			'button_icon_position',
			[
				'type'       => 'select',
				'label'      => __( 'Icon Position', 'piotnetgrid' ),
				'value'      => 'left',
				'options'    => [
					'before' => __( 'Before', 'piotnetgrid' ),
					'after'  => __( 'After', 'piotnetgrid' ),
				],
				'conditions' => [
					[
						'name'     => 'button_icon',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_control(
			'button_class',
			[
				'type'        => 'text',
				'label'       => __( 'Button Class', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => false,
			]
		);
		$this->add_control(
			'button_id',
			[
				'type'        => 'text',
				'label'       => __( 'Button ID', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => false,
			]
		);
	}

	private function add_style_controls() {
		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'button_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'button_hover_tab',
						'title' => __( 'HOVER WIDGET', 'piotnetgrid' ),
					],
					[
						'name'  => 'button_hover_card_tab',
						'title' => __( 'HOVER CARD', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_button_style_controls(
			'normal',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button',
			]
		);
		$this->add_control(
			'button_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_button_style_controls(
			'hover',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button:hover',
			]
		);
		$this->add_control(
			'button_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_card_controls = $this->add_button_style_controls(
			'hover_card',
			[
				'selectors' => '.piotnetgrid-card:hover {{WRAPPER}} .piotnetgrid-button',
			]
		);
		$this->add_control(
			'button_hover_card_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover Card', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_card_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_button_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();
		$this->add_control(
			$name.'_button_color',
			[
				'type'        => 'color',
				'label'       => __( 'Text Color', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_text_typography_controls(
			$name.'_button_text_typography',
			[
				'selectors' => $wrapper,
			]
		);
		$this->add_control(
			$name.'_button_icon_color',
			[
				'type'        => 'color',
				'label'       => __( 'Icon Color', 'piotnetgrid' ),
				'label_block' => true,
				'conditions' => [
					'terms' => [
						[
							'name'     => 'button_icon',
							'operator' => '!=',
							'value'    => '',
						],
					],
				],
				'selectors'   => [
					$wrapper . ' i' => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_icon_spacing',
			[
				'label'      => __( 'Icon Spacing', 'piotnetforms' ),
				'type'       => 'slider',
				'size_units' => [
					'px' => [
						'min'  => 0,
						'max'  => 200,
						'step' => 1,
					],
				],
				'value'      => [
					'unit' => 'px',
					'size' => '',
				],
				'conditions' => [
					'terms' => [
						[
							'name'     => 'button_icon',
							'operator' => '!=',
							'value'    => '',
						],
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-button--after i' => 'margin-left: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-button--before i' => 'margin-right: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			$name.'_button_icon_size',
			[
				'label'      => __( 'Icon Size', 'piotnetforms' ),
				'type'       => 'slider',
				'size_units' => [
					'px' => [
						'min'  => 1,
						'max'  => 50,
						'step' => 1,
					],
				],
				'value'      => [
					'unit' => 'px',
					'size' => '',
				],
				'conditions' => [
					'terms' => [
						[
							'name'     => 'button_icon',
							'operator' => '!=',
							'value'    => '',
						],
					],
				],
				'selectors'  => [
					'{{WRAPPER}} i' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'background-color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_type',
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_radius',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_padding',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		return $this->get_group_controls( $previous_controls );
	}

	public function multilingual_fields() {
		$fields = [
			[
				'field'       => 'button_text',
				'editor_type' => 'LINE',
			],
		];

		return $fields;
	}

	public function render($editor = false, $args_additional = []) {

		$settings = $this->settings;

		$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-button-wrapper' );

		$button_icon = !empty( $settings['button_icon'] ) ? $settings['button_icon'] : '';
		$button_text = !empty( $settings['button_text'] ) ? __( $settings['button_text'], 'piotnetgrid' )  : '';

		$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button' );

		if (empty($button_text) && !empty($button_icon)) {
			$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--icon-only' );
		}

		if (!empty($settings['button_link'])) {
			$this->add_render_attribute( 'button', 'href', $settings['button_link'] );
		}

		if (!empty($settings['button_link_target'])) {
			$this->add_render_attribute( 'button', 'target', $settings['button_link_target'] );
		}

		if (!empty($settings['button_id'])) {
			$this->add_render_attribute( 'button', 'id', $settings['button_id'] );
		}

		if (!empty($settings['button_class'])) {
			$this->add_render_attribute( 'button', 'class', $settings['button_class'] );
		}

		if (!empty($settings['button_align_responsive_desktop'])) {
			if ($settings['button_align_responsive_desktop'] == 'justify') {
				$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--justify' );
			}
		}

		?>
		<div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
			<?php
			if ( !empty( $settings['button_icon'] ) && !empty( $settings['button_link'])) {
				wp_enqueue_style( $this->slug . '-fontawesome-style' );

				if ( $settings['button_icon_position'] == 'before' ) {
					$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--before' );
					?>
					<a <?php echo $this->get_render_attribute_string( 'button' ); ?>><i class="<?php echo $button_icon; ?>"></i><?php echo $button_text; ?></a>
				<?php } else { $this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--after' ); ?>
					<a <?php echo $this->get_render_attribute_string( 'button' ); ?>><?php echo $button_text; ?><i class="<?php echo $button_icon; ?>"></i></a>
				<?php } ?>
				
			    <?php } elseif (!empty( $settings['button_icon'] ) && empty( $settings['button_link'])) {

				if ( $settings['button_icon_position'] == 'before' ) {
					$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--before' );
					?>
					<span <?php echo $this->get_render_attribute_string( 'button' ); ?>><i class="<?php echo $button_icon; ?>"></i><?php echo $button_text; ?></span>
				<?php } else { $this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--after' ); ?>
					<span <?php echo $this->get_render_attribute_string( 'button' ); ?>><?php echo $button_text; ?><i class="<?php echo $button_icon; ?>"></i></span>
				<?php }

                } elseif (!empty( $settings['button_link'])){ ?>
				    <a <?php echo $this->get_render_attribute_string( 'button' ); ?>><?php echo $button_text; ?></a>
			    <?php } else { ?>
				    <span <?php echo $this->get_render_attribute_string( 'button' ); ?>><?php echo $button_text; ?></span>
			    <?php }
			?>
		</div>	
		<?php
	}
	public function live_preview() {
		?>
		<?php
	}

}
