<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

require_once __DIR__ . '/../widgets/division_base.php';

class piotnetgrid_Column extends Division_Base_Widget_piotnetgrid {
	public function get_type() {
		return 'column';
	}

	public function get_class_name() {
		return 'piotnetgrid_Column';
	}

	public function get_title() {
		return 'Column';
	}

	public function get_icon() {
		return 'fas fa-columns';
	}

	public function get_categories() {
		return [ 'pafe-form-builder' ];
	}

	public function get_keywords() {
		return [ 'input', 'form', 'field' ];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'column_settings', 'Layout' );
		$this->add_setting_controls();

		$this->add_advanced_style_tab();

		$this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
		$this->add_custom_class_id();

		$this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
		$this->add_custom_css_controls();

		$this->add_animation_settings();

		$this->add_attributes_settings();

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		return $this->structure;
	}

	private function add_setting_controls() {
		$this->add_responsive_control(
			'column_width',
			[
				'type'        => 'number',
				'label'       => __( 'Column Width (%)', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					'{{WRAPPER}}' => 'width: {{VALUE}}%;',
				],
			]
		);

		$this->add_responsive_control(
			'flex_order',
			[
				'type'        => 'number',
				'label'       => __( 'Flex Order', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => '',
				'selectors'   => [
					'{{WRAPPER}}' => 'order: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'vertical_align',
			[
				'type'        => 'select',
				'label'       => __( 'Align Items', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => '',
				'options'     => [
					'' => __( 'Default', 'piotnetgrid' ),
					'center' => __( 'center', 'piotnetgrid' ),
					'flex-start' => __( 'flex-start', 'piotnetgrid' ),
					'flex-end' => __( 'flex-end', 'piotnetgrid' ),
					'baseline' => __( 'baseline', 'piotnetgrid' ),
				],
				'selectors'   => [
					'{{WRAPPER}}>.piotnetgrid-column__inner' => 'align-items: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'align_content',
			[
				'type'        => 'select',
				'label'       => __( 'Align Content', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => '',
				'options'     => [
					'' => __( 'Default', 'piotnetgrid' ),
					'center' => __( 'center', 'piotnetgrid' ),
					'flex-start' => __( 'flex-start', 'piotnetgrid' ),
					'flex-end' => __( 'flex-end', 'piotnetgrid' ),
					'space-between' => __( 'space-between', 'piotnetgrid' ),
					'space-around' => __( 'space-around', 'piotnetgrid' ),
					'space-evenly' => __( 'space-evenly', 'piotnetgrid' ),
				],
				'selectors'   => [
					'{{WRAPPER}}>.piotnetgrid-column__inner' => 'align-content: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'justify_content',
			[
				'type'        => 'select',
				'label'       => __( 'Justify Content', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'options'     => [
					''              => __( 'Default', 'piotnetgrid' ),
					'center'        => __( 'center', 'piotnetgrid' ),
					'flex-start'    => __( 'flex-start', 'piotnetgrid' ),
					'flex-end'      => __( 'flex-end', 'piotnetgrid' ),
					'space-between' => __( 'space-between', 'piotnetgrid' ),
					'space-around' => __( 'space-around', 'piotnetgrid' ),
					'space-evenly' => __( 'space-evenly', 'piotnetgrid' ),
				],
				'selectors'   => [
					'{{WRAPPER}}>.piotnetgrid-column__inner' => 'justify-content: {{VALUE}};',
				],
			]
		);
	}

	private function add_style_controls() {
		$this->add_text_typography_controls(
			'column_typography',
			[
				'selectors' => '{{WRAPPER}}',
			]
		);
		$this->add_responsive_control(
			'column_text_align',
			[
				'type'         => 'select',
				'label'        => __( 'Text Align', 'piotnetgrid' ),
				'value'        => '',
				'label_block'  => true,
				'options'      => [
					''              => __( 'Default', 'piotnetgrid' ),
					'left'          => __( 'Left', 'piotnetgrid' ),
					'center'        => __( 'Center', 'piotnetgrid' ),
					'right'         => __( 'Right', 'piotnetgrid' ),
					'justify'       => __( 'Justified', 'piotnetgrid' ),
					'space-between' => __( 'space-between', 'piotnetgrid' ),
				],
				'prefix_class' => 'piotnetgrid%s-align-',
				'selectors'    => [
					'{{WRAPPER}}' => 'text-align: {{VALUE}}',
				],
			]
		);
	}

	protected function add_advanced_controls() {
		$this->add_responsive_control(
			'advanced_margin',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Margin', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					'{{WRAPPER}}>.piotnetgrid-column__inner' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'advanced_padding',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					'{{WRAPPER}}>.piotnetgrid-column__inner' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_box_shadow_and_custom_id();
	}

	public function render_start( $editor = false ) {
		?>
			<?php if ( $editor ) : ?>
				<div class="piotnetgrid-column__controls" data-piotnetgrid-column-controls>
					<div class="piotnetgrid-column__controls-item piotnetgrid-column__controls-item--edit" title="Edit" data-piotnetgrid-control-edit>
						<i class="fas fa-th"></i>
					</div>
					<div class="piotnetgrid-column__controls-item piotnetgrid-column__controls-item--duplicate" title="Duplicate" data-piotnetgrid-control-duplicate>
						<i class="far fa-clone"></i>
					</div>
					<div class="piotnetgrid-column__controls-item piotnetgrid-column__controls-item--remove" title="Delete" data-piotnetgrid-control-remove>
						<i class="fas fa-times"></i>
					</div>
				</div>
			<?php endif; ?>
			<div class="piotnetgrid-background-overlay"></div>
			<div class="piotnetgrid-column__inner"
			<?php
			if ( $editor ) :
				?>
				 data-piotnetgrid-sortable data-piotnetgrid-inner-html<?php endif; ?>>
		<?php
	}

	public function render_end() {
		?>
			</div>
		<?php
	}

	public function render($editor = false, $args_additional = []) {
		?>
		<?php
	}

	public function live_preview() {
		?>
		<?php
	}
}
