<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Facet_Embed_Elementor extends \Elementor\Widget_Base {
	public function get_name() {
		return 'facet_embed';
	}

	public function get_title() {
		return 'Facet';
	}

	public function get_icon() {
		return 'piotnetgrid-icon piotnetgrid-icon--facet';
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'grid', 'piotnetgrid', 'piotnet' ];
	}

	public function get_script_depends() {
		return [ 
			'piotnetgrid-script',
		];
	}

	public function get_style_depends() {
		return [ 
			'piotnetgrid-style',
			'piotnetgrid-fontawesome-style',
		];
	}

	protected function _register_controls() {

		$this->start_controls_section(
			'facet_section',
			[
				'label' => __( 'Facet', 'piotnetgrid' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$facet_query = new WP_Query(
			array(
				'post_type' => 'piotnetgrid-facet',
				'posts_per_page' => -1,
			)
		);

		$facets = [];

		if($facet_query->have_posts()) : while($facet_query->have_posts()) : $facet_query->the_post();
			$facets[ get_the_ID() ] = html_entity_decode( get_the_title() );
		endwhile; endif; wp_reset_postdata();

		$this->add_control(
			'facet',
			[
				'type' => \Elementor\Controls_Manager::SELECT,
				'label' => __( 'Select a facet', 'piotnetgrid' ),
				'options' => $facets,
				'default' => '',
			]
		);

		$grid_query = new WP_Query(
			array(
				'post_type' => 'piotnetgrid',
				'posts_per_page' => -1,
			)
		);

		$grids = [
			'0' => __( 'None', 'piotnetgrid' ),
		];

		if($grid_query->have_posts()) : while($grid_query->have_posts()) : $grid_query->the_post();
			$grids[ get_the_ID() ] = html_entity_decode( get_the_title() );
		endwhile; endif; wp_reset_postdata();

		$this->add_control(
			'facet_grid',
			[
				'type' => \Elementor\Controls_Manager::SELECT,
				'label' => __( 'Apply for Grid', 'piotnetgrid' ),
				'options' => $grids,
			]
		);

		$this->end_controls_section();
	}

	public function render() {
		$settings = $this->get_settings_for_display();
		$facet = !empty($settings['facet']) ? $settings['facet'] : '';
		$grid = !empty($settings['facet_grid']) ? $settings['facet_grid'] : '';
		$editor_attr = \Elementor\Plugin::$instance->editor->is_edit_mode() ? ' editor=true' : '';
		$shortcode = '[piotnetgrid id=' . $facet . ' type=facet grid=' . $grid . $editor_attr . ']';

		if ( ! empty( $facet ) ) {
			echo '<div ' . $this->get_render_attribute_string( 'wrapper' ) . '>';
			echo do_shortcode( $shortcode );
			echo '</div>';
		}
	}
}
