<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class Piotnetgrid_Facet_Active extends Base_Widget_piotnetgrid {

	public function get_type() {
		return 'facet_active';
	}

	public function get_class_name() {
		return 'Piotnetgrid_Facet_Active';
	}

	public function get_title() {
		return 'Active Facets';
	}

	public function get_icon() {
		return [
			'type'  => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-facet-active.svg',
		];
	}

	public function get_categories() {
		return [ 'pafe-form-builder' ];
	}

	public function get_keywords() {
		return [ 'filter' ];
	}

	public function get_script() {
		return [
			'piotnetgrid-advanced-script',
		];
	}

	private function add_setting_controls() {
		$this->add_responsive_control(
			'reset_filter_align',
			[
				'type'    => 'select',
				'label'   => 'Alignment',
				'value'   => '',
				'label_block' => true,
				'render_type' => 'both', 
				'options' => [
					''        => __( 'Default', 'piotnetforms' ),
					'flex-start'    => __( 'Left', 'piotnetforms' ),
					'center'  => __( 'Center', 'piotnetforms' ),
					'flex-end'   => __( 'Right', 'piotnetforms' ),
				],
				'selectors' => [
					'{{WRAPPER}}.piotnetgrid-facet-active ul' => 'justify-content: {{VALUE}}',
				]
			]
		);
		
	}

	private function add_style_controls() {
		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'text_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'text_hover_tab',
						'title' => __( 'HOVER', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_text_style_controls(
			'normal',
			[
				'selectors' => '{{WRAPPER}}.piotnetgrid-facet-active ul.piotnetgrid-facet-list-active>li',
			]
		);

		$this->add_control(
			'text_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_text_style_controls(
			'hover',
			[
				'selectors' => '{{WRAPPER}}.piotnetgrid-facet-active ul>li:hover',
			]
		);
		
		$this->add_control(
			'text_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover Widget', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_text_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();
		$this->add_control(
			$name.'_button_color',
			[
				'type'        => 'color',
				'label'       => __( 'Text Color', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_text_typography_controls(
			$name.'_button_text_typography',
			[
				'selectors' => $wrapper,
			]
		);
		$this->add_control(
			$name.'_button_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'background-color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_type',
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_radius',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_padding',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		return $this->get_group_controls( $previous_controls );
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'filter_settings_section', 'Settings' );
		$this->add_setting_controls();

		$this->start_tab( 'style', 'Style' );
		$this->start_section( 'filter_style_section', 'Style' );
		$this->add_style_controls();

		$this->add_advanced_tab();

		return $this->structure;
	}

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
		$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-facet-active' );
		$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-active-wrapper', '' );
		$grid_post_id = !empty($args_additional['grid']) ? $args_additional['grid'] : '';
		?>
        <div <?php echo $this->get_render_attribute_string( 'wrapper' ) ?>><ul class="piotnetgrid-facet-list-active" data-piotnetgrid-facet-active="<?php echo $grid_post_id; ?>">
			<?php if($editor): ?>
				<li class="piotnetgrid-facet-active-item" data-piotnetgrid-facet-active-id="p61617717" data-piotnetgrid-facet-active-type="radio" data-piotnetgrid-facet-active-name="_achv" data-piotnetgrid-filter-value="lorem"><svg viewBox="0 0 45.334 45.334"><path d="M43.576,35.092L31.152,22.668l12.424-12.424c2.343-2.343,2.343-6.142,0-8.485c-2.343-2.343-6.142-2.343-8.485,0   L22.666,14.183L10.242,1.758c-2.344-2.343-6.143-2.343-8.485,0c-2.343,2.344-2.343,6.143,0,8.485l12.425,12.424L1.757,35.091   c-2.343,2.344-2.343,6.143,0,8.485C2.93,44.748,4.464,45.333,6,45.333c1.535,0,3.071-0.586,4.243-1.757l12.423-12.423   l12.425,12.424c1.173,1.171,2.707,1.757,4.243,1.757s3.071-0.586,4.243-1.757C45.919,41.234,45.919,37.436,43.576,35.092z" /></svg>&ensp;Facet Active</li>
			<?php endif; ?>
		</ul></div>
		<?php
	}

	public function live_preview() {
		
	}
}
