<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Facet_Embed extends Base_Widget_piotnetgrid {
	public function get_type() {
		return 'facet_embed';
	}

	public function get_class_name() {
		return 'piotnetgrid_Facet_Embed';
	}

	public function get_title() {
		return 'Facet';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-facet.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'facet' ];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'settings_section', 'Facet Settings' );
		$this->add_setting_controls();

		$this->add_advanced_tab();

		return $this->structure;
	}

	private function add_setting_controls() {

		$facet_query = new WP_Query(
			array(
				'post_type' => 'piotnetgrid-facet',
				'posts_per_page' => -1,
			)
		);

		$facets = [];

		if($facet_query->have_posts()) : while($facet_query->have_posts()) : $facet_query->the_post();
			$facets[ get_the_ID() ] = html_entity_decode( get_the_title() );
		endwhile; endif; wp_reset_postdata();

		$this->add_control(
			'facet',
			[
				'type'        => 'select',
				'label'       => __( 'Select Facet', 'piotnetgrid' ),
				'options'     => $facets,
				'description_has_value' => '<a href="' . admin_url() . 'admin.php?page=piotnetgrid&post=' . '{{VALUE}}" target="_blank">Edit this facet</a>',
			]
		);

		$grid_query = new WP_Query(
			array(
				'post_type' => 'piotnetgrid',
				'posts_per_page' => -1,
			)
		);

		$grids = [
			'0' => __( 'None', 'piotnetgrid' ),
		];

		if($grid_query->have_posts()) : while($grid_query->have_posts()) : $grid_query->the_post();
			$grids[ get_the_ID() ] = html_entity_decode( get_the_title() );
		endwhile; endif; wp_reset_postdata();

		$this->add_control(
			'facet_grid',
			[
				'type'        => 'select',
				'label'       => __( 'Apply for Grid', 'piotnetgrid' ),
				'options'     => $grids,
				'description_has_value' => '<a href="' . admin_url() . 'admin.php?page=piotnetgrid&post=' . '{{VALUE}}" target="_blank">Edit this grid</a>',
			]
		);
	}

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
		$facet = !empty($settings['facet']) ? $settings['facet'] : '';
		$grid = !empty($settings['facet_grid']) ? $settings['facet_grid'] : '';
		$editor_attr = $editor || !empty($args_additional['editor']) ? ' editor=true' : '';
		$shortcode = '[piotnetgrid id=' . $facet . ' type=facet grid=' . $grid . $editor_attr . ']';

		if ( ! empty( $facet ) && strpos( $facet, 'export' ) == false ) {
			echo '<div ' . $this->get_render_attribute_string( 'wrapper' ) . '>';
			echo do_shortcode( $shortcode );
			echo '</div>';
		}
	}

	public function live_preview() {
		?>
		<?php
	}
}
