<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class Piotnetgrid_Facet_Reset extends Base_Widget_piotnetgrid {

	public function get_type() {
		return 'reset';
	}

	public function get_class_name() {
		return 'Piotnetgrid_Facet_Reset';
	}

	public function get_title() {
		return 'Reset Facets';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-facet-reset.svg',
		];
	}

	public function get_categories() {
		return [ 'pafe-form-builder' ];
	}

	public function get_keywords() {
		return [ 'filter' ];
	}

	public function get_script() {
		return [
			'piotnetgrid-advanced-script',
		];
	}

	private function add_setting_controls() {
		$this->add_control(
			'reset_filter_text_button',
			[
				'label'       => __( 'Text', 'piotnetgrid' ),
				'type'        => 'text',
				'value'       => __( 'Reset', 'piotnetgrid' ),
			]
		);
		$this->add_responsive_control(
			'reset_filter_align',
			[
				'type'    => 'select',
				'label'   => 'Alignment',
				'value'   => '',
				'render_type' => 'both', 
				'options' => [
					''        => __( 'Default', 'piotnetforms' ),
					'left'    => __( 'Left', 'piotnetforms' ),
					'center'  => __( 'Center', 'piotnetforms' ),
					'right'   => __( 'Right', 'piotnetforms' ),
					'justify' => __( 'Justify', 'piotnetforms' ),
				],
				'selectors' => [
					'{{WRAPPER}}' => 'text-align: {{VALUE}}',
				]
			]
		);
		
	}

	private function add_style_controls() {
		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'text_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'text_hover_tab',
						'title' => __( 'HOVER', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_text_style_controls(
			'normal',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button',
			]
		);

		$this->add_control(
			'text_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_text_style_controls(
			'hover',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button:hover',
			]
		);
		
		$this->add_control(
			'text_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover Widget', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_text_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();
		$this->add_control(
			$name.'_button_color',
			[
				'type'        => 'color',
				'label'       => __( 'Text Color', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_text_typography_controls(
			$name.'_button_text_typography',
			[
				'selectors' => $wrapper,
			]
		);
		$this->add_control(
			$name.'_button_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'background-color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_type',
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_radius',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_padding',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		return $this->get_group_controls( $previous_controls );
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'filter_settings_section', 'Settings' );
		$this->add_setting_controls();

		$this->start_tab( 'style', 'Style' );
		$this->start_section( 'filter_style_section', 'Style' );
		$this->add_style_controls();

		$this->add_advanced_tab();

		return $this->structure;
	}

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
		$grid_post_id = !empty($args_additional['grid']) ? $args_additional['grid'] : '';
		if (!empty($settings['reset_filter_align_responsive_desktop'])) {
			if ($settings['reset_filter_align_responsive_desktop'] == 'justify') {
				$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button--justify' );
			}
		}
		$this->add_render_attribute( 'button', 'class', 'piotnetgrid-button' );
		$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-facet-reset-button' );
        echo '<div '.$this->get_render_attribute_string( 'wrapper' ).'><button type="button" '.$this->get_render_attribute_string( 'button' ).' data-piotnetgrid-facet-reset="'.$grid_post_id.'">'.$settings['reset_filter_text_button'].'</button></div>';
	}

	public function live_preview() {
		
	}
}
