<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Facet extends Base_Widget_piotnetgrid {
	public $post_types;

	public function __construct() {
    	parent::__construct();
    	$this->post_types = get_post_types();
	}

	public function get_type() {
		return 'facet';
	}

	public function get_class_name() {
		return 'piotnetgrid_Facet';
	}

	public function get_title() {
		return 'Facet';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-facet.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'facet' ];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'text_settings_section', 'Settings' );
		$this->add_setting_controls();

        $this->start_section('scroll_to_top_settings_section', 'Scroll To Top',
            [
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'facet_action',
                            'operator' => 'in',
                            'value'    => ['load'],
                        ],
                        [
                            'name'     => 'loading_type',
                            'operator' => 'in',
                            'value'    => ['pagination'],
                        ],
                    ],
                ]
            ]
        );
        $this->add_scroll_to_top_settings();

        $this->start_section('show_more_and_show_less_section', 'Show more/Show less Options',
            [
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'facet_action',
                            'operator' => 'in',
                            'value'    => ['filter'],
                        ],
                        [
                            'name'     => 'filter_type',
                            'operator' => 'in',
                            'value'    => ['checkboxes','radio'],
                        ],
                    ],
                ]
            ]
        );
        $this->add_show_more_or_show_less_btn_controls();

		$this->start_tab( 'style', 'Style' );

		$this->start_section(
			'facet_label_styles_section',
			'Facet Label',
			[
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['filter','sort'],
						]
					],
				],
			]
		);
		$this->add_label_style_controls();

		$this->start_section(
			'facet_field_styles_section',
			'Facet Field',
			[
				'conditions' => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['filter','sort'],
						],
						[
							'name'     => 'filter_type',
							'operator' => '!in',
							'value'    => ['slider','checkboxes', 'radio', 'buttons','color-picker','image-select','az'],
						],
					],
				]
			]
		);
		$this->add_field_style_controls();

		$this->start_section(
			'facet_slider_styles_section',
			'Facet Slider',
			[
				'conditions' => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['filter'],
						],
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => ['slider'],
						],
					],
				]
			]
		);
		$this->add_slider_style_controls();

		$this->start_section(
			'facet_pagination_styles_section',
			'Facet Pagination',
			[
				'conditions' => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['load'],
						],
						[
							'name'     => 'loading_type',
							'operator' => 'in',
							'value'    => ['pagination'],
						],
					],
				]
			]
		);
		$this->add_pagination_style_controls();

        $this->start_section(
            'facet_az_styles_section',
            'Facet A-Z',
            [
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'facet_action',
                            'operator' => 'in',
                            'value'    => ['filter'],
                        ],
                        [
                            'name'     => 'filter_type',
                            'operator' => 'in',
                            'value'    => ['az'],
                        ],
                    ],
                ]
            ]
        );
        $this->add_a_z_style_controls();

		$this->start_section(
			'facet_load_more_styles_section',
			'Facet Load More',
			[
				'conditions' => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['load'],
						],
						[
							'name'     => 'loading_type',
							'operator' => 'in',
							'value'    => ['load_more'],
						],
					],
				]
			]
		);
		$this->add_load_more_style_controls();

		$this->start_section(
			'result_counts_styles_section',
			'Result Counts',
			[
				'conditions' => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['load'],
						],
						[
							'name'     => 'loading_type',
							'operator' => 'in',
							'value'    => ['result_counts'],
						],
					],
				]
			]
		);
		$this->add_result_counts_style_controls();

		$this->start_section(
			'facet_checkbox_styles_section',
			'Facet Checkboxes / Radio',
			[
				'conditions' => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['filter'],
						],
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => ['checkboxes','radio'],
						],
					],
				]
			]
		);
		$this->add_checkbox_style_controls();

        $this->start_section(
            'facet_show_more_and_show_less_styles_section',
            'Show more/Show less Button',
            [
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'facet_action',
                            'operator' => 'in',
                            'value'    => ['filter'],
                        ],
                        [
                            'name'     => 'filter_type',
                            'operator' => 'in',
                            'value'    => ['checkboxes','radio'],
                        ],
                    ],
                ]
            ]
        );
        $this->add_show_more_and_show_less_style_controls();

		$this->start_section(
			'facet_checkbox_advanced_style_section',
			'Facet Buttons / Image Select',
			[
				'conditions' => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['filter'],
						],
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => ['buttons','image-select'],
						],
					],
				]
			]
		);
		$this->add_checkbox_advanced_style_controls();

		$this->start_section(
			'facet_color_picker_style_section',
			'Facet Color Picker',
			[
				'conditions' => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['filter'],
						],
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => ['color-picker'],
						],
					],
				]
			]
		);
		$this->add_color_picker_style_controls();

		$this->add_advanced_tab();

		return $this->structure;
	}
	
	private function add_scroll_to_top_settings() {

        $this->add_control(
            'facet_scroll_to_top',
            [
                'label' => __( 'Enable', 'piotnetgrid' ),
                'type' => 'switch',
                'default' => '',
                'label_on' => 'Yes',
                'label_off' => 'No',
                'return_value' => 'yes',
            ]
        );

        $this->add_control(
            'facet_scroll_to_top_offset_desktop',
            [
                'label' => __( 'Desktop Negative Offset Top (px)', 'piotnetgrid' ),
                'type' => 'number',
                'default' => 0,
                'condition' => [
                    'facet_scroll_to_top' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'facet_scroll_to_top_offset_tablet',
            [
                'label' => __( 'Tablet Negative Offset Top (px)', 'piotnetgrid' ),
                'type' => 'number',
                'default' => 0,
                'condition' => [
                    'facet_scroll_to_top' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'facet_scroll_to_top_offset_mobile',
            [
                'label' => __( 'Mobile Negative Offset Top (px)', 'piotnetgrid' ),
                'type' => 'number',
                'default' => 0,
                'condition' => [
                    'facet_scroll_to_top' => 'yes',
                ],
            ]
        );

    }

    private function add_show_more_or_show_less_btn_controls() {
        $this->add_control(
            'facet_limit_choices_option',
            [
                'type'        => 'number',
                'label'       => __( 'Limit Choices Number', 'piotnetgrid' ),
            ]
        );

        $this->add_control(
            'facet_show_more_button_label',
            [
                'type'        => 'text',
                'label'       => __( 'Expand Button label', 'piotnetgrid' ),
                'value'    => 'Show more',
            ]
        );

        $this->add_control(
            'facet_show_less_button_label',
            [
                'type'        => 'text',
                'label'       => __( 'Collapse Button label', 'piotnetgrid' ),
                'value'    => 'Show less',
            ]
        );
    }

	private function add_setting_controls() {

		$this->add_control(
			'facet_name',
			[
				'type'        => 'text',
				'label'       => __( 'Facet Name', 'piotnetgrid' ),
				'description' => __( 'Facet Name has to be unique, must be different with any other facets in your website', 'piotnetgrid' ),
			]
		);

		$this->add_control(
			'facet_label',
			[
				'type'        => 'text',
				'label'       => __( 'Facet Label', 'piotnetgrid' ),
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => 'in',
							'value'    => ['filter','sort'],
						]
					],
				],
			]
		);

		$this->add_control(
            'facet_label_hide',
            [
                'label' => __( 'Hide Facet Label', 'piotnetgrid' ),
                'type' => 'switch',
                'default' => '',
                'label_on' => 'Yes',
                'label_off' => 'No',
                'return_value' => 'yes',
            ]
        );

		$this->add_control(
			'facet_action',
			[
				'type'        => 'select',
				'label'       => __( 'Facet Action', 'piotnetgrid' ),
				'options'     => [
					'filter' => __( 'Filter', 'piotnetgrid' ),
					'load'   => __( 'Load', 'piotnetgrid' ),
					'sort'   => __( 'Sort', 'piotnetgrid' ),
				],
			]
		);

		$this->add_control(
			'facet_disable_add_parameter_to_url',
			[
				'type'         => 'switch',
				'label'        => __( 'Disable add the parameter to URL', 'piotnetgrid' ),
				'return_value' => 'true',
				'default'      => '',
			]
		);

		$this->add_control(
			'facet_placeholder',
			[
				'type'        => 'text',
				'label'       => __( 'Facet Placeholder', 'piotnetgrid' ),
				'value'       => __( 'All', 'piotnetgrid' ),
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => '==',
							'value'    => 'filter',
						],
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'select',
								'radio',
								'buttons',
                                'az',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'facet_sort_placeholder',
			[
				'type'        => 'text',
				'label'       => __( 'Facet Placeholder', 'piotnetgrid' ),
				'value'       => __( 'Default Sorting', 'piotnetgrid' ),
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => '==',
							'value'    => 'sort',
						],
					],
				],
			]
		);

		$this->add_control(
			'filter_type',
			[
				'type'        => 'select',
				'label'       => __( 'Filter Type', 'piotnetgrid' ),
				'options'     => [
					'select' => __( 'Select', 'piotnetgrid' ),
					'slider' => __( 'Slider', 'piotnetgrid' ),
					'radio'   => __( 'Radio', 'piotnetgrid' ),
					'checkboxes'   => __( 'Checkboxes', 'piotnetgrid' ),
					'buttons'   => __( 'Buttons', 'piotnetgrid' ),
					'color-picker'   => __( 'Color Picker', 'piotnetgrid' ),
					'image-select'   => __( 'Image Select', 'piotnetgrid' ),
					'search' => __( 'Search', 'piotnetgrid' ),
					'date' => __( 'Date', 'piotnetgrid' ),
                    'az' => __( 'A-Z', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'filter',
					],
				],
			]
		);

		$this->add_control(
			'show_choice_count',
			[
				'label'        => __( 'Show Choice Count', 'piotnetgrid' ),
				'type'         => 'switch',
				'return_value' => 'true',
				'default'      => 'true',
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'facet_action',
							'operator' => '==',
							'value'    => 'filter',
						],
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'checkboxes',
								'radio',
								'select',
								'image-select',
								'buttons',
                                'az',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'loading_type',
			[
				'type'        => 'select',
				'label'       => __( 'Loading Type', 'piotnetgrid' ),
				'options'     => [
					'pagination' => __( 'Pagination', 'piotnetgrid' ),
					'load_more' => __( 'Load More', 'piotnetgrid' ),
					'posts_per_page' => __( 'Posts Per Page', 'piotnetgrid' ),
					'result_counts' => __( 'Result Counts', 'piotnetgrid' ),
				],
				'value' => 'pagination',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'load',
					],
				],
			]
		);

		$this->add_control(
			'slider_step',
			[
				'type'        => 'number',
				'label'       => __( 'Slider Step Number', 'piotnetgrid' ),
				'value'		  => 10,
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'slider',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'slider_prefix',
			[
				'type'        => 'text',
				'label'       => __( 'Slider Prefix', 'piotnetgrid' ),
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'slider',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'slider_postfix',
			[
				'type'        => 'text',
				'label'       => __( 'Slider Postfix', 'piotnetgrid' ),
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'slider',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'slider_thousands_separator',
			[
				'type'        => 'text',
				'label'       => __( 'Slider Thousands Separator', 'piotnetgrid' ),
				'value'		  => ',',
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'slider',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'slider_decimals_places',
			[
				'type'        => 'number',
				'label'       => __( 'Slider Decimals Places', 'piotnetgrid' ),
				'value'		  => 0,
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'slider',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'slider_decimals_separator',
			[
				'type'        => 'text',
				'label'       => __( 'Slider Decimals Separator', 'piotnetgrid' ),
				'value'		  => '.',
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'slider',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'facet_placeholder_search',
			[
				'type'        => 'text',
				'label'       => __( 'Search Placeholder', 'piotnetgrid' ),
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'search',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'inline_list',
			[
				'label'        => __( 'Inline List', 'piotnetgrid' ),
				'type'         => 'switch',
				'return_value' => 'piotnetgrid-subgroup-inline',
				'default'      => '',
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'filter_type',
							'operator' => 'in',
							'value'    => [
								'checkboxes',
								'radio',
							],
						],
					],
				],
			]
		);

		$this->add_control(
			'multiple_selection',
			[
				'label' => __( 'Multiple Selection', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'true',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'filter',
					],
					[
						'name'     => 'filter_type',
						'operator' => 'in',
						'value'    => ['buttons','color-picker','image-select'],
					],
				],
			]
		);

		$post_types = get_post_types( [], 'objects' );
		$taxonomies = [
			'' => __( 'Select Taxonomy', 'piotnetgrid' ),
		];

		foreach ( $post_types as $post_type ) {
			$taxonomies_of_post_type = get_object_taxonomies( $post_type->name, 'objects' );
			foreach ($taxonomies_of_post_type as $taxonomy) {
				$taxonomies[$taxonomy->name] = $taxonomy->label . ' (' . $post_type->name . ')';
			}
		}

		$this->add_control(
			'filter_data_source',
			[
				'type'        => 'select',
				'label'       => 'Filter Data Source',
				'options'     => [
					'taxonomy' => __( 'Taxonomy', 'piotnetgrid' ),
					'post_date' => __( 'Post Date', 'piotnetgrid' ),
					'custom_field' => __( 'Custom Field', 'piotnetgrid' ),
					'author' => __( 'Author', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'filter',
					],
					[
						'name'     => 'filter_type',
						'operator' => 'in',
						'value'    => ['select','slider','radio','checkboxes','buttons','color-picker','image-select','date'],
					],
				],
			]
		);

		$this->add_control(
			'filter_data_source_custom_field',
			[
				'type'        => 'select2',
				'label'       => __( 'Custom Field Name (Key)', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'custom_field',
				'multiple' => false,
				'description' => __( 'Examples:<br>_price (Woocommerce product price)<br>wpcf-custom_field_name (Toolset)', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'filter',
					],
					[
						'name'     => 'filter_data_source',
						'operator' => '==',
						'value'    => 'custom_field',
					],
					[
						'name'     => 'filter_type',
						'operator' => 'in',
						'value'    => ['select','slider','radio','checkboxes','buttons','color-picker','image-select','date'],
					],
				],
			]
		);

		$this->add_control(
			'filter_data_source_taxonomy',
			[
				'type'        => 'select',
				'label'       => __( 'Taxonomy', 'piotnetgrid' ),
				'label_block' => true,
				'options'     => $taxonomies,
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'filter',
					],
					[
						'name'     => 'filter_data_source',
						'operator' => '==',
						'value'    => 'taxonomy',
					],
					[
						'name'     => 'filter_type',
						'operator' => 'in',
						'value'    => ['select','slider','radio','checkboxes','buttons','color-picker','image-select','date'],
					],
				],
			]
		);

		$this->add_control(
			'filter_data_source_taxonomy_exclude_terms',
			[
				'type'        => 'select2',
				'label'       => __( 'Exclude Taxonomy Terms', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'terms',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'filter',
					],
					[
						'name'     => 'filter_data_source',
						'operator' => '==',
						'value'    => 'taxonomy',
					],
					[
						'name'     => 'filter_type',
						'operator' => 'in',
						'value'    => ['select','slider','radio','checkboxes','buttons','color-picker','image-select','date'],
					],
				],
			]
		);

		$this->add_control(
			'filter_logic',
			[
				'label'   => __( 'Filter Logic', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'IN'  => __( 'OR', 'piotnetgrid' ),
					'AND' => __( 'AND', 'piotnetgrid' ),
				],
				'value'   => 'IN',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'filter',
					],
					[
						'name'     => 'filter_data_source',
						'operator' => '==',
						'value'    => 'taxonomy',
					],
					[
						'name'     => 'filter_type',
						'operator' => 'in',
						'value'    => ['checkboxes','buttons','color-picker','image-select'],
					],
				],
			]
		);

		// Color/Image Options repeater

		$this->new_group_controls();

		$this->add_control(
			'color_image_option_value',
			[
				'label'   => __( 'Option Value', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => '',
			]
		);

		$this->add_control(
			'color_image_option_label',
			[
				'label'   => __( 'Option Label', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => '',
				'placeholder' => __( 'Default', 'piotnetgrid' ),
			]
		);

		$this->add_control(
			'color_image_option_color',
			[
				'label'   => __( 'Color', 'piotnetgrid' ),
				'type'    => 'color',
				'value'   => '',
				'render_type' => 'both',
			]
		);

		$this->add_control(
			'color_image_option_image',
			[
				'label'   => __( 'Image', 'piotnetgrid' ),
				'type'    => 'media',
			]
		);

        $this->add_control(
            'repeater_id',
            [
                'type' => 'hidden',
            ],
            [
                'overwrite' => 'true',
            ]
        );
        
		$repeater_items = $this->get_group_controls();

		$this->new_group_controls();

		$this->add_control(
			'',
			[
				'type'           => 'repeater-item',
				'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
				'controls'       => $repeater_items,
				'controls_query' => '.piotnetgrid-control-repeater-field',
			]
		);

		$repeater_list = $this->get_group_controls();

		$this->add_control(
			'color_image_options',
			[
				'type'           => 'repeater',
				'label'          => __( 'Options', 'piotnetgrid' ),
				'value'          => '',
				'label_block'    => true,
				'add_label'      => __( 'Add Item', 'piotnetgrid' ),
				'controls'       => $repeater_list,
				'controls_query' => '.piotnetgrid-control-repeater-list',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'filter',
					],
					[
						'name'     => 'filter_type',
						'operator' => 'in',
						'value'    => ['color-picker','image-select','select','radio','checkboxes','buttons'],
					],
				],
			]
		);

		// end Color/Image Options repeater

		$this->add_control(
			'date_type',
			[
				'label'   => __( 'Date Type', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'range'  => __( 'Range of Dates', 'piotnetgrid' ),
					'single' => __( 'Single Date', 'piotnetgrid' ),
				],
				'value'   => 'range',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'filter',
					],
					[
						'name'     => 'filter_type',
						'operator' => 'in',
						'value'    => ['date'],
					],
				],
			]
		);

		$this->add_control(
			'date_format',
			[
				'label'   => __( 'Date Format', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => 'Y-m-d',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'filter',
					],
					[
						'name'     => 'filter_type',
						'operator' => 'in',
						'value'    => ['date'],
					],
				],
			]
		);
		$this->add_control(
			'pagination_show_all',
			[
				'label' => __( 'Show All', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'false',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'pagination',
					],
				],
			]
		);
		$this->add_control(
			'pagination_next_prev_button',
			[
				'label' => __( 'Prev and Next Button', 'piotnetgrid' ),
				'type' => 'switch',
				'value' => 'true',
				'label_on' => 'Show',
				'label_off' => 'Hiden',
				'return_value' => 'true',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'pagination',
					],
				],
			]
		);
		$this->add_control(
			'pagination_next_prev_button_only',
			[
				'label' => __( 'Only Prev and Next Button', 'piotnetgrid' ),
				'type' => 'switch',
				'value' => '',
				'label_on' => 'Show',
				'label_off' => 'Hiden',
				'return_value' => 'true',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'pagination',
					],
				],
			]
		);
		$this->add_control(
			'pagination_next_text',
			[
				'label'   => __( 'Next Text', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => '<span class="icon-angle-right"></span>',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'pagination',
					],
					[
						'name'     => 'pagination_next_prev_button',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_control(
			'pagination_prev_text',
			[
				'label'   => __( 'Previous Text', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => '<span class="icon-angle-left"></span>',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'pagination',
					],
					[
						'name'     => 'pagination_next_prev_button',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_control(
			'pagination_end_size',
			[
				'label'   => __( 'End Size', 'piotnetgrid' ),
				'type'    => 'number',
				'value'   => '2',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'pagination',
					],
				],
			]
		);
		$this->add_control(
			'pagination_mid_size',
			[
				'label'   => __( 'Mid Size', 'piotnetgrid' ),
				'type'    => 'number',
				'value'   => '1',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'pagination',
					],
				],
			]
		);
		// $this->add_control(
		// 	'pagination_add_fragment',
		// 	[
		// 		'label'   => __( 'Add Fragment', 'piotnetgrid' ),
		// 		'type'    => 'text',
		// 		'value'   => '',
		// 		'conditions'  => [
		// 			[
		// 				'name'     => 'filter_type',
		// 				'operator' => 'in',
		// 				'value'    => ['pagination'],
		// 			],
		// 		],
		// 	]
		// );
		$this->add_control(
			'pagination_before_page_number',
			[
				'label'   => __( 'Before Page Number', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => '',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'pagination',
					],
				],
			]
		);
		$this->add_control(
			'pagination_after_page_number',
			[
				'label'   => __( 'After Page Number', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => '',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'pagination',
					],
				],
			]
		);

		$this->add_control(
			'load_more_infinite_scroll',
			[
				'type'         => 'switch',
				'label'        => __( 'Infinite Scroll', 'piotnetgrid' ),
				'return_value' => 'true',
				'default'      => '',
                'conditions'  => [
                    [
                        'name'     => 'facet_action',
                        'operator' => '=',
                        'value'    => 'load',
                    ],
                    [
                        'name'     => 'loading_type',
                        'operator' => '=',
                        'value'    => 'load_more',
                    ],
                ],
			]
		);

		$this->add_control(
			'load_more_text',
			[
				'label'   => __( 'Load More Text', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => __( 'Load More', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'load_more',
					],
				],
			]
		);

		$this->add_control(
			'loading_text',
			[
				'label'   => __( 'Loading Text', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => __( 'Loading...', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'load_more',
					],
				],
			]
		);

		$this->add_control(
			'posts_per_page_options',
			[
				'label'   => __( 'Posts per page options', 'piotnetgrid' ),
				'label_block' => true,
				'type'    => 'text',
				'value'   => __( '10, 20, 30', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'posts_per_page',
					],
				],
			]
		);

		$this->add_control(
			'count_text_plural',
			[
				'type'    => 'text',
				'label'   => __( 'Count text (plural)', 'piotnetgrid' ),
				'value'   => __( '[from] - [to] of [total] results', 'piotnetgrid' ),
				'label_block' => true,
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'result_counts',
					],
				],
			]
		);

		$this->add_control(
			'count_text_singular',
			[
				'type'    => 'text',
				'label'   => __( 'Count text (singular)', 'piotnetgrid' ),
				'value'   => __( '1 result found', 'piotnetgrid' ),
				'label_block' => true,
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '=',
						'value'    => 'load',
					],
					[
						'name'     => 'loading_type',
						'operator' => '=',
						'value'    => 'result_counts',
					],
				],
			]
		);

        $this->add_control(
            'az_alphabetical',
            [
                'label'   => __( 'Alphabetical', 'piotnetgrid' ),
                'type'    => 'textarea',
                'value'   => __( '#,A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z', 'piotnetgrid' ),
                'conditions'  => [
                    [
                        'name'     => 'facet_action',
                        'operator' => '=',
                        'value'    => 'filter',
                    ],
                    [
                        'name'     => 'filter_type',
                        'operator' => '=',
                        'value'    => 'az',
                    ],
                ],
            ]
        );

		// Order repeater

		$this->new_group_controls();

		$this->add_control(
			'order_label',
			[
				'label'   => __( 'Label (Required)', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => '',
			]
		);

		$this->add_control(
			'order_by',
			[
				'label'   => __( 'Order By', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'' => __( 'None', 'piotnetgrid' ),
					'ID'  => __( 'Post ID', 'piotnetgrid' ),
					'author'  => __( 'Author', 'piotnetgrid' ),
					'title'  => __( 'Title', 'piotnetgrid' ),
					'name'  => __( 'Post Name (Post Slug)', 'piotnetgrid' ),
					'type'  => __( 'Post Type', 'piotnetgrid' ),
					'date'  => __( 'Date', 'piotnetgrid' ),
					'modified'  => __( 'Last Modified Date', 'piotnetgrid' ),
					'parent'  => __( 'Post/Page Parent ID', 'piotnetgrid' ),
					'rand'  => __( 'Random', 'piotnetgrid' ),
					'comment_count'  => __( 'Number of Comments', 'piotnetgrid' ),
					'relevance'  => __( 'Relevance', 'piotnetgrid' ),
					'menu_order'  => __( 'Menu Order', 'piotnetgrid' ),
					'meta_value'  => __( 'Custom Field', 'piotnetgrid' ),
					'meta_num'  => __( 'Numeric Custom Field', 'piotnetgrid' ),
					'post__in'  => __( 'List of Post IDs', 'piotnetgrid' ),
					'post_name__in'  => __( 'List of Post Names', 'piotnetgrid' ),
					'post_parent__in'  => __( 'List of Post Parent IDs', 'piotnetgrid' ),
				],
				'value'   => '',
			]
		);

		$this->add_control(
			'custom_field_name',
			[
				'label'   => __( 'Custom Field Name', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => '',
				'description' => __( 'Custom Field Date format for the Date Facet is Y-m-d or Y-m-d h:i:s', 'piotnetgrid' ),
				'conditions' => [
					'terms' => [
						[
							'name'     => 'order_by',
							'operator' => 'in',
							'value'    => ['meta_value', 'meta_num'],
						],
					],
				],
			]
		);

		$this->add_control(
			'order',
			[
				'label'   => __( 'Order', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'DESC' => __( 'Descending', 'piotnetgrid' ),
					'ASC'  => __( 'Ascending', 'piotnetgrid' ),
				],
				'value'   => 'DESC',
			]
		);

        $this->add_control(
            'repeater_id',
            [
                'type' => 'hidden',
            ],
            [
                'overwrite' => 'true',
            ]
        );
        
		$repeater_items = $this->get_group_controls();

		$this->new_group_controls();

		$this->add_control(
			'',
			[
				'type'           => 'repeater-item',
				'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
				'controls'       => $repeater_items,
				'controls_query' => '.piotnetgrid-control-repeater-field',
			]
		);

		$repeater_list = $this->get_group_controls();

		$this->add_control(
			'order_list',
			[
				'type'           => 'repeater',
				'label'          => __( 'Order List', 'piotnetgrid' ),
				'value'          => '',
				'label_block'    => true,
				'add_label'      => __( 'Add Item', 'piotnetgrid' ),
				'controls'       => $repeater_list,
				'controls_query' => '.piotnetgrid-control-repeater-list',
				'conditions'  => [
					[
						'name'     => 'facet_action',
						'operator' => '==',
						'value'    => 'sort',
					],
				],
			]
		);

		// end Order repeater
	}

	private function add_label_style_controls() {
		$this->add_control(
			'label_spacing',
			[
				'label'     => __( 'Spacing', 'piotnetgrid' ),
				'type'      => 'slider',
				'default'   => [
					'size' => '',
					'unit' => 'px',
				],
				'range'     => [
					'px' => [
						'min' => 0,
						'max' => 60,
					],
				],
				'render_type' => 'none',
				'selectors' => [
					'body {{WRAPPER}} .piotnetgrid-facet__label' => 'padding-bottom: {{SIZE}}{{UNIT}};',
					// for the label position = above option
				],
			]
		);

		$this->add_responsive_control(
			'label_text_align',
			[
				'type'        => 'select',
				'label'       => __( 'Text Align', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => '',
				'options'     => [
					''       => __( 'Default', 'piotnetgrid' ),
					'left'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'right'  => __( 'Right', 'piotnetgrid' ),
				],
				'render_type' => 'none',
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-facet__label' => 'text-align: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'label_color',
			[
				'label'     => __( 'Text Color', 'piotnetgrid' ),
				'type'      => 'color',
				'render_type' => 'none',
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-facet__label' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_text_typography_controls(
			'label_typography',
			[
				'render_type' => 'none',
				'selectors' => '{{WRAPPER}} .piotnetgrid-facet__label',
			]
		);
	}

    private function show_more_and_show_less_style_controls($name, $args = []){
        $wrapper = isset( $args['wrapper'] ) ? $args['wrapper'] : '{{WRAPPER}}';
        $name = !empty( $name ) ? '_' . $name : '';
        $previous_controls = $this->new_group_controls();

        $this->add_text_typography_controls(
            'facet_show_more_or_less_button_typography'.$name,
            [
                'render_type' => 'none',
                'selectors' => $wrapper,
            ]
        );

        $this->add_control(
            'facet_show_more_or_less_button_background_color'.$name,
            [
                'label'     => __( 'Background Color', 'piotnetgrid' ),
                'type'      => 'color',
                'default'   => '',
                'selectors' => [
                    $wrapper =>'background: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'facet_show_more_or_less_button_text_color'.$name,
            [
                'label'     => __( 'Text Color', 'piotnetgrid' ),
                'type'      => 'color',
                'default'   => '',
                'selectors' => [
                     $wrapper =>'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'facet_show_more_or_less_button_padding'.$name,
            [
                'type'        => 'dimensions',
                'label'       => __( 'Padding', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'  => [
                    $wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'facet_show_more_or_less_button_border_radius'.$name,
            [
                'type'        => 'dimensions',
                'label'       => __( 'Border Radius', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px', '%', 'em' ],
                'selectors'   => [
                    $wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'facet_show_more_or_less_button_border_style'.$name,
            [
                'type'      => 'select',
                'label'     => __( 'Border Type', 'piotnetgrid' ),
                'value'     => '',
                'options'   => [
                    ''       => 'None',
                    'solid'  => 'Solid',
                    'double' => 'Double',
                    'dotted' => 'Dotted',
                    'dashed' => 'Dashed',
                    'groove' => 'Groove',
                ],
                'selectors' => [
                    $wrapper => 'border-style:{{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'facet_show_more_or_less_button_border_width'.$name,
            [
                'label'      => __( 'Border Width', 'piotnetgrid' ),
                'type'       => 'dimensions',
                'size_units' => [ 'px', 'em', '%' ],
                'selectors'  => [
                    $wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'  => [
                    [
                        'name'     => 'facet_show_more_or_less_button_border_style'.$name,
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );

        $this->add_control(
            'facet_show_more_or_less_button_border_color'.$name,
            [
                'label'     => __( 'Border Color', 'piotnetgrid' ),
                'type'      => 'color',
                'selectors' => [
                    $wrapper =>'border-color: {{VALUE}};',
                ],
                'conditions'  => [
                    [
                        'name'     => 'facet_show_more_or_less_button_border_style'.$name,
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );

        return $this->get_group_controls( $previous_controls );

    }


    private function add_show_more_and_show_less_style_controls(){
        $this->add_control(
            '',
            [
                'type' => 'heading-tab',
                'tabs' => [
                    [
                        'name'   => 'show_more_and_show_less_normal_tab',
                        'title'  => __( 'NORMAL', 'piotnetgrid' ),
                        'active' => true,
                    ],
                    [
                        'name'  => 'show_more_and_show_less_hover_tab',
                        'title' => __( 'HOVER', 'piotnetgrid' ),
                    ],
                ],
            ]
        );

        $normal_controls = $this->show_more_and_show_less_style_controls(
            '',
            [
                'wrapper' => '{{WRAPPER}} .piotnetgrid-show-more-facet,{{WRAPPER}} .piotnetgrid-show-less-facet,{{WRAPPER}} .piotnetgrid-show-more-facet__preview,{{WRAPPER}} .piotnetgrid-show-less-facet__preview',
			]
        );

        $this->add_control(
            'show_more_and_show_less_normal_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Normal', 'piotnetgrid' ),
                'value'          => '',
                'active'         => true,
                'controls'       => $normal_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );

        $hover_controls = $this->show_more_and_show_less_style_controls(
            'hover',
            [
                'wrapper' => '{{WRAPPER}} .piotnetgrid-show-less-facet:hover,{{WRAPPER}} .piotnetgrid-show-more-facet:hover,{{WRAPPER}} .piotnetgrid-show-more-facet__preview:hover,{{WRAPPER}} .piotnetgrid-show-less-facet__preview:hover',
            ]
        );
        $this->add_control(
            'show_more_and_show_less_hover_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Hover', 'piotnetgrid' ),
                'value'          => '',
                'controls'       => $hover_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );
    }

	private function add_checkbox_style_controls() {

		$this->add_control(
			'checkbox_type',
			[
				'label'   => __( 'Checkbox/Radio Type', 'piotnetgrid' ),
				'label_block' => true,
				'type'    => 'select',
				'options' => [
					'native' => __( 'Native / Custom Checkbox, Radio Button', 'piotnetgrid' ),
					'square' => __( 'Square', 'piotnetgrid' ),
				],
				'default' => 'native',
			]
		);

		$checkbox_horizontal_spacing = is_rtl() ? 'margin-left: {{SIZE}}{{UNIT}};' : 'margin-right: {{SIZE}}{{UNIT}};';

		$this->add_control(
			'facet_checkbox_horizontal_spacing',
			[
				'label'      => __( 'Checkbox/Radio Button Horizontal Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'separator'  => 'before',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} input' => $checkbox_horizontal_spacing,
					'{{WRAPPER}} .piotnetgrid-facet__field-sub-icon' => $checkbox_horizontal_spacing,
				],
				'condition'  => [
					'checkbox_type' => ['native'],
				],
			]
		);

		$this->add_control(
			'facet_checkbox_item_width_type',
			[
				'label'   => __( 'Item Width Type', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'auto' => __( 'Auto', 'piotnetgrid' ),
					'custom' => __( 'Custom', 'piotnetgrid' ),
				],
			]
		);

		$this->add_responsive_control(
			'facet_checkbox_item_width',
			[
				'label'   => __( 'Item Width', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px', '%' ],
				'range'      => [
					'px' => [
						'min'  => 100,
						'max'  => 200,
						'step' => 1,
					],
					'%' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub' => 'width: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [
					'facet_checkbox_item_width_type' => 'custom',
				],
			]
		);

		$this->add_control(
			'facet_checkbox_item_vertical_spacing',
			[
				'label'      => __( 'Item Vertical Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub' => 'margin-bottom: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-facet__field-sub:last-child' => 'margin-bottom: 0;',
				],
			]
		);

		$this->add_control(
			'facet_checkbox_item_horizontal_spacing',
			[
				'label'      => __( 'Item Horizontal Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub' => 'padding: 0 {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup' => 'margin: 0 -{{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'facet_checkbox_replacement',
			[
				'label'   => __( 'Checkbox / Radio Button Replacement', 'piotnetgrid' ),
				'type'    => 'switch',
				'return_value' => 'yes',
				'default' => '',
				'condition'  => [
					'checkbox_type' => ['native'],
				],
			]
		);

		$this->add_control(
			'facet_checkbox_replacement_icon_type',
			[
				'label' => __( 'Replacement Type', 'piotnetgrid' ),
				'type'  => 'select',
				'options'   => [
					'font_awesome' => __( 'Font Awesome', 'piotnetgrid' ),
					'image'        => __( 'Image', 'piotnetgrid' ),
				],
				'default'   => 'font_awesome',
				'condition'  => [
					'facet_checkbox_replacement' => 'yes',
				],
			]
		);
		$this->add_control(
			'facet_checkbox_replacement_icon_font_awesome',
			[
				'label' => __( 'Icon Uncheck', 'piotnetgrid' ),
				'type'  => 'icon',
				'options_source' => 'fontawesome',
				'condition'      => [
					'facet_checkbox_replacement' => 'yes',
					'facet_checkbox_replacement_icon_type' => 'font_awesome',
				],
			]
		);
		$this->add_control(
			'facet_checkbox_replacement_icon_font_awesome_checked',
			[
				'label' => __( 'Icon Checked', 'piotnetgrid' ),
				'type'  => 'icon',
				'options_source' => 'fontawesome',
				'condition'      => [
					'facet_checkbox_replacement' => 'yes',
					'facet_checkbox_replacement_icon_type' => 'font_awesome',
				],
			]
		);
		$this->add_control(
			'facet_checkbox_replacement_icon_image',
			[
				'label' => __( 'Icon Image Uncheck', 'piotnetgrid' ),
				'type'  => 'media',
				'condition'      => [
					'facet_checkbox_replacement' => 'yes',
					'facet_checkbox_replacement_icon_type' => 'image',
				],
			]
		);
		$this->add_control(
			'facet_checkbox_replacement_icon_image_checked',
			[
				'label' => __( 'Icon Image Checked', 'piotnetgrid' ),
				'type'  => 'media',
				'condition'      => [
					'facet_checkbox_replacement' => 'yes',
					'facet_checkbox_replacement_icon_type' => 'image',
				],
			]
		);

		$this->add_responsive_control(
			'facet_checkbox_icon_width',
			[
				'label'   => __( 'Icon Width', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px', '%' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub-icon' => 'width: {{SIZE}}{{UNIT}}; font-size: {{SIZE}}{{UNIT}}',
				],
				'condition'  => [
					'facet_checkbox_replacement' => 'yes',
					'checkbox_type' => ['native'],
				],
			]
		);

		$this->add_control(
			'facet_checkbox_square_size',
			[
				'label'      => __( 'Size', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => 18,
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub' => 'position: relative;',
					'{{WRAPPER}} .piotnetgrid-facet__field-sub input' => 'position: absolute; top: 50%; left: 0px; transform: translateY(-50%); opacity: 0; z-index: 9;',
					'{{WRAPPER}} .piotnetgrid-facet__field-sub label' => 'display: block !important; cursor: pointer; margin: 0 auto; padding: 0px 0px 0px 30px;',
					'{{WRAPPER}} .piotnetgrid-facet__field-sub label:before' => 'content: ""; display: block; width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}}; position: absolute; top: 50%; left: 0px; transform: translateY(-50%); background: #fff; border-style: solid; border-width: 1px;',
				],
				'condition'  => [
					'checkbox_type' => 'square',
				],
			]
		);

		$this->add_control(
			'facet_checkbox_square_border_width',
			[
				'label'      => __( 'Border Width', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 10,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => 1,
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub label:before' => 'border-width: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [
					'checkbox_type' => 'square',
				],
			]
		);

		$this->add_control(
			'facet_checkbox_square_border_color',
			[
				'label'     => __( 'Border Color', 'piotnetgrid' ),
				'type'      => 'color',
				'default'   => '#006afe',
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub label:before' => 'border-color: {{VALUE}};',
				],
				'condition' => [
					'checkbox_type' => 'square',
				],
			]
		);

		$this->add_control(
			'facet_checkbox_square_background_color',
			[
				'label'     => __( 'Checked Background Color', 'piotnetgrid' ),
				'type'      => 'color',
				'default'   => '#006afe',
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub input:checked ~ label:before' => 'background: {{VALUE}};',
				],
				'condition' => [
					'checkbox_type' => 'square',
				],
			]
		);

		$this->add_control(
			'facet_checkbox_square_spacing',
			[
				'label'      => __( 'Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => 30,
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub label' => 'padding-left: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [
					'checkbox_type' => 'square',
				],
			]
		);
	}

	public function add_checkbox_advanced_style_controls() {
		$this->add_responsive_control(
			'checkbox_advanced_align_items',
			[
				'label'      => __( 'Align Items', 'piotnetgrid' ),
				'type'       => 'select',
				'label_block' => true,
				'options' => [
					'flex-start' => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end' => __( 'Right', 'piotnetgrid' ),
				],
				'selectors'  => [
					'{{WRAPPER}}.piotnetgrid-facet--checkboxes .piotnetgrid-facet__field-wrapper, {{WRAPPER}}.piotnetgrid-facet--buttons .piotnetgrid-facet__field-wrapper, {{WRAPPER}}.piotnetgrid-facet--image-select .piotnetgrid-facet__field-wrapper' => 'justify-content: {{VALUE}};',
				],
				'condition' => [
					'filter_type' => 'buttons',
				],
			]
		);

		$this->add_responsive_control(
			'checkbox_advanced_item_padding',
			[
				'label'      => __( 'Item Padding', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', 'em', '%' ],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub-label' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'checkbox_advanced_item_spacing',
			[
				'label'      => __( 'Item Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup--image-select' => 'grid-column-gap: {{SIZE}}{{UNIT}}; grid-row-gap: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'filter_type' => 'image-select',
				],
			]
		);

		$this->add_control(
			'checkbox_advanced_image_margin_bottom',
			[
				'label'      => __( 'Image Margin Bottom', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet-field__color-image img' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'filter_type' => 'image-select',
				],
			]
		);

		$this->add_responsive_control(
			'checkbox_advanced_image_columns',
			[
				'type'        => 'number',
				'label'       => __( 'Image Columns', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup--image-select' => 'grid-template-columns: repeat({{VALUE}}, 1fr);',
				],
				'condition' => [
					'filter_type' => 'image-select',
				],
			]
		);

		$this->add_control(
			'checkbox_advanced_item_spacing_button',
			[
				'label'      => __( 'Item Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub' => 'padding: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup' => 'margin: -{{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'filter_type' => 'buttons',
				],
			]
		);

		$this->add_responsive_control(
			'checkbox_advanced_item_border_radius',
			[
				'label'      => __( 'Item Border Radius', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub-label' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'style_check_heading_tab',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'style_uncheck_tab',
						'title'  => __( 'UNCHECK', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'style_checked_tab',
						'title' => __( 'CHECKED', 'piotnetgrid' ),
					],
				],
			]
		);

		$style_uncheck_controls = $this->checkbox_advanced_style_controls(
			'style_uncheck',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-facet__field-subgroup label',
			]
		);
		$this->add_control(
			'style_uncheck_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Uncheck', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $style_uncheck_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$style_checked_controls = $this->checkbox_advanced_style_controls(
			'style_checked',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-facet__field-subgroup input:checked ~ label',
			]
		);
		$this->add_control(
			'style_checked_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Checked', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $style_checked_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_color_picker_style_controls() {
		$this->add_responsive_control(
			'color_picker_align_items',
			[
				'label'      => __( 'Align Items', 'piotnetgrid' ),
				'type'       => 'select',
				'label_block' => true,
				'options' => [
					'flex-start' => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end' => __( 'Right', 'piotnetgrid' ),
				],
				'selectors'  => [
					'{{WRAPPER}}.piotnetgrid-facet--color-picker .piotnetgrid-facet__field-wrapper' => 'justify-content: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'color_picker_width',
			[
				'label'      => __( 'Item Width', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup--color-picker .piotnetgrid-facet-field__color-image' => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'color_picker_item_spacing',
			[
				'label'      => __( 'Item Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-sub' => 'padding: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup' => 'margin: -{{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'color_picker_border_radius',
			[
				'label'      => __( 'Item Normal Border Radius', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup--color-picker .piotnetgrid-facet-field__color-image' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'color_picker_tooltip_color',
			[
				'label'      => __( 'Tooltip Color', 'piotnetgrid' ),
				'type'       => 'color',
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup--color-picker .piotnetgrid-facet__field-sub-label-content' => 'background: {{VALUE}};',
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup--color-picker .piotnetgrid-facet__field-sub-label-content:before' => 'border-top-color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'color_picker_active_border_width',
			[
				'label'      => __( 'Item Active Border Width', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup--color-picker .piotnetgrid-facet-field:checked+.piotnetgrid-facet__field-sub-label .piotnetgrid-facet-field__color-image:after' => 'border-width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'color_picker_active_border_color',
			[
				'label'      => __( 'Item Active Border Color', 'piotnetgrid' ),
				'type'       => 'color',
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup--color-picker .piotnetgrid-facet-field:checked+.piotnetgrid-facet__field-sub-label .piotnetgrid-facet-field__color-image:after' => 'border-color: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'color_picker_active_border_radius',
			[
				'label'      => __( 'Item Active Border Radius', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-facet__field-subgroup--color-picker .piotnetgrid-facet-field:checked+.piotnetgrid-facet__field-sub-label .piotnetgrid-facet-field__color-image:after' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
	}

	public function checkbox_advanced_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();
		$this->add_control(
			$name.'_color',
			[
				'type'        => 'color',
				'label'       => __( 'Text Color', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_text_typography_controls(
			$name.'_text_typography',
			[
				'selectors' => $wrapper,
			]
		);
		$this->add_control(
			$name.'_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Item Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'background-color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_border_type',
			[
				'type'      => 'select',
				'label'     => __( 'Item Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Item Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Item Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		return $this->get_group_controls( $previous_controls );
	}

	private function field_style_controls(string $name, $args = []){
		$wrapper = isset( $args['wrapper'] ) ? $args['wrapper'] : '{{WRAPPER}}';
		$name = !empty( $name ) ? '_' . $name : '';
		$previous_controls = $this->new_group_controls();
		$this->add_responsive_control(
			'field_text_align' . $name,
			[
				'type'        => 'select',
				'label'       => __( 'Text Align', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => '',
				'options'     => [
					''       => __( 'Default', 'piotnetgrid' ),
					'left'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'right'  => __( 'Right', 'piotnetgrid' ),
				],
				'render_type' => 'none',
				'selectors'   => [
					$wrapper . ' .piotnetgrid-facet-field' => 'text-align: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'field_text_color' . $name,
			[
				'label'     => __( 'Text Color', 'piotnetgrid' ),
				'type'      => 'color',
				'render_type' => 'none',
				'selectors' => [
					$wrapper . ' .piotnetgrid-facet-field' => 'color: {{VALUE}};',
					$wrapper . ' .piotnetgrid-facet__field-sub label' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_text_typography_controls(
			'field_typography' . $name,
			[
				'render_type' => 'none',
				'selectors' => $wrapper . ' .piotnetgrid-facet-field, ' . $wrapper . ' .piotnetgrid-facet__field-sub label',
			]
		);

		$this->add_control(
			'field_background_color' . $name,
			[
				'label'     => __( 'Background Color', 'piotnetgrid' ),
				'type'      => 'color',
				'default'   => '#ffffff',
				'render_type' => 'none',
				'selectors' => [
					$wrapper . ' .piotnetgrid-facet-field ' => 'background-color: {{VALUE}};',
				],
				'separator' => 'before',
			]
		);

		$this->add_responsive_control(
			'input_max_width' . $name,
			[
				'label'      => __( 'Input Max Width', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px', 'em', '%' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 1500,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'default'    => [
					'unit' => 'px',
				],
				'render_type' => 'none',
				'selectors'  => [
					$wrapper . ' .piotnetgrid-facet-field ' => 'max-width: {{SIZE}}{{UNIT}}!important;',
				],
			]
		);

		$this->add_responsive_control(
			'input_padding' . $name,
			[
				'label'      => __( 'Input Padding', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', 'em', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					$wrapper . ' .piotnetgrid-facet-field ' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition'  => [
					'filter_type!' => 'checkbox',
				],
			]
		);

		$this->add_control(
			'input_placeholder_color' . $name,
			[
				'label'     => __( 'Input Placeholder Color', 'piotnetgrid' ),
				'type'      => 'color',
				'default'   => '',
				'render_type' => 'none',
				'selectors' => [
					$wrapper . ' .piotnetgrid-facet-field::placeholder' => 'color: {{VALUE}}; opacity: 1;',
					$wrapper . ' .piotnetgrid-facet-field::-webkit-input-placeholder' => 'color: {{VALUE}}; opacity: 1;',
					$wrapper . ' .piotnetgrid-facet-field::-moz-placeholder' => 'color: {{VALUE}}; opacity: 1;',
					$wrapper . ' .piotnetgrid-facet-field:-ms-input-placeholder' => 'color: {{VALUE}}; opacity: 1;',
					$wrapper . ' .piotnetgrid-facet-field:-moz-placeholder' => 'color: {{VALUE}}; opacity: 1;',
				],
			]
		);

		$this->add_control(
			'field_border_type' . $name,
			[
				'label'     => _x( 'Border Type', 'Border Control', 'elementor' ),
				'type'      => 'select',
				'options'   => [
					''       => __( 'None', 'elementor' ),
					'solid'  => _x( 'Solid', 'Border Control', 'elementor' ),
					'double' => _x( 'Double', 'Border Control', 'elementor' ),
					'dotted' => _x( 'Dotted', 'Border Control', 'elementor' ),
					'dashed' => _x( 'Dashed', 'Border Control', 'elementor' ),
					'groove' => _x( 'Groove', 'Border Control', 'elementor' ),
				],
				'render_type' => 'none',
				'selectors' => [
					$wrapper . ' .piotnetgrid-facet-field ' => 'border-style: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'field_border_width' . $name,
			[
				'label'     => _x( 'Width', 'Border Control', 'elementor' ),
				'type'      => 'dimensions',
				'render_type' => 'none',
				'selectors' => [
					$wrapper . ' .piotnetgrid-facet-field ' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'field_border_type!' => '',
				],
			]
		);

		$this->add_control(
			'field_border_color' . $name,
			[
				'label'     => _x( 'Color', 'Border Control', 'elementor' ),
				'type'      => 'color',
				'default'   => '',
				'render_type' => 'none',
				'selectors' => [
					$wrapper . ' .piotnetgrid-facet-field ' => 'border-color: {{VALUE}};',
				],
				'condition' => [
					'field_border_type!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'field_border_radius' . $name,
			[
				'label'      => __( 'Border Radius', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					$wrapper . ' .piotnetgrid-facet-field' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'field_box_shadow' . $name,
			[
				'type'        => 'box-shadow',
				'label'       => __( 'Box Shadow', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => false,
				'render_type' => 'none',
				'render_type' => 'none',
				'selectors'   => [
					$wrapper . ' .piotnetgrid-facet-field' => 'box-shadow: {{VALUE}};',
				],
			]
		);
		return $this->get_group_controls( $previous_controls );
	}

	private function add_slider_style_controls() {

		$this->add_control(
			'slider_color',
			[
				'label'     => __( 'Slider Color', 'piotnetgrid' ),
				'type'      => 'color',
				'render_type' => 'none',
				'selectors' => [
					'{{WRAPPER}} .noUi-connect' => 'background: {{VALUE}};',
					'{{WRAPPER}} .noUi-handle' => 'border-color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'slider_value_color',
			[
				'label'     => __( 'Slider Value Color', 'piotnetgrid' ),
				'type'      => 'color',
				'render_type' => 'none',
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-facet-slider-value' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_text_typography_controls(
			'slider_value_typography',
			[
				'render_type' => 'none',
				'selectors' => '{{WRAPPER}} .piotnetgrid-facet-slider-value',
			]
		);
	}

    private function az_style_controls(string $name, $args = []){
        $wrapper = isset( $args['wrapper'] ) ? $args['wrapper'] : '{{WRAPPER}}';
        $name = !empty( $name ) ? '_' . $name : '';
        $previous_controls = $this->new_group_controls();

        $this->add_control(
            'az_item_text_color' . $name,
            [
                'label'     => __( 'Text Color', 'piotnetgrid' ),
                'type'      => 'color',
                'render_type' => 'none',
                'selectors' => [
                    $wrapper => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_text_typography_controls(
            'az_item_typography' . $name,
            [
                'render_type' => 'none',
                'selectors' => $wrapper,
            ]
        );

        $this->add_control(
            'az_item_background_color' . $name,
            [
                'label'     => __( 'Background Color', 'piotnetgrid' ),
                'type'      => 'color',
                'render_type' => 'none',
                'selectors' => [
                    $wrapper => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'az_item_border_type' . $name,
            [
                'label'     => _x( 'Border Type', 'Border Control', 'elementor' ),
                'type'      => 'select',
                'options'   => [
                    ''       => __( 'None', 'elementor' ),
                    'solid'  => _x( 'Solid', 'Border Control', 'elementor' ),
                    'double' => _x( 'Double', 'Border Control', 'elementor' ),
                    'dotted' => _x( 'Dotted', 'Border Control', 'elementor' ),
                    'dashed' => _x( 'Dashed', 'Border Control', 'elementor' ),
                    'groove' => _x( 'Groove', 'Border Control', 'elementor' ),
                ],
                'render_type' => 'none',
                'selectors' => [
                    $wrapper => 'border-style: {{VALUE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'az_item_border_width' . $name,
            [
                'label'     => _x( 'Width', 'Border Control', 'elementor' ),
                'type'      => 'dimensions',
                'render_type' => 'none',
                'selectors' => [
                    $wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition' => [
                    'az_item_border_type' . $name . '!' => '',
                ],
            ]
        );

        $this->add_control(
            'az_item_border_color' . $name,
            [
                'label'     => _x( 'Color', 'Border Control', 'elementor' ),
                'type'      => 'color',
                'default'   => '',
                'render_type' => 'none',
                'selectors' => [
                    $wrapper => 'border-color: {{VALUE}};',
                ],
                'condition' => [
                    'az_item_border_type' . $name . '!' => '',
                ],
            ]
        );

        $this->add_responsive_control(
            'az_item_border_radius' . $name,
            [
                'label'      => __( 'Border Radius', 'piotnetgrid' ),
                'type'       => 'dimensions',
                'size_units' => [ 'px', '%' ],
                'render_type' => 'none',
                'selectors'  => [
                    $wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'az_item_box_shadow' . $name,
            [
                'type'        => 'box-shadow',
                'label'       => __( 'Box Shadow', 'piotnetgrid' ),
                'value'       => '',
                'label_block' => false,
                'render_type' => 'none',
                'selectors'   => [
                    $wrapper => 'box-shadow: {{VALUE}};',
                ],
            ]
        );

        return $this->get_group_controls( $previous_controls );
    }

	private function pagination_style_controls(string $name, $args = []){
		$wrapper = isset( $args['wrapper'] ) ? $args['wrapper'] : '{{WRAPPER}}';
		$name = !empty( $name ) ? '_' . $name : '';
		$previous_controls = $this->new_group_controls();

		$this->add_control(
			'pagination_item_text_color' . $name,
			[
				'label'     => __( 'Text Color', 'piotnetgrid' ),
				'type'      => 'color',
				'render_type' => 'none',
				'selectors' => [
					$wrapper => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_text_typography_controls(
			'pagination_item_typography' . $name,
			[
				'render_type' => 'none',
				'selectors' => $wrapper,
			]
		);

		$this->add_control(
			'pagination_item_background_color' . $name,
			[
				'label'     => __( 'Background Color', 'piotnetgrid' ),
				'type'      => 'color',
				'render_type' => 'none',
				'selectors' => [
					$wrapper => 'background-color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'pagination_item_border_type' . $name,
			[
				'label'     => _x( 'Border Type', 'Border Control', 'elementor' ),
				'type'      => 'select',
				'options'   => [
					''       => __( 'None', 'elementor' ),
					'solid'  => _x( 'Solid', 'Border Control', 'elementor' ),
					'double' => _x( 'Double', 'Border Control', 'elementor' ),
					'dotted' => _x( 'Dotted', 'Border Control', 'elementor' ),
					'dashed' => _x( 'Dashed', 'Border Control', 'elementor' ),
					'groove' => _x( 'Groove', 'Border Control', 'elementor' ),
				],
				'render_type' => 'none',
				'selectors' => [
					$wrapper => 'border-style: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'pagination_item_border_width' . $name,
			[
				'label'     => _x( 'Width', 'Border Control', 'elementor' ),
				'type'      => 'dimensions',
				'render_type' => 'none',
				'selectors' => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'pagination_item_border_type' . $name . '!' => '',
				],
			]
		);

		$this->add_control(
			'pagination_item_border_color' . $name,
			[
				'label'     => _x( 'Color', 'Border Control', 'elementor' ),
				'type'      => 'color',
				'default'   => '',
				'render_type' => 'none',
				'selectors' => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'condition' => [
					'pagination_item_border_type' . $name . '!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'pagination_item_border_radius' . $name,
			[
				'label'      => __( 'Border Radius', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					$wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'pagination_item_box_shadow' . $name,
			[
				'type'        => 'box-shadow',
				'label'       => __( 'Box Shadow', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => false,
				'render_type' => 'none',
				'selectors'   => [
					$wrapper => 'box-shadow: {{VALUE}};',
				],
			]
		);

		return $this->get_group_controls( $previous_controls );
	}

	private function add_pagination_style_controls() {

		$this->add_responsive_control(
			'pagination_align_items',
			[
				'label'      => __( 'Align Items', 'piotnetgrid' ),
				'label_block' => true,
				'type'       => 'select',
				'options' => [
					'' => __( 'Default', 'piotnetgrid' ),
					'flex-start' => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end' => __( 'Right', 'piotnetgrid' ),
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-pagination' => 'justify-content: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'pagination_item_spacing',
			[
				'label'      => __( 'Item Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-pagination__item' => 'padding: 0 {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-pagination' => 'margin: 0 -{{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'pagination_item_padding',
			[
				'label'      => __( 'Item Padding', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', 'em', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-pagination__item-page-numbers' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'pagination_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'pagination_hover_tab',
						'title' => __( 'HOVER', 'piotnetgrid' ),
					],
					[
						'name'  => 'pagination_current_tab',
						'title' => __( 'CURRENT', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->pagination_style_controls(
			'',
			[
				'wrapper' => '{{WRAPPER}} .piotnetgrid-pagination__item-page-numbers',
			]
		);
		$this->add_control(
			'pagination_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->pagination_style_controls(
			'hover',
			[
				'wrapper' => '{{WRAPPER}} .piotnetgrid-pagination__item-page-numbers:hover',
			]
		);
		$this->add_control(
			'pagination_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$current_controls = $this->pagination_style_controls(
			'current',
			[
				'wrapper' => '{{WRAPPER}} .piotnetgrid-pagination__item-page-numbers.piotnetgrid-pagination__item-page-numbers--current',
			]
		);
		$this->add_control(
			'pagination_current_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $current_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

    private function add_a_z_style_controls() {

        $this->add_responsive_control(
            'az_align_items',
            [
                'label'      => __( 'Align Items', 'piotnetgrid' ),
                'label_block' => true,
                'type'       => 'select',
                'options' => [
                    '' => __( 'Default', 'piotnetgrid' ),
                    'flex-start' => __( 'Left', 'piotnetgrid' ),
                    'center' => __( 'Center', 'piotnetgrid' ),
                    'flex-end' => __( 'Right', 'piotnetgrid' ),
                ],
                'selectors'  => [
                    '{{WRAPPER}} .piotnetgrid-facet__field-subgroup--az' => 'justify-content: {{VALUE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'az_item_spacing',
            [
                'label'      => __( 'Item Spacing', 'piotnetgrid' ),
                'type'       => 'slider',
                'size_units' => [ 'px' ],
                'range'      => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 100,
                        'step' => 1,
                    ],
                ],
                'default'    => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'render_type' => 'none',
                'selectors'  => [
                    '{{WRAPPER}} .piotnetgrid-facet__field-sub' => 'padding: 0 {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .piotnetgrid-facet__field-subgroup--az' => 'margin: 0 -{{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'az_item_padding',
            [
                'label'      => __( 'Item Padding', 'piotnetgrid' ),
                'type'       => 'dimensions',
                'size_units' => [ 'px', 'em', '%' ],
                'render_type' => 'none',
                'selectors'  => [
                    '{{WRAPPER}} .piotnetgrid-facet__field-sub-label' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
        $this->add_responsive_control(
            'az_item_margin',
            [
                'label'      => __( 'Item Margin', 'piotnetgrid' ),
                'type'       => 'dimensions',
                'size_units' => [ 'px', 'em', '%' ],
                'render_type' => 'none',
                'selectors'  => [
                    '{{WRAPPER}} .piotnetgrid-facet__field-sub-label' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            '',
            [
                'type' => 'heading-tab',
                'tabs' => [
                    [
                        'name'   => 'az_normal_tab',
                        'title'  => __( 'NORMAL', 'piotnetgrid' ),
                        'active' => true,
                    ],
                    [
                        'name'  => 'az_hover_tab',
                        'title' => __( 'HOVER', 'piotnetgrid' ),
                    ],
                    [
                        'name'  => 'az_current_tab',
                        'title' => __( 'CURRENT', 'piotnetgrid' ),
                    ],
                ],
            ]
        );

        $normal_controls = $this->az_style_controls(
            '',
            [
                'wrapper' => '{{WRAPPER}} .piotnetgrid-facet__field-sub-label',
            ]
        );
        $this->add_control(
            'az_normal_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Normal', 'piotnetgrid' ),
                'value'          => '',
                'active'         => true,
                'controls'       => $normal_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );

        $hover_controls = $this->az_style_controls(
            'hover',
            [
                'wrapper' => '{{WRAPPER}} .piotnetgrid-facet__field-sub:hover>.piotnetgrid-facet__field-sub-label',
            ]
        );
        $this->add_control(
            'az_hover_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Hover', 'piotnetgrid' ),
                'value'          => '',
                'controls'       => $hover_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );

       $current_controls = $this->az_style_controls(
            'current',
            [
                'wrapper' => '{{WRAPPER}} .piotnetgrid-facet__field-sub>input:checked ~ label',
            ]
        );
        $this->add_control(
            'az_current_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Current', 'piotnetgrid' ),
                'value'          => '',
                'controls'       => $current_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );
    }

	private function add_field_style_controls() {

		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'field_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'field_focus_tab',
						'title' => __( 'FOCUS', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->field_style_controls(
			'',
			[
				'wrapper' => '{{WRAPPER}}',
			]
		);
		$this->add_control(
			'field_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$focus_controls = $this->field_style_controls(
			'focus',
			[
				'wrapper' => '{{WRAPPER}}.piotnetgrid-field-focus',
			]
		);
		$this->add_control(
			'field_focus_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Focus', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $focus_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_load_more_style_controls() {
		$this->add_responsive_control(
			'load_more_align',
			[
				'type'         => 'select',
				'label'        => __( 'Alignment', 'piotnetgrid' ),
				'label_block'  => true,
				'value'        => '',
				'options'      => [
					''       => __( 'Default', 'piotnetgrid' ),
					'flex-start'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end'  => __( 'Right', 'piotnetgrid' ),
				],
				'render_type'  => 'both',
				'selectors'    => [
					'{{WRAPPER}}.piotnetgrid-facet--load_more .piotnetgrid-facet__field-wrapper' => 'justify-content: {{VALUE}}',
				],
			]
		);
		
		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'load_more_button_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'load_more_button_hover_tab',
						'title' => __( 'HOVER', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_load_more_button_style_controls(
			'normal',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button--loadmore',
			]
		);
		$this->add_control(
			'load_more_button_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_load_more_button_style_controls(
			'hover',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button--loadmore:hover',
			]
		);
		$this->add_control(
			'load_more_button_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_result_counts_style_controls() {
		$this->add_text_typography_controls(
			'result_counts_typography',
			[
				'render_type' => 'none',
				'selectors' => '{{WRAPPER}}',
			]
		);

		$this->add_control(
			'result_counts_color',
			[
				'type'    => 'color',
				'label'   => __( 'Color', 'piotnetgrid' ),
				'selectors' => [
					'{{WRAPPER}}' => 'color:{{VALUE}}',
				]
			]
		);
	}

	private function add_load_more_button_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();
		$this->add_control(
			$name.'_load_more_button_color',
			[
				'type'        => 'color',
				'label'       => __( 'Text Color', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_text_typography_controls(
			$name.'_load_more_button_text_typography',
			[
				'selectors' => $wrapper,
			]
		);
		$this->add_control(
			$name.'_load_more_button_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'background-color: {{VALUE}};',
					'{{WRAPPER}} .lds-ellipsis div' => 'background-color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_load_more_button_border_type',
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_load_more_button_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_load_more_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_load_more_button_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_load_more_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_load_more_button_border_radius',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->add_responsive_control(
			$name.'_load_more_button_padding',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		return $this->get_group_controls( $previous_controls );
	}

	public function multilingual_fields() {
		$fields = [
			[
				'field'       => 'facet_label',
				'editor_type' => 'LINE',
			],
			[
				'field'       => 'facet_placeholder',
				'editor_type' => 'LINE',
			],
			[
				'field'       => 'facet_placeholder_search',
				'editor_type' => 'LINE',
			],
			[
				'field'       => 'pagination_next_text',
				'editor_type' => 'LINE',
			],
			[
				'field'       => 'pagination_prev_text',
				'editor_type' => 'LINE',
			],
		];

		return $fields;
	}

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
		$facet_action = $settings['facet_action'];
		$filter_type = $settings['filter_type'];
		$filter_taxonomy = !empty($settings['filter_data_source_taxonomy']) ? $settings['filter_data_source_taxonomy'] : '';
		$multiple_selection = !empty($settings['multiple_selection']) ? true : false;
		$facet_value = '';
		$facet_placeholder = !empty($settings['facet_placeholder']) && in_array($filter_type, ['select', 'radio', 'buttons', 'az']) ? $settings['facet_placeholder'] : '';
		if ($filter_type == 'buttons' && $multiple_selection) {
			$facet_placeholder = '';
		}
		$grid_post_id = !empty($args_additional['grid']) ? $args_additional['grid'] : 0;
		$grid_data = json_decode( get_post_meta( $grid_post_id, '_piotnetgrid_data', true ), true );
		$grid_widgets = !empty($grid_data['widgets']) ? $grid_data['widgets'] : [];
		$piotnetgrid = !empty($args_additional['grid']) && is_numeric($args_additional['grid']) ? true : false;
		$facet_name = '_' . $settings['facet_name'];
		$paged = 1;
		$default_image_url = plugins_url() . '/piotnetgrid/assets/images/placeholder-image.png';

		$GLOBALS['piotnetgrid_facet'] = true;

		if (!empty($facet_name)) {
			$facet_value = !empty($_GET[$facet_name]) ? $_GET[$facet_name] : '';
		}

		$facet_data = array(
			'id' => $this->get_id(),
			'action' => $facet_action,
			'name' => $facet_name,
			'grid' => $grid_post_id,
		);

		if (!empty($settings['facet_scroll_to_top'])) {
		    $offset_desktop =  !empty($settings['facet_scroll_to_top_offset_desktop']) ? $settings['facet_scroll_to_top_offset_desktop'] : '0';
            $offset_tablet =  !empty($settings['facet_scroll_to_top_offset_tablet']) ? $settings['facet_scroll_to_top_offset_tablet'] : '0';
		    $offset_mobile =  !empty($settings['facet_scroll_to_top_offset_mobile']) ? $settings['facet_scroll_to_top_offset_mobile'] : '0';

            $this->add_render_attribute( 'wrapper', [
                'data-facet-pagination-scroll-to-top' => '',
                'data-facet-pagination-scroll-to-top-offset-desktop' => $offset_desktop,
                'data-facet-pagination-scroll-to-top-offset-tablet' => $offset_tablet,
                'data-facet-pagination-scroll-to-top-offset-mobile' => $offset_mobile,
            ]);
        }

		$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-grid', $grid_post_id );

		if (!empty($settings['facet_disable_add_parameter_to_url'])) {
			$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-disable-url', '' );
		}

		if (!empty($grid_post_id)) {
			if (strpos( $grid_post_id, 'piotnetgrid' ) !== 0) {
				foreach ($grid_widgets as $grid_id => $grid) {
					if ($grid['type'] == 'grid') {
						$grid_settings = $grid['settings'];
						$grid_widget = new piotnetgrid_Grid();
						$grid_widget->settings  = $grid_settings;
						$grid_widget->widget_id = $grid_id;
						$grid_widget->set_query( $grid_settings );
						$post_ids = $grid_widget->post_ids;
						$posts_per_page = $grid_widget->posts_per_page;
						$grid_args = $grid_widget->args;
						$paged = !empty($grid_args['paged']) ? $grid_args['paged'] : 1;
						$grid_widget->set_query( $grid_settings, [], false );
						$post_ids_not_facets = $grid_widget->post_ids;
						break;
					}
				}
			} else {
				if ($grid_post_id == 'piotnetgrid-id-wp') {
					global $wp_query;
					$wp_query_args = $wp_query->query_vars;
					$wp_query_args['fields'] = 'ids';
					unset($wp_query_args['number']);
					unset($wp_query_args['paged']);
					$wp_query_args['posts_per_page'] = '100000000';
					$posts_query = new WP_Query($wp_query_args);
					
					if (isset($wp_query_args['main_query']) && empty($_POST['get_posts_query'])) {
						$wp_query_args_main_query = $wp_query_args['main_query'];
						$wp_query_args_main_query['fields'] = 'ids';
						unset($wp_query_args_main_query['number']);
						unset($wp_query_args_main_query['paged']);
						$wp_query_args_main_query['posts_per_page'] = '100000000';
						$posts_query_main_query = new WP_Query($wp_query_args_main_query);
						$post_ids = array_intersect($posts_query->posts, $posts_query_main_query->posts);
						$post_ids_not_facets = $posts_query_main_query->posts;
					} else {
						$post_ids = $posts_query->posts;
						$post_ids_not_facets = $posts_query->posts;
					}
				} else {
					$wp_query_args = !empty($_POST['main_query']) ? $_POST['main_query'] : ['post_type' => 'any'];
					$wp_query_args['fields'] = 'ids';
					unset($wp_query_args['number']);
					unset($wp_query_args['paged']);
					unset($wp_query_args['no_found_rows']);
					$wp_query_args['posts_per_page'] = '100000000';
					$posts_per_page =  !empty($_POST['main_query']['posts_per_page']) ? $_POST['main_query']['posts_per_page'] : get_option( 'posts_per_page' );
					$posts_query = new WP_Query($wp_query_args);
					$post_ids_not_facets = $posts_query->posts;

					$get_facets = [];
					foreach ($_GET as $get_facet_name => $get_facet_value) {
						$get_facet_settings = get_option('_piotnetgrid_facet' . $get_facet_name);
						if (!empty($get_facet_settings)) {
							$get_facets[] = piotnetgrid_get_facet_by_name($get_facet_name, $get_facet_value);
						}
					}

					if (!empty($get_facets)) {
						$get_args_additional = piotnetgrid_facet_change_query($get_facets, []);

						foreach ($get_args_additional['query'] as $key => $value) {
							$wp_query_args[$key] = $value;
						}

						unset($wp_query_args['paged']);

						$posts_query = new WP_Query($wp_query_args);
					}

					$post_ids = $posts_query->posts;
				}
			}
		} else {
			$posts_query = new WP_Query([
				'post_type' => $this->post_types,
				'posts_per_page' => '-1',
				'fields' => 'ids',
			]);

			$post_ids = $posts_query->posts;
			$post_ids_not_facets = $posts_query->posts;
		}
		
		if (!empty($_GET)) {
			$facets_applied_not_load = [];
			foreach ($_GET as $facet_applied_name => $facet_applied_value) {
				$facet_applied_settings = get_option('_piotnetgrid_facet' . $facet_applied_name);
				if (!empty($facet_applied_settings)) {
					$facet_applied_info = piotnetgrid_get_facet_by_name($facet_applied_name, $facet_applied_value);
					if ($facet_applied_info['action'] != 'load') {
						$facets_applied_not_load[] = $facet_applied_info;
					}
				}
			}

			if (count($facets_applied_not_load) == 1 && $facets_applied_not_load[0]['name'] == $facet_name) {
				$post_ids = $post_ids_not_facets;
			}
		}

		$facet_options = [];

		if (!empty($facet_placeholder)) {
			$facet_options[] = [
				'value' => '',
				'label' => $facet_placeholder,
				'count' => -1,
			];
		}

		$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-facet' );
		switch ($facet_action) {
			case 'filter':
				if (!empty($settings['filter_type'])) {
					$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-facet--' . $settings['filter_type'] );
				}
				break;
			
			case 'load':
				if (!empty($settings['loading_type'])) {
					$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-facet--' . $settings['loading_type'] );
				}
				break;
		}
		$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-id', $this->get_id() );

		$this->add_render_attribute(
			[
				'facet_field' => [
					'class' => [
						'piotnetgrid-facet-field',
					],
					'data-piotnetgrid-facet-field' => '',
					'name' => $filter_type == 'checkboxes' ? $facet_name . '[]' : $facet_name,
					'id' => $facet_name,
				],
				'label' => [
					'for'   => $filter_type == 'checkboxes' ? $facet_name . '[]' : $facet_name,
					'class' => empty($settings['facet_label_hide']) ? 'piotnetgrid-facet__label' : 'piotnetgrid-facet__label hidden',
					'data-piotnetgrid-facet-label' => !empty($settings['facet_label']) ? $settings['facet_label'] : '',
				],
			]
		);

		switch ($facet_action) {
			case 'filter':
				$facet_data['filter_type'] = $filter_type;

				if (in_array($filter_type, ['buttons','color-picker','image-select'])) {
					if ($multiple_selection) {
						$facet_data['filter_type'] = 'checkboxes';
					} else {
						$facet_data['filter_type'] = 'radio';
					}
				}

				switch ($filter_type) {
					case 'search':
						$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-search', '' );
						break;
					case 'select':
					case 'radio':
					case 'checkboxes':
					case 'slider':
					case 'date':
					case 'buttons':
					case 'color-picker':
					case 'image-select':
						if (in_array($filter_type, ['checkboxes','buttons','color-picker','image-select']) && !empty($settings['filter_logic']) && $settings['filter_data_source'] == 'taxonomy') {
							$facet_data['filter_logic'] = $settings['filter_logic'];
						}

						if (in_array($filter_type, ['buttons','color-picker','image-select']) && empty($settings['multiple_selection'])) {
							$field_type_radio = true;
							$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-radio', '' );
						}

						$facet_data['filter_source'] = $settings['filter_data_source'];
						switch ($settings['filter_data_source']) {
							case 'taxonomy':
								$facet_data['filter_taxonomy'] = $filter_taxonomy;
								if (empty($args_additional['query']) && !empty($filter_taxonomy)) {
									if (!empty($grid_settings['current_query_enable']) || !empty($_POST['main_query'])) {
										$current_queried_object = get_queried_object();
										$current_term_id = isset($current_queried_object->term_id) ? $current_queried_object->term_id : 0;
										$current_term_slug = isset($current_queried_object->slug) ? $current_queried_object->slug : 0;
										$parents_current_term = [];
										if (!empty($current_term_id)) {
											$current_term = get_term($current_term_id);
											$parents_current_term = get_ancestors( $current_term_id, $current_term->taxonomy );
											$parents_current_term_slug = [$current_term_slug];
											foreach ($parents_current_term as $parent_term_id) {
												$parents_current_term_slug[] = get_term($parent_term_id)->slug;
											}
										}
									}

									$facet_data['exclude_terms'] = !empty($parents_current_term_slug) ? $parents_current_term_slug : [];

									if (!empty($settings['filter_data_source_taxonomy_exclude_terms'])) {
										foreach ($settings['filter_data_source_taxonomy_exclude_terms'] as $exclude_term_id) {
											$facet_data['exclude_terms'][] = get_term($exclude_term_id)->slug;
										}
									}

									$terms = get_terms( array(
									    'taxonomy' => $settings['filter_data_source_taxonomy'],
									    'hide_empty' => false,
									) );

									foreach ($terms as $term) {

										$post_ids_outofstock = piotnetgrid_get_product_variation_out_of_stock($filter_taxonomy, $term->slug);
										$post_ids_for_term = $post_ids;

										if (!empty($post_ids_outofstock)) {
											foreach ($post_ids_for_term as $key => $value) {
												if (in_array($value, $post_ids_outofstock)) {
													unset($post_ids_for_term[$key]);
												}
											}
										}

										if (!in_array($term->slug, $facet_data['exclude_terms']) || empty($parents_current_term)) {
											$term_count = 0;

											if (!empty($post_ids)) {
												$term_query = new WP_Query( array(
													'post_type' => $this->post_types,
											        'post__in' =>  $post_ids_for_term,
											        'posts_per_page' => -1,
											        'tax_query' => array(
												        array(
												            'taxonomy' => $settings['filter_data_source_taxonomy'],
												            'field'    => 'slug',
												            'terms'    => $term->slug,
												            'include_children' => false,
												        ),
												    ),
											    ));
												$term_count = $term_query->post_count;
											} else {
												$term_count = 0;
											}

											$term_query_not_facets = new WP_Query( array(
												'post_type' => $this->post_types,
										        'post__in' =>  $post_ids_not_facets,
										        'posts_per_page' => -1,
										        'tax_query' => array(
											        array(
											            'taxonomy' => $settings['filter_data_source_taxonomy'],
											            'field'    => 'slug',
											            'terms'    => $term->slug,
											            'include_children' => false,
											        ),
											    ),
										    ));
											$term_count_not_facets = $term_query_not_facets->post_count;

											if (!empty($term_count_not_facets)) {
												if (!in_array($term->slug, $facet_data['exclude_terms'])) {
													$facet_option = [
														'value' => $term->slug,
														'label' => $term->name,
														'count' => $term_count,
													];
												
													if ($filter_type == 'color-picker') {
														$facet_option['color'] = '#' . get_term_meta( $term->term_id, 'piotnetgrid_term_color', true );
													}

													if ($filter_type == 'image-select') {
														$facet_option['image'] = '';
														$term_image_id = get_term_meta( $term->term_id, 'piotnetgrid_term_image', true );
														if (!empty($term_image_id)) {
															$facet_option['image'] = wp_get_attachment_image_src( $term_image_id, 'full' )[0];
														}
													}

													$facet_options[] = $facet_option;
												}
											} else {
												$facet_data['exclude_terms'][] = $term->slug;
											}
										}
									}
								}

								if ($editor) {
									$facet_options = [];
									if (!empty($facet_placeholder)) {
										$facet_options[] = [
											'value' => '',
											'label' => $facet_placeholder,
											'count' => 75,
										];
									}
									$facet_options_editor = [
										[
											'value' => 'term_1',
											'label' => 'Term 1',
											'count' => 5,
											'color' => 'blue',
											'image' => $default_image_url,
										],
										[
											'value' => 'term_2',
											'label' => 'Term 2',
											'count' => 10,
											'color' => 'red',
											'image' => $default_image_url,
										],
										[
											'value' => 'term_3',
											'label' => 'Term 3',
											'count' => 15,
											'color' => 'purple',
											'image' => $default_image_url,
										],
										[
											'value' => 'term_4',
											'label' => 'Term 4',
											'count' => 20,
											'color' => 'yellow',
											'image' => $default_image_url,
										],
										[
											'value' => 'term_5',
											'label' => 'Term 5',
											'count' => 25,
											'color' => 'green',
											'image' => $default_image_url,
										],
									];
									$facet_options = array_merge($facet_options, $facet_options_editor);
								}

								break;

							case 'custom_field':
								$filter_custom_field_name = $settings['filter_data_source_custom_field'];
								if (!empty($filter_custom_field_name)) {
									$facet_data['filter_custom_field'] = $filter_custom_field_name;
									if (empty($args_additional['query'])) {
										global $wpdb;
										$post_ids_for_custom_field = !empty($post_ids) ? $post_ids : $post_ids_not_facets;
										if (!empty($post_ids_for_custom_field)) {
											$post_ids_in = '(' . implode(',', $post_ids_for_custom_field) .')';
											$filter_custom_field_checkbox = false;

											if ( strpos( $filter_custom_field_name, 'acf/' ) !== false && function_exists('acf_get_field_groups') ) {
												$acf_field_key = str_replace('acf/', '', $filter_custom_field_name);
												$acf_field_object = get_field_object($acf_field_key);
												$acf_field_name = $acf_field_object['name'];
												$filter_custom_field_name = $acf_field_name;

												if (in_array($acf_field_object['type'], ['checkbox','select','radio','button_group'])) {
													$filter_custom_field_checkbox = true;
													foreach ($acf_field_object['choices'] as $key => $value) {
														$facet_option = [
															'value' => $key,
															'label' => $value,
															'count' => 0,
														];

														$facet_option_value = $acf_field_object['type'] == 'select' && $acf_field_object['multiple'] != 1 || $acf_field_object['type'] == 'radio' ? $key : '"' . $key . '"';

														$facet_option_compare = $acf_field_object['type'] == 'select' && $acf_field_object['multiple'] != 1 || $acf_field_object['type'] == 'radio' ? '=' : 'LIKE';

														$facet_option_query = new WP_Query( array(
															'post_type' => $this->post_types,
															'post__in' => $post_ids_for_custom_field,
															'posts_per_page' => -1,
															'fields' => 'ids',
															'meta_query' => array(
																array(
																	'key'		=> $acf_field_name,
																	'value'		=> $facet_option_value,
																	'compare'	=> $facet_option_compare
																),
															)
														) );

														$facet_option['count'] = $facet_option_query->found_posts;

														$facet_options[] = $facet_option;
													}
												}
											}

                                            if ( strpos( $filter_custom_field_name, 'metabox/' ) !== false && function_exists('rwmb_meta') ) {
                                                $metabox_field_key = str_replace('metabox/', '', $filter_custom_field_name);
                                                $metabox_field_ids = array();
                                                $agrs = array( 'post_type' => 'meta-box' );
                                                $metabox_field_query = new WP_query($agrs);
                                                if ( $metabox_field_query->have_posts() ) {
                                                    while ( $metabox_field_query->have_posts() ) {
                                                        $metabox_field_query->the_post();
                                                        $metabox_field_ids[] = get_the_ID();
                                                    }
                                                }
                                                foreach ( $metabox_field_ids as $metabox_field_id ) {
                                                    $metabox_field_group = rwmb_meta('meta_box', array(), $metabox_field_id);
                                                    foreach ( $metabox_field_group['fields'] as $metabox_fields ) {
                                                        if ($metabox_fields['id'] == $metabox_field_key) {
                                                            $metabox_field_name = $metabox_fields['id'];
                                                            $filter_custom_field_name = $metabox_field_name;
                                                            if (in_array($metabox_fields['type'], ['checkbox_list', 'select', 'radio', 'button_group'])) {
                                                                $filter_custom_field_checkbox = true;
                                                                foreach ($metabox_fields['options'] as $key => $value) {
                                                                    $facet_option = [
                                                                        'value' => $key,
                                                                        'label' => $value,
                                                                        'count' => 0,
                                                                    ];

                                                                    $facet_option_query = new WP_Query( array(
                                                                        'post_type' => $this->post_types,
                                                                        'post__in' => $post_ids_for_custom_field,
                                                                        'posts_per_page' => -1,
                                                                        'fields' => 'ids',
                                                                        'meta_query' => array(
                                                                            array(
                                                                                'key'		=> $metabox_field_name,
                                                                                'value'		=> $key,
                                                                                'compare'	=> '='
                                                                            ),
                                                                        )
                                                                    ) );

                                                                    $facet_option['count'] = $facet_option_query->found_posts;

                                                                    $facet_options[] = $facet_option;
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }

                                            if ( strpos( $filter_custom_field_name, 'jetengine/' ) !== false && function_exists('jet_engine') ) {
                                                $jetengine_field_key = str_replace('jetengine/', '', $filter_custom_field_name);
                                                $jetengine_field_groups = get_option('jet_engine_meta_boxes');
                                                foreach ( $jetengine_field_groups as $jetengine_field_group ) {
                                                    $jetengine_fields = $jetengine_field_group['meta_fields'];
                                                    foreach ( $jetengine_fields as $jetengine_field ) {
                                                        if ( ($jetengine_field['name'] == $jetengine_field_key) ) {
                                                            $jetengine_field_name = $jetengine_field['name'];
                                                            $filter_custom_field_name = $jetengine_field_name;

                                                            if (in_array($jetengine_field['type'], ['checkbox','select','radio'])) {
                                                                $filter_custom_field_checkbox = true;
                                                                foreach ($jetengine_field['options'] as $value) {
                                                                    $facet_option = [
                                                                        'value' => $value['key'],
                                                                        'label' => $value['value'],
                                                                        'count' => 0,
                                                                    ];

                                                                    $facet_option_value = $jetengine_field['type'] == 'select' || $jetengine_field['type'] == 'radio' ? $value['key'] : '"' . $value['key'] . '"' . ';s:4:"true';

                                                                    $facet_option_compare = $jetengine_field['type'] == 'select' && !$jetengine_field['is_multiple'] || $jetengine_field['type'] == 'radio' ? '=' : 'LIKE';

                                                                    $facet_option_query = new WP_Query( array(
                                                                        'post_type' => $this->post_types,
                                                                        'post__in' => $post_ids_for_custom_field,
                                                                        'posts_per_page' => -1,
                                                                        'fields' => 'ids',
                                                                        'meta_query' => array(
                                                                            array(
                                                                                'key'		=> $jetengine_field_name,
                                                                                'value'		=> $facet_option_value,
                                                                                'compare'	=> $facet_option_compare
                                                                            ),
                                                                        )
                                                                    ) );


                                                                    $facet_option['count'] = $facet_option_query->found_posts;

                                                                    $facet_options[] = $facet_option;
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }

											if (!$filter_custom_field_checkbox) {
												$filter_custom_field_values = $wpdb->get_results( 'SELECT meta_value AS "value", meta_value AS "label", count(*) AS "count" FROM ' . $wpdb->prefix . 'postmeta WHERE post_id IN ' . $post_ids_in . ' AND meta_key = "' . $filter_custom_field_name . '" AND meta_value <> "" GROUP BY meta_value', ARRAY_A );
												$facet_options = array_merge($facet_options, $filter_custom_field_values);

												if (empty($filter_custom_field_values)) {
													$post_ids_in = '(' . implode(',', $post_ids_not_facets) .')';
													$filter_custom_field_values = $wpdb->get_results( 'SELECT meta_value AS "value", count(*) AS "count" FROM ' . $wpdb->prefix . 'postmeta WHERE post_id IN ' . $post_ids_in . ' AND meta_key = "' . $filter_custom_field_name . '" AND meta_value <> "" GROUP BY meta_value', ARRAY_A );

													foreach ($filter_custom_field_values as $filter_custom_field_value) {
														$facet_options[] = [
															'value' => $filter_custom_field_value['value'],
															'label' => $filter_custom_field_value['value'],
															'count' => 0,
														];
													}
												}
											}
											
										}
									}
								}

								if ($editor) {
									$facet_options = [];
									if (!empty($facet_placeholder)) {
										$facet_options[] = [
											'value' => '',
											'label' => $facet_placeholder,
											'count' => 75,
										];
									}
									$facet_options_editor = [
										[
											'value' => '',
											'label' => $facet_placeholder,
											'count' => 75,
										],
										[
											'value' => 'field_1',
											'label' => 'Field 1',
											'count' => 5,
											'color' => 'blue',
										],
										[
											'value' => 'field_2',
											'label' => 'Field 2',
											'count' => 10,
											'color' => 'red',
										],
										[
											'value' => 'field_3',
											'label' => 'Field 3',
											'count' => 15,
											'color' => 'purple',
										],
										[
											'value' => 'field_4',
											'label' => 'Field 4',
											'count' => 20,
											'color' => 'yellow',
										],
										[
											'value' => 'field_5',
											'label' => 'Field 5',
											'count' => 25,
											'color' => 'green',
										],
									];
									$facet_options = array_merge($facet_options, $facet_options_editor);
								}
								
								break;

							case 'author':
								$authors = [];

								foreach ($post_ids_not_facets as $post_id) {
									$author_id = get_post_field( 'post_author', $post_id );
									$author = get_user_by( 'ID', $author_id );
									$authors[$author_id]['name'] = $author->display_name; 
								}

								foreach ($authors as $key => $author) {
									if (!empty($post_ids)) {
										$author_query = new WP_Query( array(
									        'post__in' => $post_ids,
									        'posts_per_page' => -1,
									        'author' => $key,
									    ));
									}
									
								    $facet_options[] = [
										'value' => $key,
										'label' => $authors[$key]['name'],
										'count' => !empty($post_ids) ? $author_query->post_count : 0,
									];
								}

								if ($editor) {
									$facet_options = [];
									if (!empty($facet_placeholder)) {
										$facet_options[] = [
											'value' => '',
											'label' => $facet_placeholder,
											'count' => 75,
										];
									}
									$facet_options_editor = [
										[
											'value' => '',
											'label' => $facet_placeholder,
											'count' => 75,
										],
										[
											'value' => 'author_1',
											'label' => 'Author 1',
											'count' => 5,
										],
										[
											'value' => 'author_2',
											'label' => 'Author 2',
											'count' => 10,
										],
										[
											'value' => 'author_3',
											'label' => 'Author 3',
											'count' => 15,
										],
										[
											'value' => 'author_4',
											'label' => 'Author 4',
											'count' => 20,
										],
										[
											'value' => 'author_5',
											'label' => 'Author 5',
											'count' => 25,
										],
									];
									$facet_options = array_merge($facet_options, $facet_options_editor);
								}

								break;
						}

						break;

                    case 'az':

                    	$facet_data['filter_source'] = 'az';
                    	$az_list = !empty($settings['az_alphabetical']) ? $settings['az_alphabetical'] : '#,A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z';
                        $az_list = explode(',', $az_list);
                        $facet_data['az_list'] = $az_list;

                        if (empty($args_additional['query'])) {
                            foreach ($az_list as $az_text) {
                                $az_query = new WP_Query( array(
                                    'post_type' => $this->post_types,
                                    'post__in' =>  $post_ids,
                                    'posts_per_page' => -1,
                                    'piotnetgrid_query_starts_with' => $az_text,
                                    'fields' => 'ids',
                                ));
                                $facet_options[] = [
                                    'value' => $az_text,
                                    'label' => $az_text,
                                    'count' => count($az_query->posts),
                                ];
                            }
                        }

                        break;
				}

				break;

			case 'sort':
				$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-sort', '' );
				break;

			case 'load':
				$facet_data['loading_type'] = $settings['loading_type'];

				switch ($settings['loading_type']) {
					case 'load_more':
						$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-facet-load-more' );
						$this->add_render_attribute( 'field_wrapper', 'class', 'piotnetgrid-button-wrapper' );
						if (!empty($settings['load_more_infinite_scroll'])) {
							$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-load-more-infinite-scroll', '' );
						}
						break;
					case 'pagination':
						$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-pagination', '' );
						break;
					case 'posts_per_page':
						$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-posts-per-page', '' );
						break;
				}

				break;
		}

		if (!empty($args_additional['query'])) {
			if (isset($args_additional['facet_options'])) {
				$facet_options = array_merge($facet_options, $args_additional['facet_options']);
				
				if (!empty($args_additional['exclude_terms'])) {
					$facet_data['exclude_terms'] = $args_additional['exclude_terms'];
					foreach ($facet_options as $key => $facet_option) {
						if (in_array($facet_option['value'], $args_additional['exclude_terms'])) {
							unset($facet_options[$key]);
						}
					}
				}
			}
			if (isset($args_additional['post_ids'])) {
				$post_ids = $args_additional['post_ids'];
			}
			if (isset($args_additional['query']['posts_per_page'])) {
				$posts_per_page = $args_additional['query']['posts_per_page'];
			}
			if (isset($args_additional['query']['paged'])) {
				$paged = $args_additional['query']['paged'];
			}
		}

		if (isset($args_additional['value'])) {
			$facet_value = $args_additional['value'];
		}

		// Color/Image Options

		if (!empty($facet_options) && !empty($settings['color_image_options']) && !$editor) {
			foreach ($settings['color_image_options'] as $color_image_option) {
				foreach ($facet_options as $key => $facet_option) {
					if ($facet_option['value'] == $color_image_option['color_image_option_value']) {
						if (!empty($color_image_option['color_image_option_label'])) {
							$facet_options[$key]['label'] = $color_image_option['color_image_option_label'];
						}

						if (!empty($color_image_option['color_image_option_color'])) {
							$facet_options[$key]['color'] = $color_image_option['color_image_option_color'];
						}
					
						if (!empty($color_image_option['color_image_option_image']['url'])) {
							unset($facet_options[$key]['color']);
							$facet_options[$key]['image'] = $color_image_option['color_image_option_image']['url'];
						}
					}
				}
			}
		}

		if ($filter_type == 'slider') {
			$slider_values = [];
			foreach ($facet_options as $facet_option) {
				if (is_numeric($facet_option['value'])) {
					$slider_values[] = $facet_option['value'];
				}
			}

			if (!empty($slider_values)) {
				$slider_options = [
					'min' => min($slider_values),
					'max' => max($slider_values),
					'step' => !empty($settings['slider_step']) ? $settings['slider_step'] : 10,
					'prefix' => !empty($settings['slider_prefix']) ? htmlentities($settings['slider_prefix'], ENT_COMPAT, 'UTF-8') : '',
					'postfix' => !empty($settings['slider_postfix']) ? htmlentities($settings['slider_postfix'], ENT_COMPAT, 'UTF-8') : '',
					'thousands_separator' => !empty($settings['slider_thousands_separator']) ? $settings['slider_thousands_separator'] : '',
					'decimals_separator' => !empty($settings['slider_decimals_separator']) ? $settings['slider_decimals_separator'] : '',
					'decimals_places' => !empty($settings['slider_decimals_places']) ? $settings['slider_decimals_places'] : 0,
				];
			} else {
				$slider_options = [
					'min' => 0,
					'max' => 0,
				];
			}
			
			if (!empty($facet_value)) {
				if (is_array($facet_value)) {
					$facet_value_slider = $facet_value;
				} else {
					$facet_value_slider = explode(',', $facet_value);
				}
				$slider_options['from'] = $facet_value_slider[0];
				$slider_options['to'] = $facet_value_slider[1];
			}

			if ($editor) {
				$slider_options = [
					'min' => 0,
					'max' => 1000,
					'prefix' => !empty($settings['slider_prefix']) ? htmlentities($settings['slider_prefix'], ENT_COMPAT, 'UTF-8') : '',
					'postfix' => !empty($settings['slider_postfix']) ? htmlentities($settings['slider_postfix'], ENT_COMPAT, 'UTF-8') : '',
					'thousands_separator' => !empty($settings['slider_thousands_separator']) ? $settings['slider_thousands_separator'] : '',
					'decimals_separator' => !empty($settings['slider_decimals_separator']) ? $settings['slider_decimals_separator'] : '',
					'decimals_places' => !empty($settings['slider_decimals_places']) ? $settings['slider_decimals_places'] : 0,
				];
			}

			$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet-slider', json_encode($slider_options) );
		}

		if (!empty($args_additional['cache'])) {
			$facet_data['cache'] = $args_additional['cache'];
			$facet_data['cache_expiration'] = $args_additional['cache_expiration'];
		}

		$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-facet', json_encode($facet_data) );

		$label_extend_html = '';

		if ($filter_type == 'checkboxes' || $filter_type == 'radio') {
			$checkbox_replacement_class = !empty($settings['facet_checkbox_replacement']) && $settings['checkbox_type'] == 'native' ? ' piotnetgrid-facet__field-subgroup--checkbox-replacement' : '';

			if ( !empty($settings['facet_checkbox_replacement']) && $settings['checkbox_type'] == 'native' ) {
				if ( $settings['facet_checkbox_replacement_icon_type'] == 'font_awesome' ) {
					if ($settings['facet_checkbox_replacement_icon_font_awesome'] || $settings['facet_checkbox_replacement_icon_font_awesome_checked']) {
						wp_enqueue_style( $this->slug . '-fontawesome-style' );
						$label_extend_html_wrapper_class = $settings['facet_checkbox_replacement_icon_font_awesome_checked'] ? 'piotnetgrid-facet__field-sub-icon piotnetgrid-facet__field-sub-icon-has-checked' : 'piotnetgrid-facet__field-sub-icon';

						if ($settings['facet_checkbox_replacement_icon_font_awesome_checked'] && !$settings['facet_checkbox_replacement_icon_font_awesome']) {
							$label_extend_html_wrapper_class .= ' piotnetgrid-facet__field-sub-icon--only-checked';
						}

						$label_extend_html = '<span class="' . $label_extend_html_wrapper_class . '">';

						if ($settings['facet_checkbox_replacement_icon_font_awesome']) {
							$label_extend_html .= '<i class="piotnetgrid-facet__field-sub-icon-font-awesome piotnetgrid-facet__field-sub-icon-font-awesome--normal ' . $settings['facet_checkbox_replacement_icon_font_awesome'] . '"></i>';
						}

						if ($settings['facet_checkbox_replacement_icon_font_awesome_checked']) {
							$label_extend_html .= '<i class="piotnetgrid-facet__field-sub-icon-font-awesome piotnetgrid-facet__field-sub-icon-font-awesome--checked ' . $settings['facet_checkbox_replacement_icon_font_awesome_checked'] . '"></i>';
						}

						$label_extend_html .= '</span>';
					}
				} else {
					if ($settings['facet_checkbox_replacement_icon_image'] || $settings['facet_checkbox_replacement_icon_image_checked']) {
						$label_extend_html_wrapper_class = $settings['facet_checkbox_replacement_icon_image_checked'] ? 'piotnetgrid-facet__field-sub-icon piotnetgrid-facet__field-sub-icon-has-checked' : 'piotnetgrid-facet__field-sub-icon';

						if ($settings['facet_checkbox_replacement_icon_image_checked'] && !$settings['facet_checkbox_replacement_icon_image']) {
							$label_extend_html_wrapper_class .= ' piotnetgrid-facet__field-sub-icon--only-checked';
						}

						$label_extend_html = '<span class="' . $label_extend_html_wrapper_class . '">';

						if ($settings['facet_checkbox_replacement_icon_image']) {
							$label_extend_html .= '<img class="piotnetgrid-facet__field-sub-icon-image piotnetgrid-facet__field-sub-icon-image--normal" src="' . $settings['facet_checkbox_replacement_icon_image']['url'] . '">';
						}

						if ($settings['facet_checkbox_replacement_icon_image_checked']) {
							$label_extend_html .= '<img class="piotnetgrid-facet__field-sub-icon-image piotnetgrid-facet__field-sub-icon-image--checked" src="' . $settings['facet_checkbox_replacement_icon_image_checked']['url'] . '">';
						}

						$label_extend_html .= '</span>';
					}
				}
			}
		}

		$this->add_render_attribute( 'field_wrapper', 'class', 'piotnetgrid-facet__field-wrapper' );

		$show_choice_count = !empty($settings['show_choice_count']) ? true : false;
		?>
			<div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
				<?php if (!empty($settings['facet_label'])) : ?>
					<label <?php echo $this->get_render_attribute_string( 'label' ); ?>><?php _e( $settings['facet_label'], 'piotnetgrid' ); ?></label>
				<?php endif; ?>
				<div <?php echo $this->get_render_attribute_string( 'field_wrapper' ); ?>>
				<?php
					switch ($facet_action) {
						case 'filter':
							switch ($filter_type) {
								case 'select':
									?>
										<select <?php echo $this->get_render_attribute_string( 'facet_field' ); ?>>
											<?php foreach ($facet_options as $facet_option) : ?>
												<option value="<?php echo $facet_option['value']; ?>"<?php if ($facet_value == $facet_option['value']) { echo ' selected'; } ?><?php if ($facet_option['count'] == 0) { echo ' disabled'; } ?> data-piotnetgrid-option-label="<?php echo $facet_option['label']; ?>"><?php echo $facet_option['label']; ?><?php if ($show_choice_count && $facet_option['count'] !== -1) : ?> (<?php echo $facet_option['count']; ?>)<?php endif; ?></option>
											<?php endforeach; ?>
										</select>
									<?php
									break;

								case 'radio':
								case 'checkboxes':
								case 'buttons':
								case 'color-picker':
								case 'image-select':
                                case 'az':

									$this->add_render_attribute( 'facet_field_subgroup', 'class', 'piotnetgrid-facet__field-subgroup' );

									if ($filter_type == 'radio' || $filter_type == 'checkboxes') {
										$checkbox_type = !empty($settings['checkbox_type']) ? $settings['checkbox_type'] : 'native';
										$this->add_render_attribute( 'facet_field_subgroup', 'class', 'piotnetgrid-facet__field-subgroup--' . $checkbox_type );
										$this->add_render_attribute( 'facet_field_subgroup', 'class', $checkbox_replacement_class );
										if (!empty($settings['inline_list'])) {
											$this->add_render_attribute( 'facet_field_subgroup', 'class', 'piotnetgrid-facet__field-subgroup--inline' );
										}

                                        if (!empty($settings['facet_limit_choices_option'])) {
                                            $this->add_render_attribute( 'facet_field_subgroup', 'data-piotnetgrid-limit-choices-option', $settings['facet_limit_choices_option']);
                                        }

									} else {
										$this->add_render_attribute( 'facet_field_subgroup', 'class', 'piotnetgrid-facet__field-subgroup--' . $filter_type );
										$this->add_render_attribute( 'facet_field_subgroup', 'class', 'piotnetgrid-facet__field-subgroup--inline' );
									}

									$field_type = 'radio';

									if ($filter_type == 'checkboxes' || in_array($filter_type, ['buttons','color-picker','image-select'])) {
										$field_type = 'checkbox';
									}

									$this->add_render_attribute( 'facet_field', 'type', $field_type );

									if ($field_type == 'checkbox') {
										$facet_value = is_array($facet_value) ? $facet_value : explode(',', $facet_value);
									}
									?>
										<div <?php echo $this->get_render_attribute_string( 'facet_field_subgroup' ); ?>>
											<?php foreach ($facet_options as $facet_option_key => $facet_option) : ?>
												<div class="piotnetgrid-facet__field-sub">
													<?php
														$this->remove_render_attribute( 'facet_field', 'id' );
														$this->remove_render_attribute( 'facet_field', 'value' );
														$this->remove_render_attribute( 'facet_field', 'checked' );
														$this->remove_render_attribute( 'facet_field', 'disabled' );
														$this->remove_render_attribute( 'facet_field', 'data-piotnetgrid-option-label' );

														$this->add_render_attribute( 'facet_field', 'id', $facet_name . '-' . $facet_option_key );
														$this->add_render_attribute( 'facet_field', 'value', $facet_option['value'] );

														$facet_option_disabled = false;

														if ($field_type == 'radio' && $facet_value == $facet_option['value'] || $field_type == 'checkbox' && in_array( $facet_option['value'], $facet_value)) {
															$this->add_render_attribute( 'facet_field', 'checked', '' );
															if (!empty($facet_placeholder) && is_array($facet_value) && $facet_value[0] == '') {
																$this->remove_render_attribute( 'facet_field', 'checked' );
															}
														}
														if ($facet_option['count'] == 0) {
															$this->add_render_attribute( 'facet_field', 'disabled', '' );
															$facet_option_disabled = true;
														}

														if (in_array($filter_type, ['color-picker','image-select'])) {
															$label_extend_html = '';

															if ($filter_type == 'color-picker') {
																$facet_option_color = !empty($facet_option['color']) ? $facet_option['color'] : $facet_option['value'];
																$label_extend_html = '<span class="piotnetgrid-facet-field__color-image" style="background-color:' . $facet_option_color . '"></span>';
															}
															if ($filter_type == 'image-select') {
																$label_extend_html = '<span class="piotnetgrid-facet-field__color-image"><img src="' . $facet_option['image'] . '"></span>';
															}
														}

														$facet_option['label'] = !empty($facet_option['label']) ? $facet_option['label'] : $facet_option['value'];
														$this->add_render_attribute( 'facet_field', 'data-piotnetgrid-option-label', $facet_option['label'] );
													?>
													<input <?php echo $this->get_render_attribute_string( 'facet_field' ); ?>>
													<label for="<?php echo $facet_name . '-' . $facet_option_key; ?>" class="piotnetgrid-facet__field-sub-label<?php if($facet_option_disabled) { echo ' piotnetgrid-facet__field-sub-label--disabled'; } ?>"  data-piotnetgrid-option-label="<?php echo $facet_option['label']; ?>">
														<?php echo $label_extend_html; ?>
														<span class="piotnetgrid-facet__field-sub-label-content">
															<span class="piotnetgrid-facet__field-sub-label-text"><?php echo $facet_option['label']; ?></span><?php if ($show_choice_count && $facet_option['count'] !== -1) : ?> <span class="piotnetgrid-facet__field-sub-label-count">(<?php echo $facet_option['count']; ?>)</span><?php endif; ?>
														</span>
													</label>
												</div>

											<?php endforeach; ?>
                                            <?php if (!empty($settings['facet_limit_choices_option'])) : ?>
                                            <div class="piotnetgrid-facet__show-more-less">
                                                <button class="piotnetgrid-facet__show-more-less-btn piotnetgrid-facet__show-more-less-btn--more"><?php if (!empty($settings['facet_show_more_button_label'])){ echo $settings['facet_show_more_button_label']; } else { echo __('Show more','piotnetgrid'); } ?></button>
                                                <button class="piotnetgrid-facet__show-more-less-btn piotnetgrid-facet__show-more-less-btn--less"><?php if (!empty($settings['facet_show_less_button_label'])){ echo $settings['facet_show_less_button_label'];} else { echo __('Show less','piotnetgrid'); } ?></button>
                                            </div>
                                            <?php endif; ?>
										</div>
									<?php

									break;
								
								case 'search':
									if (!empty($settings['facet_placeholder_search'])) {
										$this->add_render_attribute( 'facet_field', 'placeholder', __( $settings['facet_placeholder_search'], 'piotnetgrid' ) );
									}
									?>
										<input type="text" <?php echo $this->get_render_attribute_string( 'facet_field' ); ?> value="<?php echo $facet_value; ?>">
									<?php
									break;

								case 'slider':
										$this->add_render_attribute( 'facet_field', 'class', 'piotnetgrid-facet-field--slider' );
									?>
										<input type="hidden" <?php echo $this->get_render_attribute_string( 'facet_field' ); ?>>
										<div class="piotnetgrid-facet-slider" data-piotnetgrid-facet-slider-bar></div>
										<div class="piotnetgrid-facet-slider-value" data-piotnetgrid-facet-slider-value></div>
									<?php
									break;

								case 'date':
									wp_enqueue_style( $this->slug . '-flatpickr-style' );
									wp_enqueue_script( $this->slug . '-flatpickr-script' );

									$facet_date_options = [];
									$facet_dates = [];
									$facet_dates_format = [];

									switch ($settings['filter_data_source']) {
										case 'post_date':
											foreach ($post_ids as $post_id) {
												$facet_dates[] = get_the_date('Ymd', $post_id);
												$facet_dates_format[] = get_the_date('Y-m-d', $post_id);
											}
											break;

										case 'custom_field':
											foreach ($facet_options as $facet_option_key => $facet_option) {
												$facet_dates[] = date('Ymd',strtotime($facet_option['value']));
												$facet_dates_format[] = date('Y-m-d',strtotime($facet_option['value']));
											}
											break;
									}

									$facet_dates = array_unique($facet_dates);
									$facet_dates_format = array_unique($facet_dates_format);

									if ($settings['date_type'] == 'single') {
										$facet_date_options['enable'] = $facet_dates_format;
									} else {
										if (!empty($facet_dates)) {
											$facet_date_options['mode'] = 'range';
											$facet_date_options['minDate'] = date('Y-m-d',strtotime(min($facet_dates)));
											$facet_date_options['maxDate'] = date('Y-m-d',strtotime(max($facet_dates)));
										}
									}
									
									$this->add_render_attribute( 'facet_field', 'data-piotnetgrid-facet-date', json_encode($facet_date_options) );
									?>
										<input type="text" <?php echo $this->get_render_attribute_string( 'facet_field' ); ?> value="<?php echo $facet_value; ?>"><span class="piotnetgrid-remove-date-field">
											<svg viewBox="0 0 45.334 45.334" style="width: 10px;height: 10px;">
												<path d="M43.576,35.092L31.152,22.668l12.424-12.424c2.343-2.343,2.343-6.142,0-8.485c-2.343-2.343-6.142-2.343-8.485,0   L22.666,14.183L10.242,1.758c-2.344-2.343-6.143-2.343-8.485,0c-2.343,2.344-2.343,6.143,0,8.485l12.425,12.424L1.757,35.091   c-2.343,2.344-2.343,6.143,0,8.485C2.93,44.748,4.464,45.333,6,45.333c1.535,0,3.071-0.586,4.243-1.757l12.423-12.423   l12.425,12.424c1.173,1.171,2.707,1.757,4.243,1.757s3.071-0.586,4.243-1.757C45.919,41.234,45.919,37.436,43.576,35.092z" />
											</svg>
										</span>
									<?php
									break;
							}
							break;
						
						case 'sort':
							if (!empty($settings['order_list'])) {
								$facet_options = [];
								$order_list = $settings['order_list'];
								foreach ($order_list as $order) {
									$order_value = $order['order_by'] . '_' . strtolower($order['order']);

									if ($order['order_by'] == 'meta_value' || $order['order_by'] == 'meta_num') {
										$order_custom_field_name = $order['custom_field_name'];
										$order_value = $order['order_by'] . '_' . $order_custom_field_name . '_' . strtolower($order['order']);
									}

									$facet_options[] = [
										'label' => $order['order_label'],
										'value' => $order_value,
									];
								}
							}
							?>
								<select <?php echo $this->get_render_attribute_string( 'facet_field' ); ?>>
									<option value=""><?php echo !empty($settings['facet_sort_placeholder']) ? $settings['facet_sort_placeholder'] : ''; ?></option>
									<?php foreach ($facet_options as $facet_option) : ?>
										<option value="<?php echo $facet_option['value']; ?>"<?php if ($facet_value == $facet_option['value']) { echo ' selected'; } ?> data-piotnetgrid-option-label="<?php echo $facet_option['label']; ?>"><?php echo $facet_option['label']; ?></option>
									<?php endforeach; ?>
								</select>
							<?php
							break;

						case 'load':
							switch ($settings['loading_type']) {
								case 'pagination':
									if ($editor) {
										$posts_per_page = 2;
										$post_ids = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20];
									}
									if (!empty($posts_per_page) && $posts_per_page != -1) :
										$page_numbers = ceil( count($post_ids) / $posts_per_page );
										echo $this->paginate_links( array(
											'total' => $page_numbers,
											'current' => !empty($facet_value) ? $facet_value : 1,
											'show_all' => !empty($settings['pagination_show_all']) ? $settings['pagination_show_all'] : false,
											'prev_next' => !empty($settings['pagination_next_prev_button']) ? true : false,
											'prev_text_only' => !empty($settings['pagination_next_prev_button_only']) ? true : false,
											'prev_text' => !empty($settings['pagination_prev_text']) && !empty($settings['pagination_next_prev_button']) ? $settings['pagination_prev_text'] : '&laquo; Previous',
											'next_text' => !empty($settings['pagination_next_text']) && !empty($settings['pagination_next_prev_button']) ? $settings['pagination_next_text'] : 'Next &raquo;',
											'end_size' => !empty($settings['pagination_end_size']) ? $settings['pagination_end_size'] : 2,
											'mid_size' => !empty($settings['pagination_mid_size']) ? $settings['pagination_mid_size'] : 1,
											'add_fragment' => !empty($settings['pagination_add_fragment']) ? $settings['pagination_add_fragment']: '',
											'before_page_number' => !empty($settings['pagination_before_page_number']) ? $settings['pagination_before_page_number'] : '',
											'after_page_number' => !empty($settings['pagination_after_page_number']) ? $settings['pagination_after_page_number'] : '',
											'is_editor' => $editor
										));
									?>
										<input type="hidden" <?php echo $this->get_render_attribute_string( 'facet_field' ); ?> value="<?php echo $facet_value; ?>">
									<?php
									endif;

									break;
								
								case 'load_more':
									if ($editor) {
										$posts_per_page = 2;
										$post_ids = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20];
									}
									if (!empty($posts_per_page) && $posts_per_page != -1) :
										$page_numbers = ceil( count($post_ids) / $posts_per_page );
										$page_current = !empty($facet_value) ? $facet_value : $paged;

										if (!empty($GLOBALS['piotnetgrid_load_more'])) {
											$page_current = $GLOBALS['piotnetgrid_load_more'];
										}

										if ($page_current < $page_numbers) :
											$load_more_data = [
												'max' => $page_numbers,
											];
											$this->add_render_attribute( 'load_more_button', 'data-piotnetgrid-facet-load-more', json_encode($load_more_data) );
											$this->add_render_attribute( 'load_more_button', 'class', 'piotnetgrid-button piotnetgrid-button--loadmore' );

											if ($page_current == 1) {
												$page_current = '';
											}
									?>
											<?php if (!empty($settings['load_more_infinite_scroll'])) : ?>
												<div class="lds-ellipsis"><div></div><div></div><div></div><div></div></div>
											<?php endif; ?>
											<button <?php echo $this->get_render_attribute_string( 'load_more_button' ); ?>>
												<span class="piotnetgrid-button--loadmore__text"><?php echo $settings['load_more_text']; ?></span>
												<span class="piotnetgrid-button--loadmore__text-loading"><?php echo $settings['loading_text']; ?></span>
											</button>
											<input type="hidden" <?php echo $this->get_render_attribute_string( 'facet_field' ); ?> value="<?php echo $page_current; ?>">
									<?php
										endif;
									endif;

									break;

								case 'posts_per_page':
									$posts_per_page_options = !empty($settings['posts_per_page_options']) ? explode(',',$settings['posts_per_page_options']) : [10,20,30];
									?>
										<select <?php echo $this->get_render_attribute_string( 'facet_field' ); ?>>
											<?php foreach ($posts_per_page_options as $posts_per_page_option) : $posts_per_page_option = trim($posts_per_page_option); ?>
												<option value="<?php echo $posts_per_page_option; ?>"<?php if ($facet_value == $posts_per_page_option) { echo ' selected'; } ?>><?php echo $posts_per_page_option; ?></option>
											<?php endforeach; ?>
										</select>
									<?php
									break;

								case 'result_counts':
									$result_count_text = array(
										'plural' => $settings['count_text_plural'],
										'singular' => $settings['count_text_singular'],
									);

									if (!empty($facet_applied_info) && !empty($facet_applied_info['loading_type']) && $facet_applied_info['loading_type'] == 'pagination') {
										$paged = intval($facet_applied_info['value']);
									}

									if (!empty($GLOBALS['piotnetgrid_load_more'])) {
										$paged = intval($GLOBALS['piotnetgrid_load_more']);
									}

									$result_counts_total = count($post_ids);
									$result_counts_from = (($paged - 1) * $posts_per_page) + 1;

									if (!empty($GLOBALS['piotnetgrid_load_more']) || !empty($args_additional['facet_current_type']) && $args_additional['facet_current_type'] == 'load_more') {
										$result_counts_from = 1;
									}

									$result_counts_to = $result_counts_total >= $posts_per_page ? $paged * $posts_per_page : $result_counts_total;
									if ($result_counts_to > $result_counts_total) {
										$result_counts_to = $result_counts_total;
									}

									if ($result_counts_total == 0) {
										$result_counts_html = '';
									} elseif ($result_counts_total == 1) {
										$result_counts_html = $settings['count_text_singular'];
									} else {
										$result_counts_html = $settings['count_text_plural'];
									}

									$result_counts_html = str_replace('[from]', $result_counts_from, $result_counts_html);
									$result_counts_html = str_replace('[to]', $result_counts_to, $result_counts_html);
									$result_counts_html = str_replace('[total]', $result_counts_total, $result_counts_html);

									$this->add_render_attribute( 'result_counts', 'data-piotnetgrid-result-counts', json_encode($result_count_text) );
									?>
							        <div <?php echo $this->get_render_attribute_string( 'result_counts' ) ?>><?php echo $result_counts_html; ?></div>
									<?php
									break;
							}

							break;
					}
				?>
				</div>
			</div>
		<?php
		unset($GLOBALS['piotnetgrid_facet']);
		unset($GLOBALS['piotnetgrid']);
	}

	public function live_preview() {
	}

	public function paginate_links( $args = '' ) {
	    $defaults = array(
	        'total'              => 2,
	        'current'            => 1,
	        'show_all'           => false,
	        'prev_next'          => true,
	        'prev_text'          => __( '&laquo; Previous' ),
	        'next_text'          => __( 'Next &raquo;' ),
	        'end_size'           => 2,
	        'mid_size'           => 1,
	        'add_fragment'       => '',
	        'before_page_number' => '',
	        'after_page_number'  => '',
	    );
	 
	    $args = wp_parse_args( $args, $defaults );
	 
	    // Who knows what else people pass in $args.
	    $total = (int) $args['total'];
	    if ( $total < 2 ) {
	        return;
	    }
	    $current  = (int) $args['current'];
	    $end_size = (int) $args['end_size']; // Out of bounds? Make it the default.
	    if ( $end_size < 1 ) {
	        $end_size = 1;
	    }
	    $mid_size = (int) $args['mid_size'];
	    if ( $mid_size < 0 ) {
	        $mid_size = 2;
	    }
	 
	    $r          = '';
	    $page_links = array();
	    $dots       = false;

	    if ( $args['prev_next'] && $current && 1 < $current || $args['is_editor'] || $args['prev_text_only']) :
	        // $link = str_replace( '%_%', 2 == $current ? '' : $args['format'], $args['base'] );
	        // $link = str_replace( '%#%', $current - 1, $link );
	        // $link .= $args['add_fragment'];

	        $prev_classes = !($args['prev_next'] && $current && 1 < $current) && $args['prev_text_only'] ? ' piotnetgrid-pagination__item-page-numbers--disabled' : '';

	        $page_links[] = sprintf(
	            '<div class="piotnetgrid-pagination__item-page-numbers piotnetgrid-pagination__item-page-numbers--prev' . $prev_classes . '" data-piotnetgrid-facet-pagination-page-number="%s">%s</div>',
	            $current - 1,
	            __($args['prev_text'],'piotnetgrid')
	        );
	    endif;
		
		if (!$args['prev_text_only']) {
		    for ( $n = 1; $n <= $total; $n++ ) :
		        if ( $n == $current ) :
		            $page_links[] = sprintf(
		                '<div class="piotnetgrid-pagination__item-page-numbers piotnetgrid-pagination__item-page-numbers--current" data-piotnetgrid-facet-pagination-page-number="%s">%s</div>',
		                $n,
		                $args['before_page_number'] . number_format_i18n( $n ) . $args['after_page_number']
		            );
		 
		            $dots = true;
		        else :
		            if ( $args['show_all'] || ( $n <= $end_size || ( $current && $n >= $current - $mid_size && $n <= $current + $mid_size ) || $n > $total - $end_size ) ) :
		                // $link = str_replace( '%_%', 1 == $n ? '' : $args['format'], $args['base'] );
		                // $link = str_replace( '%#%', $n, $link );
		                // if ( $add_args ) {
		                //     $link = add_query_arg( $add_args, $link );
		                // }
		                // $link .= $args['add_fragment'];
		 
		                $page_links[] = sprintf(
		                    '<div class="piotnetgrid-pagination__item-page-numbers" data-piotnetgrid-facet-pagination-page-number="%s">%s</div>',
		                    $n,
		                    $args['before_page_number'] . number_format_i18n( $n ) . $args['after_page_number']
		                );
		 
		                $dots = true;
		            elseif ( $dots && ! $args['show_all'] ) :
		                $page_links[] = '<div class="piotnetgrid-pagination__item-page-numbers piotnetgrid-pagination__item-page-numbers--dots">' . __( '&hellip;' ) . '</div>';
		 
		                $dots = false;
		            endif;
		        endif;
		    endfor;
	    }
	 
	    if ( $args['prev_next'] && $current && $current < $total || $args['prev_text_only'] ) :
	        // $link = str_replace( '%_%', $args['format'], $args['base'] );
	        // $link = str_replace( '%#%', $current + 1, $link );
	        // $link .= $args['add_fragment'];
	 		
	 		$next_classes = !($args['prev_next'] && $current && $current < $total) && $args['prev_text_only'] ? ' piotnetgrid-pagination__item-page-numbers--disabled' : '';

	        $page_links[] = sprintf(
	            '<div class="piotnetgrid-pagination__item-page-numbers piotnetgrid-pagination__item-page-numbers--next' . $next_classes . '" data-piotnetgrid-facet-pagination-page-number="%s">%s</div>',
	            $current + 1,
	            __($args['next_text'], 'piotnetgrid')
	        );
	    endif;
	 
	    $r .= "<div class='piotnetgrid-pagination'><div class='piotnetgrid-pagination__item'>";
        $r .= implode( "</div><div class='piotnetgrid-pagination__item'>", $page_links );
        $r .= "</div></div>";
	 
	    return $r;
	}

}
