<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Gallery extends Base_Widget_piotnetgrid {
    public function get_type() {
        return 'gallery';
    }

    public function get_class_name() {
        return 'piotnetgrid_Gallery';
    }

    public function get_title() {
        return 'Gallery';
    }

    public function get_icon() {
        return [
            'type' => 'image',
            'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-gallery.svg',
        ];
    }

    public function get_categories() {
        return [ 'piotnetgrid' ];
    }

    public function get_keywords() {
        return [ 'gallery' ];
    }

    public function register_controls() {
        $this->start_tab( 'settings', 'Settings' );

        $this->start_section( 'gallery_settings_section', 'Gallery Settings' );
        $this->add_setting_controls();

        $this->start_section( 'gallery_gallery_settings_section', 'Carousel Settings' );
        $this->add_layout_controls();

        $this->start_tab( 'style', 'Style' );

        $this->start_section( 'gallery_styles_section', 'Gallery Style' );
        $this->add_gallery_style_controls();

        $this->start_section( 'arrows_styles_section', 'Arrows Style' );
        $this->add_arrows_style_controls();

        $this->start_section( 'dots_styles_section', 'Dots Style' );
        $this->add_dots_style_controls();

        $this->add_advanced_style_tab();

        $this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
        $this->add_custom_class_id();

        $this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
        $this->add_custom_css_controls();

        $this->add_animation_settings();

        $this->add_attributes_settings();

        $this->start_section( 'conditional_visibility', 'Conditional Visibility' );
        $this->add_conditional_visibility_controls();

        $this->add_trigger_action_carousel();


        return $this->structure;
    }
    private function add_setting_controls() {

        $this->add_control(
            'gallery_dynamic_data',
            [
                'type'        => 'select',
                'label'       => __( 'Gallery Dynamic Data', 'piotnetgrid' ),
                'options'     => [
                    '' => __( 'None', 'piotnetgrid' ),
                    'acf_field' => __( 'ACF Gallery Field', 'piotnetgrid' ),
                    'metabox_field' => __( 'Metabox Gallery Field', 'piotnetgrid' ),
                    'pods_field' => __( 'Pods Gallery Field', 'piotnetgrid' ),
                    'toolset_field' => __( 'Toolset Gallery Field', 'piotnetgrid' ),
                    'jetengine_field' => __( 'JetEngine Gallery Field', 'piotnetgrid' ),
                    'woocommerce_gallery' => __( 'Woocomemrce Gallary Images', 'piotnetgrid' ),
                ],
            ]
        );

        $this->add_control(
            'gallery_dynamic_data_field_name',
            [
                'type'        => 'text',
                'label'       => __( 'Field Name', 'piotnetgrid' ),
                'conditions'  => [
                    [
                        'name'     => 'gallery_dynamic_data',
                        'operator' => 'in',
                        'value'    => [
                            'acf_field',
                            'metabox_field',
                            'pods_field',
                            'toolset_field',
                            'jetengine_field',
                        ],
                    ],
                ],
            ]
        );

        $image_sizes = $this->get_additional_image_sizes();

        $this->add_control(
            'gallery_image_size',
            [
                'type'    => 'select',
                'label'   => __( 'Image Size', 'piotnetgrid' ),
                'value'   => 'full',
                'options' => $image_sizes,
            ]
        );
        $this->add_responsive_control(
            'gallery_aspect_ratio',
            [
                'label' => __( 'Gallery Aspect Ratio', 'pafe' ),
                'label_block' => true,
                'type' => 'number',
                'description' => 'Aspect Ratio = Height / Width * 100. E.g Width = 100, Height = 100 => Ratio = 100; Width = 100, Height = 50 => Ratio = 50',
                'value'   => '',
                'render_type' => 'both',
                'selectors' => [
                    '{{WRAPPER}} .piotnetgrid-gallery__aspect-ratio:after' => 'padding-top: {{VALUE}}%;',
                ],
            ]
        );
    }

    private function add_layout_controls() {

        $this->add_control(
            'grid_gallery_carousel_hide_on_desktop',
            [
                'label' => __( 'Disable on Desktop', 'piotnetgrid' ),
                'type' => 'switch',
                'default' => '',
                'label_on' => 'Yes',
                'label_off' => 'No',
                'return_value' => 'yes',
            ]
        );
        $this->add_control(
            'grid_gallery_carousel_hide_on_tablet',
            [
                'label' => __( 'Disable on Tablet', 'piotnetgrid' ),
                'type' => 'switch',
                'default' => '',
                'label_on' => 'Yes',
                'label_off' => 'No',
                'return_value' => 'yes',
            ]
        );

        $this->add_control(
            'grid_gallery_carousel_hide_on_mobile',
            [
                'label' => __( 'Disable on Mobile', 'piotnetgrid' ),
                'type' => 'switch',
                'default' => '',
                'label_on' => 'Yes',
                'label_off' => 'No',
                'return_value' => 'yes',
            ]
        );

        $this->add_responsive_control(
            'grid_gallery_carousel_slides_to_show',
            [
                'label'       => __( 'Slides to Show', 'piotnetgrid' ),
                'type'      => 'number',
                'label_block' => true,
                'description' => __( 'Set the value with decimal places like 2.3 or 2.5 to show half of the next slide', 'piotnetgrid' ),
                'default'   => 1,
                'min'       => 1,
            ]
        );
        $this->add_responsive_control(
            'grid_gallery_carousel_slides_to_scroll',
            [
                'label'       => __( 'Slides To Scroll', 'piotnetgrid' ),
                'type'      => 'number',
                'default'   => 1,
                'label_block' => true,
                'min'       => 1,
            ]
        );
        $this->add_responsive_control(
            'grid_gallery_carousel_slides_spacing',
            [
                'label'       => __( 'Slides Spacing', 'piotnetgrid' ),
                'type'        => 'slider',
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => 1,
                        'max' => 200,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-slide' => 'margin: 0 {{SIZE}}{{UNIT}};',
                ],
            ]
        );
        $this->add_control(
            'grid_gallery_carousel_autoplay',
            [
                'label'       => __( 'Autoplay', 'piotnetgrid' ),
                'type'        => 'select',
                'options'     => [
                    'yes'=>'Yes',
                    'no'=>'No',
                ],
                'value'   => 'no',
            ]
        );
        $this->add_control(
            'grid_gallery_carousel_pause_on_hover',
            [
                'label'       => __( 'Pause on Hover', 'piotnetgrid' ),
                'type'        => 'select',
                'options'     => [
                    'yes'=>'Yes',
                    'no'=>'No',
                ],
                'value'   => 'yes',
            ]
        );

        $this->add_control(
            'grid_gallery_carousel_infinite_loop',
            [
                'label'       => __( 'Infinite Loop', 'piotnetgrid' ),
                'type'        => 'select',
                'options'     => [
                    'yes'=>'Yes',
                    'no'=>'No',
                ],
                'value'   => 'no',
            ]
        );
        $this->add_control(
            'grid_gallery_carousel_autoplay_speed',
            [
                'label'       => __( 'Autoplay Speed', 'piotnetgrid' ),
                'type'        => 'number',
                'value'       => 5000,
            ]
        );
        $this->add_control(
            'grid_gallery_carousel_animation_speed',
            [
                'label'       => __( 'Animation Speed', 'piotnetgrid' ),
                'type'        => 'number',
                'value'       => 500,
            ]
        );
        $this->add_control(
            'grid_gallery_carousel_navigation',
            [
                'label' => __( 'Navigation', 'piotnetgrid' ),
                'type' => 'select',
                'value' => 'both',
                'options' => [
                    'both' => __( 'Arrows and Dots', 'piotnetgrid' ),
                    'arrows' => __( 'Arrows', 'piotnetgrid' ),
                    'dots' => __( 'Dots', 'piotnetgrid' ),
                    'none' => __( 'None', 'piotnetgrid' ),
                ],
            ]
        );

        $this->add_control(
            'grid_gallery_carousel_custom_next_prev_button',
            [
                'label' => __( 'Custom Next / Previous Button', 'piotnetgrid' ),
                'label_block' => true,
                'type' => 'html',
                'raw' => '<a href="https://piotnetgrid.com/docs/create-a-grid/#carousel-custom-next-previous-button" target="_blank">Document here</a>',
            ]
        );
    }


    private function add_gallery_style_controls() {

        $this->add_control(
            'gallery_border_style',
            [
                'type'      => 'select',
                'label'     => __( 'Border Type', 'piotnetgrid' ),
                'value'     => '',
                'options'   => [
                    ''       => 'None',
                    'solid'  => 'Solid',
                    'double' => 'Double',
                    'dotted' => 'Dotted',
                    'dashed' => 'Dashed',
                    'groove' => 'Groove',
                ],
                'selectors' => [
                    '{{WRAPPER}} .piotnetgrid-gallery__item img' => 'border-style:{{VALUE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'gallery_border_color',
            [
                'type'        => 'color',
                'label'       => __( 'Border Color', 'piotnetgrid' ),
                'value'       => '',
                'label_block' => true,
                'selectors'   => [
                    '{{WRAPPER}} .piotnetgrid-gallery__item img' => 'border-color: {{VALUE}};',
                ],
                'conditions'  => [
                    [
                        'name'     => 'gallery_border_style',
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );

        $this->add_responsive_control(
            'gallery_border_width',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Border Width', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px', '%', 'em' ],
                'selectors'   => [
                    '{{WRAPPER}} .piotnetgrid-gallery__item img' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'  => [
                    [
                        'name'     => 'gallery_border_style',
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );

        $this->add_responsive_control(
            'gallery_border_radius',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Border Radius', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px', '%', 'em' ],
                'selectors'   => [
                    '{{WRAPPER}} .piotnetgrid-gallery img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
    }

    private function add_arrows_style_controls() {
        $this->add_control(
            'piotnet_grid_slick_arrows_color',
            [
                'type'      => 'color',
                'label'     => __( 'Color', 'piotnetgrid' ),
                'selectors' => [
                    '{{WRAPPER}} .slick-next:before, {{WRAPPER}} .slick-prev:before' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_control(
            'piotnet_grid_slick_arrows_color_background',
            [
                'type'      => 'color',
                'label'     => __( 'Background Color', 'piotnetgrid' ),
                'selectors' => [
                    '{{WRAPPER}} .slick-arrow' => 'background-color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_size',
            [
                'type'        => 'slider',
                'label'       => __( 'Size', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => 1,
                        'max' => 300,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-next:before, {{WRAPPER}} .slick-prev:before' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_padding',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Padding', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-arrow' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_horizontal_position',
            [
                'type'        => 'slider',
                'label'       => __( 'Horizontal Position', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => -100,
                        'max' => 100,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-prev' => 'left: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .slick-next' => 'right: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'piotnet_grid_slick_arrows_border_style',
            [
                'type'      => 'select',
                'label'     => __( 'Border Type', 'piotnetgrid' ),
                'value'     => '',
                'options'   => [
                    ''       => 'None',
                    'solid'  => 'Solid',
                    'double' => 'Double',
                    'dotted' => 'Dotted',
                    'dashed' => 'Dashed',
                    'groove' => 'Groove',
                ],
                'selectors' => [
                    '{{WRAPPER}} .slick-arrow' => 'border-style:{{VALUE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_border_color',
            [
                'type'        => 'color',
                'label'       => __( 'Border Color', 'piotnetgrid' ),
                'value'       => '',
                'label_block' => true,
                'selectors'   => [
                    '{{WRAPPER}} .slick-arrow' => 'border-color: {{VALUE}};',
                ],
                'conditions'  => [
                    [
                        'name'     => 'piotnet_grid_slick_arrows_border_style',
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_border_width',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Border Width', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px', '%', 'em' ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-arrow' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'  => [
                    [
                        'name'     => 'piotnet_grid_slick_arrows_border_style',
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_border_radius',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Border Radius', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px', '%', 'em' ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-arrow' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_opacity',
            [
                'type'        => 'slider',
                'label'       => __( 'Opacity', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1,
                        'step' => 0.1,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-arrow' => 'opacity:{{SIZE}};',
                ],
            ]
        );
    }

    private function add_dots_style_controls() {
        $this->add_control(
            'piotnet_grid_slick_dots_color',
            [
                'type'      => 'color',
                'label'     => __( 'Background Color', 'piotnetgrid' ),
                'selectors' => [
                    '{{WRAPPER}} .slick-dots li button:before' => 'color: {{VALUE}}',
                ],
            ]
        );
        $this->add_control(
            'piotnet_grid_slick_dots_color_active',
            [
                'type'      => 'color',
                'label'     => __( 'Background Color Active', 'piotnetgrid' ),
                'selectors' => [
                    '{{WRAPPER}} .slick-dots li.slick-active button:before' => 'color: {{VALUE}}!important',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_opacity',
            [
                'type'        => 'slider',
                'label'       => __( 'Opacity', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1,
                        'step' => 0.1,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-dots li button:before' => 'opacity:{{SIZE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_opacity_active',
            [
                'type'        => 'slider',
                'label'       => __( 'Opacity Active', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1,
                        'step' => 0.1,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-dots li.slick-active button:before' => 'opacity:{{SIZE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_size',
            [
                'type'        => 'slider',
                'label'       => __( 'Size', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-dots li button:before' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_spacing',
            [
                'type'        => 'slider',
                'label'       => __( 'Spacing', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-dots li' => 'margin: 0 {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_bottom_position',
            [
                'label'       => __( 'Bottom Position', 'piotnetgrid' ),
                'type'        => 'text',
                'label_block' => true,
                'description' => 'E.g 20px, -20px, 10%',
                'selectors' => [
                    '{{WRAPPER}} .slick-dots' => 'bottom: {{VALUE}};',
                ],
            ]
        );
    }


    public function render($editor = false, $args_additional = []) {
        $settings = $this->settings;
        $demo_image_url = plugins_url() . '/piotnetgrid/assets/images/placeholder-image.png';
        $this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-gallery' );
        if (!empty($settings['gallery_aspect_ratio_responsive_desktop'])) {
            $this->add_render_attribute( 'wrapper-aspect-ratio', 'class', 'piotnetgrid-gallery__aspect-ratio' );
        } else {
            $this->add_render_attribute( 'wrapper-aspect-ratio', 'class', 'piotnetgrid-gallery__content' );
        }
        $this->add_render_attribute( 'wrapper-gallery', 'class', 'piotnetgrid-gallery__inner' );
        $this->add_render_attribute( 'wrapper-gallery', 'data-piotnetgrid-gallery', '' );

        $post_id = !empty($GLOBALS['preview_card_post_id']) ? $GLOBALS['preview_card_post_id'] : get_the_ID();
        $image_size = $settings['gallery_image_size'];
        $image_links = array();
        if ( !$editor || !empty($GLOBALS['preview_card_post_id']) ) {
            switch ($settings['gallery_dynamic_data']) {
                case 'acf_field':
                    if ( function_exists( 'get_field' ) ) {
                        $image_urls = get_field($settings['gallery_dynamic_data_field_name'], $post_id);
                        foreach ( $image_urls as $image_url ) {
                            $value = wp_get_attachment_image_src( attachment_url_to_postid( $image_url ), $image_size );
                            $image_links[] =  $value[0];
                        }
                    }
                break;
                case 'metabox_field':
                    if ( function_exists( 'rwmb_get_value' ) ) {
                        $image_urls = rwmb_get_value($settings['gallery_dynamic_data_field_name'], array( 'size' => 'large' ), $post_id);
                        foreach ( $image_urls as $image_url ) {
                            $value = wp_get_attachment_image_src(  attachment_url_to_postid( $image_url['file'] ), $image_size );
                            $image_links[] =  $value[0];
                        }
                    }
                break;
                case 'pods_field':
                    if ( function_exists( 'pods_field' ) ) {
                        $image_urls = pods_field(get_post_type(), $post_id, $settings['gallery_dynamic_data_field_name'], false);
                        foreach ($image_urls as $image_url) {
                            $value = wp_get_attachment_image_src($image_url, $image_size);
                            $image_links[] = $value[0];
                        }
                    }
                break;
                case 'toolset_field':
                    $image_urls = get_post_meta( $post_id, 'wpcf-' . $settings['gallery_dynamic_data_field_name'], false );
                    foreach ($image_urls as $image_url) {
                        $value = wp_get_attachment_image_src( attachment_url_to_postid( $image_url ), $image_size );
                        $image_links[] = $value[0];
                    }
                break;
                case 'jetengine_field':
                    $image_array = get_post_meta( $post_id, $settings['gallery_dynamic_data_field_name'], false );
                    $image_urls = explode(',',$image_array[0]);
                    foreach ($image_urls as $image_url) {
                        $value = wp_get_attachment_image_src( $image_url, $image_size );
                        $image_links[] = $value[0];
                    }
                break;
                case 'woocommerce_gallery':
                    global $product;
                    if (!$product) {
                        $product = wc_get_product( $post_id );
                    }
                    if (!$product) {
                        return;
                    }
                    $product_featured_image = wp_get_attachment_image_src( get_post_thumbnail_id( $post_id ), 'full' );
                    $image_links[] = $product_featured_image[0];
                    $attachment_ids = $product->get_gallery_image_ids();
                    if (!empty($attachment_ids) && is_array( $attachment_ids )) {
                        foreach( $attachment_ids as $attachment_id ) {
                            $image_links[] = wp_get_attachment_url( $attachment_id );
                        }
                    }
                break;
            }
        }

        $grid_carousel_navigation = !empty($settings['grid_gallery_carousel_navigation']) ? $settings['grid_gallery_carousel_navigation'] : 'both';
        $show_dots = ( in_array( $grid_carousel_navigation, [ 'dots', 'both' ] ) );
        $show_arrows = ( in_array( $grid_carousel_navigation, [ 'arrows', 'both' ] ) );
        $slick_slides_to_show_desktop = (!empty($settings['grid_gallery_carousel_slides_to_show_responsive_desktop'])) ? floatval( $settings['grid_gallery_carousel_slides_to_show_responsive_desktop'] ) : '3';
        $slick_slides_to_show_tablet = (!empty($settings['grid_gallery_carousel_slides_to_show_responsive_tablet'])) ? floatval( $settings['grid_gallery_carousel_slides_to_show_responsive_tablet'] ) : '2';
        $slick_slides_to_show_mobile = (!empty($settings['grid_gallery_carousel_slides_to_show_responsive_mobile'])) ? floatval( $settings['grid_gallery_carousel_slides_to_show_responsive_mobile'] ) : '1';

        $slick_slides_to_scroll_desktop = (!empty($settings['grid_gallery_carousel_slides_to_scroll_responsive_desktop'])) ? floatval( $settings['grid_gallery_carousel_slides_to_scroll_responsive_desktop'] ) : '3';
        $slick_slides_to_scroll_tablet = (!empty($settings['grid_gallery_carousel_slides_to_scroll_responsive_tablet'])) ? floatval( $settings['grid_gallery_carousel_slides_to_scroll_responsive_tablet'] ) : '2';
        $slick_slides_to_scroll_mobile = (!empty($settings['grid_gallery_carousel_slides_to_scroll_responsive_mobile'])) ? floatval( $settings['grid_gallery_carousel_slides_to_scroll_responsive_mobile'] ) : '1';

        if ( !empty($settings['grid_gallery_carousel_hide_on_desktop']) ) {
            $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-gallery-carousel-disable-on-desktop', $settings['grid_gallery_carousel_hide_on_desktop'] );
        }
        if ( !empty($settings['grid_gallery_carousel_hide_on_tablet']) ) {
            $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-gallery-carousel-disable-on-tablet', $settings['grid_gallery_carousel_hide_on_tablet'] );
        }
        if ( !empty($settings['grid_gallery_carousel_hide_on_mobile']) ) {
            $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-gallery-carousel-disable-on-mobile', $settings['grid_gallery_carousel_hide_on_mobile'] );
        }

        $slick_options = [
            'autoplay' => ( 'yes' === $settings['grid_gallery_carousel_autoplay'] ),
            'infinite' => ( 'yes' === $settings['grid_gallery_carousel_infinite_loop'] ),
            'pauseOnHover' => ( 'yes' === $settings['grid_gallery_carousel_pause_on_hover'] ),
            'arrows' => $show_arrows,
            'dots' =>   $show_dots,
            'slidesToShow' => $slick_slides_to_show_desktop,
            'slidesToScroll' => $slick_slides_to_scroll_desktop,
            'responsive'=> [
                [
                    'breakpoint' => '1170',
                    'settings' => [
                        'slidesToShow' => $slick_slides_to_show_desktop,
                        'slidesToScroll' => $slick_slides_to_scroll_desktop,
                    ]
                ],
                [
                    'breakpoint' => '991',
                    'settings' => [
                        'slidesToShow' => $slick_slides_to_show_tablet,
                        'slidesToScroll' => $slick_slides_to_scroll_tablet,

                    ]
                ],
                [
                    'breakpoint' => '767',
                    'settings' => [
                        'slidesToShow' => $slick_slides_to_show_mobile,
                        'slidesToScroll' => $slick_slides_to_scroll_mobile,
                    ]
                ],
            ],

        ];

        $slick_options['autoplaySpeed'] = (!empty($settings['grid_gallery_carousel_autoplay_speed'])) ? absint( $settings['grid_gallery_carousel_autoplay_speed'] ) : '3000';
        $slick_options['speed'] = (!empty($settings['grid_gallery_carousel_animation_speed'])) ? absint( $settings['grid_gallery_carousel_animation_speed'] ) : '300';
        wp_enqueue_script( $this->slug . '-slick-script' );
        wp_enqueue_style( $this->slug . '-slick-style' );
        $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-gallery-carousel', json_encode($slick_options) );

        ?>
        <?php if ( !$editor || !empty($GLOBALS['preview_card_post_id']) ) : ?>
            <div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
                <div <?php echo $this->get_render_attribute_string( 'wrapper-gallery' ); ?>>
                <?php foreach ( $image_links as $image_link ) : ?>
                <div <?php echo $this->get_render_attribute_string( 'wrapper-aspect-ratio' ); ?>>
                    <img src="<?php echo $image_link; ?>" alt="">
                </div>
                <?php endforeach; ?>
                </div>
            </div>
        <?php else : ?>
            <div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
                <div <?php echo $this->get_render_attribute_string( 'wrapper-gallery' ); ?>>
                   <img src="<?php echo $demo_image_url; ?>" alt="">
                   <img src="<?php echo $demo_image_url; ?>" alt="">
                   <img src="<?php echo $demo_image_url; ?>" alt="">
                   <img src="<?php echo $demo_image_url; ?>" alt="">
                </div>
            </div>
        <?php endif; ?>
        <?php
    }

    public function live_preview() {

    }

}
