<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Grid extends Base_Widget_piotnetgrid {
	public function get_type() {
		return 'grid';
	}

	public function get_class_name() {
		return 'piotnetgrid_Grid';
	}

	public function get_title() {
		return 'Grid';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-grid.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'grid' ];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'grid_settings_section', 'Query' );
		$this->add_setting_controls();

		$this->start_section( 'grid_layout_section', 'Layout' );
		$this->add_layout_controls();

		$this->start_section( 'grid_animation_section', 'Card Animation' );
		$this->add_animation_controls();

		$this->start_section(
			'grid_cache_section',
			'Cache',
			[
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'source',
                            'operator' => '!=',
                            'value'    => 'taxonomy_terms',
                        ],
                    ],
                ],
			]
		);
		$this->add_cache_controls();

		$this->start_section(
			'grid_dots_styles_section',
			'Carousel Dots Style',
			[
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
			]
		);
		$this->add_dots_style_controls();

		$this->start_section(
			'grid_arrows_styles_section',
			'Carousel Arrows Style',
			[
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
			]
		);
        $this->add_arrows_style_controls();

        $this->start_section(
        	'no_results_found_section',
        	'No Results Found Alert',
        	[
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'source',
                            'operator' => '!=',
                            'value'    => 'taxonomy_terms',
                        ],
                    ],
                ],
			]
        );
		$this->add_no_results_found_controls();

		$this->start_section(
			'recently_viewed_posts_title_style_section',
			'Recently Viewed Posts Title Style',
			[
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'recently_viewed_posts',
                            'operator' => '==',
                            'value'    => 'yes',
                        ],
                        [
                            'name'     => 'source',
                            'operator' => '!=',
                            'value'    => 'taxonomy_terms',
                        ],
                    ],
                ],
			]
		);

		$this->add_recently_viewed_posts_title_style_controls();

        $this->add_advanced_tab();

		return $this->structure;
	}
	
	private function add_setting_controls() {

		$this->add_control(
			'source',
			[
				'type'        => 'select',
				'label'       => __( 'Source', 'piotnetgrid' ),
				'label_block' => true,
				'options' => [
					'' => __( 'Posts, Product, Custom Post Type', 'piotnetgrid' ),
					'taxonomy_terms' => __( 'Taxonomy Terms', 'piotnetgrid' ),
				],
			]
		);

		$card_templates = [
			'' => __( 'Select', 'piotnetgrid' ),
		];

		$card_templates_query = new WP_Query(
			array(
				'post_type' => 'piotnetgrid-card',
				'posts_per_page' => -1,
			)
		);

		if($card_templates_query->have_posts()) : while($card_templates_query->have_posts()) : $card_templates_query->the_post();
			$card_templates[ get_the_ID() ] = html_entity_decode( get_the_title() );
		endwhile; endif; wp_reset_postdata();

		$this->add_control(
			'card_template',
			[
				'type'        => 'select',
				'label'       => __( 'Card Template', 'piotnetgrid' ),
				'label_block' => true,
				'options' => $card_templates,
				'description_has_value' => '<a href="' . admin_url() . 'admin.php?page=piotnetgrid&post=' . '{{VALUE}}" target="_blank">Edit this card template</a>',
			]
		);

		$this->add_control(
			'add_post_classes_to_the_card',
			[
				'label' => __( 'Add post classes to the card', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '=',
                        'value'    => '',
                    ],
				],
			]
		);

		$post_types = get_post_types( [], 'objects' );
		$taxonomies = [];

		foreach ( $post_types as $post_type ) {
			$taxonomies_of_post_type = get_object_taxonomies( $post_type->name, 'objects' );
			foreach ($taxonomies_of_post_type as $taxonomy) {
				$taxonomies[$taxonomy->name] = $taxonomy->label . ' (' . $post_type->name . ')';
			}
		}

		$this->add_control(
			'terms_grid_taxonomy',
			[
				'type'        => 'select',
				'label'       => __( 'Taxonomy', 'piotnetgrid' ),
				'label_block' => true,
				'options'     => $taxonomies,
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '=',
                        'value'    => 'taxonomy_terms',
                    ],
				],
			]
		);

		$this->add_control(
			'terms_grid_parent',
			[
				'type'        => 'select2',
				'label'       => __( 'Parent Term (Select only one term)', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'terms',
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '=',
                        'value'    => 'taxonomy_terms',
                    ],
				],
			]
		);

		$this->add_control(
			'terms_grid_include',
			[
				'type'        => 'select2',
				'label'       => __( 'Include Taxonomy Terms', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'terms',
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '=',
                        'value'    => 'taxonomy_terms',
                    ],
				],
			]
		);

		$this->add_control(
			'terms_grid_exclude',
			[
				'type'        => 'select2',
				'label'       => __( 'Exclude Taxonomy Terms', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'terms',
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '=',
                        'value'    => 'taxonomy_terms',
                    ],
				],
			]
		);

		$this->add_control(
			'terms_grid_hide_empty',
			[
				'label' => __( 'Hide Empty', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '=',
                        'value'    => 'taxonomy_terms',
                    ],
				],
			]
		);

		$this->add_control(
			'terms_grid_number',
			[
				'label' => __( 'Maximum number of terms', 'piotnetgrid' ),
				'type' => 'number',
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '=',
                        'value'    => 'taxonomy_terms',
                    ],
				],
			]
		);

		$this->add_control(
			'current_query_enable',
			[
				'label' => __( 'Current Query', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '!=',
                        'value'    => 'taxonomy_terms',
                    ],
				],
			]
		);

		$this->add_control(
			'custom_query_enable',
			[
				'label' => __( 'Custom Query', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'custom_query',
			[
				'label' => __( 'Custom Query', 'piotnetgrid' ),
				'label_block' => true,
				'type' => 'textarea',
				'default' => '',
				'description' => 'https://developer.wordpress.org/reference/classes/wp_query/',
				'placeholder' => 'array("post_type" => "post")',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'wishlist_posts_enable',
			[
				'label' => __( 'Wishlist Posts', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'recently_viewed_posts',
			[
				'label' => __( 'Recently Viewed Posts', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'recently_viewed_posts_title',
			[
				'label' => __( 'Recently Viewed Posts Title', 'piotnetgrid' ),
				'label_block' => true,
				'type' => 'text',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'recently_viewed_posts',
							'operator' => '=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$post_types       = get_post_types( [], 'objects' );
		$post_types_array = [];
		$post_types_array['any'] = __( 'Any', 'piotnetgrid' );

		foreach ( $post_types as $post_type ) {
			if ($post_type->public) {
				$post_types_array[ $post_type->name ] = $post_type->label;
			}
		}

		$this->add_control(
			'post_type',
			[
				'label'   => __( 'Post Type', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => $post_types_array,
				'value'   => 'post',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$post_statuses = get_post_statuses();

		$this->add_control(
			'post_status',
			[
				'type'        => 'select2',
				'label'       => __( 'Post Status', 'piotnetgrid' ),
				'label_block' => true,
				'options' => $post_statuses,
				'value' => 'publish',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'include_posts',
			[
				'type'        => 'select2',
				'label'       => __( 'Include Posts', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'posts',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'exclude_posts',
			[
				'type'        => 'select2',
				'label'       => __( 'Exclude Posts', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'posts',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'include_terms',
			[
				'type'        => 'select2',
				'label'       => __( 'Include Taxonomy Terms', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'terms',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'include_terms_relation',
			[
				'type'        => 'select',
				'label'       => __( 'Include Taxonomy Terms Relation', 'piotnetgrid' ),
				'label_block' => true,
				'options'     => [
					'OR' => __( 'OR', 'piotnetgrid' ),
					'AND' => __( 'AND', 'piotnetgrid' ),
				],
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'exclude_terms',
			[
				'type'        => 'select2',
				'label'       => __( 'Exclude Taxonomy Terms', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'terms',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'posts_per_page',
			[
				'type'        => 'text',
				'label'       => __( 'Posts Per Page', 'piotnetgrid' ),
				'value'       => '10',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'offset',
			[
				'type'        => 'number',
				'label'       => __( 'First posts skipping quantity (Offset)', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => '',
				'min'		  => 0,
				'description' => __( 'Use this setting to skip over the first posts (e.g. "2" to skip over 2 posts).', 'piotnetgrid' ),
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'offset_first',
			[
				'type'        => 'number',
				'label'       => __( 'First posts skipping quantity on each page of the pagination', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => '',
				'min'		  => 0,
				'description' => __( 'You should use it if you have many grid layouts on this grid', 'piotnetgrid' ),
				'conditions' => [
					'terms' => [
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'offset_last',
			[
				'type'        => 'number',
				'label'       => __( 'Last posts skipping quantity on each page of the pagination', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => '',
				'min'		  => 0,
				'description' => __( 'You should use it if you have many grid layouts on this grid', 'piotnetgrid' ),
				'conditions' => [
					'terms' => [
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'ignore_sticky_posts',
			[
				'label' => __( 'Ignore Sticky Posts', 'piotnetgrid' ),
				'type' => 'switch',
				'value' => 'yes',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		$this->add_control(
			'meta_query_relation',
			[
				'label'   => __( 'Custom Fields (Meta) Relation', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'AND' => __( 'AND', 'piotnetgrid' ),
					'OR'  => __( 'OR', 'piotnetgrid' ),
				],
				'value'   => 'AND',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
					],
				],
			]
		);

		//repeater

		$this->new_group_controls();

		$this->add_control(
			'meta_key',
			[
				'label'   => __( 'Field Name (Key)', 'piotnetgrid' ),
				'label_block' => true,
				'type'    => 'text',
			]
		);

		$this->add_control(
			'meta_type',
			[
				'label'   => __( 'Field Type', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'CHAR' => __( 'Character', 'piotnetgrid' ),
					'NUMERIC'  => __( 'Numberic', 'piotnetgrid' ),
					'BINARY' => __( 'Binary', 'piotnetgrid' ),
					'DATE'  => __( 'Date', 'piotnetgrid' ),
					'DATETIME' => __( 'Date Time', 'piotnetgrid' ),
					'TIME'  => __( 'Time', 'piotnetgrid' ),
					'DECIMAL' => __( 'Decimal', 'piotnetgrid' ),
					'SIGNED'  => __( 'Signed', 'piotnetgrid' ),
					'UNSIGNED'  => __( 'Unsigned', 'piotnetgrid' ),
				],
				'value'   => 'CHAR',
			]
		);

		$this->add_control(
			'meta_value',
			[
				'label'   => __( 'Field Value', 'piotnetgrid' ),
				'label_block' => true,
				'type'    => 'text',
			]
		);

		$this->add_control(
			'meta_compare',
			[
				'label'   => __( 'Compare', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'=' => __( '=', 'piotnetgrid' ),
					'!='  => __( '!=', 'piotnetgrid' ),
					'>' => __( '>', 'piotnetgrid' ),
					'>='  => __( '>=', 'piotnetgrid' ),
					'<' => __( '<', 'piotnetgrid' ),
					'<='  => __( '<=', 'piotnetgrid' ),
					'LIKE' => __( 'Like', 'piotnetgrid' ),
					'NOT LIKE'  => __( 'Not Like', 'piotnetgrid' ),
					'IN'  => __( 'In', 'piotnetgrid' ),
					'NOT IN'  => __( 'Not In', 'piotnetgrid' ),
					'BETWEEN'  => __( 'Between', 'piotnetgrid' ),
					'NOT BETWEEN'  => __( 'Not Between', 'piotnetgrid' ),
					'EXISTS'  => __( 'Exists', 'piotnetgrid' ),
					'NOT EXISTS'  => __( 'Not Exists', 'piotnetgrid' ),
				],
				'value'   => '=',
			]
		);

        $this->add_control(
            'repeater_id',
            [
                'type' => 'hidden',
            ],
            [
                'overwrite' => 'true',
            ]
        );

		$repeater_items = $this->get_group_controls();

		$this->new_group_controls();

		$this->add_control(
			'',
			[
				'type'           => 'repeater-item',
				'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
				'controls'       => $repeater_items,
				'controls_query' => '.piotnetgrid-control-repeater-field',
			]
		);

		$repeater_list = $this->get_group_controls();

		$this->add_control(
			'meta_query_list',
			[
				'type'           => 'repeater',
				'label'          => __( 'Custom Fields (Meta)', 'piotnetgrid' ),
				'value'          => '',
				'label_block'    => true,
				'add_label'      => __( 'Add Item', 'piotnetgrid' ),
				'controls'       => $repeater_list,
				'controls_query' => '.piotnetgrid-control-repeater-list',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
					],
				],
			]
		);

		//end repeater

		//repeater

		$this->add_control(
			'terms_grid_orderby',
			[
				'label' => __( 'Order By', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'' => __( 'None', 'piotnetgrid' ),
					'name'  => __( 'Name', 'piotnetgrid' ),
					'slug'  => __( 'Slug', 'piotnetgrid' ),
					'term_group'  => __( 'Term Group', 'piotnetgrid' ),
					'term_id'  => __( 'Term ID', 'piotnetgrid' ),
					'id'  => __( 'ID', 'piotnetgrid' ),
					'description'  => __( 'Description', 'piotnetgrid' ),
					'parent'  => __( 'Parent', 'piotnetgrid' ),
					'term_order'  => __( 'Term Order', 'piotnetgrid' ),
					'count'  => __( 'Count', 'piotnetgrid' ),
					'include'  => __( 'Include', 'piotnetgrid' ),
					'meta_value'  => __( 'Meta Value', 'piotnetgrid' ),
					'meta_value_num'  => __( 'Meta Value Num', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '=',
                        'value'    => 'taxonomy_terms',
                    ],
				],
			]
		);

		$this->add_control(
			'terms_grid_order',
			[
				'label'   => __( 'Order', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'ASC'  => __( 'Ascending', 'piotnetgrid' ),
					'DESC' => __( 'Descending', 'piotnetgrid' ),
				],
				'conditions'  => [
					[
                        'name'     => 'source',
                        'operator' => '=',
                        'value'    => 'taxonomy_terms',
                    ],
				],
			]
		);

		$this->new_group_controls();

		$this->add_control(
			'order_by',
			[
				'label'   => __( 'Order By', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'' => __( 'None', 'piotnetgrid' ),
					'ID'  => __( 'Post ID', 'piotnetgrid' ),
					'author'  => __( 'Author', 'piotnetgrid' ),
					'title'  => __( 'Title', 'piotnetgrid' ),
					'name'  => __( 'Post Name (Post Slug)', 'piotnetgrid' ),
					'type'  => __( 'Post Type', 'piotnetgrid' ),
					'date'  => __( 'Date', 'piotnetgrid' ),
					'modified'  => __( 'Last Modified Date', 'piotnetgrid' ),
					'parent'  => __( 'Post/Page Parent ID', 'piotnetgrid' ),
					'rand'  => __( 'Random', 'piotnetgrid' ),
					'comment_count'  => __( 'Number of Comments', 'piotnetgrid' ),
					'relevance'  => __( 'Relevance', 'piotnetgrid' ),
					'menu_order'  => __( 'Menu Order', 'piotnetgrid' ),
					'meta_value'  => __( 'Custom Field', 'piotnetgrid' ),
					'meta_value_num'  => __( 'Numeric Custom Field', 'piotnetgrid' ),
					'post__in'  => __( 'List of Post IDs', 'piotnetgrid' ),
					'post_name__in'  => __( 'List of Post Names', 'piotnetgrid' ),
					'post_parent__in'  => __( 'List of Post Parent IDs', 'piotnetgrid' ),
				],
				'value'   => '',
			]
		);

		$this->add_control(
			'custom_field_name',
			[
				'label'   => __( 'Custom Field Name', 'piotnetgrid' ),
				'type'    => 'text',
				'value'   => '',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'order_by',
							'operator' => 'in',
							'value'    => ['meta_value', 'meta_value_num'],
						],
					],
				],
			]
		);

		$this->add_control(
			'order',
			[
				'label'   => __( 'Order', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'DESC' => __( 'Descending', 'piotnetgrid' ),
					'ASC'  => __( 'Ascending', 'piotnetgrid' ),
				],
				'value'   => 'DESC',
			]
		);

        $this->add_control(
            'repeater_id',
            [
                'type' => 'hidden',
            ],
            [
                'overwrite' => 'true',
            ]
        );

		$repeater_items = $this->get_group_controls();

		$this->new_group_controls();

		$this->add_control(
			'',
			[
				'type'           => 'repeater-item',
				'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
				'controls'       => $repeater_items,
				'controls_query' => '.piotnetgrid-control-repeater-field',
			]
		);

		$repeater_list = $this->get_group_controls();

		$this->add_control(
			'order_list',
			[
				'type'           => 'repeater',
				'label'          => __( 'Order List', 'piotnetgrid' ),
				'value'          => '',
				'label_block'    => true,
				'add_label'      => __( 'Add Item', 'piotnetgrid' ),
				'controls'       => $repeater_list,
				'controls_query' => '.piotnetgrid-control-repeater-list',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'custom_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
							'name'     => 'current_query_enable',
							'operator' => '!=',
							'value'    => 'yes',
						],
						[
	                        'name'     => 'source',
	                        'operator' => '!=',
	                        'value'    => 'taxonomy_terms',
	                    ],
					],
				],
			]
		);

		//end repeater

	}

    private function add_animation_controls() {
        $this->add_control(
            'grid_animation_enable',
            [
                'label' => __( 'Animation Enable', 'piotnetgrid' ),
                'type' => 'switch',
                'default' => '',
                'label_on' => 'Yes',
                'label_off' => 'No',
                'return_value' => 'yes',
            ]
        );
        $this->add_control(
            'grid_animation_effects',
            [
                'label'   => __( 'Effects', 'piotnetgrid' ),
                'type'    => 'select',
                'options' => [
                    'animate__fadeIn'  => __( 'Fade In', 'piotnetgrid' ),
                    'animate__fadeInDown'  => __( 'Fade In Down', 'piotnetgrid' ),
                    'animate__fadeInUp'  => __( 'Fade In Up', 'piotnetgrid' ),
                    'animate__fadeOutDown'  => __( 'Fade Out Down', 'piotnetgrid' ),
                    'animate__fadeOutUp'  => __( 'Fade Out Up', 'piotnetgrid' ),
                    'animate__zoomIn'  => __( 'Zoom In', 'piotnetgrid' ),
                    'animate__slideInDown'  => __( 'Slide In Down', 'piotnetgrid' ),
                    'animate__slideInUp'  => __( 'Slide In Up', 'piotnetgrid' ),
                    'animate__bounce' => __( 'Bounce', 'piotnetgrid' ),
                    'animate__bounceIn'  => __( 'Bounce In', 'piotnetgrid' ),
                ],
                'value'   => 'animate__fadeInUp',
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_animation_enable',
                            'operator' => '==',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );

        $this->add_control(
            'grid_animation_speed',
            [
                'label'   => __( 'Speed', 'piotnetgrid' ),
                'type'    => 'select',
                'options' => [
                    'animate__faster' => __( 'Faster', 'piotnetgrid' ),
                    'animate__fast'  => __( 'Fast', 'piotnetgrid' ),
                    'animate__slow'  => __( 'Slow', 'piotnetgrid' ),
                    'animate__slower'  => __( 'Slower', 'piotnetgrid' ),
                ],
                'value'   => 'animate__fast',
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_animation_enable',
                            'operator' => '==',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_control(
            'grid_animation_delay',
            [
                'label'   => __( 'Delay (ms)', 'piotnetgrid' ),
                'type'    => 'text',
                'value'   => '',
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_animation_enable',
                            'operator' => '==',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );

    }

	private function add_layout_controls() {

		$this->add_control(
			'grid_masonry_enable',
			[
				'label' => __( 'Masonry Enable', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
			]
		);

		$this->add_responsive_control(
			'grid_columns',
			[
				'type'        => 'number',
				'label'       => __( 'Columns', 'piotnetgrid' ),
				'value'       => 1,
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-grid__posts' => 'grid-template-columns: repeat({{VALUE}}, minmax(0, 1fr));',
				],
			]
		);

		$this->add_responsive_control(
			'grid_columns_gap',
			[
				'type'        => 'slider',
				'label'       => __( 'Columns Gap', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-grid__posts' => 'grid-column-gap: {{SIZE}}{{UNIT}};',
				],
            ]
		);

		$this->add_responsive_control(
			'grid_row_gap',
			[
				'type'        => 'slider',
				'label'       => __( 'Row Gap', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-grid__posts' => 'grid-row-gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'grid_carousel_enable',
			[
				'label' => __( 'Carousel Enable', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
			]
		);

        $this->add_control(
			'grid_carousel_hide_on_desktop',
			[
				'label' => __( 'Disable on Desktop', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
			]
		);
        $this->add_control(
			'grid_carousel_hide_on_tablet',
			[
				'label' => __( 'Disable on Tablet', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
			]
		);

        $this->add_control(
			'grid_carousel_hide_on_mobile',
			[
				'label' => __( 'Disable on Mobile', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
			]
		);

        $this->add_responsive_control(
            'grid_carousel_slides_to_show',
            [
                'label'       => __( 'Slides to Show', 'piotnetgrid' ),
                'type'      => 'number',
                'label_block' => true,
                'description' => __( 'Set the value with decimal places like 2.3 or 2.5 to show half of the next slide', 'piotnetgrid' ),
                'default'   => 1,
                'min'       => 1,
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_responsive_control(
            'grid_carousel_slides_to_scroll',
            [
                'label'       => __( 'Slides To Scroll', 'piotnetgrid' ),
                'type'      => 'number',
                'default'   => 1,
                'label_block' => true,
                'min'       => 1,
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_responsive_control(
            'grid_carousel_slides_spacing',
            [
                'label'       => __( 'Slides Spacing', 'piotnetgrid' ),
                'type'        => 'slider',
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => 1,
                        'max' => 200,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-slide' => 'margin: 0 {{SIZE}}{{UNIT}};',
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_control(
            'grid_carousel_autoplay',
            [
                'label'       => __( 'Autoplay', 'piotnetgrid' ),
                'type'        => 'select',
                'options'     => [
                    'yes'=>'Yes',
                    'no'=>'No',
                ],
                'value'   => 'no',
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_control(
            'grid_carousel_pause_on_hover',
            [
                'label'       => __( 'Pause on Hover', 'piotnetgrid' ),
                'type'        => 'select',
                'options'     => [
                    'yes'=>'Yes',
                    'no'=>'No',
                ],
                'value'   => 'yes',
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );

        $this->add_control(
            'grid_carousel_infinite_loop',
            [
                'label'       => __( 'Infinite Loop', 'piotnetgrid' ),
                'type'        => 'select',
                'options'     => [
                    'yes'=>'Yes',
                    'no'=>'No',
                ],
                'value'   => 'no',
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_control(
            'grid_carousel_autoplay_speed',
            [
                'label'       => __( 'Autoplay Speed', 'piotnetgrid' ),
                'type'        => 'number',
                'value'       => 5000,
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_control(
            'grid_carousel_animation_speed',
            [
                'label'       => __( 'Animation Speed', 'piotnetgrid' ),
                'type'        => 'number',
                'value'       => 500,
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_control(
            'grid_carousel_navigation',
            [
                'label' => __( 'Navigation', 'piotnetgrid' ),
                'type' => 'select',
                'value' => 'both',
                'options' => [
                    'both' => __( 'Arrows and Dots', 'piotnetgrid' ),
                    'arrows' => __( 'Arrows', 'piotnetgrid' ),
                    'dots' => __( 'Dots', 'piotnetgrid' ),
                    'none' => __( 'None', 'piotnetgrid' ),
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );

        $this->add_control(
            'grid_carousel_custom_next_prev_button',
            [
                'label' => __( 'Custom Next / Previous Button', 'piotnetgrid' ),
                'label_block' => true,
                'type' => 'html',
                'raw' => '<a href="https://piotnetgrid.com/docs/create-a-grid/#carousel-custom-next-previous-button" target="_blank">Document here</a>',
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'grid_carousel_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
	}

	private function add_cache_controls() {
		$this->add_control(
			'cache_enable',
			[
				'label' => __( 'Enable', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
			]
		);

		$this->add_control(
			'cache_boost_loading',
			[
				'label' => __( 'Boost Cache Loading', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'description' => __( 'Enable for fastest 40ms loading time in best conditions', 'piotnetgrid' ),
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'cache_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
			]
		);

		$this->add_control(
			'cache_expiration',
			[
				'label' => __( 'Expiration (seconds)', 'piotnetgrid' ),
				'type' => 'text',
				'value' => __( '259200', 'piotnetgrid' ),
				'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'cache_enable',
                            'operator' => '=',
                            'value'    => 'yes',
                        ],
                    ],
                ],
			]
		);
	}

	private function add_pagination_controls() {
		$this->add_control(
			'pagination',
			[
				'type'    => 'select',
				'label'   => __( 'Pagination', 'piotnetgrid' ),
				'options' => [
					'' => __( 'None', 'piotnetgrid' ),
					'numbers_previous_next' => __( 'Numbers + Previous/Next', 'piotnetgrid' ),
					'numbers' => __( 'Numbers', 'piotnetgrid' ),
					'previous_next' => __( 'Previous/Next', 'piotnetgrid' ),
					'load_more' => __( 'Load More', 'piotnetgrid' ),
				],
				'value' => 'numbers_previous_next',
			]
		);

		$this->add_control(
			'pagination_shorten',
			[
				'label' => __( 'Shorten', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'pagination',
							'operator' => 'in',
							'value'    => ['numbers', 'numbers_previous_next'],
						],
					],
				],
			]
		);

		$this->add_control(
			'pagination_previous_label',
			[
				'label' => __( 'Previous Label', 'piotnetgrid' ),
				'type' => 'text',
				'value' => __( 'Previous', 'piotnetgrid' ),
				'conditions' => [
					'terms' => [
						[
							'name'     => 'pagination',
							'operator' => 'in',
							'value'    => ['previous_next', 'numbers_previous_next'],
						],
					],
				],
			]
		);

		$this->add_control(
			'pagination_next_label',
			[
				'label' => __( 'Previous Label', 'piotnetgrid' ),
				'type' => 'text',
				'value' => __( 'Previous', 'piotnetgrid' ),
				'conditions' => [
					'terms' => [
						[
							'name'     => 'pagination',
							'operator' => 'in',
							'value'    => ['previous_next', 'numbers_previous_next'],
						],
					],
				],
			]
		);

		$this->add_control(
			'pagination_load_more_text',
			[
				'label' => __( 'Load more Text', 'piotnetgrid' ),
				'type' => 'text',
				'value' => __( 'Load more', 'piotnetgrid' ),
				'conditions' => [
					'terms' => [
						[
							'name'     => 'pagination',
							'operator' => '==',
							'value'    => 'load_more',
						],
					],
				],
			]
		);
	}

    private function add_arrows_style_controls() {
        $this->add_control(
            'piotnet_grid_slick_arrows_color',
            [
                'type'      => 'color',
                'label'     => __( 'Color', 'piotnetgrid' ),
                'selectors' => [
                    '{{WRAPPER}} .slick-next:before, {{WRAPPER}} .slick-prev:before' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_control(
            'piotnet_grid_slick_arrows_color_background',
            [
                'type'      => 'color',
                'label'     => __( 'Background Color', 'piotnetgrid' ),
                'selectors' => [
                    '{{WRAPPER}} .slick-arrow' => 'background-color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_size',
            [
                'type'        => 'slider',
                'label'       => __( 'Size', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => 1,
                        'max' => 300,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-next:before, {{WRAPPER}} .slick-prev:before' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_padding',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Padding', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-arrow' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_horizontal_position',
            [
                'type'        => 'slider',
                'label'       => __( 'Horizontal Position', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => -100,
                        'max' => 100,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-prev' => 'left: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .slick-next' => 'right: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
			'piotnet_grid_slick_arrows_border_style',
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					'{{WRAPPER}} .slick-arrow' => 'border-style:{{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'piotnet_grid_slick_arrows_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					'{{WRAPPER}} .slick-arrow' => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => 'piotnet_grid_slick_arrows_border_style',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);

		$this->add_responsive_control(
			'piotnet_grid_slick_arrows_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					'{{WRAPPER}} .slick-arrow' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => 'piotnet_grid_slick_arrows_border_style',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);

		$this->add_responsive_control(
			'piotnet_grid_slick_arrows_border_radius',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					'{{WRAPPER}} .slick-arrow' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

        $this->add_responsive_control(
            'piotnet_grid_slick_arrows_opacity',
            [
                'type'        => 'slider',
                'label'       => __( 'Opacity', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1,
                        'step' => 0.1,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-arrow' => 'opacity:{{SIZE}};',
                ],
            ]
        );
    }

    private function add_dots_style_controls() {
        $this->add_control(
            'piotnet_grid_slick_dots_color',
            [
                'type'      => 'color',
                'label'     => __( 'Background Color', 'piotnetgrid' ),
                'selectors' => [
                    '{{WRAPPER}} .slick-dots li button:before' => 'color: {{VALUE}}',
                ],
            ]
        );
        $this->add_control(
            'piotnet_grid_slick_dots_color_active',
            [
                'type'      => 'color',
                'label'     => __( 'Background Color Active', 'piotnetgrid' ),
                'selectors' => [
                    '{{WRAPPER}} .slick-dots li.slick-active button:before' => 'color: {{VALUE}}!important',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_opacity',
            [
                'type'        => 'slider',
                'label'       => __( 'Opacity', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1,
                        'step' => 0.1,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-dots li button:before' => 'opacity:{{SIZE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_opacity_active',
            [
                'type'        => 'slider',
                'label'       => __( 'Opacity Active', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1,
                        'step' => 0.1,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-dots li.slick-active button:before' => 'opacity:{{SIZE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_size',
            [
                'type'        => 'slider',
                'label'       => __( 'Size', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-dots li button:before' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_spacing',
            [
                'type'        => 'slider',
                'label'       => __( 'Spacing', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors'   => [
                    '{{WRAPPER}} .slick-dots li' => 'margin: 0 {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'piotnet_grid_slick_dots_bottom_position',
            [
                'label'       => __( 'Bottom Position', 'piotnetgrid' ),
                'type'        => 'text',
                'label_block' => true,
                'description' => 'E.g 20px, -20px, 10%',
                'selectors' => [
                    '{{WRAPPER}} .slick-dots' => 'bottom: {{VALUE}};',
                ],
            ]
        );
    }

    private function add_no_results_found_controls() {
    	$this->add_control(
			'no_results_found_content',
			[
				'type'        => 'textarea',
				'label'       => 'Content',
				'label_block' => true,
				'value'		  => __( 'Sorry, no results found!', 'piotnetgrid' ),
				'placeholder' => '',
				'dynamic'     => true,
			]
		);

		$this->add_control(
			'no_results_found_preview',
			[
				'label' => __( 'Preview', 'piotnetgrid' ),
				'type' => 'switch',
				'default' => '',
				'label_on' => 'Yes',
				'label_off' => 'No',
				'return_value' => 'yes',
			]
		);

		$this->add_responsive_control(
			'no_results_found_align',
			[
				'type'         => 'select',
				'label'        => 'Alignment',
				'label_block'  => true,
				'options'      => [
					''        => __( 'Default', 'piotnetgrid' ),
					'left'    => __( 'Left', 'piotnetgrid' ),
					'center'  => __( 'Center', 'piotnetgrid' ),
					'right'   => __( 'Right', 'piotnetgrid' ),
					'justify' => __( 'Justified', 'piotnetgrid' ),
				],
				'selectors'    => [
					'{{WRAPPER}} .piotnetgrid-no-results-found' => 'text-align: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'no_results_found_color',
			[
				'type'      => 'color',
				'label'     => 'Text Color',
				'dynamic'	=> true,
				'dynamic_field'	=> true,
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-no-results-found' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_text_typography_controls(
			'no_results_found_typography',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-no-results-found',
			]
		);
	}

	private function add_recently_viewed_posts_title_style_controls() {
		
		$this->add_control(
            'recently_viewed_posts_title_color',
            [
                'type'      => 'color',
                'label'     => __( 'Color', 'piotnetgrid' ),
                'selectors' => [
                    '{{WRAPPER}} .piotnetgrid-grid__title' => 'color: {{VALUE}}',
                ],
            ]
        );

    	$this->add_text_typography_controls(
			'recently_viewed_posts_title_typography',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-grid__title',
			]
		);

		$this->add_responsive_control(
            'recently_viewed_posts_title_spacing',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Spacing', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'   => [
                    '{{WRAPPER}} .piotnetgrid-grid__title' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
	}

	public function multilingual_fields() {
		$fields = [
			[
				'field'       => 'no_results_found_content',
				'editor_type' => 'LINE',
			],
			[
				'field'       => 'recently_viewed_posts_title',
				'editor_type' => 'LINE',
			],
		];

		return $fields;
	}

	public $post_ids;
	public $query;
	public $args;
	public $current_query;
	public $posts_per_page;
	public $lang;
	public $paged;

	public function set_query($settings = [], $args_additional = [], $facet_applied = true, $lang = '') {

		if (!empty($lang)) {
			$this->lang = $lang;
			if ( function_exists('icl_object_id') ) {
			    global $sitepress;
			    if ($sitepress) {
			    	$sitepress->switch_lang($lang);
			    }
			}
		}

		$args = [];
		$current_query = [];

		if (!empty($settings['current_query_enable'])) {
			global $wp_query;
			$args = $wp_query->query_vars;
			$args['piotnetgrid'] = true;

			foreach ($args as $key => $arg) {
				if (empty($arg)) {
					unset($args[$key]);
				}
			}
			unset($args['number']);

			if (isset($args['wc_query'])) {
				$args['post_type'] = 'product';
			}

			if (!empty($settings['posts_per_page'])) {
				$args['posts_per_page'] = $settings['posts_per_page'];
			}

			if (!empty($settings['offset'])) {
				$args['offset'] = $settings['offset'];
			}

			$this->current_query = $args;
		}

		if (!empty($settings['custom_query_enable']) && !empty($settings['custom_query'])) {
			$custom_query = stripslashes( $settings['custom_query'] );
			eval("\$args = $custom_query;");
			$this->current_query = $args;
		}

		if (empty($args) && empty($settings['current_query_enable']) && empty($settings['custom_query_enable'])) {
			$args = array(
				'post_type' => $settings['post_type'],
			);

			if (!empty($settings['wishlist_posts_enable'])) {
				$args['post__in'] = !empty($_SESSION['piotnetgrid_wishlist']) ? $_SESSION['piotnetgrid_wishlist'] : [10000000000];
			}

			if (!empty($settings['recently_viewed_posts'])) {
				$args['post__in'] = !empty($_SESSION['piotnetgrid_recently_viewed'][$settings['post_type']]) ? $_SESSION['piotnetgrid_recently_viewed'][$settings['post_type']] : [10000000000];
			}

			if (!empty($settings['include_posts'])) {
				foreach ($settings['include_posts'] as $include_post) {
					$args['post__in'][] = $include_post;
				}
			}

			if (!empty($settings['exclude_posts'])) {
				$args['post__not_in'] = $settings['exclude_posts'];
			}

			$tax_query = [];

			if (!empty($settings['include_terms'])) {
				$tax_query['relation'] = 'AND';
				$tax_query_include = array(
					'relation' => $settings['include_terms_relation'],
				);

				foreach ($settings['include_terms'] as $term_id) {
					$term = get_term($term_id);
					$tax_query_include[] = [
						'taxonomy' => $term->taxonomy,
						'field' => 'term_id',
						'terms' => $term->term_id,
					];
				}

				$tax_query[] = $tax_query_include;
			}

			if (!empty($settings['exclude_terms'])) {
				$tax_query['relation'] = 'AND';
				$tax_query_exclude = array(
					'relation' => 'AND',
				);

				foreach ($settings['exclude_terms'] as $term_id) {
					$term = get_term($term_id);
					$tax_query_exclude[] = [
						'taxonomy' => $term->taxonomy,
						'field' => 'term_id',
						'terms' => array($term->term_id),
						'operator' => 'NOT IN',
					];
				}

				$tax_query[] = $tax_query_exclude;
			}

			if (!empty($tax_query)) {
				$args['tax_query'] = $tax_query;
			}

			if (!empty($settings['post_status'])) {
				$args['post_status'] = $settings['post_status'];
			}

			if (!empty($settings['ignore_sticky_posts'])) {
				$args['ignore_sticky_posts'] = 1;
			}

			if (!empty($settings['meta_query_list'])) {
				$meta_query = [
					'relation' => $settings['meta_query_relation'],
				];
				$meta_query_list = $settings['meta_query_list'];
				foreach ($meta_query_list as $meta_query_item) {
					$meta_query[$meta_query_item['meta_key']] = [
						'key' => $meta_query_item['meta_key'],
						'value' => $meta_query_item['meta_value'],
			            'type' => $meta_query_item['meta_type'],
			            'compare' => $meta_query_item['meta_compare'],
					];
				}
				$args['meta_query'] = $meta_query;
			}

			if (!empty($settings['order_list'])) {
				$order_by = [];
				$order_list = $settings['order_list'];
				foreach ($order_list as $order) {
					if ($order['order_by'] == 'meta_value' || $order['order_by'] == 'meta_value_num') {
						$order_custom_field_name = $order['custom_field_name'];
						if (!empty($order_custom_field_name)) {
							if (!isset($args['meta_query'][$order_custom_field_name])) {
								if (empty($args['meta_query'])) {
									$args['meta_query'] = [
										'relation' => 'AND',
									];
								}
								$args['meta_query'][$order_custom_field_name] = [
						            'key' => $order_custom_field_name,
						            'compare' => 'EXISTS',
						            'type' => $order['order_by'] == 'meta_value_num' ? 'NUMBERIC' : 'CHAR',
								];
							}
							$order_by[$order_custom_field_name] = $order['order'];
						}
					} else {
						$order_by[$order['order_by']] = $order['order'];
					}
				}

				$args['orderby'] = $order_by;
			}
		}

		if (empty($settings['custom_query_enable'])) {
			if (!empty($settings['posts_per_page'])) {
				$args['posts_per_page'] = $settings['posts_per_page'];
			}

			if (!empty($settings['offset'])) {
				$args['offset'] = $settings['offset'];
			}
		}

		if ($facet_applied) {
			if (!empty($_GET)) {
				$facets = [];
				foreach ($_GET as $facet_name => $facet_value) {
					$facet_settings = get_option('_piotnetgrid_facet' . $facet_name);
					if (!empty($facet_settings)) {
						$facets[] = piotnetgrid_get_facet_by_name($facet_name, $facet_value);
					}
				}
			}

			if (empty($args_additional['query']) && !empty($facets)) {
				$args_additional = piotnetgrid_facet_change_query($facets, $args_additional);
				foreach ($args_additional['query'] as $key => $value) {
					if (isset($args[$key])) {
						if (is_array($value)) {
							$args[$key] = array_merge_recursive($args[$key],$value);
						} else {
							$args[$key] = $value;
						}
					} else {
						$args[$key] = $value;
					}
				}
				$args_additional = [];
			}
		}

		if (!empty($args_additional['query'])) {
			$this->current_query = $args_additional['current_query'];
			$args = array_merge($args,$args_additional['query']);

			if (!empty($settings['current_query_enable'])) {
				foreach ($args_additional['current_query'] as $key => $value) {
					if (isset($args_additional['query'][$key])) {
						if (is_array($value)) {
							$args_additional['query'][$key] = array_merge_recursive($args_additional['query'][$key],$value);
						}
					} else {
						$args_additional['query'][$key] = $value;
					}
				}

				$args = $args_additional['query'];
			}
		}

		if (!empty($args['tax_query']['relation']) && is_array($args['tax_query']['relation'])) {
			$args['tax_query']['relation'] = $args['tax_query']['relation'][0];
		}

		if (!empty($args['taxonomy']) && !empty($args['term']) && !empty($args['tax_query'])) {
			$args['tax_query'][] = [
				'taxonomy' => $args['taxonomy'],
				'terms' => $args['term'],
				'field' => 'slug',
			];
			unset($args['taxonomy']);
			unset($args['term']);
		}

		$args['piotnetgrid'] = true;

		if (class_exists( 'WooCommerce' ) && $args['post_type'] == 'product' && get_option('woocommerce_hide_out_of_stock_items') == 'yes') {
			if (!empty($args['meta_query'])) {
				$args['meta_query'] = [
					'relation' => 'AND',
					array(
					    'key' => '_stock_status',
					    'value' => 'instock',
					    'compare' => '='
				    ),
				    $args['meta_query'],
				];
			} else {
				$args['meta_query'] = [
					'relation' => 'AND',
					array(
					    'key' => '_stock_status',
					    'value' => 'instock',
					    'compare' => '='
				    ),
				];
			}
		}

		// echo '<pre>';
		// print_r($args);
		// echo '</pre>';

		$this->args = $args;
		$query = new WP_Query($args);

		if ($query->found_posts == 0 && !empty($args['s'])) {
			global $wpdb;
			$post_ids_fulltext = [];
			$search_fulltext = $wpdb->get_results( "SELECT ID FROM " . $wpdb->prefix . "posts WHERE MATCH " . $wpdb->prefix . "posts.post_title, " . $wpdb->prefix . "posts.post_content AGAINST ('" . $args["s"] . "')", ARRAY_A );

			foreach ($search_fulltext as $search_fulltext_item) {
				$post_ids_fulltext[] = $search_fulltext_item['ID'];
			}

			if (!empty($post_ids_fulltext)) {
				if (!empty($args['post__in'])) {
					$post_ids_fulltext = array_intersect($args['post__in'],$post_ids_fulltext);
				}
				unset($args['s']);
				$args['post__in'] = $post_ids_fulltext;
				$this->args = $args;
				$query = new WP_Query($args);
			}
		}

		$this->posts_per_page = !empty($args['posts_per_page']) ? $args['posts_per_page'] : get_option( 'posts_per_page' );
		$args['fields'] = 'ids';
		$args['posts_per_page'] = '-1';
		foreach ($args as $key => $arg) {
			if (empty($arg)) {
				unset($args[$key]);
			}
		}
		unset($args['offset']);
		$posts = new WP_Query($args);
		$this->post_ids = $posts->posts;
		$this->query = $query;
		$this->paged = !empty($args['paged']) ? $args['paged'] : 1;
	}

	public function render($editor = false,$args_additional = []) {
		$settings = $this->settings;
		$lang = !empty($args_additional) && !empty($args_additional['lang']) ? $args_additional['lang'] : '';
		$this->set_query($settings, $args_additional, true, $lang);
		
		if (!empty($settings['card_template']) && strpos( $settings['card_template'], 'export' ) == false) :
			$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-grid' );

		    if (!empty($settings['grid_masonry_enable'])) {
		        $piotnetgrid_masonry = [
                    'trueOrder'=> true,
                    'waitForImages' => false,
                    /*'useContainerForBreakpoints' => true,
                    'cancelLegacy' => true,
                    'useOwnImageLoader' => true,*/
                    'columns' => (int)(!empty($settings['grid_columns_responsive_desktop']) ? $settings['grid_columns_responsive_desktop'] : '3'),
                    'margin' => [
                        'x' => (int)(!empty($settings['grid_columns_gap_responsive_desktop']['size']) ? $settings['grid_columns_gap_responsive_desktop']['size'] : '10'),
                        'y' => (int)(!empty($settings['grid_row_gap_responsive_desktop']['size']) ? $settings['grid_row_gap_responsive_desktop']['size'] : '10'),
                    ],
                    'debug'=> true,
                    'mobileFirst' => false,
                    'breakAt' => [
                        '1170'=> [
                            'margin' => [
                                'x' => (int)(!empty($settings['grid_columns_gap_responsive_desktop']['size']) ? $settings['grid_columns_gap_responsive_desktop']['size'] : '10'),
                                'y' => (int)(!empty($settings['grid_row_gap_responsive_desktop']['size']) ? $settings['grid_row_gap_responsive_desktop']['size'] : '10'),
                              ],
                              'columns' => (int)(!empty($settings['grid_columns_responsive_desktop']) ? $settings['grid_columns_responsive_desktop'] : '3')
                        ],
                        '767'=> [
                            'margin' => [
                                'x' => (int)(!empty($settings['grid_columns_gap_responsive_tablet']['size']) ? $settings['grid_columns_gap_responsive_tablet']['size'] : '10'),
                                'y' => (int)(!empty($settings['grid_row_gap_responsive_tablet']['size']) ? $settings['grid_row_gap_responsive_tablet']['size'] : '10'),
                              ],
                              'columns' => (int)(!empty($settings['grid_columns_responsive_tablet']) ? $settings['grid_columns_responsive_tablet'] : '2')
                        ],
                        '480'=> [
                            'margin' => [
                                'x' => (int)(!empty($settings['grid_columns_gap_responsive_mobile']['size']) ? $settings['grid_columns_gap_responsive_mobile']['size'] : '10'),
                                'y' => (int)(!empty($settings['grid_row_gap_responsive_mobile']['size']) ? $settings['grid_row_gap_responsive_mobile']['size'] : '10'),
                              ],
                              'columns' => (int)(!empty($settings['grid_columns_responsive_mobile']) ? $settings['grid_columns_responsive_mobile'] : '1')
                        ],
                    ]
                ];

		        $this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-grid--masonry' );
		        $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-masonry', json_encode($piotnetgrid_masonry) );
		    }

            $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-id', $this->get_id() );

			if (!empty($this->current_query)) {
				$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-current-query', json_encode($this->current_query) );
			}

			if (!empty($settings['cache_enable'])) {
				$cache_boost_loading = !empty($settings['cache_boost_loading']) ? true : false;
				$cache_expiration = isset($settings['cache_expiration']) ? $settings['cache_expiration'] : 0;
				$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-cache', json_encode(['boost_loading' => $cache_boost_loading, 'expiration' => $cache_expiration]) );
			}

			if ($editor && !empty($settings['current_query_enable'])) {
				$this->query = new WP_Query(array(
					'post_type' => 'post',
					'posts_per_page' => !empty($settings['posts_per_page']) ? $settings['posts_per_page'] : get_option( 'posts_per_page' ),
				));
			}

			$card_template_id = $settings['card_template'];
			$card_template = '[piotnetgrid id=' . $card_template_id . ' type=card]';

			if ($editor) {
				$card_template_exploded = explode(']', $card_template);
				$card_template =  $card_template_exploded[0] . ' editor=true]';
			}

			$upload = wp_upload_dir();
			$upload_dir = $upload['baseurl'];
			$upload_dir = $upload_dir . '/piotnetgrid/css/';
			$card_template_file = $upload_dir . $card_template_id . ".css";

			if (!empty($args_additional['editor'])) {
				echo "<link rel='stylesheet' href='" . $card_template_file . '?ver=' . get_post_meta( $card_template_id, '_piotnetgrid-revision-version', true ) . "' media='all'>";
			}

			wp_enqueue_style( 'piotnetgrid-style-' . $card_template_id, $card_template_file, [], get_post_meta( $card_template_id, '_piotnetgrid-revision-version', true ) );

			if (!empty($settings['grid_carousel_enable'])) {
				$grid_carousel_navigation = !empty($settings['grid_carousel_navigation']) ? $settings['grid_carousel_navigation'] : 'both';
                $show_dots = ( in_array( $grid_carousel_navigation, [ 'dots', 'both' ] ) );
                $show_arrows = ( in_array( $grid_carousel_navigation, [ 'arrows', 'both' ] ) );
                $slick_slides_to_show_desktop = (!empty($settings['grid_carousel_slides_to_show_responsive_desktop'])) ? floatval( $settings['grid_carousel_slides_to_show_responsive_desktop'] ) : '3';
                $slick_slides_to_show_tablet = (!empty($settings['grid_carousel_slides_to_show_responsive_tablet'])) ? floatval( $settings['grid_carousel_slides_to_show_responsive_tablet'] ) : '2';
                $slick_slides_to_show_mobile = (!empty($settings['grid_carousel_slides_to_show_responsive_mobile'])) ? floatval( $settings['grid_carousel_slides_to_show_responsive_mobile'] ) : '1';

                $slick_slides_to_scroll_desktop = (!empty($settings['grid_carousel_slides_to_scroll_responsive_desktop'])) ? floatval( $settings['grid_carousel_slides_to_scroll_responsive_desktop'] ) : '3';
                $slick_slides_to_scroll_tablet = (!empty($settings['grid_carousel_slides_to_scroll_responsive_tablet'])) ? floatval( $settings['grid_carousel_slides_to_scroll_responsive_tablet'] ) : '2';
                $slick_slides_to_scroll_mobile = (!empty($settings['grid_carousel_slides_to_scroll_responsive_mobile'])) ? floatval( $settings['grid_carousel_slides_to_scroll_responsive_mobile'] ) : '1';

                if ( !empty($settings['grid_carousel_hide_on_desktop']) ) {
                    $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-carousel-disable-on-desktop', $settings['grid_carousel_hide_on_desktop'] );
                }
                if ( !empty($settings['grid_carousel_hide_on_tablet']) ) {
                    $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-carousel-disable-on-tablet', $settings['grid_carousel_hide_on_tablet'] );
                }
                if ( !empty($settings['grid_carousel_hide_on_mobile']) ) {
                    $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-carousel-disable-on-mobile', $settings['grid_carousel_hide_on_mobile'] );
                }

                $slick_options = [
                    'autoplay' => ( 'yes' === $settings['grid_carousel_autoplay'] ),
                    'infinite' => ( 'yes' === $settings['grid_carousel_infinite_loop'] ),
                    'pauseOnHover' => ( 'yes' === $settings['grid_carousel_pause_on_hover'] ),
                    'arrows' => $show_arrows,
                    'dots' =>   $show_dots,
                    'slidesToShow' => $slick_slides_to_show_desktop,
                    'slidesToScroll' => $slick_slides_to_scroll_desktop,
                    'responsive'=> [
                        [
                            'breakpoint' => '1170',
                            'settings' => [
                                'slidesToShow' => $slick_slides_to_show_desktop,
                                'slidesToScroll' => $slick_slides_to_scroll_desktop,
                            ]
                        ],
                        [
                            'breakpoint' => '991',
                            'settings' => [
                                'slidesToShow' => $slick_slides_to_show_tablet,
                                'slidesToScroll' => $slick_slides_to_scroll_tablet,

                            ]
                        ],
                        [
                            'breakpoint' => '767',
                            'settings' => [
                                'slidesToShow' => $slick_slides_to_show_mobile,
                                'slidesToScroll' => $slick_slides_to_scroll_mobile,
                            ]
                        ],
                    ],

                ];

                $slick_options['autoplaySpeed'] = (!empty($settings['grid_carousel_autoplay_speed'])) ? absint( $settings['grid_carousel_autoplay_speed'] ) : '3000';
                $slick_options['speed'] = (!empty($settings['grid_carousel_animation_speed'])) ? absint( $settings['grid_carousel_animation_speed'] ) : '300';
                wp_enqueue_script( $this->slug . '-slick-script' );
				wp_enqueue_style( $this->slug . '-slick-style' );
				$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-carousel', json_encode($slick_options) );
			}
			//Animation
			if (!empty($settings['grid_animation_enable'])) {
			    $animation_effect = !empty($settings['grid_animation_effects']) ? $settings['grid_animation_effects'] : 'animate__fadeInUp';
                $animation_speed = !empty($settings['grid_animation_speed']) ? $settings['grid_animation_speed'] : 'animate__fast';
                $animation_delay = !empty($settings['grid_animation_delay']) ? $settings['grid_animation_delay'] : '0';

                wp_enqueue_style( $this->slug . '-animate-style' );
                $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-animation', $animation_effect );
                $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-animation-speed', $animation_speed );
                $this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-animation-delay', $animation_delay );
            }

			$offset_first = !empty($settings['offset_first']) ? intval($settings['offset_first']) : '';
			$offset_last = !empty($settings['offset_last']) ? intval($settings['offset_last']) : '';
			$have_posts = $this->query->have_posts();
			if (!$have_posts || $editor && !empty($settings['no_results_found_preview'])) {
				$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-grid--no-results-found' );
			}

			if ( function_exists('icl_object_id') ) {
			    global $sitepress;
			    if ($sitepress) {
			    	$current_lang = $sitepress->get_current_language();
				    $default_lang = $sitepress->get_default_language();
				    if (strcmp($current_lang, $default_lang) != 0) {
				    	$this->lang = $current_lang;
				    }
			    }
			}

			if (!empty($this->lang)) {
				if ( function_exists('icl_object_id') ) {
					$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-grid-lang', $this->lang );
				}
			}

			$this->add_render_attribute( 'wrapper-posts', 'class', 'piotnetgrid-grid__posts' );
			$this->add_render_attribute( 'wrapper-posts', 'data-piotnetgrid-grid-posts', '' );

            if (!$editor){
                require_once (__DIR__ . '/../templates/piotnet-grid-lightbox.php');
            }
            echo '<style>.pswp {display: none;}</style>';
		?>
			<div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
			<?php
				if (empty($settings['source'])) {
					if (!$editor || $editor && empty($settings['no_results_found_preview']) ) {
					?>
						<?php
							if ($have_posts || !$have_posts && $editor) :
								if (!empty($settings['recently_viewed_posts']) && !empty($settings['recently_viewed_posts_title'])) :
						?>
	                    		<div class="piotnetgrid-grid__title"><?php echo $settings['recently_viewed_posts_title']; ?></div>
	                	<?php
	                			endif;
	                		endif;
	            		?>
						<?php
						if($have_posts) {
	                    ?>
	                	<div <?php echo $this->get_render_attribute_string( 'wrapper-posts' ); ?>>
	                    <?php
							$post_index = 0;
							?>
							<?php
							while($this->query->have_posts()) :
								$post_index = $post_index + 1;
								$this->query->the_post();

								$post_crossed = false;

								if (empty($offset_first) || $post_index > $offset_first) {
									$post_crossed = true;
								}

								if ($post_crossed) {
									if (empty($offset_last) || $this->posts_per_page - $post_index >= $offset_last) {
										$post_crossed = true;
									} else {
										$post_crossed = false;
									}
								}

								if ($post_crossed) {
									$card_classes = !empty($settings['add_post_classes_to_the_card']) ? implode( ' ', get_post_class() ) . ' piotnetgrid-card' : 'piotnetgrid-card';

	                                echo '<article class="' . $card_classes . '"';
	                                if (!empty($settings['grid_animation_enable']) && !$editor) {
	                                	echo ' style="opacity: 0"';
	                                }
	                                echo '>';
	                                    echo do_shortcode($card_template);
	                                echo '</article>';
	                            }
							endwhile;
						?>
						</div>
						<?php
						}
					}

					if(!$have_posts && empty($settings['recently_viewed_posts']) || $editor && !empty($settings['no_results_found_preview']) && empty($settings['recently_viewed_posts'])) :
						?>
						<div class="piotnetgrid-no-results-found">
							<?php
								$no_results_found_content = !empty($settings['no_results_found_content']) ? $settings['no_results_found_content'] : '';
								_e( $no_results_found_content, 'piotnetgrid' );
							?>
						</div>
						<?php
					endif;
					wp_reset_postdata();
			} else {
				if ($settings['source'] == 'taxonomy_terms') {
					$terms = [];

					$terms_args = array(
					    'taxonomy' => !empty($settings['terms_grid_taxonomy']) ? $settings['terms_grid_taxonomy'] : 'category',
					    'hide_empty' => !empty($settings['terms_grid_hide_empty']) ? true : false,
					    'include' => !empty($settings['terms_grid_include']) ? $settings['terms_grid_include'] : [],
					    'exclude' => !empty($settings['terms_grid_exclude']) ? $settings['terms_grid_exclude'] : [],
					    'number' => !empty($settings['terms_grid_number']) ? $settings['terms_grid_number'] : 0,
					    'order' => !empty($settings['terms_grid_order']) ? $settings['terms_grid_order'] : 'ASC',
					);

					if (!empty($settings['terms_grid_orderby'])) {
						$terms_args['orderby'] = $settings['terms_grid_orderby'];
					}

					if (!empty($settings['terms_grid_parent'])) {
						$terms_args['parent'] = $settings['terms_grid_parent'][0];
					}

					if (!empty($settings['meta_query_list'])) {
						$meta_query = [
							'relation' => $settings['meta_query_relation'],
						];
						$meta_query_list = $settings['meta_query_list'];
						foreach ($meta_query_list as $meta_query_item) {
							$meta_query[$meta_query_item['meta_key']] = [
								'key' => $meta_query_item['meta_key'],
								'value' => $meta_query_item['meta_value'],
					            'type' => $meta_query_item['meta_type'],
					            'compare' => $meta_query_item['meta_compare'],
							];
						}
						$terms_args['meta_query'] = $meta_query;
					}

					$term_query = new WP_Term_Query($terms_args);
					?>
					<div <?php echo $this->get_render_attribute_string( 'wrapper-posts' ); ?>>
						<?php
						foreach ($term_query->terms as $term) {
							echo '<div class="piotnetgrid-card"';
	                        if (!empty($settings['grid_animation_enable']) && !$editor) {
	                        	echo ' style="opacity: 0"';
	                        }
	                        echo '>';
	                        	$args_shortcode = [
	                        		'id' => $card_template_id,
	                        		'type' => 'card',
	                    		];

								if ($editor) {
									$args_shortcode['editor'] = 'true';
								}

								$GLOBALS['term'] = $term;
	                        	echo do_shortcode($card_template);
	                        echo '</div>';
						}
						?>
					</div>
					<?php
				}
			}

			echo '</div>';
		else :
			_e('Please edit this widget and select Card Template','piotnetgrid');
		endif;
	}

	public function live_preview() {
	}

}
