<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Image extends Base_Widget_piotnetgrid {
	public function get_type() {
		return 'image';
	}

	public function get_class_name() {
		return 'piotnetgrid_Image';
	}

	public function get_title() {
		return 'Image';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-image.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'image' ];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'image_settings_section', 'Settings' );
		$this->add_setting_controls();

        $this->start_section( 'grid_photoswipe_section', 'Lightbox Image' );
        $this->add_photoswipe_controls();


		$this->start_tab( 'style', 'Style' );
		$this->start_section( 'image_styles_section', 'Image Style' );
		$this->add_style_controls();
		$this->start_section( 'caption_styles_section', 'Caption Style' );
		$this->add_caption_style_controls();

		$this->add_advanced_style_tab();

		$this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
		$this->add_custom_class_id();

		$this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
		$this->add_custom_css_controls();

		$this->add_animation_settings();

		$this->add_attributes_settings();

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		$this->add_trigger_action_carousel();

		return $this->structure;
	}

    private function add_photoswipe_controls() {
        $this->add_control(
            'piotnetgrid_photoswipe_enable',
            [
                'label' => __( 'Lightbox Enable', 'piotnetgrid' ),
                'type' => 'switch',
                'default' => '',
                'label_on' => 'Yes',
                'label_off' => 'No',
                'return_value' => 'yes',
            ]
        );
        $this->add_control(
            'piotnetgrid_photoswipe_image_caption',
            [
                'type'        => 'select',
                'label'       => __( 'Caption', 'piotnetgrid' ),
                'options'     => [
                    'none' => __( 'None', 'piotnetgrid' ),
                    'attachment' => __( 'Attachment Caption', 'piotnetgrid' ),
                    'description' => __( 'Attachment Description', 'piotnetgrid' ),
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'piotnetgrid_photoswipe_enable',
                            'operator' => '==',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_control(
            'piotnetgrid_photoswipe_image_title',
            [
                'type'        => 'select',
                'label'       => __( 'Title', 'piotnetgrid' ),
                'options'     => [
                    'none' => __( 'None', 'piotnetgrid' ),
                    'attachment' => __( 'Attachment Title', 'piotnetgrid' ),
                    'post_data_title' => __( 'Title From Post Data', 'piotnetgrid' ),
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'piotnetgrid_photoswipe_enable',
                            'operator' => '==',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_control(
            'piotnetgrid_pswp_dynamic_title',
            [
                'type'        => 'text',
                'label'       => __( 'Post Title', 'piotnetgrid' ),
                'dynamic'     => true,
                'conditions'  => [
                    'terms' => [
                        [
                            'name'     => 'piotnetgrid_photoswipe_image_title',
                            'operator' => 'in',
                            'value'    => [
                                'post_data_title',
                            ],
                        ],
                        [
                            'name'     => 'piotnetgrid_photoswipe_enable',
                            'operator' => '==',
                            'value'    => 'yes',
                        ],
                    ],

                ],
            ]
        );

        $this->add_control(
            'piotnetgrid_photoswipe_background_color',
            [
                'type'        => 'color',
                'label'       => __( 'Background Color', 'piotnetgrid' ),
                'value'       => '',
                'label_block' => true,
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'piotnetgrid_photoswipe_enable',
                            'operator' => '==',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
        $this->add_control(
            'piotnetgrid_photoswipe_background_opacity',
            [
                'type'        => 'number',
                'label'       => __( 'Background Opacity', 'piotnetgrid' ),
                'label_block' => true,
                'value'       => '1',
                'min'		  => 0,
                'max'         => 1,
                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'piotnetgrid_photoswipe_enable',
                            'operator' => '==',
                            'value'    => 'yes',
                        ],
                    ],
                ],
            ]
        );
    }
	private function add_setting_controls() {
		$this->add_control(
			'image_select',
			[
				'type'        => 'media',
				'label'       => __( 'Choose Image', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'placeholder' => '',
			]
		);

		$this->add_control(
			'image_dynamic_data',
			[
				'type'        => 'select',
				'label'       => __( 'Image Dynamic Data', 'piotnetgrid' ),
				'options'     => [
					'' => __( 'None', 'piotnetgrid' ),
					'featured_image' => __( 'Featured Image', 'piotnetgrid' ),
					'author_profile_picture' => __( 'Author Profile Picture', 'piotnetgrid' ),
					'post_custom_field' => __( 'Post Custom Field', 'piotnetgrid' ),
					'acf_field' => __( 'ACF Field', 'piotnetgrid' ),
					'metabox_field' => __( 'Metabox Field', 'piotnetgrid' ),
					'pods_field' => __( 'Pods Field', 'piotnetgrid' ),
					'toolset_field' => __( 'Toolset Field', 'piotnetgrid' ),
					'jetengine_field' => __( 'JetEngine Field', 'piotnetgrid' ),
					'term_image' => __( 'Term Image', 'piotnetgrid' ),
					'woocommerce_category_thumbnail' => __( 'Woocommerce Category Thumbnail', 'piotnetgrid' ),
				],
			]
		);

		$this->add_control(
			'image_dynamic_data_field_name',
			[
				'type'        => 'text',
				'label'       => __( 'Field Name', 'piotnetgrid' ),
				'conditions'  => [
					[
						'name'     => 'image_dynamic_data',
						'operator' => 'in',
						'value'    => [
							'post_custom_field',
							'acf_field',
							'metabox_field',
							'pods_field',
							'toolset_field',
							'jetengine_field',
						],
					],
				],
			]
		);

		if ( class_exists( 'woocommerce' ) ) {
			$this->add_control(
				'switch_wc_product_image_on_hover',
				[
					'type'         => 'switch',
					'label'        => __( 'Switch Woocommerce Product Image on Hover ', 'piotnetgrid' ),
					'value'        => '',
					'label_on'     => 'Yes',
					'label_off'    => 'No',
					'return_value' => 'yes',
					'conditions'  => [
						[
							'name'     => 'image_dynamic_data',
							'operator' => '=',
							'value'    => 'featured_image',
						],
					],
				]
			);

			$this->add_control(
				'variation_wc_image',
				[
					'type'         => 'switch',
					'label'        => __( 'Get image from Woocommerce variation swatches selected', 'piotnetgrid' ),
					'value'        => '',
					'label_on'     => 'Yes',
					'label_off'    => 'No',
					'return_value' => 'yes',
				]
			);
		}
        $image_sizes = $this->get_additional_image_sizes();
	    $this->add_control(
			'image_size',
			[
				'type'    => 'select',
				'label'   => __( 'Image Size', 'piotnetgrid' ),
				'value'   => 'full',
				'options' => $image_sizes,
				'conditions'  => [
					[
						'name'     => 'image_dynamic_data',
						'operator' => '!=',
						'value'    => 'author_profile_picture',
					],
				],
			]
		);

		$this->add_control(
			'image_size_author_profile_picture',
			[
				'type'    => 'text',
				'label'   => __( 'Image Size (px)', 'piotnetgrid' ),
				'value'   => '150',
				'conditions'  => [
					[
						'name'     => 'image_dynamic_data',
						'operator' => '==',
						'value'    => 'author_profile_picture',
					],
				],
			]
		);

		$this->add_responsive_control(
			'image_aspect_ratio',
			[
				'label' => __( 'Image Aspect Ratio', 'pafe' ),
				'label_block' => true,
				'type' => 'number',
				'description' => 'Aspect Ratio = Height / Width * 100. E.g Width = 100, Height = 100 => Ratio = 100; Width = 100, Height = 50 => Ratio = 50',
				'value'   => '',
				'render_type' => 'both',
				'selectors' => [
					'{{WRAPPER}}.piotnetgrid-image:after' => 'padding-top: {{VALUE}}%;',
				],
			]
		);

		$this->add_control(
			'image_link_type',
			[
				'type'    => 'select',
				'label'   => __( 'Link', 'piotnetgrid' ),
				'value'   => '',
				'options' => [
					''       => 'None',
					'custom' => 'Custom URL',
					'file'   => 'Media File',
				],
			]
		);

		$this->add_control(
			'image_link',
			[
				'type'        => 'text',
				'label'       => __( 'Link URL', 'piotnetgrid' ),
				'value'       => '',
				'dynamic'	  => true,
				'label_block' => false,
				'placeholder' => '',
				'conditions'  => [
					[
						'name'     => 'image_link_type',
						'operator' => '==',
						'value'    => 'custom',
					],
				],
			]
		);

		$this->add_control(
			'image_link_target',
			[
				'type'       => 'select',
				'label'      => __( 'Link Target', 'piotnetgrid' ),
				'value'      => '',
				'options'    => [
					'_self'  => 'Self',
					'_blank' => 'Blank',
				],
				'conditions' => [
					[
						'name'     => 'image_link_type',
						'operator' => '!=',
						'value'    => ['', 'file'],
					],
				],
			]
		);

		$this->add_control(
			'image_caption',
			[
				'type'        => 'text',
				'label'       => __( 'Caption', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => false,
				'placeholder' => '',
			]
		);
        $this->add_control(
            'grid_lazy_load_enable',
            [
                'label' => __( 'Lazy Load Enable', 'piotnetgrid' ),
                'type' => 'switch',
                'default' => '',
                'label_on' => 'Yes',
                'label_off' => 'No',
                'return_value' => 'yes',
            ]
        );

	}

	private function add_style_controls() {
		$this->add_control(
			'image_custom_size',
			[
				'type'         => 'switch',
				'label'        => __( 'Custom Size', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'yes',
			]
		);

		$this->add_control(
			'image_custom_size_with_center_mode',
			[
				'type'         => 'switch',
				'label'        => __( 'Custom Size with Center Mode', 'piotnetgrid' ),
				'value'        => '',
				'label_on'     => 'Yes',
				'label_off'    => 'No',
				'return_value' => 'yes',
			]
		);

		$this->add_responsive_control(
			'image_width',
			[
				'type'        => 'slider',
				'label'       => __( 'Width', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'px' => [
						'min'  => 1,
						'max'  => 1000,
						'step' => 1,
					],
					'%'  => [
						'min'  => 1,
						'max'  => 100,
						'step' => 1,
					],
				],
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-image__content img' => 'width:{{SIZE}}{{UNIT}}',
				],
				'conditions'  => [
					[
						'name'     => 'image_custom_size',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);

		$this->add_responsive_control(
			'image_height',
			[
				'type'        => 'slider',
				'label'       => __( 'Height', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'px' => [
						'min'  => 1,
						'max'  => 1000,
						'step' => 1,
					],
					'%'  => [
						'min'  => 1,
						'max'  => 100,
						'step' => 1,
					],
				],
				'selectors'   => [
					'{{WRAPPER}}.piotnetgrid-image:after' => 'padding-top: {{SIZE}}{{UNIT}}',
				],
				'conditions'  => [
					[
						'name'     => 'image_custom_size_with_center_mode',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);

		$this->add_control(
			'image_border_style',
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-image__content img' => 'border-style:{{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'image_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-image__content img' => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => 'image_border_style',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);

		$this->add_responsive_control(
			'image_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-image__content img' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => 'image_border_style',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);

		$this->add_responsive_control(
			'image_border_radius',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-image__content img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'image_text_align',
			[
				'type'         => 'select',
				'label'        => __( 'Alignment', 'piotnetgrid' ),
				'value'        => 'left',
				'label_block' => true,
				'options'      => [
					''   => __( 'Default', 'piotnetgrid' ),
					'left'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'right'  => __( 'Right', 'piotnetgrid' ),
				],
				'prefix_class' => 'piotnetgrid%s-align-',
				'selectors'    => [
					'{{WRAPPER}} .piotnetgrid-image__content' => 'text-align: {{VALUE}}',
				],
			]
		);
	}

	private function add_caption_style_controls() {
		$this->add_control(
			'image_caption_color',
			[
				'type'        => 'color',
				'label'       => __( 'Color', 'piotnetgrid' ),
				'value'       => '#000',
				'label_block' => true,
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-image__caption' => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_text_typography_controls(
			'image_caption_typography',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-image__caption',
			]
		);
		$this->add_responsive_control(
			'image_caption_text_align',
			[
				'type'         => 'select',
				'label'        => __( 'Alignment', 'piotnetgrid' ),
				'value'        => 'left',
				'label_block' => true,
				'options'      => [
					''   => __( 'Default', 'piotnetgrid' ),
					'left'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'right'  => __( 'Right', 'piotnetgrid' ),
				],
				'prefix_class' => 'piotnetgrid%s-align-',
				'selectors'    => [
					'{{WRAPPER}} .piotnetgrid-image__caption' => 'text-align: {{VALUE}}',
				],
			]
		);
		$this->add_control(
			'caption_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-image__caption' => 'background-color: {{VALUE}};',
				],
			]
		);
	}

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
		$image_size = $settings['image_size'];
		$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-image' );
		$default_image_url = plugins_url() . '/piotnetgrid/assets/images/placeholder-image.png';
		$image_url = ! empty( $settings['image_select']['url'] ) ? $settings['image_select']['url'] : $default_image_url;
		$post_id = !empty($GLOBALS['preview_card_post_id']) ? $GLOBALS['preview_card_post_id'] : get_the_ID();
		$variation_wc_image = class_exists( 'woocommerce' ) && !empty($settings['variation_wc_image']) ? true : false;

		if ( !$editor || !empty($GLOBALS['preview_card_post_id']) ) {
			switch ($settings['image_dynamic_data']) {
				case 'featured_image':
					$image_url = wp_get_attachment_image_src( get_post_thumbnail_id( $post_id ), $settings['image_size'] );
					$image_url = $image_url[0];
					if (class_exists( 'woocommerce' ) && !empty($settings['switch_wc_product_image_on_hover']) && get_post_type($post_id) == 'product') {
						$secondary_image_id = wc_get_product($post_id)->get_gallery_image_ids()[0] ;
						if ( $secondary_image_id ) {
							$secondary_image_url = wp_get_attachment_image_src( $secondary_image_id, $settings['image_size'] )[0];
						}
					}
					break;

				case 'author_profile_picture':
					$image_size_author_profile_picture = !empty($settings['image_size_author_profile_picture']) ? $settings['image_size_author_profile_picture'] : 150;
					$image_url = get_avatar_url(get_the_author_meta('ID'), array('size' => $image_size_author_profile_picture));
					break;

				case 'post_custom_field':
                    $value = get_post_meta( $post_id, $settings['image_dynamic_data_field_name'], true );
                    $image_url = wp_get_attachment_image_src( attachment_url_to_postid( $value ), $settings['image_size'] );
                    $image_url = $image_url[0];
                    break;

                case 'toolset_field':
                    $value = get_post_meta( $post_id, 'wpcf-' . $settings['image_dynamic_data_field_name'], true );
                    $image_url = wp_get_attachment_image_src( attachment_url_to_postid( $value ), $settings['image_size'] );
                    $image_url = $image_url[0];
                    break;

                case 'metabox_field':
                    if ( function_exists( 'rwmb_meta_legacy' ) ) {
                        $value = rwmb_meta_legacy( $settings['image_dynamic_data_field_name'], array( 'limit' => 1 ), $post_id );
                    }
                    $image_url = wp_get_attachment_image_src( $value, $settings['image_size'] );
                    $image_url = $image_url[0];
                    break;

                case 'pods_field':
                    if ( function_exists( 'pods_field' ) ) {
                        $value = pods_field( get_post_type(), $post_id, $settings['image_dynamic_data_field_name'],true );
                    }
                    $image_url = wp_get_attachment_image_src( attachment_url_to_postid( $value['guid'] ), $settings['image_size'] );
                    $image_url = $image_url[0];
                    break;

                case 'jetengine_field':
                    $image_url = wp_get_attachment_image_src( get_post_meta( $post_id, $settings['image_dynamic_data_field_name'], true ), $settings['image_size'] );
                    $image_url = $image_url[0];
                    break;

                case 'acf_field':
                    if ( function_exists( 'get_field' ) ) {
                        $value = get_field($settings['image_dynamic_data_field_name'], $post_id);

                    }
                    $image_url = wp_get_attachment_image_src( attachment_url_to_postid( $value['url'] ), $settings['image_size'] );
                    $image_url = $image_url[0];
                    break;

                case 'term_image':
                	global $term;
                	if (!empty($term)) {
                		$image_id = get_term_meta( $term->term_id, 'piotnetgrid_term_image', true );
                		if (!empty($image_id)) {
                			$image_url = wp_get_attachment_image_src( $image_id, $settings['image_size'] );
                    		$image_url = $image_url[0];
                		}
                	}

                    break;

                case 'woocommerce_category_thumbnail':
                	global $term;
                    $image_url = wp_get_attachment_image_src( get_term_meta( $term->term_id, 'thumbnail_id', true ), $settings['image_size'] );
                    $image_url = $image_url[0];
                    break;
            }
		}

		if ($settings['image_dynamic_data'] !== 'author_profile_picture') {
			if ( !$image_url ) {
				$image_url = $default_image_url;
			}
		}


		$image_link  = ! empty( $settings['image_link'] ) && ! empty( $settings['image_link_type'] ) ? $settings['image_link'] : '';
		$link_target = ! empty( $settings['image_link_target'] ) ? $settings['image_link_target'] : '';
		$image_caption = ! empty( $settings['image_caption'] ) ? $settings['image_caption'] : '';

		if (!empty($settings['image_aspect_ratio_responsive_desktop']) || !empty($settings['image_custom_size_with_center_mode'])) {
			$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-image--aspect-ratio' );
		}
        if (!empty($settings['grid_lazy_load_enable']) && !$editor){
            $this->add_render_attribute( 'image-tag', 'class', 'piotnetgrid-image__tag--lazy' );
            $this->add_render_attribute( 'image-tag', 'data-src', $image_url );
            if (!empty($secondary_image_url)) {
            	$this->add_render_attribute( 'image-tag-secondary', 'class', 'piotnetgrid-image__tag--lazy piotnetgrid-image__tag--secondary' );
            	$this->add_render_attribute( 'image-tag-secondary', 'data-src', $secondary_image_url );
            }
        } else {
        	$this->add_render_attribute( 'image-tag', 'class', 'piotnetgrid-image__tag' );
            $this->add_render_attribute( 'image-tag', 'src', $image_url );
            if (!empty($secondary_image_url)) {
            	$this->add_render_attribute( 'image-tag-secondary', 'class', 'piotnetgrid-image__tag piotnetgrid-image__tag--secondary' );
            	$this->add_render_attribute( 'image-tag-secondary', 'src', $secondary_image_url );
            }
        }
        //Photoswipe
        $piotnetgrid_pswp = $settings['piotnetgrid_photoswipe_enable'];
        if( !empty($piotnetgrid_pswp) ) {

            $bgOpacity = !empty($settings['piotnetgrid_photoswipe_background_opacity']) ? $settings['piotnetgrid_photoswipe_background_opacity'] : 1;
            $this->add_render_attribute( 'pswp-data', 'data-grid-pswp-bg-color', $settings['piotnetgrid_photoswipe_background_color'] );
            $this->add_render_attribute( 'pswp-data', 'data-grid-pswp-bg-opacity', $bgOpacity );

            $id = attachment_url_to_postid($image_url);
            $title_source = $settings['piotnetgrid_photoswipe_image_title'];
            $caption_source = $settings['piotnetgrid_photoswipe_image_caption'];
            $image = get_post($id);
            $pswp_title = '';
            $pswp_caption = '';
            $image_meta  = wp_get_attachment_metadata( $id );
            if( $title_source == 'attachment' ) {
                $pswp_title = $image->post_title;
            }
            if( $title_source == 'post_data_title' ) {
                $pswp_title = $settings['piotnetgrid_pswp_dynamic_title'];
            }


            if( $caption_source == 'attachment' ) {
                $pswp_caption = $image->post_excerpt;
            }
            if( $caption_source == 'description' ) {
                $pswp_caption = $image->post_content;
            }


            wp_enqueue_script( $this->slug . '-photoswipe-script' );
            wp_enqueue_style( $this->slug . '-photoswipe-style' );

        }

        if ($variation_wc_image) {
        	$this->add_render_attribute( 'image-tag-variation', 'class', 'piotnetgrid-image__tag piotnetgrid-image__tag--variation' );
        	$this->add_render_attribute( 'image-tag-variation', 'data-piotnetgrid-variation-image', '' );
        }
		?>
		<div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
			<?php if ( empty( $image_link ) ) : ?>
				<div class="piotnetgrid-image__content">
                    <?php if( !empty($piotnetgrid_pswp)) { ?>
                    <a data-href="<?php echo $image_url; ?>" class="piotnetgrid-photoswipe" data-width="<?php echo $image_meta['width']; ?>" data-height="<?php echo $image_meta['height']; ?>"  itemprop="contentUrl" <?php echo $this->get_render_attribute_string('pswp-data'); ?>>
                        <?php } ?>
                        <img <?php echo $this->get_render_attribute_string('image-tag'); ?>>
                        <?php if (!empty($secondary_image_url)) : ?>
                            <img <?php echo $this->get_render_attribute_string('image-tag-secondary');?>>
                        <?php endif; ?>
                        <?php if (!empty($variation_wc_image)) : ?>
                            <img <?php echo $this->get_render_attribute_string('image-tag-variation');?>>
                        <?php endif; ?>
                        <?php if( !empty($settings['piotnetgrid_photoswipe_enable'])) :

                        ?>
                        <?php if( $title_source != 'none' || $caption_source != 'none' ) : ?>
                            <div class="piotnetgrid-pswp__text" style="display: none;">
                                <?php if( $title_source != 'none' ) : ?>
                                    <div class="piotnetgrid-pswp__title"><?php echo $pswp_title; ?></div>
                                <?php endif; ?>
                                <?php if( $caption_source != 'none' ) : ?>
                                    <div class="piotnetgrid-pswp__caption"><?php echo $pswp_caption; ?> </div>
                                <?php endif; ?>
                            </div>
                        <?php endif; ?>
                        </a>
                        <?php endif; ?>
				</div>
				<?php if ( ! empty( $image_caption ) ) : ?>
					<div class="piotnetgrid-image__caption"><?php echo $image_caption; ?></div>
				<?php endif; ?>
			<?php else : ?>
				<div class="piotnetgrid-image__content">
                    <a target="<?php echo $link_target; ?>" href="<?php echo $image_link; ?>" <?php if (!empty($piotnetgrid_pswp)) : ?> data-href="<?php echo $image_url; ?>" class="piotnetgrid-photoswipe" data-width="<?php echo $image_meta['width']; ?>" data-height="<?php echo $image_meta['height']; ?>"  itemprop="contentUrl" <?php echo $this->get_render_attribute_string('pswp-data'); endif; ?> >
                        <?php if( $title_source != 'none' || $caption_source != 'none' ) : ?>
                            <div class="piotnetgrid-pswp__text" style="display: none;">
                                <?php if( $title_source != 'none' ) : ?>
                                    <div class="piotnetgrid-pswp__title"><?php echo $pswp_title; ?></div>
                                <?php endif; ?>
                                <?php if( $caption_source != 'none' ) : ?>
                                    <div class="piotnetgrid-pswp__caption"><?php echo $pswp_caption; ?> </div>
                                <?php endif; ?>
                            </div>
                        <?php endif; ?>
                        <img <?php echo $this->get_render_attribute_string('image-tag'); ?>>
                        <?php if (!empty($secondary_image_url)) : ?>
                            <img <?php echo $this->get_render_attribute_string('image-tag-secondary'); ?>>
                        <?php endif; ?>
                        <?php if (!empty($variation_wc_image)) : ?>
                            <img <?php echo $this->get_render_attribute_string('image-tag-variation'); ?>>
                        <?php endif; ?>
                    </a>
				</div>
				<?php if ( ! empty( $image_caption ) ) : ?>
					<div class="piotnetgrid-image__caption"><?php echo $image_caption; ?></div>
				<?php endif; ?>
			<?php endif; ?>
		</div>
		<?php
	}

	public function live_preview() {
		
	}

}
