<?php

if (!defined('ABSPATH')) {
    exit;
}

class PiotnetgridDynamicImageOxygen extends OxyEl {

    public $css_added = false;

    function init() {
        $this->El->useAJAXControls();
    }

    function name() {
        return __('Dynamic Image', 'piotnetgrid');
    }

    function slug() {
        return "piotnetgrid_dynamic_image_oxygen";
    }

    function button_place() {
        return "piotnetgrid::basic";
    }

    function button_priority() {
        return '';
    }

    function isBuilderEditorActive() {
        if (isset($_GET['oxygen_iframe']) || defined('OXY_ELEMENTS_API_AJAX')) {
            return true;
        }

        return false;
    }

    function icon() {
        return plugin_dir_url(__FILE__) . '../../../assets/icons/i-oxygen-dynamic-image.svg';
    }

    private function get_additional_image_sizes() {

        global $_wp_additional_image_sizes;

        $image_sizes = array(
            'full' => 'Full',
        );

        foreach (get_intermediate_image_sizes() as $_size) {
            if (in_array($_size, array('thumbnail', 'medium', 'medium_large', 'large'))) {
                $image_sizes[$_size] = ucwords(str_replace('_', ' ', $_size)) . ' - ' . get_option("{$_size}_size_w") . ' x ' . get_option("{$_size}_size_h");
            } elseif (isset($_wp_additional_image_sizes[$_size])) {
                $image_sizes[$_size] = ucwords(str_replace('_', ' ', $_size)) . ' - ' . $_wp_additional_image_sizes[$_size]['width'] . ' x ' . $_wp_additional_image_sizes[$_size]['height'];
            }
        }
        return $image_sizes;
    }

    function controls() {

        $dynamic_data = [
            '' => __('None', 'piotnetgrid'),
            'featured_image' => __('Featured Image', 'piotnetgrid'),
            'author_profile_picture' => __('Author Profile Picture', 'piotnetgrid'),
            'post_custom_field' => __('Post Custom Field', 'piotnetgrid'),
            'acf_field' => __('ACF Field', 'piotnetgrid'),
            'metabox_field' => __('Metabox Field', 'piotnetgrid'),
            'pods_field' => __('Pods Field', 'piotnetgrid'),
            'toolset_field' => __('Toolset Field', 'piotnetgrid'),
            'jetengine_field' => __('JetEngine Field', 'piotnetgrid'),
        ];

        $image_sizes = $this->get_additional_image_sizes();

        // $setting_section = $this->addControlSection("setting_section", __("Setting"), "assets/icon.png", $this);

        $this->addOptionControl(
            array(
                'name' => __('Dynamic data', 'piotnetgrid'),
                'slug' => 'dynamic_data',
                'type' => 'dropdown',
                'value' => $dynamic_data,
                'default' => '',
                'css' => false
            )
        )->rebuildElementOnChange();

        $this->addOptionControl(
            [
                'type' => 'textfield',
                'name' => __('Field Name', 'piotnetgrid'),
                'slug' => 'field_name',
                'default' => '',
                'css' => false,
                'condition' => 'dynamic_data=post_custom_field||dynamic_data=acf_field||dynamic_data=metabox_field||dynamic_data=pods_field||dynamic_data=toolset_field||dynamic_data=jetengine_field||dynamic_data=jetengine_field',
            ]
        )->rebuildElementOnChange();

        $this->addOptionControl(
            array(
                'name' => __('Image Size', 'piotnetgrid'),
                'slug' => 'image_size',
                'type' => 'dropdown',
                'value' => $image_sizes,
                'default' => 'full',
                'css' => false,
                'condition' => 'dynamic_data!=author_profile_picture'
            )
        )->rebuildElementOnChange();

        $this->addOptionControl(
            array(
                'name' => __('Link', 'piotnetgrid'),
                'slug' => 'image_link_type',
                'type' => 'dropdown',
                'value' => [
                    '' => 'None',
                    'post_url' => 'Post URL'
                ],
                'default' => '',
                'css' => false,
            )
        );

        $this->addOptionControl(
            array(
                'name' => __('Link Target', 'piotnetgrid'),
                'slug' => 'image_link_target',
                'type' => 'dropdown',
                'value' => [
                    '_self' => 'Self',
                    '_blank' => 'Blank',
                ],
                'default' => '_self',
                'css' => false,
                'condition' => 'image_link_type!=file'
            )
        );

        $this->addOptionControl(
            array(
                "type" => "textfield",
                "name" => __("Image Size"),
                "slug" => "image_size_author_profile_picture",
                "default" => "150",
                'condition' => 'dynamic_data=author_profile_picture'
            )
        )->rebuildElementOnChange();

        $this->addStyleControl(
            array(
                "name" => __("Aspect Ratio"),
                "selector" => ".piotnetgrid-image::after",
                "property" => "padding-top",
                "units" => "%",
                'description' => 'Aspect Ratio = Height / Width * 100. E.g Width = 100, Height = 100 => Ratio = 100; Width = 100, Height = 50 => Ratio = 50. Please set width and max-width to 100% in Advanced Tab if you use this option'
            )
        )->rebuildElementOnChange();
    }

    function render($options, $defaults, $content) {
        global $term;
        $post_id = get_the_ID();
        $image_link = '';
        $link_target = ! empty( $options['image_link_target'] ) ? $options['image_link_target'] : '';

        if ( $options['image_link_type'] === 'post_url' ) {
            $image_link = get_permalink($post_id);
        }

        switch ($options['dynamic_data']) {
            case 'featured_image':
                $image_url = wp_get_attachment_image_src(get_post_thumbnail_id($post_id), $options['image_size']);
                $image_url = $image_url[0];
                break;

            case 'author_profile_picture':
                $image_size_author_profile_picture = !empty($options['image_size_author_profile_picture']) ? $options['image_size_author_profile_picture'] : 150;
                $image_url = get_avatar_url(get_the_author_meta('ID'), array('size' => $image_size_author_profile_picture));
                break;

            case 'post_custom_field':
                $value = get_post_meta($post_id, $options['field_name'], true);
                if (is_array($value)) {
                    $image_url = wp_get_attachment_image_src(attachment_url_to_postid($value), $options['image_size']);
                } else {
                    $image_url = wp_get_attachment_image_src($value, $options['image_size']);
                }
                $image_url = $image_url[0];

                break;

            case 'toolset_field':
                $value = get_post_meta($post_id, 'wpcf-' . $options['field_name'], true);
                if (is_array($value)) {
                    $image_url = wp_get_attachment_image_src(attachment_url_to_postid($value), $options['image_size']);
                } else {
                    $image_url = wp_get_attachment_image_src($value, $options['image_size']);
                }
                $image_url = $image_url[0];
                break;

            case 'metabox_field':
                if (function_exists('rwmb_meta_legacy')) {
                    $value = rwmb_meta_legacy($options['field_name'], array('limit' => 1), $post_id);
                }
                $image_url = wp_get_attachment_image_src($value, $options['image_size']);
                $image_url = $image_url[0];
                break;

            case 'pods_field':
                if (function_exists('pods_field')) {
                    $value = pods_field(get_post_type(), $post_id, $options['field_name'], true);
                }
                if (is_array($value)) {
                    $image_url = wp_get_attachment_image_src(attachment_url_to_postid($value['guid']), $options['image_size']);
                } else {
                    $image_url = wp_get_attachment_image_src($value, $options['image_size']);
                }
                $image_url = $image_url[0];
                break;

            case 'jetengine_field':
                $value = get_post_meta($post_id, $options['field_name'], true);
                $image_url = wp_get_attachment_image_src($value, $options['image_size']);
                $image_url = $image_url[0];
                break;

            case 'acf_field':
                if (function_exists('get_field')) {
                    $value = get_field($options['field_name'], $post_id);
                }
                if (is_array($value)) {
                    $image_url = wp_get_attachment_image_src(attachment_url_to_postid($value['url']), $options['image_size']);
                } else {
                    $image_url = wp_get_attachment_image_src($value, $options['image_size']);
                }
                $image_url = $image_url[0];

                break;
        }

        ?>
        <div class="piotnetgrid-image <?php if (!empty($options['slug_piotnetgridimageafter_padding_top'])) { echo ' piotnetgrid-image--aspect-ratio'; } ?> ">
        <?php if ( !empty( $image_link ) ) : ?>
            <div class="piotnetgrid-image__content">
                <a target="<?php echo $link_target; ?>" href="<?php echo $image_link; ?>">
                    <img class="piotnetgrid-image__tag" src="<?php echo $image_url; ?>">
                </a>
            </div>
        <?php else: ?>
            <div class="piotnetgrid-image__content">
                    <img class="piotnetgrid-image__tag" src="<?php echo $image_url; ?>">
            </div>
        <?php endif; ?>
        </div>
        <?php
    }
}

new PiotnetgridDynamicImageOxygen();