<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Product_Badge extends Base_Widget_piotnetgrid {
    public function get_type() {
        return 'piotnetgrid-product-badge';
    }

    public function get_class_name() {
        return 'piotnetgrid_Product_Badge';
    }

    public function get_title() {
        return 'Product Badge';
    }

    public function get_icon() {
        return [
            'type' => 'image',
            'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-product-badge.svg',
        ];
    }

    public function get_categories() {
        return [ 'piotnetgrid' ];
    }

    public function get_keywords() {
        return [ 'badge' ];
    }

    public function register_controls() {
        $this->start_tab( 'settings', 'Settings' );
        $this->start_section( 'text_settings_section', 'Settings' );
        $this->add_setting_controls();

        $this->start_tab( 'style', 'Style' );
        $this->start_section( 'text_styles_section', 'Style' );
        $this->add_style_controls();

        $this->add_advanced_style_tab();

        $this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
        $this->add_custom_class_id();

        $this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
        $this->add_custom_css_controls();

        $this->add_animation_settings();

        $this->add_attributes_settings();

        $this->start_section( 'conditional_visibility', 'Conditional Visibility' );
        $this->add_conditional_visibility_controls();

        return $this->structure;
    }

    private function add_setting_controls() {

        $this->add_control(
            'product_badge_product_status',
            [
                'type'        => 'select',
                'label'       => __( 'Product Status', 'piotnetgrid' ),
                'default' => 'text',
                'options'     => [
                    'sale' => 'Sale',
                    'non-sale' => 'Non-sale',
                    'out-of-stock' => 'Out-of-stock',
                    'featured' => 'Featured',
                ],
            ]
        );
        $this->add_control(
            'product_badge_type',
            [
                'type'        => 'select',
                'label'       => __( 'Type', 'piotnetgrid' ),
                'default' => 'text',
                'options'     => [
                    'text' => 'Text',
                    'icon' => 'Icon',
                    'image' => 'Image',
                ],
            ]
        );

        $this->add_control(
            'product_badge_text',
            [
                'type'        => 'text',
                'label'       => __( 'Text','piotnetgrid' ),
                'value'       => 'Sale',
                'label_block' => true,
                'placeholder' => '',
                'dynamic'     => true,
                'condition'  => [
                    'product_badge_type'=> 'text'
                ],
            ]
        );

        $this->add_control(
            'product_badge_icon',
            [
                'type'           => 'icon',
                'label'          => __( 'Icon','piotnetgrid' ),
                'value'          => 'fas fa-star',
                'options_source' => 'fontawesome',
                'condition'  => [
                    'product_badge_type'=> 'icon'
                ],
            ]
        );

        $this->add_control(
            'product_badge_image',
            [
                'type'        => 'media',
                'label'       => __( 'Choose Image', 'piotnetgrid' ),
                'value'       => '',
                'label_block' => true,
                'placeholder' => '',
                'condition'  => [
                    'product_badge_type'=> 'image'
                ],
            ]
        );
    }

    private function add_style_controls() {
       
        $this->add_control(
            '',
            [
                'type' => 'heading-tab',
                'tabs' => [
                    [
                        'name'   => 'product_badge_normal_tab',
                        'title'  => __( 'NORMAL', 'piotnetgrid' ),
                        'active' => true,
                    ],
                    [
                        'name'  => 'product_badge_hover_tab',
                        'title' => __( 'HOVER', 'piotnetgrid' ),
                    ],
                ],
            ]
        );
        $normal_controls = $this->add_product_badge_style_controls(
            'normal',
            [
                'selectors' => '{{WRAPPER}} .piotnetgrid-product-badge__inner',
            ]
        );
        $this->add_control(
            'product_badge_normal_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Normal', 'piotnetgrid' ),
                'value'          => '',
                'active'         => true,
                'controls'       => $normal_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );

        $hover_controls = $this->add_product_badge_style_controls(
            'hover',
            [
                'selectors' => '{{WRAPPER}} .piotnetgrid-product-badge__inner:hover',
            ]
        );
        $this->add_control(
            'product_badge_hover_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Hover', 'piotnetgrid' ),
                'value'          => '',
                'controls'       => $hover_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );
       
    }

    private function add_product_badge_style_controls(string $name, $args = []) {
        $wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
        $previous_controls = $this->new_group_controls();
        $this->add_control(
            'badge_text_color_'.$name,
            [
                'label'       => __( 'Text Color', 'piotnetgrid' ),
                'type'        => 'color',
                'label_block' => true,
                'selectors'   => [
                    $wrapper => 'color: {{VALUE}};',
                ],

                'conditions' => [
                    'terms' => [
                        [
                            'name'     => 'product_badge_type',
                            'operator' => 'in',
                            'value'    => [
                                'text',
                                'icon',
                            ],
                        ],
                    ],
                ],
            ]
        );
        $this->add_text_typography_controls(
            'badge_text_typography_'.$name,
            [
                'selectors' => $wrapper,
            ]
        );
        $this->add_responsive_control(
            'badge_text_align_'.$name,
            [
                'type'        => 'select',
                'label'       => __( 'Text Align', 'piotnetgrid' ),
                'label_block' => true,
                'value'       => '',
                'options'     => [
                    ''       => __( 'Default', 'piotnetgrid' ),
                    'left'   => __( 'Left', 'piotnetgrid' ),
                    'center' => __( 'Center', 'piotnetgrid' ),
                    'right'  => __( 'Right', 'piotnetgrid' ),
                ],
                'selectors'   => [
                    $wrapper => 'text-align: {{VALUE}};',
                ],
            ]
        );
        $this->add_control(
            'badge_background_color_'.$name,
            [
                'type'        => 'color',
                'label'       => __( 'Background Color', 'piotnetgrid' ),
                'label_block' => true,
                'selectors'   => [
                    $wrapper => 'background-color: {{VALUE}};',
                ],
            ]
        );
        $this->add_control(
            'badge_border_type_'.$name,
            [
                'type'      => 'select',
                'label'     => __( 'Border Type', 'piotnetgrid' ),
                'value'     => '',
                'options'   => [
                    ''       => 'None',
                    'solid'  => 'Solid',
                    'double' => 'Double',
                    'dotted' => 'Dotted',
                    'dashed' => 'Dashed',
                    'groove' => 'Groove',
                ],
                'selectors' => [
                    $wrapper => 'border-style:{{VALUE}};',
                ],
            ]
        );
        $this->add_control(
            'badge_border_color_'.$name,
            [
                'type'        => 'color',
                'label'       => __( 'Border Color', 'piotnetgrid' ),
                'value'       => '',
                'label_block' => true,
                'selectors'   => [
                    $wrapper => 'border-color: {{VALUE}};',
                ],
                'conditions'  => [
                    [
                        'name'     => 'badge_border_type_'.$name,
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );
        $this->add_responsive_control(
            'badge_border_width_'.$name,
            [
                'type'        => 'dimensions',
                'label'       => __( 'Border Width', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px', '%', 'em' ],
                'selectors'   => [
                    $wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'  => [
                    [
                        'name'     => 'badge_border_type_'.$name,
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );
        $this->add_responsive_control(
            'badge_border_radius_'.$name,
            [
                'type'        => 'dimensions',
                'label'       => __( 'Border Radius', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'   => [
                    $wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
        $this->add_responsive_control(
            'badge_width_'.$name,
            [
                'type'        => 'slider',
                'label'       => __( 'Width', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min'  => 1,
                        'max'  => 1000,
                        'step' => 1,
                    ],
                ],
                'selectors'   => [
                    $wrapper => 'width:{{SIZE}}{{UNIT}};',
                ],
            ]
        );
        $this->add_responsive_control(
            'badge_padding_'.$name,
            [
                'type'        => 'dimensions',
                'label'       => __( 'Padding', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'   => [
                    $wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
        $this->add_responsive_control(
            'badge_margin_'.$name,
            [
                'type'        => 'dimensions',
                'label'       => __( 'Margin', 'piotnetforms' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'   => [
                    $wrapper => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        return $this->get_group_controls( $previous_controls );
    }

    public function render($editor = false, $args_additional = []) {
        $settings = $this->settings;
        $product_status = $settings['product_badge_product_status'];
        global $product;
        $this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-product-badge' );
        ?>
        <div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?> >
            <?php if ( $editor ) : ?>
                <div class="piotnetgrid-product-badge__inner">
                    <?php if ( $settings['product_badge_type'] == 'text' ) : ?>
                        <?php echo $settings['product_badge_text']; ?>
                    <?php elseif ( $settings['product_badge_type'] == 'icon' ) : ?>
                        <i class="<?php echo $settings['product_badge_icon']; ?>"></i>
                    <?php elseif ( $settings['product_badge_type'] == 'image' ) : ?>
                        <img src="<?php echo $settings['product_badge_image']['url']; ?>">
                    <?php endif; ?>
                </div>
            <?php else : ?>
                <?php if ( $product ) : ?>
                    <?php if ( ( $product->is_on_sale() && $product_status == 'sale') || ( ! $product->is_on_sale() && $product_status == 'non-sale')|| ( ! $product->is_in_stock() && $product_status == 'out-of-stock')|| ( $product->is_featured() && $product_status == 'featured')  ) : ?>
                        <div class="piotnetgrid-product-badge__inner">
                            <?php if ( $settings['product_badge_type'] == 'text' ) : ?>
                                <?php echo $settings['product_badge_text']; ?>
                            <?php elseif ( $settings['product_badge_type'] == 'icon' ) : ?>
                                <i class="<?php echo $settings['product_badge_icon']; ?>"></i>
                            <?php elseif ( $settings['product_badge_type'] == 'image' ) : ?>
                                <img src="<?php echo $settings['product_badge_image']['url']; ?>">
                            <?php endif; ?>
                        </div>
                    <?php endif; ?>
                <?php endif; ?>
            <?php endif; ?>
        </div>
        <?php
    }

    public function live_preview() {

    }
}