<?php
if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Product_Rating extends Base_Widget_piotnetgrid {
	public function get_type() {
		return 'product rating';
	}

	public function get_class_name() {
		return 'piotnetgrid_Product_Rating';
	}

	public function get_title() {
		return 'Product Rating';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-product-rating.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'grid' ];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'product_rating_settings_section', 'Settings' );
		$this->add_setting_controls();

		$this->add_advanced_style_tab();

		$this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
		$this->add_custom_class_id();

		$this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
		$this->add_custom_css_controls();

		$this->add_animation_settings();

		$this->add_attributes_settings();

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		return $this->structure;
	}

	private function add_setting_controls() {
		
        $this->add_responsive_control(
			'star_rating_alignment',
			[
				'type'         => 'select',
				'label'        => __( 'Alignment', 'piotnetgrid' ),
				'value'        => 'left',
				'label_block'  => true,
				'options'      => [
					'flex-start' => __( 'Left', 'piotnetgrid' ),
					'center'  => __( 'Center', 'piotnetgrid' ),
					'flex-end'   => __( 'Right', 'piotnetgrid' ),
				],
				'selectors'    => [
					'{{WRAPPER}} .woocommerce-product-rating' => 'justify-content: {{VALUE}}',
				],
			]
		);

		$this->add_style_controls();

	}

	private function add_style_controls() {

		$this->add_control(
            '',
            [
                'type' => 'heading-tab',
                'tabs' => [
                    [
                        'name'   => 'product_rating_normal_tab',
                        'title'  => __( 'NORMAL', 'piotnetgrid' ),
                        'active' => true,
                    ],
                    [
                        'name'  => 'product_rating_hover_tab',
                        'title' => __( 'HOVER', 'piotnetgrid' ),
                    ],
                ],
            ]
        );
        $normal_controls = $this->add_product_rating_style_controls(
            'normal',
            [
                'selectors' => '{{WRAPPER}} .woocommerce-product-rating .star-rating',
            ]
        );
        $this->add_control(
            'product_rating_normal_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Normal', 'piotnetgrid' ),
                'value'          => '',
                'active'         => true,
                'controls'       => $normal_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );

        $hover_controls = $this->add_product_rating_style_controls(
            'hover',
            [
                'selectors' => '{{WRAPPER}} .woocommerce-product-rating .star-rating:hover',
            ]
        );

        $this->add_control(
            'product_rating_hover_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Hover', 'piotnetgrid' ),
                'value'          => '',
                'controls'       => $hover_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );

	}

	private function add_product_rating_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
        $previous_controls = $this->new_group_controls();

        $this->add_control(
			'stars_color_'.$name,
			[
				'label' => __( 'Star Color', 'piotnetgrid' ),
				'type' => 'color',
				'default' =>'#006afe',
				'selectors' => [
					$wrapper => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'empty_star_color_'.$name,
			[
				'label' => __( 'Empty Star Color', 'piotnetgrid' ),
				'type' => 'color',
				'selectors' => [
					$wrapper.'::before' => 'color: {{VALUE}}',
				],
			]
		);


	    $this->add_responsive_control(
			'star_size_'.$name,
			[
				'type'        => 'slider',
				'label'       => __( 'Star Size', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'%'  => [
						'min'  => 50,
						'max'  => 300,
						'step' => 1,
					],
					'px' => [
						'min'  => 10,
						'max'  => 100,
						'step' => 1,
					],
				],
				'selectors'   => [
					$wrapper => 'font-size:{{SIZE}}{{UNIT}}',
				],
			]
		);

        return $this->get_group_controls( $previous_controls );
    }

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
        $this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-star-rating' );
        global $product;
		if ( !$product ) { ?>
		    <div <?php echo $this->get_render_attribute_string( 'wrapper'); ?> >
		        <div class="woocommerce-product-rating">
			         <div class="star-rating">
				       <span style="width:70%"></span>
			         </div>	
			    </div>
	        </div>
		 
	    <?php

        return '';
    }
       ?>
		<div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
            <?php
				if ( ! wc_review_ratings_enabled() ) {
					return;
				}

				$rating_count = $product->get_rating_count();
				$review_count = $product->get_review_count();
				$average      = $product->get_average_rating();

				if ( $rating_count > 0 ) : ?>

					<div class="woocommerce-product-rating">
						<?php echo wc_get_rating_html( $average, $rating_count ); // WPCS: XSS ok. ?>
					</div>

				<?php endif; ?>
        </div>
        <?php  
	}

	public function live_preview() { ?>
		<%
		  view.add_attribute('wrapper', 'class', 'piotnetgrid-star-rating');
		%>
		<div <%= view.render_attributes('wrapper') %>>
			<div class="woocommerce-product-rating">
				<div class="star-rating">
					<span style="width:70%"></span>
				</div>	
			</div>
	    </div>

		<?php

    }
}  