<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Section extends Division_Base_Widget_piotnetgrid {
	public function get_type() {
		return 'section';
	}

	public function get_class_name() {
		return 'piotnetgrid_Section';
	}

	public function get_title() {
		return 'Section';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-section.svg',
		];
	}

	public function get_categories() {
		return [ 'pafe-form-builder' ];
	}

	public function get_keywords() {
		return [ 'input', 'form', 'field' ];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'section_settings', 'Layout' );
		$this->add_setting_controls();

		$this->start_section( 'section_search_results_section', 'Search Results Section' );
		$this->add_search_results_section_controls();

		$this->start_section( 'collapse_section', 'Collapse Section' );
		$this->add_collapse_section_controls();

		$this->start_section( 'auto_scrolling_effect_section', 'Auto Scrolling Effect' );
		$this->add_auto_scrolling_effect_controls();

		$this->add_advanced_style_tab();

		$this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
		$this->add_custom_class_id();

		$this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
		$this->add_custom_css_controls();

		$this->add_animation_settings();

		$this->add_attributes_settings();

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		return $this->structure;
	}

	private function add_setting_controls() {
		$this->add_control(
			'section_content_width_type',
			[
				'type'        => 'select',
				'label'       => __( 'Content Width', 'piotnetgrid' ),
				'value'       => 'boxed',
				'label_block' => true,
				'options'     => [
					'boxed'      => 'Boxed',
					'full-width' => 'Full Width',
				],
			]
		);
		$this->add_responsive_control(
			'section_content_width',
			[
				'type'        => 'slider',
				'label'       => __( 'Width', 'piotnetgrid' ),
				'value'       => [
					'unit' => 'px',
					'size' => '',
				],
				'label_block' => true,
				'size_units'  => [
					'px' => [
						'min'  => 500,
						'max'  => 1600,
						'step' => 1,
					],
				],
				'selectors'   => [
					'{{WRAPPER}}>.piotnetgrid-section__container' => 'max-width: {{SIZE}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => 'section_content_width_type',
						'operator' => '==',
						'value'    => 'boxed',
					],
				],
			]
		); // FIXME conditions not working

		$this->add_control(
			'section_link_url',
			[
				'type'        => 'text',
				'label'       => __( 'Section Link URL', 'piotnetgrid' ),
				'dynamic'     => true,
				'label_block' => true,
				'description' => __( 'Please make sure there are not anchor tags in this section, if not the layout will be broken', 'piotnetgrid' ),
			]
		);

		$this->add_control(
			'section_link_target',
			[
				'type'       => 'select',
				'label'      => __( 'Section Link Target', 'piotnetgrid' ),
				'value'      => '',
				'options'    => [
					'_self'  => 'Self',
					'_blank' => 'Blank',
				],
				'conditions' => [
					[
						'name'     => 'section_link_url',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
	}

	private function add_search_results_section_controls() {
		$this->add_control(
			'search_results_section_enable',
			[
				'type'         => 'switch',
				'label'        => __( 'Enable', 'piotnetgrid' ),
				'return_value' => 'true',
				'default'      => '',
			]
		);

		$this->add_control(
			'search_results_section_facet_name',
			[
				'type'       => 'text',
				'label'      => __( 'Search Facet Name', 'piotnetgrid' ),
				'value'      => '',
				'description'  => __( 'This section only show on the frontend when the Search Facet has value', 'piotnetgrid' ),
				'conditions' => [
					[
						'name'     => 'search_results_section_enable',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
	}

	private function add_collapse_section_controls() {
		$this->add_control(
			'collapse_section_notice',
			[
				'type'    => 'html',
				'label'   => __( 'This feature only works on the frontend', 'piotnetgrid' ),
				'label_block'  => true,
			]
		);

		$this->add_control(
			'collapse_section_devices',
			[
				'type'    => 'select2',
				'label'   => __( 'Collapse on Devices', 'piotnetgrid' ),
				'options' => [
					'mobile' => __( 'Mobile', 'piotnetgrid' ),
					'tablet' => __( 'Tablet', 'piotnetgrid' ),
					'desktop' => __( 'Desktop', 'piotnetgrid' ),
				]
			]
		);

		$this->add_control(
			'collapse_section_button_text',
			[
				'type'         => 'text',
				'label'        => __( 'Collapse Button Text', 'piotnetgrid' ),
				'value' 	   => __( 'Collapse', 'piotnetgrid' ),
			]
		);

		$this->add_responsive_control(
			'collapse_section_button_align',
			[
				'type'         => 'select',
				'label'        => __( 'Collapse Button Alignment', 'piotnetgrid' ),
				'label_block'  => true,
				'value'        => '',
				'options'      => [
					''       => __( 'Default', 'piotnetgrid' ),
					'flex-start'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end'  => __( 'Right', 'piotnetgrid' ),
					'justify'  => __( 'Justify', 'piotnetgrid' ),
				],
				'prefix_class' => 'piotnetgrid%s-align-',
				'render_type'  => 'both',
				'selectors'    => [
					'{{WRAPPER}} .piotnetgrid-button-wrapper' => 'justify-content: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'collapse_section_button_icon',
			[
				'type'           => 'icon',
				'label'          => __( 'Collapse Button Icon', 'piotnetgrid' ),
				'value'          => '',
				'label_block'    => false,
				'placeholder'    => '',
				'options_source' => 'fontawesome',
			]
		);

		$this->add_control(
			'collapse_section_button_icon_position',
			[
				'type'       => 'select',
				'label'      => __( 'Icon Position', 'piotnetgrid' ),
				'value'      => 'left',
				'options'    => [
					'before' => __( 'Before', 'piotnetgrid' ),
					'after'  => __( 'After', 'piotnetgrid' ),
				],
				'conditions' => [
					[
						'name'     => 'collapse_section_button_icon',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);

		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'collapse_section_button_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'collapse_section_button_hover_tab',
						'title' => __( 'HOVER', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_button_style_controls(
			'normal',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button--collapse-section.piotnetgrid-button',
			]
		);
		$this->add_control(
			'collapse_section_button_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_button_style_controls(
			'hover',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-button--collapse-section.piotnetgrid-button:hover',
			]
		);
		$this->add_control(
			'collapse_section_button_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_button_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();
		$this->add_control(
			$name.'_button_color',
			[
				'type'        => 'color',
				'label'       => __( 'Text Color', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_text_typography_controls(
			$name.'_button_text_typography',
			[
				'selectors' => $wrapper,
			]
		);
		$this->add_control(
			$name.'_button_icon_color',
			[
				'type'        => 'color',
				'label'       => __( 'Icon Color', 'piotnetgrid' ),
				'label_block' => true,
				'conditions' => [
					'terms' => [
						[
							'name'     => 'collapse_section_button_icon',
							'operator' => '!=',
							'value'    => '',
						],
					],
				],
				'selectors'   => [
					$wrapper . ' i' => 'color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_icon_spacing',
			[
				'label'      => __( 'Icon Spacing', 'piotnetforms' ),
				'type'       => 'slider',
				'size_units' => [
					'px' => [
						'min'  => 0,
						'max'  => 200,
						'step' => 1,
					],
				],
				'value'      => [
					'unit' => 'px',
					'size' => '',
				],
				'conditions' => [
					'terms' => [
						[
							'name'     => 'collapse_section_button_icon',
							'operator' => '!=',
							'value'    => '',
						],
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-button--after i' => 'margin-left: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-button--before i' => 'margin-right: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			$name.'_button_icon_size',
			[
				'label'      => __( 'Icon Size', 'piotnetforms' ),
				'type'       => 'slider',
				'size_units' => [
					'px' => [
						'min'  => 1,
						'max'  => 50,
						'step' => 1,
					],
				],
				'value'      => [
					'unit' => 'px',
					'size' => '',
				],
				'conditions' => [
					'terms' => [
						[
							'name'     => 'collapse_section_button_icon',
							'operator' => '!=',
							'value'    => '',
						],
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-button--collapse-section i' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'background-color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_type',
			[
				'type'      => 'select',
				'label'     => __( 'Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_button_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_button_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_border_radius',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Border Radius', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->add_responsive_control(
			$name.'_button_padding',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Padding', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px' ],
				'selectors'   => [
					$wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		return $this->get_group_controls( $previous_controls );
	}

	private function add_style_typography_controls() {

		$this->add_responsive_control(
			'section_text_align',
			[
				'type'         => 'select',
				'label'        => 'Text Alignment',
				'label_block'  => true,
				'options'      => [
					''        => __( 'Default', 'piotnetgrid' ),
					'left'    => __( 'Left', 'piotnetgrid' ),
					'center'  => __( 'Center', 'piotnetgrid' ),
					'right'   => __( 'Right', 'piotnetgrid' ),
					'justify' => __( 'Justified', 'piotnetgrid' ),
				],
				'prefix_class' => 'piotnetgrid%s-align-',
				'selectors'    => [
					'{{WRAPPER}}' => 'text-align: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'section_text_color',
			[
				'type'      => 'color',
				'label'     => 'Text Color',
				'selectors' => [
					'{{WRAPPER}}' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_text_typography_controls(
			'section_text_typography',
			[
				'selectors' => '{{WRAPPER}}',
			]
		);
	}

	private function add_auto_scrolling_effect_controls() {
		$this->add_control(
			'auto_scrolling_effect_enable',
			[
				'type'         => 'switch',
				'label'        => __( 'Enable', 'piotnetgrid' ),
				'return_value' => 'true',
				'default'      => '',
			]
		);
	}

	public function render_start( $editor = false ) {
			$settings = $this->settings;
		?>
			<?php if ( $editor ) : ?>
				<div class="piotnetgrid-section__controls">
					<div class="piotnetgrid-section__controls-item piotnetgrid-section__controls-item--edit" title="Edit" data-piotnetgrid-control-edit>
						<i class="fas fa-th"></i>
					</div>
					<div class="piotnetgrid-section__controls-item piotnetgrid-section__controls-item--duplicate" title="Duplicate" data-piotnetgrid-control-duplicate>
						<i class="far fa-clone"></i>
					</div>
					<div class="piotnetgrid-section__controls-item piotnetgrid-section__controls-item--remove" title="Delete" data-piotnetgrid-control-remove>
						<i class="fas fa-times"></i>
					</div>
				</div>
			<?php endif; ?>
			<div class="piotnetgrid-background-overlay"></div>
			<?php
				$collapse_section_devices = !empty($settings['collapse_section_devices']) ? $settings['collapse_section_devices'] : [];
				if (!empty($collapse_section_devices)) :
					$this->add_render_attribute( 'collapse_section_button_wrapper', 'class', 'piotnetgrid-button-wrapper' );

					$this->add_render_attribute( 'collapse_section_button', 'class', 'piotnetgrid-button');
					$this->add_render_attribute( 'collapse_section_button', 'class', 'piotnetgrid-button--collapse-section' );
					$this->add_render_attribute( 'collapse_section_button', 'data-piotnetgrid-collapse-section-button', '' );

					foreach ($collapse_section_devices as $collapse_section_device) {
						$this->add_render_attribute( 'collapse_section_button', 'class', 'piotnetgrid-button--collapse-section-' . $collapse_section_device );
					}

					$button_text = !empty( $settings['collapse_section_button_text'] ) ? $settings['collapse_section_button_text'] : '';
					$button_icon = !empty( $settings['collapse_section_button_icon'] ) ? '<i class="' . $settings['collapse_section_button_icon'] . '"></i>' : '';
					$button_icon_before = !empty($button_icon) && $settings['collapse_section_button_icon_position'] == 'before' ? $button_icon : '';
					$button_icon_after = !empty($button_icon) && $settings['collapse_section_button_icon_position'] == 'after' ? $button_icon : '';

					if (!empty($button_icon)) {
						wp_enqueue_style( $this->slug . '-fontawesome-style' );
					}

					if (!empty($button_icon_before)) {
						$this->add_render_attribute( 'collapse_section_button', 'class', 'piotnetgrid-button--before' );
					}

					if (!empty($button_icon_after)) {
						$this->add_render_attribute( 'collapse_section_button', 'class', 'piotnetgrid-button--after' );
					}

					if (empty($button_text) && !empty($button_icon)) {
						$this->add_render_attribute( 'collapse_section_button', 'class', 'piotnetgrid-button--icon-only' );
					}

					if (!empty($settings['collapse_section_button_align_responsive_desktop'])) {
						if ($settings['collapse_section_button_align_responsive_desktop'] == 'justify') {
							$this->add_render_attribute( 'collapse_section_button', 'class', 'piotnetgrid-button--justify' );
						}
					}	
			?>
				<div <?php echo $this->get_render_attribute_string( 'collapse_section_button_wrapper' ); ?>>
					<span <?php echo $this->get_render_attribute_string( 'collapse_section_button' ); ?>><?php echo $button_icon_before; ?><?php echo $button_text; ?><?php echo $button_icon_after; ?></span>
				</div>
				<div class="piotnetgrid-collapse-section-bg" data-piotnetgrid-collapse-section-bg>
					<svg width="20px" height="20px" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg"><g id="Group-14" fill="#ffffff" fill-rule="nonzero"><polygon id="Line" transform="translate(10.047288, 9.952712) rotate(45.000000) translate(-10.047288, -9.952712) " points="9.04728777 -2.85541277 9.04728777 22.7608372 11.0472878 22.7608372 11.0472878 -2.85541277"></polygon><polygon id="Line-2" transform="translate(10.000000, 10.000000) rotate(45.000000) translate(-10.000000, -10.000000) " points="-2.875 11 22.875 11 22.875 9 -2.875 9"></polygon></g></svg>
				</div>
			<?php endif; ?>
			<?php
				$this->add_render_attribute( 'section_container', 'class', 'piotnetgrid-section__container' );

				if ( $settings['section_content_width_type'] == 'full-width' ) {
					$this->add_render_attribute( 'section_container', 'class', 'piotnetgrid-section__container-full' );
				}

				if ( $editor ) {
					$this->add_render_attribute( 'section_container', 'data-piotnetgrid-section-container', '' );
					$this->add_render_attribute( 'section_container', 'data-piotnetgrid-inner-html', '' );
				}

				if (!empty($collapse_section_devices)) {
					$this->add_render_attribute( 'section_container', 'data-piotnetgrid-collapse-section', '' );
					foreach ($collapse_section_devices as $collapse_section_device) {
						$this->add_render_attribute( 'section_container', 'class', 'piotnetgrid-collapse-section--' . $collapse_section_device );
					}
				}

				if (!empty($settings['auto_scrolling_effect_enable'])) {
					$this->add_render_attribute( 'section_container', 'data-piotnetgrid-auto-scrolling', '' );
				}					
			?>
			<div <?php echo $this->get_render_attribute_string( 'section_container' ); ?>>
		<?php
	}

	public function render_end() {
		?>
			</div>
		<?php
	}

	public function render($editor = false, $args_additional = []) {
		?>
		<?php
	}

	public function live_preview() {
		?>
		<?php
	}
}
