<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Single_Settings extends Base_Widget_piotnetgrid {

	protected $is_add_conditional_logic = false;

	public $is_global = true;

	public function get_type() {
		return 'single-settings';
	}

	public function get_class_name() {
		return 'piotnetgrid_Single_Settings';
	}

	public function get_title() {
		return 'Single Settings';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-field.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'settings' ];
	}

	public function register_controls() {
		$post_id = !empty($_GET['post']) ? $_GET['post'] : '';

		if (get_post_type($post_id) == 'piotnetgrid-archive') {
			$this->start_tab( 'settings', 'Archive Template Settings' );
			$this->start_section( 'section_display_conditions', 'Display Conditions' );
			$this->add_display_conditions();
		}
		
		if (get_post_type($post_id) == 'piotnetgrid-card') {
			$this->start_tab( 'settings', 'Preview Card Settings' );
			$this->start_section( 'section_preview_post', 'Preview Post' );
			$this->add_preview_post_controls();
		}

		return $this->structure;
	}

	private function add_preview_post_controls() {
		$this->add_control(
			'preview_post',
			[
				'type'        => 'select2',
				'label'       => __( 'Select a Post to Preview', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'posts',
				'multiple' => false,
				'description' => __( 'Please click Save button and then reload this page', 'piotnetgrid' ),
			]
		);
	}

	private function add_display_conditions() {

		//repeater

		$this->new_group_controls();

		$this->add_control(
			'condition_type',
			[
				'label'   => __( 'Condition Type', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => [
					'include' => __( 'Include', 'piotnetgrid' ),
					'exclude'  => __( 'Exclude', 'piotnetgrid' ),
				],
				'value'   => 'include',
			]
		);

		$post_types       = get_post_types( [], 'objects' );
		$archive_list = [
			'all_archives' => 'All Archives',
		];

		foreach ( $post_types as $post_type ) {
			if ($post_type->public) {
				$post_type_name = $post_type->name;
				switch ($post_type_name) {
					case 'piotnetgrid':
					case 'piotnetgrid-card':
					case 'piotnetgrid-facet':
					case 'piotnetgrid-archive':
					case 'page':
						break;
					
					default:
						$taxonomies = get_object_taxonomies( $post_type_name );
						if (!empty($taxonomies)) {

							$post_type_label = $post_type->name == 'product' ? $post_type->label . ' Archive' . ' ' . __('(Shop Page)','piotnetgrid') : $post_type->label . ' Archive';

							$archive_taxonomies = [
								'label' => $post_type->label . ' Archive',
								'options' => [
									$post_type->name . '_archive_post_type' => $post_type_label,
								],
							];

							foreach ($taxonomies as $taxonomy) {
								$taxonomy_object = get_taxonomy( $taxonomy );
								$archive_taxonomies['options'][$taxonomy . '_archive_taxonomy'] = $taxonomy_object->label;
							}

							$archive_list[ $post_type->name ] = $archive_taxonomies;
						} else {
							$archive_list[ $post_type->name . '_archive_post_type' ] = $post_type->label . ' Archive';
						}
						
						break;
				}
			}
		}

		$this->add_control(
			'condition_archive',
			[
				'label'   => __( 'Archive', 'piotnetgrid' ),
				'type'    => 'select',
				'options' => $archive_list,
				'value'   => 'all_archives',
			]
		);

		$this->add_control(
			'condition_terms',
			[
				'type'        => 'select2',
				'label'       => __( 'Terms', 'piotnetgrid' ),
				'label_block' => true,
				'query' => 'terms',
				'conditions' => [
					'terms' => [
						[
							'name'     => 'condition_archive',
							'operator' => '!=',
							'value'    => 'all_archives',
						],
					],
				],
			]
		);

        $this->add_control(
            'repeater_id',
            [
                'type' => 'hidden',
            ],
            [
                'overwrite' => 'true',
            ]
        );
        
		$repeater_items = $this->get_group_controls();

		$this->new_group_controls();

		$this->add_control(
			'',
			[
				'type'           => 'repeater-item',
				'remove_label'   => __( 'Remove Item', 'piotnetgrid' ),
				'controls'       => $repeater_items,
				'controls_query' => '.piotnetgrid-control-repeater-field',
			]
		);

		$repeater_list = $this->get_group_controls();

		$this->add_control(
			'display_conditions',
			[
				'type'           => 'repeater',
				'label'          => __( 'Conditions', 'piotnetgrid' ),
				'value'          => '',
				'label_block'    => true,
				'add_label'      => __( 'Add Item', 'piotnetgrid' ),
				'controls'       => $repeater_list,
				'controls_query' => '.piotnetgrid-control-repeater-list',
			]
		);

		//end repeater

	}

	public function render($editor = false, $args_additional = []) {
	}

	public function live_preview() {
	}
}
