<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Term_List extends Base_Widget_piotnetgrid {
    public function get_type() {
        return 'term_list';
    }

    public function get_class_name() {
        return 'piotnetgrid_Term_List';
    }

    public function get_title() {
        return 'Term List';
    }

    public function get_icon() {
        return [
            'type' => 'image',
            'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-term-list.svg',
        ];
    }

    public function get_categories() {
        return [ 'piotnetgrid' ];
    }

    public function get_keywords() {
        return [ 'term' ];
    }

    public function register_controls() {
        $this->start_tab( 'settings', 'Settings' );
        $this->start_section( 'text_settings_section', 'Settings' );
        $this->add_setting_controls();

        $this->start_tab( 'style', 'Style' );
        $this->start_section( 'text_styles_section', 'Style' );
        $this->add_style_controls();

        $this->add_advanced_style_tab();

        $this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
        $this->add_custom_class_id();

        $this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
        $this->add_custom_css_controls();

        $this->add_animation_settings();

        $this->add_attributes_settings();

        $this->start_section( 'conditional_visibility', 'Conditional Visibility' );
        $this->add_conditional_visibility_controls();

        return $this->structure;
    }

    private function add_setting_controls() {

        $post_types = get_post_types( [], 'objects' );
        $taxonomies = [];

        foreach ( $post_types as $post_type ) {
            $taxonomies_of_post_type = get_object_taxonomies( $post_type->name, 'objects' );
            foreach ($taxonomies_of_post_type as $taxonomy) {

                $taxonomies[$taxonomy->name] = $taxonomy->label . ' (' . $post_type->name . ')';
            }
        }

        $this->add_control(
            'piotnet_grid_taxonomies_list',
            [
                'type'        => 'select',
                'label'       => __( 'Taxonomy List', 'piotnetgrid' ),
                'options'     => $taxonomies,
            ]
        );
        $this->add_control(
            'piotnet_grid_term_link',
            [
                'type'         => 'switch',
                'label'        => __( 'Term URL', 'piotnetgrid' ),
                'value'        => '',
                'label_on'     => 'Yes',
                'label_off'    => 'No',
                'return_value' => 'yes',
            ]
        );
    }

    private function add_style_controls() {
        $this->add_responsive_control(
            'term_list_text_align',
            [
                'type'         => 'select',
                'label'        => __( 'Alignment', 'piotnetgrid' ),
                'value'        => 'left',
                'label_block'  => true,
                'options'      => [
                    'flex-start' => __( 'Left', 'piotnetgrid' ),
                    'center'  => __( 'Center', 'piotnetgrid' ),
                    'flex-end'   => __( 'Right', 'piotnetgrid' ),
                    'space-between' => __( 'Space-Between', 'piotnetgrid' ),
                    'space-around' => __( 'Space-Around', 'piotnetgrid' ),
                ],
                'selectors'    => [
                    '{{WRAPPER}} .piotnetgrid-grid-term-list' => 'justify-content: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'term_list_margin',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Margin', 'piotnetforms' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'   => [
                    '{{WRAPPER}} .piotnetgrid-grid-term-list' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            '',
            [
                'type' => 'heading-tab',
                'tabs' => [
                    [
                        'name'   => 'item_normal_tab',
                        'title'  => __( 'NORMAL', 'piotnetgrid' ),
                        'active' => true,
                    ],
                    [
                        'name'  => 'item_hover_tab',
                        'title' => __( 'HOVER', 'piotnetgrid' ),
                    ],
                ],
            ]
        );

        $normal_controls = $this->add_terms_style_controls(
            'normal',
            [
                'selectors' => '{{WRAPPER}} .piotnetgrid-grid-term-list-item__inner',
            ]
        );
        $this->add_control(
            'item_normal_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Normal', 'piotnetgrid' ),
                'value'          => '',
                'active'         => true,
                'controls'       => $normal_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );

        $hover_controls = $this->add_terms_style_controls(
            'hover',
            [
                'selectors' => '{{WRAPPER}} .piotnetgrid-grid-term-list-item__inner:hover',
            ]
        );
        $this->add_control(
            'item_hover_tab',
            [
                'type'           => 'content-tab',
                'label'          => __( 'Hover', 'piotnetgrid' ),
                'value'          => '',
                'controls'       => $hover_controls,
                'controls_query' => '.piotnetgrid-start-controls-tab',
            ]
        );
    }

    private function add_terms_style_controls(string $name, $args = []) {
        $wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
        $previous_controls = $this->new_group_controls();
        $this->add_control(
            $name.'_item_color',
            [
                'label'       => __( 'Text Color', 'piotnetgrid' ),
                'type'        => 'color',
                'label_block' => true,
                'selectors'   => [
                    $wrapper => 'color: {{VALUE}};',
                ],
            ]
        );
        $this->add_text_typography_controls(
            $name.'_item_text_typography',
            [
                'selectors' => $wrapper,
            ]
        );
        $this->add_control(
            $name.'_item_background_color',
            [
                'type'        => 'color',
                'label'       => __( 'Background Color', 'piotnetgrid' ),
                'label_block' => true,
                'selectors'   => [
                    $wrapper => 'background-color: {{VALUE}};',
                ],
            ]
        );
        $this->add_control(
            $name.'_item_border_type',
            [
                'type'      => 'select',
                'label'     => __( 'Border Type', 'piotnetgrid' ),
                'value'     => '',
                'options'   => [
                    ''       => 'None',
                    'solid'  => 'Solid',
                    'double' => 'Double',
                    'dotted' => 'Dotted',
                    'dashed' => 'Dashed',
                    'groove' => 'Groove',
                ],
                'selectors' => [
                    $wrapper => 'border-style:{{VALUE}};',
                ],
            ]
        );
        $this->add_control(
            $name.'_item_border_color',
            [
                'type'        => 'color',
                'label'       => __( 'Border Color', 'piotnetgrid' ),
                'value'       => '',
                'label_block' => true,
                'selectors'   => [
                    $wrapper => 'border-color: {{VALUE}};',
                ],
                'conditions'  => [
                    [
                        'name'     => $name.'_item_border_type',
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );
        $this->add_responsive_control(
            $name.'_item_border_width',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Border Width', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px', '%', 'em' ],
                'selectors'   => [
                    $wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'  => [
                    [
                        'name'     => $name.'_item_border_type',
                        'operator' => '!=',
                        'value'    => '',
                    ],
                ],
            ]
        );
        $this->add_responsive_control(
            $name.'_item_border_radius',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Border Radius', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'   => [
                    $wrapper => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
        $this->add_responsive_control(
            $name.'_item_padding',
            [
                'type'        => 'dimensions',
                'label'       => __( 'Padding', 'piotnetgrid' ),
                'value'       => [
                    'unit'   => 'px',
                    'top'    => '',
                    'right'  => '',
                    'bottom' => '',
                    'left'   => '',
                ],
                'label_block' => true,
                'size_units'  => [ 'px' ],
                'selectors'   => [
                    $wrapper => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
        $this->add_responsive_control(
            $name.'_item_spacing',
            [
                'type'        => 'slider',
                'label'       => __( 'Spacing', 'piotnetgrid' ),
                'value'       => [
                    'unit' => 'px',
                    'size' => '',
                ],
                'label_block' => true,
                'size_units'  => [
                    'px' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1,
                    ],
                ],
                'selectors'   => [
                    $wrapper => 'margin-right:{{SIZE}}{{UNIT}}',
                ],
            ]
        );

        return $this->get_group_controls( $previous_controls );
    }

    public function render($editor = false, $args_additional = []) {
        $settings = $this->settings;
        $taxonomies_list = $settings['piotnet_grid_taxonomies_list'];
        $term_url =  $settings['piotnet_grid_term_link'];
        $post_id = !empty($GLOBALS['preview_card_post_id']) ? $GLOBALS['preview_card_post_id'] : get_the_ID();

        if ( !empty($taxonomies_list) && $taxonomies_list != 'elementor_library_category' && $taxonomies_list != 'elementor_library_type'  ) {
            $terms = get_the_terms( $post_id, $taxonomies_list );
        }

        ?>

        <div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
            <div class="piotnetgrid-grid-term-list">
            <?php if ( $editor && empty($GLOBALS['preview_card_post_id']) ) : ?>
                <div class="piotnetgrid-grid-term-list-item"><span class="piotnetgrid-grid-term-list-item__inner">Tag 1</span></div>
                <div class="piotnetgrid-grid-term-list-item"><span class="piotnetgrid-grid-term-list-item__inner">Tag 2</span></div>
                <div class="piotnetgrid-grid-term-list-item"><span class="piotnetgrid-grid-term-list-item__inner">Tag 3</span></div>
            <?php endif; ?>
            
            <?php foreach ($terms as $term) : ?>
                <?php if ( $term_url == 'yes' ) : ?>
                <a href="<?php echo get_term_link( $term->term_id ); ?>" class="piotnetgrid-grid-term-list-item">
                    <span class="piotnetgrid-grid-term-list-item__inner" piotnetgrid-grid-term-data-id=<?php echo $term->term_id; ?> > <?php echo $term->name; ?> </span>
                </a>
                <?php else: ?>
                <div class="piotnetgrid-grid-term-list-item">
                    <span class="piotnetgrid-grid-term-list-item__inner" piotnetgrid-grid-term-data-id=<?php echo $term->term_id; ?> > <?php echo $term->name; ?> </span>
                </div>
                <?php endif; ?>
            <?php endforeach;?>

            </div>
        </div>
        <?php
    }
    public function live_preview() {
        ?>
        <div <%= view.render_attributes('wrapper') %>>
            <div class="piotnetgrid-grid-term-list">
                <div class="piotnetgrid-grid-term-list-item"><span class="piotnetgrid-grid-term-list-item__inner">Tag 1</span></div>
                <div class="piotnetgrid-grid-term-list-item"><span class="piotnetgrid-grid-term-list-item__inner">Tag 2</span></div>
                <div class="piotnetgrid-grid-term-list-item"><span class="piotnetgrid-grid-term-list-item__inner">Tag 3</span></div>
            </div>
		</div>
        <?php
    }
}