<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Text extends Base_Widget_piotnetgrid {

	public function get_type() {
		return 'text';
	}

	public function get_class_name() {
		return 'piotnetgrid_Text';
	}

	public function get_title() {
		return 'Text';
	}

	public function get_icon() {
		return [
			'type'  => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-text.svg',
		];
	}

	public function get_categories() {
		return [ 'pafe-form-builder' ];
	}

	public function get_keywords() {
		return [ 'text' ];
	}

	private function add_setting_controls() {
		$this->add_control(
			'text_content',
			[
				'type'        => 'textarea',
				'label'       => 'Content',
				'label_block' => true,
				'placeholder' => '',
				'dynamic'     => true,
			]
		);
		$this->add_control(
			'text_html_tag',
			[
				'type'    => 'select',
				'label'   => 'Tag',
				'value'   => 'h2',
				'options' => [
					'h1'   => 'H1',
					'h2'   => 'H2',
					'h3'   => 'H3',
					'h4'   => 'H4',
					'h5'   => 'H5',
					'h6'   => 'H6',
					'div'  => 'div',
					'span' => 'span',
					'p'    => 'p',
				],
			]
		);
		$this->add_control(
			'text_link',
			[
				'type'        => 'text',
				'label'       => __( 'Link URL', 'piotnetgrid' ),
				'dynamic'     => true,
			]
		);
		$this->add_control(
			'text_link_target',
			[
				'type'       => 'select',
				'label'      => __( 'Link Target', 'piotnetgrid' ),
				'value'      => '_self',
				'options'    => [
					'_self'  => 'Self',
					'_blank' => 'Blank',
				],
			]
		);
		$this->add_responsive_control(
			'text_align',
			[
				'type'         => 'select',
				'label'        => 'Alignment',
				'label_block'  => true,
				'options'      => [
					''        => __( 'Default', 'piotnetgrid' ),
					'left'    => __( 'Left', 'piotnetgrid' ),
					'center'  => __( 'Center', 'piotnetgrid' ),
					'right'   => __( 'Right', 'piotnetgrid' ),
					'justify' => __( 'Justified', 'piotnetgrid' ),
				],
				'prefix_class' => 'piotnetgrid%s-align-',
				'selectors'    => [
					'{{WRAPPER}}' => 'text-align: {{VALUE}}',
				],
			]
		);
	}

	private function add_style_controls() {
		$this->add_control(
			'',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'text_normal_tab',
						'title'  => __( 'NORMAL', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'text_hover_tab',
						'title' => __( 'HOVER WIDGET', 'piotnetgrid' ),
					],
					[
						'name'  => 'text_hover_card_tab',
						'title' => __( 'HOVER CARD', 'piotnetgrid' ),
					],
				],
			]
		);

		$normal_controls = $this->add_text_style_controls(
			'normal',
			[
				'selectors' => '{{WRAPPER}}',
			]
		);

		$this->add_control(
			'text_normal_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Normal', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $normal_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_controls = $this->add_text_style_controls(
			'hover',
			[
				'selectors' => '{{WRAPPER}}:hover',
			]
		);

		$this->add_control(
			'text_hover_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover Widget', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$hover_card_controls = $this->add_text_style_controls(
			'hover_card',
			[
				'selectors' => '{{WRAPPER_CARD}}:hover {{WIDGET}}',
			]
		);

		$this->add_control(
			'text_hover_card_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Hover Card', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $hover_card_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	private function add_text_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();

		$this->add_control(
			'text_color_' . $name,
			[
				'type'      => 'color',
				'label'     => 'Text Color',
				'dynamic'	=> true,
				'dynamic_field'	=> true,
				'selectors' => [
					$wrapper => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_text_typography_controls(
			'text_typography_' . $name,
			[
				'selectors' => $wrapper,
			]
		);

		$this->add_control(
			'custom_css_' . $name,
			[
				'type'      => 'textarea',
				'label'     => 'Custom CSS',
				'label_block' => true,
				'placeholder' => 'color: blue;',
				'selectors' => [
					$wrapper => '{{VALUE}}',
				],
			]
		);

		return $this->get_group_controls( $previous_controls );
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'text_settings_section', 'Settings' );
		$this->add_setting_controls();

		$this->add_advanced_style_tab();

		$this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
		$this->add_custom_class_id();

		$this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
		$this->add_custom_css_controls();

		$this->add_animation_settings();

		$this->add_attributes_settings();

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		$this->add_trigger_action_carousel();

		return $this->structure;
	}

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
		$text_link  = ! empty( $settings['text_link'] ) ? $settings['text_link'] : '';
		$text_link_target = ! empty( $settings['text_link_target'] ) ? $settings['text_link_target'] : '';
		$preview_card_post_id = !empty($GLOBALS['preview_card_post_id']) ? $GLOBALS['preview_card_post_id'] : get_the_ID();
		if ( ! empty( $settings['text_content'] ) ) {
			$text_content = !empty($text_link) ? '<a class="piotnetgrid-link" href="' . $text_link . '" target="' . $text_link_target . '">' . $settings['text_content'] . '</a>' : $settings['text_content'];
			$text_content = piotnetgrid_dynamic_tags( $text_content, $preview_card_post_id );
			echo '<' . $settings['text_html_tag'] . ' ' . $this->get_render_attribute_string( 'wrapper' ) . '>' . $text_content . '</' . $settings['text_html_tag'] . '>';
		}
	}

	public function live_preview() {
		?>
		<<%=data['widget_settings']['text_html_tag']%> <%= view.render_attributes('wrapper') %>><%= data['widget_settings']['text_content'] %></<%=data['widget_settings']['text_html_tag'] %>>
		<?php
	}
}
