<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class Piotnetgrid_Variation_Swatches extends Base_Widget_piotnetgrid {
	public function get_type() {
		return 'variation-swatches';
	}

	public function get_class_name() {
		return 'Piotnetgrid_Variation_Swatches';
	}

	public function get_title() {
		return 'Variation Swatches';
	}

	public function get_icon() {
		return [
			'type' => 'image',
			'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-facet-active.svg',
		];
	}

	public function get_categories() {
		return [ 'piotnetgrid' ];
	}

	public function get_keywords() {
		return [ 'button' ];
	}

	public function get_script() {
		return [
			'piotnetgrid-advanced-script',
		];
	}

	public function register_controls() {
		$this->start_tab( 'settings', 'Settings' );

		$this->start_section( 'label_section', 'Label' );
		$this->add_label_style_controls();

		$this->start_section( 'spacing_section', 'Spacing' );
		$this->add_spacing_control();

		$this->start_section( 'color_section', 'Color Variation' );
		$this->add_color_picker_style_controls();

		$this->start_section( 'button_section', 'Button Variation' );
		$this->add_button_style_controls();

		$this->start_section( 'image_section', 'Image Variation' );
		$this->add_image_style_controls();

		$this->start_section( 'select_section', 'Select Variation' );
		$this->add_select_style_controls();

		$this->add_advanced_style_tab();

		$this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
		$this->add_custom_class_id();

		$this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
		$this->add_custom_css_controls();

		$this->add_animation_settings();

		$this->add_attributes_settings();

		$this->start_section( 'conditional_visibility', 'Conditional Visibility' );
		$this->add_conditional_visibility_controls();

		return $this->structure;
	}

	private function add_spacing_control() {
		$this->add_control(
			'spacing',
			[
				'label'     => __( 'Spacing', 'piotnetgrid' ),
				'type'      => 'slider',
				'default'   => [
					'size' => '',
					'unit' => 'px',
				],
				'range'     => [
					'px' => [
						'min' => 0,
						'max' => 60,
					],
				],
				'render_type' => 'none',
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
			]
		);
	}

	private function add_label_style_controls() {

		$this->add_control(
            'label_hide',
            [
                'label' => __( 'Hide Label', 'piotnetgrid' ),
                'type' => 'switch',
                'default' => '',
                'label_on' => 'Yes',
                'label_off' => 'No',
                'return_value' => 'yes',
            ]
        );

		$this->add_control(
			'label_spacing',
			[
				'label'     => __( 'Spacing', 'piotnetgrid' ),
				'type'      => 'slider',
				'default'   => [
					'size' => '',
					'unit' => 'px',
				],
				'range'     => [
					'px' => [
						'min' => 0,
						'max' => 60,
					],
				],
				'render_type' => 'none',
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute-label' => 'padding-bottom: {{SIZE}}{{UNIT}};',
				],
				'conditions' => [
					[
						'name'     => 'label_hide',
						'operator' => '!=',
						'value'    => 'yes',
					],
				],
			]
		);

		$this->add_responsive_control(
			'label_text_align',
			[
				'type'        => 'select',
				'label'       => __( 'Text Align', 'piotnetgrid' ),
				'label_block' => true,
				'value'       => '',
				'options'     => [
					''       => __( 'Default', 'piotnetgrid' ),
					'left'   => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'right'  => __( 'Right', 'piotnetgrid' ),
				],
				'render_type' => 'none',
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute-label' => 'text-align: {{VALUE}};',
				],
				'conditions' => [
					[
						'name'     => 'label_hide',
						'operator' => '!=',
						'value'    => 'yes',
					],
				],
			]
		);

		$this->add_control(
			'label_color',
			[
				'label'     => __( 'Text Color', 'piotnetgrid' ),
				'type'      => 'color',
				'render_type' => 'none',
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute-label' => 'color: {{VALUE}};',
				],
				'conditions' => [
					[
						'name'     => 'label_hide',
						'operator' => '!=',
						'value'    => 'yes',
					],
				],
			]
		);

		$this->add_text_typography_controls(
			'label_typography',
			[
				'render_type' => 'none',
				'selectors' => '{{WRAPPER}} .piotnetgrid-variation-swatches__attribute-label',
				'conditions' => [
					[
						'name'     => 'label_hide',
						'operator' => '!=',
						'value'    => 'yes',
					],
				],
			]
		);
	}

	private function add_color_picker_style_controls() {
		$this->add_responsive_control(
			'color_picker_align_items',
			[
				'label'      => __( 'Align Items', 'piotnetgrid' ),
				'type'       => 'select',
				'label_block' => true,
				'options' => [
					'' => __( 'Default', 'piotnetgrid' ),
					'flex-start' => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end' => __( 'Right', 'piotnetgrid' ),
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--color .piotnetgrid-variation-swatches__attribute-options' => 'justify-content: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'color_picker_width',
			[
				'label'      => __( 'Item Width', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__field-label-color' => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'color_picker_item_spacing',
			[
				'label'      => __( 'Item Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--color .piotnetgrid-variation-swatches__attribute-option' => 'padding: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--color .piotnetgrid-variation-swatches__attribute-options' => 'margin: -{{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'color_picker_border_radius',
			[
				'label'      => __( 'Item Normal Border Radius', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__field-label-color' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'color_picker_tooltip_color',
			[
				'label'      => __( 'Tooltip Color', 'piotnetgrid' ),
				'type'       => 'color',
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--color .piotnetgrid-variation-swatches__field-label-content' => 'background: {{VALUE}};',
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--color .piotnetgrid-variation-swatches__field-label-content:before' => 'border-top-color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'color_picker_active_border_width',
			[
				'label'      => __( 'Item Active Border Width', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__field-label-color:after' => 'border-width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'color_picker_active_border_color',
			[
				'label'      => __( 'Item Active Border Color', 'piotnetgrid' ),
				'type'       => 'color',
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--color .piotnetgrid-variation-swatches__attribute-field:checked + label .piotnetgrid-variation-swatches__field-label-color:after' => 'border-color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'color_picker_active_border_radius',
			[
				'label'      => __( 'Item Active Border Radius', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__field-label-color:after' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
	}

	public function add_button_style_controls() {
		$this->add_responsive_control(
			'button_align_items',
			[
				'label'      => __( 'Align Items', 'piotnetgrid' ),
				'type'       => 'select',
				'label_block' => true,
				'options' => [
					'' => __( 'Default', 'piotnetgrid' ),
					'flex-start' => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end' => __( 'Right', 'piotnetgrid' ),
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--button .piotnetgrid-variation-swatches__attribute-options' => 'justify-content: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'button_item_padding',
			[
				'label'      => __( 'Item Padding', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', 'em', '%' ],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--button .piotnetgrid-variation-swatches__field-label' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'button_item_spacing',
			[
				'label'      => __( 'Item Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--button .piotnetgrid-variation-swatches__attribute-option' => 'padding: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--button .piotnetgrid-variation-swatches__attribute-options' => 'margin: -{{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'button_item_margin_bottom',
			[
				'label'      => __( 'Item Margin Bottom', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--button .piotnetgrid-variation-swatches__attribute-option' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'button_item_border_radius',
			[
				'label'      => __( 'Item Border Radius', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--button .piotnetgrid-variation-swatches__field-label' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'style_check_heading_tab',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'style_uncheck_tab',
						'title'  => __( 'UNCHECK', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'style_checked_tab',
						'title' => __( 'CHECKED', 'piotnetgrid' ),
					],
				],
			]
		);

		$style_uncheck_controls = $this->checked_style_controls(
			'style_uncheck',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--button .piotnetgrid-variation-swatches__field-label',
			]
		);
		$this->add_control(
			'style_uncheck_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Uncheck', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $style_uncheck_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$style_checked_controls = $this->checked_style_controls(
			'style_checked',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--button input:checked ~ label',
			]
		);
		$this->add_control(
			'style_checked_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Checked', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $style_checked_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	public function add_image_style_controls() {
		$this->add_responsive_control(
			'image_align_items',
			[
				'label'      => __( 'Align Items', 'piotnetgrid' ),
				'type'       => 'select',
				'label_block' => true,
				'options' => [
					'' => __( 'Default', 'piotnetgrid' ),
					'flex-start' => __( 'Left', 'piotnetgrid' ),
					'center' => __( 'Center', 'piotnetgrid' ),
					'flex-end' => __( 'Right', 'piotnetgrid' ),
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image .piotnetgrid-variation-swatches__attribute-options' => 'justify-content: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'image_item_padding',
			[
				'label'      => __( 'Item Padding', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', 'em', '%' ],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image .piotnetgrid-variation-swatches__field-label' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'image_item_spacing',
			[
				'label'      => __( 'Item Spacing', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image .piotnetgrid-variation-swatches__attribute-option' => 'padding: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image .piotnetgrid-variation-swatches__attribute-options' => 'margin: -{{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'image_item_margin_bottom',
			[
				'label'      => __( 'Item Margin Bottom', 'piotnetgrid' ),
				'type'       => 'slider',
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image .piotnetgrid-variation-swatches__attribute-option' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'image_item_border_radius',
			[
				'label'      => __( 'Item Border Radius', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', '%' ],
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image .piotnetgrid-variation-swatches__field-label' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'image_item_tooltip_color',
			[
				'label'      => __( 'Tooltip Color', 'piotnetgrid' ),
				'type'       => 'color',
				'render_type' => 'none',
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image .piotnetgrid-variation-swatches__field-label-content' => 'background: {{VALUE}};',
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image .piotnetgrid-variation-swatches__field-label-content:before' => 'border-top-color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'image_style_check_heading_tab',
			[
				'type' => 'heading-tab',
				'tabs' => [
					[
						'name'   => 'image_style_uncheck_tab',
						'title'  => __( 'UNCHECK', 'piotnetgrid' ),
						'active' => true,
					],
					[
						'name'  => 'image_style_checked_tab',
						'title' => __( 'CHECKED', 'piotnetgrid' ),
					],
				],
			]
		);

		$style_uncheck_controls = $this->checked_style_controls(
			'image_style_uncheck',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image .piotnetgrid-variation-swatches__field-label',
			]
		);
		$this->add_control(
			'image_style_uncheck_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Uncheck', 'piotnetgrid' ),
				'value'          => '',
				'active'         => true,
				'controls'       => $style_uncheck_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);

		$style_checked_controls = $this->checked_style_controls(
			'image_style_checked',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--image input:checked ~ label',
			]
		);
		$this->add_control(
			'image_style_checked_tab',
			[
				'type'           => 'content-tab',
				'label'          => __( 'Checked', 'piotnetgrid' ),
				'value'          => '',
				'controls'       => $style_checked_controls,
				'controls_query' => '.piotnetgrid-start-controls-tab',
			]
		);
	}

	public function checked_style_controls(string $name, $args = []) {
		$wrapper = isset( $args['selectors'] ) ? $args['selectors'] : '{{WRAPPER}}';
		$previous_controls = $this->new_group_controls();

		if ($name == 'style_uncheck' || $name == 'style_checked') {
			$this->add_control(
				$name.'_color',
				[
					'type'        => 'color',
					'label'       => __( 'Text Color', 'piotnetgrid' ),
					'label_block' => true,
					'selectors'   => [
						$wrapper => 'color: {{VALUE}};',
					],
				]
			);
			$this->add_text_typography_controls(
				$name.'_text_typography',
				[
					'selectors' => $wrapper,
				]
			);
		}
		
		$this->add_control(
			$name.'_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Item Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'background-color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_border_type',
			[
				'type'      => 'select',
				'label'     => __( 'Item Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					$wrapper => 'border-style:{{VALUE}};',
				],
			]
		);
		$this->add_control(
			$name.'_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Item Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					$wrapper => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			$name.'_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Item Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					$wrapper => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => $name.'_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		return $this->get_group_controls( $previous_controls );
	}

	public function add_select_style_controls() {

		$this->add_responsive_control(
			'select_padding',
			[
				'label'      => __( 'Item Padding', 'piotnetgrid' ),
				'type'       => 'dimensions',
				'size_units' => [ 'px', 'em', '%' ],
				'selectors'  => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--select select' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'select_color',
			[
				'type'        => 'color',
				'label'       => __( 'Text Color', 'piotnetgrid' ),
				'label_block' => true,
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--select select' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_text_typography_controls(
			'select_text_typography',
			[
				'selectors' => '{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--select select',
			]
		);
		
		$this->add_control(
			'select_background_color',
			[
				'type'        => 'color',
				'label'       => __( 'Item Background Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--select select' => 'background-color: {{VALUE}};',
				],
			]
		);
		$this->add_control(
			'select_border_type',
			[
				'type'      => 'select',
				'label'     => __( 'Item Border Type', 'piotnetgrid' ),
				'value'     => '',
				'options'   => [
					''       => 'None',
					'solid'  => 'Solid',
					'double' => 'Double',
					'dotted' => 'Dotted',
					'dashed' => 'Dashed',
					'groove' => 'Groove',
				],
				'selectors' => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--select select' => 'border-style:{{VALUE}};',
				],
			]
		);
		$this->add_control(
			'select_border_color',
			[
				'type'        => 'color',
				'label'       => __( 'Item Border Color', 'piotnetgrid' ),
				'value'       => '',
				'label_block' => true,
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--select select' => 'border-color: {{VALUE}};',
				],
				'conditions'  => [
					[
						'name'     => 'select_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
		$this->add_responsive_control(
			'select_border_width',
			[
				'type'        => 'dimensions',
				'label'       => __( 'Item Border Width', 'piotnetgrid' ),
				'value'       => [
					'unit'   => 'px',
					'top'    => '',
					'right'  => '',
					'bottom' => '',
					'left'   => '',
				],
				'label_block' => true,
				'size_units'  => [ 'px', '%', 'em' ],
				'selectors'   => [
					'{{WRAPPER}} .piotnetgrid-variation-swatches__attribute--select select' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'conditions'  => [
					[
						'name'     => 'select_border_type',
						'operator' => '!=',
						'value'    => '',
					],
				],
			]
		);
	}

	public function render($editor = false, $args_additional = []) {
		$settings = $this->settings;
		$post_id = !empty($GLOBALS['preview_card_post_id']) ? $GLOBALS['preview_card_post_id'] : get_the_ID();
		global $product;
		if (!$product && !empty($GLOBALS['preview_card_post_id'])) {
			$product = wc_get_product( $post_id );
		}
		$this->add_render_attribute( 'wrapper', 'class', 'piotnetgrid-variation-swatches' );
		if ( $product && $product->is_type('variable') || $editor ) :
			$default_image_url = plugins_url() . '/piotnetgrid/assets/images/placeholder-image.png';
        ?>
        	<?php
        	if (!$editor || $product && $product->is_type('variable')) {
        		global $post, $woocommerce;

				$get_variations = count( $product->get_children() ) <= apply_filters( 'woocommerce_ajax_variation_threshold', 30, $product );
				$available_variations = $get_variations ? $product->get_available_variations() : false;
				$attributes           = $product->get_variation_attributes();
				$selected_attributes  = $product->get_default_attributes();
				$attribute_keys  = array_keys( $attributes );
				$variations_json = wp_json_encode( $available_variations );
				$variations_attr = function_exists( 'wc_esc_json' ) ? wc_esc_json( $variations_json ) : _wp_specialchars( $variations_json, ENT_QUOTES, 'UTF-8', true );

				$this->add_render_attribute( 'wrapper', 'data-piotnetgrid-product-variations', $variations_attr );
        	} else {
        		$attributes = [
        			'color' => ['Red','Blue','Yellow'],
        			'button' => ['Small','Medium','Large'],
        			'image' => ['Term 1', 'Term 2', 'Term 3'],
        			'select' => ['Term 1', 'Term 2', 'Term 3'],
        		];
        	}

			if ( !empty( $available_variations ) || $editor ) :
			?>

			<div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
				<div class="piotnetgrid-variation-swatches__attributes">
					<?php
						foreach ( $attributes as $attribute_name => $options ) :
							$attribute_name = str_replace(' ', '-', $attribute_name);
							if (!$editor || $product && $product->is_type('variable')) {
								$attribute_id = wc_attribute_taxonomy_id_by_name( $attribute_name );
								$attribute_type = !empty( get_option( "wc_attribute_piotnetgrid_attribute_type_$attribute_id" ) ) ? get_option( "wc_attribute_piotnetgrid_attribute_type_$attribute_id" ) : 'select';
								$label =  wc_attribute_label( $attribute_name );
							} else {
								$attribute_id = 'editor';
								$attribute_type = $attribute_name;
								$label = ucfirst($attribute_name);
							}
					?>
						<div class="piotnetgrid-variation-swatches__attribute piotnetgrid-variation-swatches__attribute--<?php echo $attribute_type; ?>">
							<?php if(empty($settings['label_hide'])) : ?>
								<div class="piotnetgrid-variation-swatches__attribute-label">
									<?php echo $label; ?>
								</div>
							<?php endif; ?>
							<div class="piotnetgrid-variation-swatches__attribute-options">
								<?php
								if (!$editor || $product && $product->is_type('variable')) {
									if (taxonomy_exists( $attribute_name )) {
										$terms = wc_get_product_terms(
											$product->get_id(),
											$attribute_name,
											array(
												'fields' => 'all',
											)
										);

										$options = [];
										foreach ( $terms as $term ) {
											$options[] = $term->slug;
										}
									}
								}
								?>
								<?php if ($attribute_type == 'select') : ?>
									<select data-piotnetgrid-variation-swatches-attribute-field='<?php echo json_encode($options); ?>' name="attribute_<?php echo strtolower($attribute_name); ?>" id="<?php echo $product->id . '_' . strtolower($attribute_name); ?>">
										<option value=""><?php echo __('Choose an option','piotnetgrid') ?></option>
								<?php endif; ?>
								<?php
								foreach ( $options as $key => $option ) :
									$term = get_term_by('slug', $option, $attribute_name);
									if ($term) {
										$term_name = $term->name;
									} else {
										$term_name = esc_html( apply_filters( 'woocommerce_variation_option_name', $option, null, $attribute_name, $product ) );
									}
								?>
									<?php if ($attribute_type == 'select') : ?>
										<option value="<?php echo $option; ?>"><?php echo $term_name; ?></option>
									<?php else : ?>
										<div class="piotnetgrid-variation-swatches__attribute-option">
											<input class="piotnetgrid-variation-swatches__attribute-field" data-piotnetgrid-variation-swatches-attribute-field type="checkbox" name="attribute_<?php echo strtolower($attribute_name); ?>" id="<?php echo $product->id . '_' . strtolower($attribute_name) . '_' . $key; ?>" value="<?php echo $option; ?>">
											<label for="<?php echo $product->id . '_' . strtolower($attribute_name) . '_' . $key; ?>" class="piotnetgrid-variation-swatches__field-label">
												<?php
													if($term || $editor) :
														if (!$editor) {
															$color = $attribute_type == 'color' ? '#' . get_term_meta( $term->term_id, 'piotnetgrid_term_color', true ) : '#fff';
															$image_url = $attribute_type == 'image' && !empty(get_term_meta( $term->term_id, 'piotnetgrid_term_image', true )) ? wp_get_attachment_image_src( get_term_meta( $term->term_id, 'piotnetgrid_term_image', true ) )[0] : $default_image_url;
														} else {
															$color = strtolower($option);
															$image_url = $default_image_url;
														}
												?>
													<span class="piotnetgrid-variation-swatches__field-label-<?php echo $attribute_type; ?>"<?php if($attribute_type == 'color') { echo 'style="background-color:' . $color . '"'; } ?>>
														<?php if($attribute_type == 'image') : ?>
															<img src="<?php echo $image_url; ?>">
														<?php endif; ?>
													</span>
												<?php endif; ?>
												<span class="piotnetgrid-variation-swatches__field-label-content">
													<span class="piotnetgrid-variation-swatches__field-label-text"><?php echo $term_name; ?></span>
												</span>
											</label>
										</div>
									<?php endif; ?>
								<?php endforeach; ?>

								<?php if ($attribute_type == 'select') : ?>
									</select>
								<?php endif; ?>
							</div>
						</div>
					<?php endforeach; ?>
				</div>
			</div>
			<?php endif; ?>
        <?php
        endif;
	}

	public function live_preview() {
	}
}
