<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

class piotnetgrid_Video extends Base_Widget_piotnetgrid {
    public function get_type() {
        return 'video';
    }

    public function get_class_name() {
        return 'piotnetgrid_Video';
    }

    public function get_title() {
        return 'Video';
    }

    public function get_icon() {
        return [
            'type' => 'image',
            'value' => plugin_dir_url( __FILE__ ) . '../../assets/icons/i-video.svg',
        ];
    }

    public function get_categories() {
        return [ 'piotnetgrid' ];
    }

    public function get_keywords() {
        return [ 'video' ];
    }

    public function register_controls() {
        $this->start_tab( 'settings', 'Settings' );

        $this->start_section( 'piotnet_grid_video_settings_section', 'Video Settings' );
        $this->add_setting_controls();

        // $this->start_tab( 'style', 'Style' );

        // $this->start_section( 'piotnet_grid_video_styles_section', 'Video Style' );
        // $this->add_video_style_controls();

        $this->add_advanced_style_tab();

        $this->start_section( 'custom_class_id_section', 'Custom Class/ID' );
        $this->add_custom_class_id();

        $this->start_section( 'advanced_custom_css_section', 'Advanced Custom CSS' );
        $this->add_custom_css_controls();

        $this->add_animation_settings();

        $this->add_attributes_settings();

        $this->start_section( 'conditional_visibility', 'Conditional Visibility' );
        $this->add_conditional_visibility_controls();

        $this->add_trigger_action_carousel();

        return $this->structure;
    }
    private function add_setting_controls() {

        $this->add_control(
            'piotnetgrid_video_dynamic_data',
            [
                'type'        => 'select',
                'label'       => __( 'Video Dynamic Data', 'piotnetgrid' ),
                'options'     => [
                    '' => __( 'None', 'piotnetgrid' ),
                    'post_custom_field' => __( 'Post Custom Field', 'piotnetgrid' ),
                    'acf_field' => __( 'ACF Field', 'piotnetgrid' ),
                    'metabox_field' => __( 'Metabox Field', 'piotnetgrid' ),
                    'pods_field' => __( 'Pods Field', 'piotnetgrid' ),
                    'toolset_field' => __( 'Toolset Field', 'piotnetgrid' ),
                    'jetengine_field' => __( 'JetEngine Field', 'piotnetgrid' ),
                ],
            ]
        );

        $this->add_control(
            'piotnetgrid_video_dynamic_data_field_name',
            [
                'type'        => 'text',
                'label'       => __( 'Field Name', 'piotnetgrid' ),
            ]
        );
    }

    private function curl_get($url) {

        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);

        $return = curl_exec($curl);
        curl_close($curl);

        return $return;

    }
    private function vimeo_object($video_url) {
        $oembed_endpoint = 'http://vimeo.com/api/oembed';
        // Create the URLs
        $xml_url = $oembed_endpoint . '.xml?url=' . rawurlencode($video_url) . '&width=640';
        // Load in the oEmbed XML
        $oembed = simplexml_load_string($this->curl_get($xml_url));
        return $oembed;
    }

    private function add_video_style_controls() {

    }

    public function render($editor = false, $args_additional = []) {
        $settings = $this->settings;
        $video_url = "";
        $iframe = "";
        $post_id = !empty($GLOBALS['preview_card_post_id']) ? $GLOBALS['preview_card_post_id'] : get_the_ID();
        $default_image_url = plugins_url() . '/piotnetgrid/assets/images/placeholder-video.png';

        if ( !$editor || !empty($GLOBALS['preview_card_post_id']) ) {
            switch ($settings['piotnetgrid_video_dynamic_data']) {
                case 'jetengine_field':
                case 'post_custom_field':
                    $video_url = get_post_meta( $post_id, $settings['piotnetgrid_video_dynamic_data_field_name'], true );
                    break;

                case 'toolset_field':
                    $video_url = get_post_meta( $post_id, 'wpcf-' . $settings['piotnetgrid_video_dynamic_data_field_name'], true );
                    break;

                case 'metabox_field':
                    if ( function_exists( 'rwmb_meta_legacy' ) ) {
                        $video_url = rwmb_meta_legacy( $settings['piotnetgrid_video_dynamic_data_field_name'] );
                    }
                    break;

                case 'pods_field':
                    if ( function_exists( 'pods_field' ) ) {
                        $video_url = pods_field( get_post_type(), $post_id, $settings['piotnetgrid_video_dynamic_data_field_name'],true );
                    }

                    break;

                case 'acf_field':
                    if ( function_exists('get_field') ) {
                        $video_url = get_field($settings['piotnetgrid_video_dynamic_data_field_name']);
                    }
                    break;
            }
        }
        if (!empty($video_url) ) {
            if ( strpos($video_url, 'vimeo.com') !== false) {
                $video_object = $this->vimeo_object($video_url);
                $video_id = $video_object->video_id;
                $auto = '';
                if ($settings['piotnetgrid_video_autoplay'] == 'yes') {
                    $auto = 'autoplay;';
                }
                $iframe = '<iframe src="https://player.vimeo.com/video/'.$video_id.'?title=0&byline=0&portrait=0" frameborder="0" allow="'.$auto.' fullscreen" allowfullscreen></iframe>';
            } elseif ( strpos($video_url, 'youtube.com') !== false ) {
                parse_str( parse_url( $video_url, PHP_URL_QUERY ), $my_array_of_vars );
                $video_id = $my_array_of_vars['v'];
                $auto = '';
                if ($settings['piotnetgrid_video_autoplay'] == 'yes') {
                    $auto = 'autoplay;';
                }
                if ( !empty($video_id) ) {
                    $iframe = '<iframe src="https://www.youtube.com/embed/'.$video_id.'?&autoplay=1" frameborder="0" allow="accelerometer;'.$auto.' encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>';
                }
            } else {
                if ($settings['piotnetgrid_video_autoplay'] == 'yes') {
                    $auto = 'autoplay;';
                }
                $iframe = '<video src="'.$video_url.'"'.$auto.' width="100%" controls></video>';
            }
        }
        ?>
            <div class="piotnetgrid-video" data-piotnetgrid-video>
                <?php
                    if ( !$editor) {
                        echo $iframe;
                    } else {
                        echo '<img src="'.$default_image_url.'" />';
                    }
                ?>
            </div>
        <?php
    }

    public function live_preview() {

    }

}
