<?php
/**
 * Plugin Name: Piotnet Grid
 * Description: Piotnet Grid
 * Plugin URI:  https://piotnetgrid.com/
 * Version:     1.1.4
 * Author:      Piotnet
 * Author URI:  https://piotnet.com/
 * Text Domain: piotnetgrid
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) ) { exit; }

require_once __DIR__ . '/inc/variables.php';
require_once __DIR__ . '/inc/license.php';

define( 'PIOTNETGRID_VERSION', '1.1.4' );

class piotnetgrid extends piotnetgrid_Variables_Pro {

	public $get_posts_query = [];

	public function __construct() {

		parent::__construct();

		add_action( 'plugins_loaded', [ $this, 'init' ] );
		register_activation_hook( __FILE__, [ $this, 'plugin_activate' ] );
	}

	public function init() {

		add_action( 'init', [ $this, 'register_post_type' ] );
		add_action('init', [ $this, 'gutenberg_block' ]);
        //add_action( 'init', [ $this, 'ajax_wp_template' ]);
        add_action( 'init', [ $this, 'get_html' ]);

		add_action('init', [ $this, 'init_session' ]);
		if ( ! defined( 'DOING_AJAX' ) || false === DOING_AJAX ) {
			add_action('wp_loaded', [ $this, 'close_session' ], 30);
		}

		if ( version_compare( $GLOBALS['wp_version'], '5.8-alpha-1', '<' ) ) {
			add_filter( 'block_categories', [ $this, 'gutenberg_block_categories' ], 10, 2 );
		} else {
			add_filter( 'block_categories_all', [ $this, 'gutenberg_block_categories' ], 10, 2 );
		}
		
		if ( function_exists('icl_object_id') ) {
			add_action( 'init', function() {
				remove_all_filters( 'rest_url' );
			} );
		}

		add_action( 'init', [ $this, 'load_textdomain' ] );

		add_filter( 'single_template', [ $this, 'single_template' ] );

		require_once __DIR__ . '/inc/managers/controls.php';

		require_once __DIR__ . '/inc/class/piotnetgrid-editor.php';

		require_once __DIR__ . '/inc/ajax/preview.php';

		require_once __DIR__ . '/inc/ajax/get-json-file.php';

		require_once __DIR__ . '/inc/ajax/save.php';

		require_once __DIR__ . '/inc/ajax/save-draft.php';

        require_once __DIR__ . '/inc/ajax/export.php';

        require_once __DIR__ . '/inc/ajax/duplicate.php';

		require_once __DIR__ . '/inc/shortcode/shortcode-widget.php';

		require_once __DIR__ . '/inc/ajax/select-autocomplete.php';

		require_once __DIR__ . '/inc/ajax/piotnetgrid/add-to-wishlist.php';

		require_once __DIR__ . '/inc/ajax/piotnetgrid/quick-view.php';

		require_once __DIR__ . '/inc/ajax/piotnetgrid/clear-cache.php';

		require_once __DIR__ . '/inc/ajax/piotnetgrid/check-facet-name.php';

		require_once __DIR__ . '/inc/ajax/piotnetgrid/import-demo.php';

		require_once __DIR__ . '/inc/ajax/piotnetgrid/add-to-cart.php';

		require_once __DIR__ . '/inc/functions.php';

		require_once __DIR__ . '/inc/metaboxs/metabox-shortcode-in-post.php';

		add_action( 'wp_enqueue_scripts', [ $this, 'load_jquery' ] );

		add_action( 'admin_enqueue_scripts', [ $this, 'admin_enqueue' ] );

		add_action( 'admin_enqueue_scripts', [ $this, 'add_admin_scripts' ], 10, 1 );

		add_action( 'admin_footer', [ $this, 'admin_footer' ], 10, 1 );

		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue' ] );

		add_action( 'admin_menu', [ $this, 'admin_menu' ], 600 );

		add_filter( 'the_content', [ $this, 'add_wrapper' ] );

		add_filter( 'manage_piotnetgrid_posts_columns', [ $this, 'set_custom_edit_columns' ] );
		add_action( 'manage_piotnetgrid_posts_custom_column', [ $this, 'custom_column' ], 10, 2 );

		add_filter( 'manage_piotnetgrid-card_posts_columns', [ $this, 'set_custom_edit_columns' ] );
		add_action( 'manage_piotnetgrid-card_posts_custom_column', [ $this, 'custom_column' ], 10, 2 );

		add_filter( 'manage_piotnetgrid-facet_posts_columns', [ $this, 'set_custom_edit_columns' ] );
		add_action( 'manage_piotnetgrid-facet_posts_custom_column', [ $this, 'custom_column' ], 10, 2 );

		add_filter( 'manage_piotnetgrid-template_posts_columns', [ $this, 'set_custom_edit_columns' ] );
		add_action( 'manage_piotnetgrid-template_posts_custom_column', [ $this, 'custom_column' ], 10, 2 );

		add_action( 'wp_footer', [ $this, 'enqueue_footer' ], 600 );

		add_action( 'wp_head', [ $this, 'enqueue_head' ] );

		$upload     = wp_upload_dir();
		$upload_dir = $upload['basedir'];
		$upload_dir = $upload_dir . '/piotnetgrid';
		if ( ! is_dir( $upload_dir ) ) {
			mkdir( $upload_dir, 0755 );
			mkdir( $upload_dir . '/css', 0755 );
			mkdir( $upload_dir . '/files', 0755 );
		} else {
			if ( ! is_dir( $upload_dir . '/files') ) {
				mkdir( $upload_dir . '/files', 0755 );
			}
			if ( @chmod( $upload_dir, 0700 ) ) {
				@chmod( $upload_dir, 0755 );
				@chmod( $upload_dir . '/css', 0755 );
				@chmod( $upload_dir . '/files', 0755 );
			}
		}

		$cache_dir = WP_CONTENT_DIR . '/cache';
		if ( ! is_dir( $cache_dir ) ) {
			@mkdir( $cache_dir, 0755 );
			@mkdir( $cache_dir . '/piotnetgrid', 0755 );
			@mkdir( $cache_dir . '/piotnetgrid/json', 0755 );
		} else {
			if ( ! is_dir( $cache_dir . '/piotnetgrid/json' ) ) {
				@mkdir( $cache_dir . '/piotnetgrid', 0755 );
				@mkdir( $cache_dir . '/piotnetgrid/json', 0755 );
			}
		}

		add_action( 'in_plugin_update_message-piotnetgrid-pro/piotnetgrid-pro.php', [ $this, 'update_message' ], 10, 2 );
		add_filter( 'plugin_row_meta', [ $this, 'plugin_row_meta' ], 10, 2 );
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), [ $this, 'plugin_action_links' ], 10, 1 );

		add_action( 'admin_init', [ $this, 'plugin_redirect' ] );

		add_filter( 'post_row_actions', [ $this, 'modify_list_row_actions' ], 10, 2 );

		add_filter( 'page_row_actions', [ $this, 'modify_list_row_actions' ], 10, 2 );

		add_filter( 'body_class', [ $this, 'add_body_class' ] );

		require_once( __DIR__ . '/inc/ajax/piotnetgrid/piotnetgrid-facet.php' );

		add_action( 'rest_api_init', function () {
			register_rest_route( 'piotnetgrid/v1', '/refresh', array(
				'methods' => 'POST',
				'callback' => [ $this, 'piotnetgrid_rest_api' ],
				'permission_callback' => '__return_true',
			) );
		} );

    	if (function_exists('get_field')) {
    		add_filter('acf/settings/remove_wp_meta_box', '__return_false');
    	}

    	add_action( 'restrict_manage_posts', [ $this, 'piotnetgrid_filter' ] );
    	add_filter( 'parse_query', [ $this, 'piotnetgrid_filter_posts' ] );

		add_filter('manage_piotnetgrid-data_posts_columns', [$this,'piotnetgrid_filter_column'], 10);
		add_action('manage_piotnetgrid-data_posts_custom_column', [$this,'piotnetgrid_filter_column_content'], 10, 2);

		add_action('admin_footer', [$this,'piotnetgrid_filter_export_btn'] );

		add_action( 'save_post_piotnetgrid', [$this,'save_form'], 10, 3 );

        // add_action( 'init', [ $this, 'check_starter_forms' ] );

        add_filter( 'template_include', [ $this, 'filter_archive_template' ], 999 );
        add_filter( 'woocommerce_locate_template', [ $this, 'filter_archive_template_woo' ], 999 );

		add_action( 'current_screen', [ $this, 'add_term_meta_boxes' ] );
		add_action( 'created_term', [ $this, 'save_termmeta' ] );
		add_action( 'edited_term',  [ $this, 'save_termmeta' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'category_colorpicker_enqueue' ] );
		add_action( 'admin_print_scripts', [ $this, 'colorpicker_init_inline' ], 20 );

		// Quick View
		// Image.

		if ( class_exists( 'woocommerce' ) ) {
			add_theme_support( 'wc-product-gallery-zoom' );
			add_theme_support( 'wc-product-gallery-lightbox' );
			add_theme_support( 'wc-product-gallery-slider' );
			add_action( 'piotnetgrid_quick_view_product_image', 'woocommerce_show_product_sale_flash', 10 );
			add_action( 'piotnetgrid_quick_view_product_image', 'woocommerce_show_product_images', 20 );
		}

		add_action( 'admin_bar_menu', [ $this, 'add_menu_admin_bar_link' ],81 );

		if ( defined('ELEMENTOR_VERSION') ) {
			add_action( 'elementor/elements/categories_registered', [ $this, 'add_elementor_widget_categories' ] );
			add_action( 'elementor/widgets/widgets_registered', [ $this, 'elementor_widgets' ] );
			add_action( 'elementor/editor/before_enqueue_styles', [ $this, 'enqueue_editor_elementor' ] );
		}

		$theme_template = wp_get_theme()->Template;
		if($theme_template == 'bricks'){
			add_filter('bricks/frontend/render_data', [$this, 'piotnetgrid_disable_content_footer'], 10, 2);
		}

		if (class_exists('OxyEl')) {
			add_action('oxygen_add_plus_sections', [ $this, 'oxygen_register_add_plus_section' ] );
			add_action( 'oxygen_add_plus_piotnetgrid_section_content', [ $this, 'oxygen_register_subsection' ] );
			require_once __DIR__ . '/inc/widgets/oxygen/facet-oxygen.php';
			require_once __DIR__ . '/inc/widgets/oxygen/grid-oxygen.php';
			require_once __DIR__ . '/inc/widgets/oxygen/template-oxygen.php';
			require_once __DIR__ . '/inc/widgets/oxygen/dynamic-image-oxygen.php';
		}

		add_action('admin_head-edit.php', [ $this, 'add_import_button' ] );

		add_action( 'woocommerce_after_add_attribute_fields', [ $this, 'add_wc_attribute_piotnetgrid_attribute_type' ] );
		add_action( 'woocommerce_after_edit_attribute_fields', [ $this, 'edit_wc_attribute_piotnetgrid_attribute_type' ] );
		add_action( 'woocommerce_attribute_added', [ $this, 'save_wc_attribute_piotnetgrid_attribute_type' ] );
		add_action( 'woocommerce_attribute_updated', [ $this, 'save_wc_attribute_piotnetgrid_attribute_type' ] );
		add_action( 'woocommerce_attribute_deleted', function ( $id ) {
		    delete_option( "wc_attribute_piotnetgrid_attribute_type_$id" );
		} );

        add_filter( 'posts_where', [ $this, 'query_starts_with'], 10, 2 );

		add_action( 'loop_end', [ $this, 'grid_query' ] );
		//add_action( 'woocommerce_product_loop_end', [ $this, 'grid_query' ] );
		add_action( 'pre_get_posts',  [$this, 'update_query_vars_main_query'], 999 );

		add_action( 'updated_post_meta', [ $this, 'oxygen_updated_post_meta' ], 10, 4 );
		add_filter( 'pre_do_shortcode_tag', [ $this, 'filter_pre_do_shortcode_tag' ], 10, 4 );

		add_filter( 'woocommerce_shortcode_products_query', [ $this, 'filter_woocommerce_shortcode_products_query' ], 10 );
		add_filter( 'woocommerce_product_loop_end', [ $this, 'filter_woocommerce_product_loop_end' ], 10 );

        $this->setup_updater();
	}

    private function setup_updater() {
        require_once ( 'updater.php' );
        $plugin_slug = plugin_basename( __FILE__ );
        new Piotnetgrid_Updater( $plugin_slug, PIOTNETGRID_VERSION );
    }

	public function piotnetgrid_disable_content_footer($content, $post){
		if($post->post_type == 'piotnetgrid'){
			$doc = new DOMDocument();
			libxml_use_internal_errors(true);
			$doc->loadHTML($content);
			$xpath = new DOMXPath($doc);
			$nlist = $xpath->query("//div[@id='piotnetgrid']");
			if($nlist->length > 0){
				for($i=0; $i<$nlist->length; $i++){
					$node = $nlist->item($i);
					$node->parentNode->removeChild($node);
				}
			}
			$content = $doc->saveHTML();
		}
		return $content;
	}
	
	public function get_shortcode_attributes( $tag, $text ) {
	    preg_match_all( '/' . get_shortcode_regex() . '/s', $text, $matches );
	    $out = array();
	    if( isset( $matches[2] ) )
	    {
	        foreach( (array) $matches[2] as $key => $value )
	        {
	            if( $tag === $value )
	                $out[] = shortcode_parse_atts( $matches[3][$key] );
	        }
	    }
	    return $out;
	}

	public function tag_contents($string, $tag_open, $tag_close){
		foreach (explode($tag_open, $string) as $key => $value) {
			if(strpos($value, $tag_close) !== FALSE){
			    $result[] = substr($value, 0, strpos($value, $tag_close));;
			}
		}
		return $result;
	}

	public function oxygen_updated_post_meta( $meta_id, $post_id, $meta_key, $meta_value ) {
	    if ( 'ct_builder_shortcodes' == $meta_key ) {
	    	global $wpdb;
			// $wpdb->query( "DELETE FROM " . $wpdb->prefix . "options WHERE option_name LIKE '_piotnetgrid_facet_oxygen_{$post_id}%'" );
			$grids_oxygen = [];
	        $piotnetgrid_shortcodes = $this->tag_contents( $meta_value, '[oxy-piotnetgrid_facet_oxygen ', '[/oxy-piotnetgrid_facet_oxygen]' );
	        if (is_array($piotnetgrid_shortcodes)) {
		        foreach ($piotnetgrid_shortcodes as $shortcode) {
		        	$shortcode = '[oxy-piotnetgrid_facet_oxygen ' . $shortcode . '[/oxy-piotnetgrid_facet_oxygen]';
		        	$shortcode = $this->get_shortcode_attributes( 'oxy-piotnetgrid_facet_oxygen', $shortcode )[0];
		        	$ct_options = json_decode($shortcode['ct_options'], true);
		        	$ct_options = $ct_options['original'];
		        	if ($ct_options['oxy-piotnetgrid_facet_oxygen_facet_grid'] == '0_oxygen' && !empty($ct_options['oxy-piotnetgrid_facet_oxygen_facet_grid_oxygen'])) {
		        		$grid_oxygen = $ct_options['oxy-piotnetgrid_facet_oxygen_facet_grid_oxygen'];
		        		$grids_oxygen[] = $grid_oxygen;
		        		update_option('_piotnetgrid_facet_oxygen_' . $post_id . '_' . $ct_options['oxy-piotnetgrid_facet_oxygen_facet'] . '_' . $grid_oxygen, $grid_oxygen);
		        	}
		        }
	        }

	        $grids_oxygen_list = @$wpdb->get_results( 'SELECT * FROM ' . $wpdb->prefix . 'options WHERE option_name LIKE "_piotnetgrid_facet_oxygen_' . $post_id .'%"', ARRAY_A );

	        foreach ($grids_oxygen_list as $grids_oxygen_item) {
	        	if (!in_array($grids_oxygen_item['option_value'], $grids_oxygen)) {
	        		delete_option($grids_oxygen_item['option_name']);
	        	}
	        }
	    }
	}

	public function filter_pre_do_shortcode_tag( $false, $tag, $attr, $m ) {
	    if ((strpos( $tag, 'oxy_dynamic_list' ) !== false || strpos( $tag, 'oxy_posts_grid' ) !== false || strpos( $tag, 'oxy-woo-products') !== false) && !isset($GLOBALS['grid_id'])) {

	    	global $wpdb;
	    	$ct_options = json_decode($attr['ct_options'], true);
	    	$grid_id = '';
	    	if (!empty($ct_options['classes'])) {
	    		foreach ($ct_options['classes'] as $element_class) {
	    			if (strpos( $element_class, 'piotnetgrid-id' ) !== false) {
	    				$grid_id = $element_class;
    				}
	    		}
	    	}

	    	$facets_oxygen = @$wpdb->get_results( 'SELECT option_name FROM ' . $wpdb->prefix . 'options WHERE option_value = "' . $grid_id . '"', ARRAY_A );
	    	$facets_oxygen_post_ids = [];
	    	$post_ids = [];

	    	foreach ($facets_oxygen as $facet_oxygen) {
	    		if (strpos( $facet_oxygen['option_name'], '_piotnetgrid_facet_oxygen_' ) !== false) {
	    			$facet_oxygen_data = explode('_', str_replace('_piotnetgrid_facet_oxygen_', '', $facet_oxygen['option_name']));
	    			$facets_oxygen_post_ids[] = $facet_oxygen_data[1];
	    		}
	    	}

	    	if (!empty($facets_oxygen_post_ids)) {
	    		$GLOBALS['ct_options'] = $ct_options['original'];
    			$GLOBALS['grid_id'] = $grid_id;
    			$GLOBALS['grid_id_woocommerce'] = $grid_id;

	    		foreach ($_GET as $key => $value) {
		    		$facet_post_ids = piotnetgrid_get_facet_post_ids($key, $value, $facets_oxygen_post_ids);
		    		if (!empty($facet_post_ids)) {
		    			if (empty($post_ids)) {
		    				$post_ids = $facet_post_ids;
		    			} else {
		    				$post_ids = array_intersect($post_ids, $facet_post_ids);
		    			}
		    		}
		    	}

		    	if (!empty($post_ids)) {
		    		$GLOBALS['piotnetgrid_post_ids'] = $post_ids;

			    	$shortcode = $m[0];

			    	return do_shortcode($shortcode);
		    	} else {
			    	return $false;
			    	unset($GLOBALS['grid_id']);
					unset($GLOBALS['ct_options']);
			    }
	    	} else {
		    	return $false;
		    }

	    } else {
	    	return $false;
	    }
	}

	public function filter_woocommerce_shortcode_products_query($query_args) {
		$GLOBALS['piotnetgrid_query'] = $query_args;

		if (!empty($GLOBALS['grid_id_woocommerce'])) {
			$GLOBALS['woocommerce_shortcode'] = true;
			$facets = piotnetgrid_get_facets();

			if (!empty($facets)) {
				$query_args['no_found_rows'] = true;

				$args_additional = piotnetgrid_facet_change_query($facets, []);

				foreach ($args_additional['query'] as $key => $value) {
					$query_args[$key] = $value;
				}
			}

			if (!empty($GLOBALS['piotnetgrid_load_more'])) {
				$query_args['paged'] = $GLOBALS['piotnetgrid_load_more'];
			}
		}

		return $query_args;
	}

	public function filter_woocommerce_product_loop_end($loop_end) {
		if (!empty($GLOBALS['piotnetgrid_query'])) {
			$grid_query = [
				'piotnetgrid_id' => !empty($GLOBALS['grid_id_woocommerce']) ? $GLOBALS['grid_id_woocommerce'] : 'wp',
				'query' => $GLOBALS['piotnetgrid_query'],
				'main_query' => $GLOBALS['piotnetgrid_query'],
			];

			unset($GLOBALS['grid_id_woocommerce']);
			unset($GLOBALS['woocommerce_shortcode']);

			$loop_end = $loop_end . "<div data-piotnetgrid-query='" . json_encode($grid_query) . "'></div>";
		}

		return $loop_end;
	}

	public function grid_query($wp_query) {
		if (!wp_doing_ajax() && empty($_POST['piotnetgrid'])) {
			if (empty($_GET['action']) || !empty($_GET['action']) && strpos( $_GET['action'], 'oxy_' ) === false) {
				$wp_query_vars = $wp_query->query_vars;

				if ( empty($wp_query->get( 'piotnetgrid' )) && empty($wp_query->get( 'piotnetgrid_id' )) ) {
					$main_query = $wp_query->query;

					if (empty($main_query['posts_per_page']) && !empty($wp_query_vars['posts_per_page'])) {
						$main_query['posts_per_page'] = $wp_query_vars['posts_per_page'];
					}

					$grid_query = [
						'piotnetgrid_id' => !empty($GLOBALS['grid_id']) ? $GLOBALS['grid_id'] : 'wp',
						'query' => $wp_query_vars,
						'main_query' => $main_query,
					];

					if (!empty($GLOBALS['ct_options'])) {
						$grid_main_query = $this->oxygen_get_query($GLOBALS['ct_options']);
						if (!empty($grid_main_query)) {
							$grid_query['main_query'] = $grid_main_query;
						}
					}

					unset($GLOBALS['grid_id']);
					unset($GLOBALS['ct_options']);

					if (!is_admin()) {
						if ( defined('ELEMENTOR_VERSION') || defined('FL_BUILDER_VERSION')) {
							if(headers_sent()) {
								if (!defined('FL_THEME_BUILDER_VERSION')) {
					                echo "<div data-piotnetgrid-query='" . json_encode($grid_query) . "'></div>";
					            }
							}
						} else {
		                    echo "<div data-piotnetgrid-query='" . json_encode($grid_query) . "'></div>";
						}
					}
		        }

		        if ( !empty($wp_query->get( 'piotnetgrid_id' )) ) {
		        	$grid_query = [
						'piotnetgrid_id' => $wp_query->get( 'piotnetgrid_id' ),
						'query' => $wp_query_vars,
						'main_query' => $wp_query_vars,
					];

					if (!is_admin()) {
			            if ( defined('ELEMENTOR_VERSION') ||  defined('FL_BUILDER_VERSION')) {
							if(headers_sent()) {
								if (!defined('FL_THEME_BUILDER_VERSION')) {
					                echo "<div data-piotnetgrid-query='" . json_encode($grid_query) . "'></div>";
					            }
							}
						} else {
		                    echo "<div data-piotnetgrid-query='" . json_encode($grid_query) . "'></div>";
						}
					}
		        }

		        if (!empty($GLOBALS['piotnetgrid_post_ids'])) {
		        	unset($GLOBALS['piotnetgrid_post_ids']);
		        }
			}
		}
	}

	public function ajax_wp_template() {
		if (!empty($_POST['get_posts_query'])) {
			ob_start();
			add_filter( 'show_admin_bar', '__return_false' );
			$this->get_posts_query = json_decode($_POST['get_posts_query'],true);
			add_action( 'pre_get_posts', [$this, 'update_query_vars'], 999 );
			add_action( 'shutdown', [ $this, 'inject_template' ], 0 );
            ob_start();
		}
	}

	public function get_html() {
		if (!empty($_POST['get_html'])) {
			ob_start();

	        if (!empty($GLOBALS['piotnetgrid_load_more'])) {
	        	unset($GLOBALS['piotnetgrid_load_more']);
	        }

			if (!empty($_POST['load_more'])) {
				$GLOBALS['piotnetgrid_load_more'] = $_POST['load_more'];
			}
			
			add_filter( 'show_admin_bar', '__return_false' );
			add_action( 'shutdown', [ $this, 'inject_template' ], 0 );
            ob_start();
		}
	}

	public function update_query_vars($query) {
		if (!empty($this->get_posts_query)) {
			foreach ($this->get_posts_query as $key => $value) {
				$query->set( $key, $value );
			}
		}
	}

	public function update_query_vars_main_query($query) {
		if (!is_admin() && empty($query->get('piotnetgrid')) && empty($GLOBALS['piotnetgrid'])) {
			if (empty($GLOBALS['piotnetgrid_facet']) && !empty($GLOBALS['piotnetgrid_load_more'])) {
				$query->set( 'paged', $GLOBALS['piotnetgrid_load_more'] );
			}

			if ($query->is_main_query() && !empty($_GET) && !empty($query->get('wc_query')) || $query->is_main_query() && !empty($_GET) && !empty($GLOBALS['grid_id']) ) {
				$facets = [];
				foreach ($_GET as $facet_name => $facet_value) {
					$facet_settings = get_option('_piotnetgrid_facet' . $facet_name);
					if (!empty($facet_settings)) {
						$facets[] = piotnetgrid_get_facet_by_name($facet_name, $facet_value);
					}
				}

				if (!empty($facets)) {
					// $query->set( 'piotnetgrid_id', 'wp' );
					$query->set( 'main_query', $query->query_vars );

					if (empty($query->get('wc_query'))) {
						$query->set( 'no_found_rows', true );
					}

					$args_additional = piotnetgrid_facet_change_query($facets, []);

					foreach ($args_additional['query'] as $key => $value) {
						$query->set( $key, $value );
					}
					// echo '<pre>';
					// print_r($query);
					// echo '</pre>';
				}
			}

			if (!empty($GLOBALS['grid_id']) && empty($query->get('wc_query')) && empty($GLOBALS['woocommerce_shortcode']) && empty($GLOBALS['piotnetgrid_facet'])) {
				if (!empty($GLOBALS['grid_id']) || !empty($_POST['get_html'])) {
					$query->set( 'no_found_rows', true );
				}

				$facets = [];
				foreach ($_GET as $facet_name => $facet_value) {
					$facet_settings = get_option('_piotnetgrid_facet' . $facet_name);
					if (!empty($facet_settings)) {
						$facets[] = piotnetgrid_get_facet_by_name($facet_name, $facet_value);
					}
				}

				if (!empty($facets)) {
					$args_additional = piotnetgrid_facet_change_query($facets, []);

					foreach ($args_additional['query'] as $key => $value) {
						//if (in_array($key, ['paged', 'posts_per_page'])) {
							$query->set( $key, $value );
						//}
					}
				}

				if (!empty($GLOBALS['piotnetgrid_load_more'])) {
					$query->set( 'paged', $GLOBALS['piotnetgrid_load_more'] );
				}

				if (!empty($GLOBALS['piotnetgrid_post_ids'])) {
					$post_in = $query->get('post__in');

					if (!empty($post_in)) {
						$post_in = array_intersect($post_in, $GLOBALS['piotnetgrid_post_ids']);
					} else {
						$post_in = $GLOBALS['piotnetgrid_post_ids'];
					}

					$query->set( 'post__in', $post_in );
				}
			}

			if ( class_exists( 'WooCommerce' ) && empty($_GET['ct_builder']) && empty($_GET['oxygen_iframe']) && $query->get('post_type') == 'product' ) {
				if (empty($_GET['_locale']) || !empty($_GET['_locale']) && $_GET['_locale'] !== 'user') {
					$tax_query = !empty($query->get('tax_query')) ? $query->get('tax_query') : [];
					$tax_query[] = array(
				        'taxonomy'  => 'product_visibility',
				        'terms'     => array( 'exclude-from-catalog' ),
				        'field'     => 'name',
				        'operator'  => 'NOT IN',
				    );
					$query->set( 'tax_query', $tax_query );
				}
			}
		}
	}

	public function oxygen_get_query($options , $debug=false) {
        $args = [];

        // manual
        if (isset($options['query_args']) && isset($options['wp_query']) && $options['wp_query']=='manual') {

            $args_inline = $options['query_args'];
            /* https://wordpress.stackexchange.com/questions/120407/how-to-fix-pagination-for-custom-loops
            apparently doesn't work on static front pages? */
            $args_inline .= get_query_var( 'paged' ) ? '&paged='.get_query_var( 'paged' ) : '';

            //$this->query = new WP_Query($args);

            //$query = new WP_Query($args);

            $args_array = explode('&', $args_inline);
            foreach ($args_array as $args_item) {
            	$args_item_array = explode('=', $args_item);
            	$args[$args_item_array[0]] = $args_item_array[1];
            }

            if (!empty($options['query_post_ids'])) {
                $args['post__in'] = explode(",",$options['query_post_ids']);
            }

        } elseif (isset($options['wp_query']) && $options['wp_query']=='custom') {

            $args = array();

            // post type
            if (!empty($options['query_post_ids'])) {
                $args['post__in'] = explode(",",$options['query_post_ids']);
            }
            else {
                $args['post_type'] = $options['query_post_types'];
            }

            // filtering
            if (!empty($options['query_taxonomies_any'])) {
	            if (is_array($options['query_taxonomies_any'])) {

	                $taxonomies = array();
	                $args['tax_query'] = array(
	                    'relation' => 'OR',
	                );

	                // sort IDs by taxonomy slug
	                foreach ($options['query_taxonomies_any'] as $value) {
	                    $value = explode(",",$value);
	                    $key = $value[0];
	                    if ($key == "tag") {
	                        $key = "post_tag";
	                    }
	                    $taxonomies[$key][] = $value[1];
	                }

	                foreach ($taxonomies as $key => $value) {
	                    $args['tax_query'][] = array(
	                        'taxonomy' => $key,
	                        'terms'    => $value,
	                    );
	                }
	            }
            }

            if (!empty($options['query_taxonomies_all']) && is_array($options['query_taxonomies_all'])) {

                $taxonomies = array();
                $args['tax_query'] = array(
                    'relation' => 'AND',
                );

                // sort IDs by taxonomy slug
                foreach ($options['query_taxonomies_all'] as $value) {
                    $value = explode(",",$value);
                    $key = $value[0];
                    if ($key == "tag") {
                        $key = "post_tag";
                    }
                    $taxonomies[$key][] = $value[1];
                }

                foreach ($taxonomies as $key => $value) {
                    $args['tax_query'][] = array(
                        'taxonomy' => $key,
                        'terms'    => $value,
                        'operator' => 'AND'
                    );
                }
            }
            if (!empty($options['query_authors'])) {
                $args['author__in'] = $options['query_authors'];
            }

            // order
            $args['order']   = !empty($options['query_order']) ? $options['query_order'] : 'ASC';
            $args['orderby'] = !empty($options['query_order_by']) ? $options['query_order_by'] : '';

            if (!empty($options['query_ignore_sticky_posts']) && $options['query_ignore_sticky_posts']==='true') {
                $args['ignore_sticky_posts'] = true;
            }

            if (!empty($options['query_count']) && $options['query_count'] && is_numeric($options['query_count']) && intval($options['query_count']) > 0) {
                $args['posts_per_page'] = $options['query_count'];
            } else {
                $args['nopaging'] = true;
            }

            // pagination
            if (get_query_var('paged') && (!isset($args['nopaging']) || !$args['nopaging'])) {
                $args['paged'] = get_query_var( 'paged' );
            }

            //$this->query = new WP_Query($args);
            //$query = new WP_Query($args);

        } else {

            // use the current default query
            // global $wp_query;

            // $query = $wp_query;
            //$query = new WP_Query($args);
        }

        return $args;
    }

	public function inject_template() {
		$html = ob_get_clean();
		preg_match( "/<body(.*?)>(.*?)<\/body>/s", $html, $matches );

        if ( ! empty( $matches ) ) {
            $html = trim( $matches[2] );
        }

        $output = [];

        $output['html'] = $html;

        if (!empty($_POST['cache'])) {
        	$request_url = $_POST['request_url'];
        	$current_url_path = $_POST['current_url_path'];
        	$cache_expiration = $_POST['cache_expiration'];
        	$grid_post_id = $_POST['grid_post_id'];

			$cache_id = piotnetgrid_set_cache($request_url, $current_url_path, $cache_expiration, $grid_post_id);

			if ( is_dir( WP_CONTENT_DIR . '/cache/piotnetgrid/json' ) ) {
				$cache_file = piotnetgrid_get_cache($request_url, $grid_post_id);
				$cache_url = WP_CONTENT_DIR . '/cache/piotnetgrid/json/' . $cache_file;
				@file_put_contents( $cache_url, json_encode($output, JSON_UNESCAPED_UNICODE) );
				$output['cache_file'] = str_replace('.json', '', $cache_file);
			}
		}

        wp_send_json( $output );
	}

    public function query_starts_with( $where, $query ) {
        global $wpdb;

        $starts_with = esc_sql( $query->get( 'piotnetgrid_query_starts_with' ) );

        if ( $starts_with ) {
            $where .= " AND $wpdb->posts.post_title LIKE '$starts_with%'";
        }

        return $where;
    }

	public function load_textdomain() {
		load_plugin_textdomain( 'piotnetgrid', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' ); 
	}

	public function oxygen_register_add_plus_section() {
		CT_Toolbar::oxygen_add_plus_accordion_section( 'piotnetgrid', __('Piotnet Grid', 'piotnetgrid') );
	}

	public function oxygen_register_subsection() {
		do_action( 'oxygen_add_plus_piotnetgrid_basic' );
	}

	public function add_elementor_widget_categories( $elements_manager ) {

		$elements_manager->add_category(
			'piotnetgrid',
			[
				'title' => __( 'Piotnetgrid', 'piotnetgrid' ),
				'icon' => 'fa fa-plug',
			]
		);

	}

	public function enqueue_editor_elementor() {

		wp_enqueue_style( 'editor-elementor-piotnetgrid', plugin_dir_url( __FILE__ ) . 'assets/css/editor-elementor.css', [], PIOTNETGRID_VERSION );

	}

	public function elementor_widgets() {
		require_once( __DIR__ . '/inc/widgets/elementor/facet-elementor.php' );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \piotnetgrid_Facet_Embed_Elementor() );

		require_once( __DIR__ . '/inc/widgets/elementor/grid-elementor.php' );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \piotnetgrid_Grid_Embed_Elementor() );

		require_once( __DIR__ . '/inc/widgets/elementor/template-elementor.php' );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \piotnetgrid_Template_Elementor() );
	}

	public function gutenberg_block() {

		if ( ! function_exists( 'register_block_type' ) ) {
			return;
		}

		wp_register_script(
			'piotnetgrid-gutenberg-facet',
			plugins_url( 'assets/js/gutenberg/facet.js', __FILE__ ),
			[ 'wp-i18n', 'wp-element', 'wp-blocks', 'wp-components', 'wp-editor' ],
			filemtime( plugin_dir_path( __FILE__ ) . 'assets/js/gutenberg/facet.js' )
		);

		register_block_type( 'piotnetgrid/facet', array(
			'editor_script' => 'piotnetgrid-gutenberg-facet',
			'render_callback' => [ $this, 'gutenberg_render_facet']
		) );

		wp_register_script(
			'piotnetgrid-gutenberg-grid',
			plugins_url( 'assets/js/gutenberg/grid.js', __FILE__ ),
			[ 'wp-i18n', 'wp-element', 'wp-blocks', 'wp-components', 'wp-editor' ],
			filemtime( plugin_dir_path( __FILE__ ) . 'assets/js/gutenberg/grid.js' )
		);

		register_block_type( 'piotnetgrid/grid', array(
			'editor_script' => 'piotnetgrid-gutenberg-grid',
			'render_callback' => [ $this, 'gutenberg_render_grid']
		) );

		wp_register_script(
			'piotnetgrid-gutenberg-template',
			plugins_url( 'assets/js/gutenberg/template.js', __FILE__ ),
			[ 'wp-i18n', 'wp-element', 'wp-blocks', 'wp-components', 'wp-editor' ],
			filemtime( plugin_dir_path( __FILE__ ) . 'assets/js/gutenberg/template.js' )
		);

		register_block_type( 'piotnetgrid/template', array(
			'editor_script' => 'piotnetgrid-gutenberg-template',
			'render_callback' => [ $this, 'gutenberg_render_template']
		) );

		if ( function_exists( 'wp_set_script_translations' ) ) {
			wp_set_script_translations( 'piotnetgrid-gutenberg-grid', 'piotnetgrid' );
			wp_set_script_translations( 'piotnetgrid-gutenberg-facet', 'piotnetgrid' );
			wp_set_script_translations( 'piotnetgrid-gutenberg-template', 'piotnetgrid' );
		}

	}

	public function gutenberg_render_grid( $attributes, $content ) {
		ob_start();

		if (!empty($attributes['grid_id'])) {
			$shortcode = '[piotnetgrid id=' . $attributes['grid_id'] . ']';

			echo '<div>';
			echo do_shortcode( $shortcode );
			echo '</div>';
		}

		return ob_get_clean();
	}

	public function gutenberg_render_template( $attributes, $content ) {
		ob_start();

		if (!empty($attributes['template_id'])) {
			$shortcode = '[piotnetgrid id=' . $attributes['template_id'] . ' type=template]';

			echo '<div>';
			echo do_shortcode( $shortcode );
			echo '</div>';
		}

		return ob_get_clean();
	}

	public function gutenberg_render_facet( $attributes, $content ) {
		ob_start();

		if (!empty($attributes['grid_id']) && !empty($attributes['facet_id'])) {
			$shortcode = '[piotnetgrid id=' . $attributes['facet_id'] . ' type=facet grid=' . $attributes['grid_id'] . ']';

			echo '<div>';
			echo do_shortcode( $shortcode );
			echo '</div>';
		}

		return ob_get_clean();
	}

	public function gutenberg_block_categories( $categories ) {
		return array_merge(
			$categories,
			[
				[
					'slug'  => 'piotnetgrid',
					'title' => __( 'Piotnetgrid', 'piotnetgrid' ),
				],
			]
		);
	}

	public function add_menu_admin_bar_link() {
		global $wp_admin_bar;

		$args = array(
			'id' => 'piotnetgrid',
			'title' => 'Piotnetgrid',
		);
		$wp_admin_bar->add_menu( $args );

		$args = array(
			'id' => 'piotnetgrid_clear_cache',
			'title' => 'Clear Cache All',
			'href' => esc_url( get_admin_url( null, 'admin-ajax.php?action=piotnetgrid_clear_cache&page=all' ) ),
			'parent'=> 'piotnetgrid',
		);
		$wp_admin_bar->add_node( $args );

		$args = array(
			'id' => 'piotnetgrid_clear_cache_this_page',
			'title' => 'Clear Cache on This Page',
			'href' => esc_url( get_admin_url( null, 'admin-ajax.php?action=piotnetgrid_clear_cache&page=this' ) ),
			'parent'=> 'piotnetgrid',
		);
		$wp_admin_bar->add_node( $args );
	}
		
	public function init_session() {
	    if(!session_id()) {
	        session_start();
	    }
	}

	public function close_session() {
		if (session_status() == PHP_SESSION_ACTIVE) {
			session_write_close();
		}
	}

	public function create_analytics_table() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'piotnetgrid_analytics';
		
		$charset_collate = $wpdb->get_charset_collate();

		if($wpdb->get_var( "show tables like '$table_name'" ) != $table_name) {
			$sql = "CREATE TABLE $table_name (
				id bigint(20) NOT NULL AUTO_INCREMENT,
				facet_name text NOT NULL,
				facet_value text NOT NULL,
				facet_value_label text NOT NULL,
				facet_label text NOT NULL,
				url text NOT NULL,
				grouped boolean NOT NULL,
				no_results boolean NOT NULL,
				time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
				PRIMARY KEY  (id)
			) $charset_collate;";

			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );

			add_option( 'piotnetgrid_analytics_table_version', PIOTNETGRID_VERSION );
		}
	}

	public function create_cache_table() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'piotnetgrid_cache';
		
		$charset_collate = $wpdb->get_charset_collate();

		if($wpdb->get_var( "show tables like '$table_name'" ) != $table_name) {
			$sql = "CREATE TABLE $table_name (
				id bigint(20) NOT NULL AUTO_INCREMENT,
				name text NOT NULL,
				file text NOT NULL,
				url_path text NOT NULL,
				expiration bigint(10) NOT NULL,
				PRIMARY KEY  (id)
			) $charset_collate;";

			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );

			add_option( 'piotnetgrid_cache_table_version', PIOTNETGRID_VERSION );
		}
	}

	public function filter_archive_template($template) {
		if ( is_archive() ) {
			$template_posts = get_posts([
				'post_type' => 'piotnetgrid-archive',
				'posts_per_page' => -1,
				'fields' => 'ids',
			]);

			foreach ($template_posts as $post_id) {
				$raw_single_settings = get_post_meta( $post_id, '_piotnetgrid_single_settings', true );
				$single_settings = json_decode( $raw_single_settings, true );
				$condition_errors = 0;
				$condition_passed = 0;

				if (!empty($single_settings['single-settings']['fields']['display_conditions'])) {
					$display_conditions = $single_settings['single-settings']['fields']['display_conditions'];
					foreach ($display_conditions as $condition) {
						$condition_type = $condition['condition_type'];
						$condition_archive = $condition['condition_archive'];
						$condition_terms = !empty($condition['condition_terms']) ? $condition['condition_terms'] : [];

						if ($condition_type == 'include' && $condition_archive == 'all_archives') {
							$condition_passed++;
						}

						if (stripos( $condition_archive, '_archive_taxonomy' ) !== false) {
							$condition_tax = str_replace('_archive_taxonomy', '', $condition_archive);
							if (empty($condition_terms) && is_tax($condition_tax) || !empty($condition_terms) && is_tax($condition_tax, $condition_terms) || empty($condition_terms) && $condition_tax == 'category' && is_category() || !empty($condition_terms) && $condition_tax == 'category' && is_category($condition_terms)) {
								if ($condition_type == 'include') {
									$condition_passed++;
								} else {
									$condition_errors++;
								}
							}
						}

						if (stripos( $condition_archive, '_archive_post_type' ) !== false) {
							$condition_post_type = str_replace('_archive_post_type', '', $condition_archive);
							if (is_post_type_archive($condition_post_type)) {
								if ($condition_type == 'include') {
									$condition_passed++;
								} else {
									$condition_errors++;
								}
							}
						}
					}
				}

				if ($condition_passed > 0 && $condition_errors <= 0) {
					$template_id = $post_id;
					$template = plugin_dir_path( __FILE__ ) . 'inc/templates/archive-template.php';
					$template = load_template( $template, true, array('template_id' => $template_id) );
					break;
				}
			}
		}

		return $template;
	}

	public function filter_archive_template_woo( $template ) {
		if ( is_archive() ) {
			$template_posts = get_posts([
				'post_type' => 'piotnetgrid-archive',
				'posts_per_page' => -1,
				'fields' => 'ids',
			]);

			foreach ($template_posts as $post_id) {
				$raw_single_settings = get_post_meta( $post_id, '_piotnetgrid_single_settings', true );
				$single_settings = json_decode( $raw_single_settings, true );
				$condition_errors = 0;
				$condition_passed = 0;

				if (!empty($single_settings['single-settings']['fields']['display_conditions'])) {
					$display_conditions = $single_settings['single-settings']['fields']['display_conditions'];
					foreach ($display_conditions as $condition) {
						$condition_type = $condition['condition_type'];
						$condition_archive = $condition['condition_archive'];
						$condition_terms = !empty($condition['condition_terms']) ? $condition['condition_terms'] : [];

						if ($condition_type == 'include' && $condition_archive == 'all_archives') {
							$condition_passed++;
						}

						if (stripos( $condition_archive, '_archive_taxonomy' ) !== false) {
							$condition_tax = str_replace('_archive_taxonomy', '', $condition_archive);
							if (empty($condition_terms) && is_tax($condition_tax) || !empty($condition_terms) && is_tax($condition_tax, $condition_terms) || empty($condition_terms) && $condition_tax == 'category' && is_category() || !empty($condition_terms) && $condition_tax == 'category' && is_category($condition_terms)) {
								if ($condition_type == 'include') {
									$condition_passed++;
								} else {
									$condition_errors++;
								}
							}
						}

						if (stripos( $condition_archive, '_archive_post_type' ) !== false) {
							$condition_post_type = str_replace('_archive_post_type', '', $condition_archive);
							if (is_post_type_archive($condition_post_type)) {
								if ($condition_type == 'include') {
									$condition_passed++;
								} else {
									$condition_errors++;
								}
							}
						}
					}
				}

				if ($condition_passed > 0 && $condition_errors <= 0) {
					$template = plugin_dir_path( __FILE__ ) . 'inc/templates/archive-woocommerce.php';
					break;
				}
			}
		}

		return $template;
	}

	public function piotnetgrid_rest_api( $data ) {
		$response = piotnetgrid_facet($data);
		$result = new WP_REST_Response($response, 200);
		$result->set_headers(array('Cache-Control' => 'max-age=3600'));

		return $result;
	}

	public function piotnetgrid_woocommerce_checkout_redirect(){
		if ( class_exists( 'WooCommerce' ) ) {
			/* do nothing if we are not on the appropriate page */
			if( !is_wc_endpoint_url( 'order-received' ) || empty( $_GET['key'] ) ) {
				return;
			}
		 
			$order_id = wc_get_order_id_by_order_key( $_GET['key'] );
			$order = wc_get_order( $order_id );
		    $order_items = $order->get_items();

		    foreach ($order_items as $key => $value) {
	            $redirect_url = wc_get_order_item_meta( $key, 'piotnetgrid_woocommerce_checkout_redirect', true );

	            if (!empty($redirect_url)) {
	            	wc_delete_order_item_meta( $key, 'piotnetgrid_woocommerce_checkout_redirect' );
	            	wp_redirect( $redirect_url );
	            }
	        }
        }
	}

	public function piotnetgrid_woocommerce_add_to_cart( $item_data, $cart_item ) {
	    if ( empty( $cart_item['fields'] ) ) {
	        return $item_data;
	    }

	    $fields = apply_filters( 'piotnetgrid/form_builder/woocommerce_add_to_cart_fields', $cart_item['fields'] );

	    foreach ($fields as $item) {
	    	$item_data[] = array(
		        'key'     => $item['label'],
		        'value'   => $item['value'],
		        'display' => '',
		    );
	    }
	 
	    return $item_data;
	}

	function check_starter_forms() {
	    $option_key = 'piotnetgrid_starter_imported';
        $starter_forms_imported = get_option($option_key, null);
        if (!isset($starter_forms_imported)) {
            $this->import_starter_forms();
        }
        $data = [
            "importedDate" => time()
        ];
        update_option($option_key, $data);
    }

    function import_starter_forms() {
	    $dir = __DIR__ . "/assets/demos/starter/";
        $files = array_diff(scandir($dir), array('.', '..'));
        if (count($files) === 0) {
            return;
        }

        foreach ($files as $file) {
            $content = file_get_contents( $dir . $file );
            $data = json_decode( $content, true );

            $post = [
                'post_title'  => $data['title'],
                'post_status' => 'publish',
                'post_type'   => $data['post_type'],
            ];

            $post_id = wp_insert_post( $post );
            piotnetgrid_do_import( $post_id, $data );
        }
    }

    public function save_form( $post_id, $post, $update ) {
	    if ( $update ) {
	        $raw_data = get_post_meta( $post_id, '_piotnetgrid_data', true );
	        if (!empty(get_post_meta( $post_id, '_piotnetgrid_form_id', true ))) {
				$data_str = str_replace('"form_id":"' . get_post_meta( $post_id, '_piotnetgrid_form_id', true ), '"form_id":"' . get_the_title($post_id), $raw_data);
				$data_str = str_replace('"piotnetgrid_booking_form_id":"' . get_post_meta( $post_id, '_piotnetgrid_form_id', true ), '"piotnetgrid_booking_form_id":"' . get_the_title($post_id), $data_str);
				$data_str = str_replace('"piotnetgrid_woocommerce_checkout_form_id":"' . get_post_meta( $post_id, '_piotnetgrid_form_id', true ), '"piotnetgrid_woocommerce_checkout_form_id":"' . get_the_title($post_id), $data_str);
				$data_str = str_replace('"piotnetgrid_conditional_logic_form_form_id":"' . get_post_meta( $post_id, '_piotnetgrid_form_id', true ), '"piotnetgrid_conditional_logic_form_form_id":"' . get_the_title($post_id), $data_str);
				$data_str = str_replace('"piotnetgrid_repeater_form_id":"' . get_post_meta( $post_id, '_piotnetgrid_form_id', true ), '"piotnetgrid_repeater_form_id":"' . get_the_title($post_id), $data_str);
				update_post_meta( $post_id, '_piotnetgrid_data', wp_slash( $data_str ) );
	        }
	        
	        update_post_meta( $post_id, '_piotnetgrid_form_id', get_the_title($post_id) );
	    }
	}

	public function add_contextmenu() {
		ob_start();
		?>
		<div class="piotnetgrid-contextmenu" data-piotnetgrid-contextmenu>
			<div class="piotnetgrid-contextmenu__item" data-piotnetgrid-contextmenu-action="copy-style">Copy Style</div>
			<div class="piotnetgrid-contextmenu__item" data-piotnetgrid-contextmenu-action="paste-style">Paste Style</div>
		</div>
		<?php
		return ob_get_clean();
	}

	public function add_wrapper( $content ) {
		$post_id = get_the_ID();

        $raw_data = get_post_meta( $post_id, '_piotnetgrid_data', true );
        $data = json_decode( $raw_data, true );
        $widget_content = !empty($data['content']) ? $data['content'] : '';

        $preview_card_post_id = '';
        $raw_single_settings = get_post_meta( $post_id, '_piotnetgrid_single_settings', true );

        if (!empty($raw_single_settings)) {
        	$single_settings = json_decode( $raw_single_settings, true );
			$preview_card_post_id = !empty($single_settings['single-settings']['fields']['preview_post']) ? $single_settings['single-settings']['fields']['preview_post'] : '';
			$GLOBALS['preview_card_post_id'] = $preview_card_post_id;
        }

		if ( ! empty( $widget_content ) ) {
			if ( isset( $_GET['action'] ) && $_GET['action'] == 'piotnetgrid' ) {
				$editor = new piotnetgrid_Editor();
				if ( is_user_logged_in() ) {
					if ( current_user_can( 'edit_others_posts' ) ) {
						$content = piotnetgrid_dynamic_tags( $editor->editor_preview( $widget_content ) );
						$sortable = get_post_type() == 'piotnetgrid-card' ? '' : 'data-piotnetgrid-sortable';
						if (get_post_type() == 'piotnetgrid-card') {
							$content = '<div class="piotnetgrid-card" data-piotnetgrid-sortable>' . $content . '</div>';
						}
						$content = '<div class="piotnetgrid-widget-preview piotnetgrid piotnetgrid-' . $post_id . '" id="piotnetgrid" data-piotnetgrid-widget-preview ' . $sortable . '>' . $content . '</div>';
						$content .= $this->add_contextmenu();
					}
				}
			}

			if ( !isset( $_GET['action'] ) ) {
				wp_enqueue_style( 'piotnetgrid-style' );
				wp_enqueue_style( 'piotnetgrid-global-style' );
				wp_enqueue_script( 'piotnetgrid-script' );

				$content = $this->piotnetgrid_render_loop( $widget_content, $post_id, $preview_card_post_id );
				if (get_post_type() == 'piotnetgrid-card') {
					$content = '<div id="piotnetgrid" class="piotnetgrid piotnetgrid-' . $post_id . '"><article class="piotnetgrid-card">' . $content . '</article></div>';
				} else {
					$content = '<div id="piotnetgrid" class="piotnetgrid piotnetgrid-' . $post_id . '">' . $content . '</div>';
				}

				$upload     = wp_upload_dir();
				$upload_dir = $upload['baseurl'];
				$upload_dir = $upload_dir . '/piotnetgrid/css/';

				$css_file = $upload_dir . $post_id . '.css';
				
				wp_enqueue_style( $this->slug . '-style-' . $post_id, $css_file, [], get_post_meta( $post_id, '_piotnetgrid-revision-version', true ) );
			}
		} else {
			if ( is_user_logged_in() ) {
				if ( current_user_can( 'edit_others_posts' ) ) {
					if ( isset( $_GET['action'] ) && $_GET['action'] == 'piotnetgrid' ) {
						$sortable = get_post_type() == 'piotnetgrid-card' ? '' : 'data-piotnetgrid-sortable';
						$content = '<div class="piotnetgrid-widget-preview piotnetgrid piotnetgrid-' . $post_id . '" id="piotnetgrid" data-piotnetgrid-widget-preview ' . $sortable . '>';
						if (get_post_type() == 'piotnetgrid-card') {
							$content .= '<div class="piotnetgrid-card" data-piotnetgrid-sortable>';
						}
						$content .= '</div>';
						if (get_post_type() == 'piotnetgrid-card') {
							$content .= '</div>';
						}
						$content .= $this->add_contextmenu();
					}
				}
			}
		}

		return $content;
	}

	public function load_jquery() {
	    if ( ! wp_script_is( 'jquery', 'enqueued' )) {

	        //Enqueue
	        wp_enqueue_script( 'jquery' );

	    }
	}

	public function add_body_class( $classes ) {
		$classes[] = 'piotnetgrid-edit';
		return $classes;
	}

	public function modify_list_row_actions( $actions, $post ) {
		$post_type = $post->post_type;
		if ( in_array($post_type, ['piotnetgrid','piotnetgrid-card','piotnetgrid-facet','piotnetgrid-archive','piotnetgrid-template']) ) {
			$url = admin_url() . 'admin.php?page=piotnetgrid&post=' . $post->ID;

			$url_html = '<a href="' . $url . '">' . __( 'Edit With Piotnet Grid', 'piotnetgrid' ) . '</a>';

            $url_export_html = '<a href="' . esc_url( get_admin_url( null, 'admin-ajax.php?action=piotnetgrid_export&id=' ) ) . $post->ID . '&post_type=' . $post_type . '">' . __( 'Export', 'piotnetgrid' ) . '</a>';

            $duplicate_html = '<a href="' . esc_url( get_admin_url( null, 'admin-ajax.php?action=piotnetgrid_duplicate&id=' ) ) . $post->ID . '&post_type=' . $post_type . '">' . __( 'Duplicate', 'piotnetgrid' ) . '</a>';

			$actions['edit_with_piotnetgrid'] = $url_html;
			$actions['export_piotnetgrid'] = $url_export_html;
			$actions['duplicate_piotnetgrid'] = $duplicate_html;
		}

		return $actions;
	}

	public function plugin_action_links( $links ) {
		$links[]           = '<a href="' . esc_url( get_admin_url( null, 'admin.php?page=piotnetgrid' ) ) . '">' . esc_html__( 'Settings', 'piotnetgrid' ) . '</a>';
        if( !Piotnetgrid_License_Service::has_valid_license() ) {
			$links[] = '<a href="' . esc_url( get_admin_url( null, 'admin.php?page=piotnetgrid' ) ) . '" class="piotnetgrid-plugins-gopro">' . esc_html__( 'Activate License', 'piotnetgrid' ) . '</a>';
		}
		return $links;

	}

	public function update_message( $data, $response ) {
		echo '<br> ';
		printf(
			__( 'To enable updates, please login your account on the <a href="%1$s">Plugin Settings</a> page. If you have not purchased yet, please visit <a href="%2$s">https://piotnetgrid.com</a>. If you can not update, please download new version on <a href="https://piotnetgrid.com/my-account/">https://piotnetgrid.com/my-account/</a>.', 'piotnetgrid' ),
			admin_url( 'admin.php?page=piotnetgrid' ),
			'https://piotnetgrid.com'
		);
	}

	public function enqueue_frontend() {
		wp_enqueue_script( $this->slug . '-script' );
		wp_enqueue_style( $this->slug . '-style' );
		if ( file_exists( wp_upload_dir()['basedir'] . '/piotnetgrid/css/global.css' ) ) {
			wp_enqueue_style( $this->slug . '-global-style' );
		}
	}

	public function enqueue_frontend_all() {
		wp_enqueue_script( $this->slug . '-flatpickr-script' );
		wp_enqueue_script( $this->slug . '-slick-script' );
		wp_enqueue_script( $this->slug . '-script' );
		wp_enqueue_style( $this->slug . '-style' );
		wp_enqueue_style( $this->slug . '-flatpickr-style' );
		wp_enqueue_style( $this->slug . '-jquery-ui' );
		wp_enqueue_style( $this->slug . '-slick-style' );
		wp_enqueue_style( $this->slug . '-animate-style' );
		wp_enqueue_style( $this->slug . '-fontawesome-style' );
		wp_enqueue_style( $this->slug . '-global-style' );
	}

	public function enqueue() {
		wp_register_script( $this->slug . '-script', plugin_dir_url( __FILE__ ) . 'assets/js/minify/frontend.min.js', [ 'jquery' ], PIOTNETGRID_VERSION );
		wp_register_script( $this->slug . '-flatpickr-script', plugin_dir_url( __FILE__ ) . 'assets/js/minify/frontend/flatpickr.min.js', [ 'jquery' ], PIOTNETGRID_VERSION );
		wp_register_script( $this->slug . '-slick-script', plugin_dir_url( __FILE__ ) . 'assets/js/src/lib/slick.js', [ 'jquery' ], PIOTNETGRID_VERSION );
		wp_register_script( $this->slug . '-photoswipe-script', plugin_dir_url( __FILE__ ) . 'assets/js/minify/photoswipe.js', [ 'jquery' ], PIOTNETGRID_VERSION );
		wp_register_script( $this->slug . '-advanced-script', plugin_dir_url( __FILE__ ) . 'assets/js/minify/frontend/advanced.min.js', [ 'jquery' ], PIOTNETGRID_VERSION );

		wp_register_style( $this->slug . '-style', plugin_dir_url( __FILE__ ) . 'assets/css/minify/frontend.min.css', [], PIOTNETGRID_VERSION );
		wp_register_style( $this->slug . '-animate-style', plugin_dir_url( __FILE__ ) . 'assets/css/minify/animate.min.css', [], PIOTNETGRID_VERSION );
		wp_register_style( $this->slug . '-photoswipe-style', plugin_dir_url( __FILE__ ) . 'assets/css/minify/photoswipe.min.css', [], PIOTNETGRID_VERSION );
		wp_register_style( $this->slug . '-jquery-ui', plugin_dir_url( __FILE__ ) . 'assets/css/jquery-ui.css', [], PIOTNETGRID_VERSION );
		wp_register_style( $this->slug . '-flatpickr-style', plugin_dir_url( __FILE__ ) . 'assets/css/minify/flatpickr.min.css', [], PIOTNETGRID_VERSION );
		wp_register_style( $this->slug . '-slick-style', plugin_dir_url( __FILE__ ) . 'assets/css/minify/slick.min.css', [], PIOTNETGRID_VERSION );
		wp_register_style( $this->slug . '-fontawesome-style', plugin_dir_url( __FILE__ ) . 'assets/css/minify/fontawesome.min.css', [], PIOTNETGRID_VERSION );

		if ( file_exists( wp_upload_dir()['basedir'] . '/piotnetgrid/css/global.css' ) ) {
			wp_register_style( $this->slug . '-global-style', wp_upload_dir()['baseurl'] . '/piotnetgrid/css/global.css', [], intval( get_option( 'piotnetgrid-global-css-version' ) ) );
		}

		if ( is_user_logged_in() ) {
			if ( current_user_can( 'edit_others_posts' ) ) {
				if ( isset( $_GET['action'] ) && $_GET['action'] == 'piotnetgrid' ) {
					$this->enqueue_frontend_all();
					$this->admin_enqueue();
				}
			}
		}

		global $post;
		if ( is_object($post) ) {
            if ( has_shortcode( $post->post_content, 'piotnetgrid') ) {
                $this->enqueue_frontend();
            }
            if (!empty($_GET['ct_builder'])) {
            	$this->enqueue_frontend_all();
            }
        }

		$shortcode = get_post_meta( get_the_ID(), '_piotnetgrid_shortcode_in_post', true );
		if (!empty($shortcode)) {
			$this->enqueue_frontend();
			$shortcode = explode('|', $shortcode);
			foreach ($shortcode as $shortcode_item) {
				$shortcode_atts = shortcode_parse_atts($shortcode_item);
				if (!empty($shortcode_atts['id'])) {
					$post_id = intval($shortcode_atts['id']);
					$upload     = wp_upload_dir();
					$upload_dir = $upload['baseurl'];
					$upload_dir = $upload_dir . '/piotnetgrid/css/';
					$css_file = $upload_dir . $post_id . '.css';

					wp_enqueue_style( $this->slug . '-style-' . $post_id, $css_file, [], get_post_meta( $post_id, '_piotnetgrid-revision-version', true ) );
				}
			}
		}
	}

	public function add_admin_scripts( $hook ) {
		global $post;

		if ( isset($_GET['page']) && isset($_GET['post']) ) {
			if ( $_GET['page'] == 'piotnetgrid' ) {
				wp_enqueue_script( $this->slug . '-jquery-ui-script', plugin_dir_url( __FILE__ ) . 'assets/js/src/lib/jquery-ui.js', [ 'jquery' ], PIOTNETGRID_VERSION );
				wp_enqueue_script( $this->slug . '-editor-script', plugin_dir_url( __FILE__ ) . 'assets/js/minify/editor.min.js', [ 'jquery' ], PIOTNETGRID_VERSION );
				wp_enqueue_script( $this->slug . '-editor-forms-script', plugin_dir_url( __FILE__ ) . 'assets/js/minify/preview.min.js', [ 'jquery' ], PIOTNETGRID_VERSION );
				wp_enqueue_style( $this->slug . '-admin-style', plugin_dir_url( __FILE__ ) . 'assets/css/minify/admin.min.css', [], PIOTNETGRID_VERSION );
				wp_enqueue_script( $this->slug . '-script', plugin_dir_url( __FILE__ ) . 'assets/js/minify/frontend.min.js', [ 'jquery' ], PIOTNETGRID_VERSION );
				wp_enqueue_style( $this->slug . '-style', plugin_dir_url( __FILE__ ) . 'assets/css/minify/frontend.min.css', [], PIOTNETGRID_VERSION );
				wp_enqueue_style( $this->slug . '-style', plugin_dir_url( __FILE__ ) . 'assets/css/minify/animate.min.css', [], PIOTNETGRID_VERSION );
				wp_enqueue_media();
			}
		}

		wp_enqueue_script( $this->slug . '-admin-script', plugin_dir_url( __FILE__ ) . 'assets/js/minify/admin.min.js', [ 'jquery' ], PIOTNETGRID_VERSION );
	}

	public function enqueue_head() {
		if ( is_user_logged_in() ) {
			if ( current_user_can( 'edit_others_posts' ) ) {
				if ( isset( $_GET['action'] ) && $_GET['action'] == 'piotnetgrid' ) {
					//                  echo '<style data-piotnetgrid-widget-css-head>' . get_post_meta( get_the_ID(), '_piotnetgrid-widgets-css', true ) . '</style>';

					$post_id = get_the_ID();

					if ( $post_id != false ) {
						$widget_settings = get_post_meta( $post_id, '_piotnetgrid-widget-settings', true );

						if ( ! empty( $widget_settings ) ) {
							$widget_settings = json_decode( $widget_settings, true );
							if ( ! empty( $widget_settings['fonts'] ) ) {
								$fonts = $widget_settings['fonts'];
								foreach ( $fonts as $font ) {
									echo '<link href="' . $font . '" rel="stylesheet">';
								}
							}
						}
					}
				}
			}
		}

		global $wpdb;
		$current_url = (isset($_SERVER['HTTPS']) ? "https" : "http") . "://$_SERVER[HTTP_HOST]".strtok($_SERVER["REQUEST_URI"],'?');
		$current_time = strtotime("now");
		$caches = @$wpdb->get_results( 'SELECT file FROM ' . $wpdb->prefix . 'piotnetgrid_cache WHERE url_path = "' . $current_url . '" AND (expiration>' . $current_time . ' OR expiration=0)', ARRAY_A );
		
		$cache_files = [];
		foreach ($caches as $cache) {
			$cache_files[] = $cache['file'];
		}

		$global_settings = get_option( 'piotnetgrid_global_settings' );
		$global_settings_data = !empty($global_settings) ? json_decode(stripslashes($global_settings), true) : [];

		$header_script = '';
		$header_script .= '<script type="text/javascript">';
		$header_script .= 'var ajaxurl = "' . admin_url('admin-ajax.php') . '";';
		$header_script .= 'var piotnetgridRestURL = "' . get_rest_url() . 'piotnetgrid/v1/refresh";';
		$header_script .= 'var piotnetgridWpcontentURL = "' . content_url() . '";';
		if ( is_user_logged_in() && current_user_can( 'edit_others_posts' ) ) {
			$header_script .= 'var piotnetgridAdminURL = "' . admin_url() . '";';
		}
		$header_script .= 'window["cache_files"] = ' . json_encode($cache_files) . ';';
		$header_script .= '</script>';
		echo $header_script;

		$header_style = '';
		$header_style .= '<style>';
		$header_style .= '@keyframes piotnetfadein {0% { opacity: 0} 90% { opacity: 0} 100% { opacity: 1}} .piotnetgrid-fadein {animation: piotnetfadein 0.6s;}';
		if (!empty($global_settings_data['global-general']['fields']['custom_css'])) {
			$header_style .= $global_settings_data['global-general']['fields']['custom_css'];
		}
		$header_style .= '</style>';
		echo $header_style;

		if (is_singular()) {
			$post_id = get_the_ID();
			$post_type = get_post_type();

			if (empty($_SESSION['piotnetgrid_recently_viewed'][$post_type])) {
				$_SESSION['piotnetgrid_recently_viewed'][$post_type] = [get_the_ID()];
			} else {
				if (!in_array($post_id, $_SESSION['piotnetgrid_recently_viewed'][$post_type])) {
					$_SESSION['piotnetgrid_recently_viewed'][$post_type][] = $post_id;
				}
			}
		}

		$GLOBALS['piotnetgrid'] = true;
	}

	public function enqueue_footer() {
		$global_settings = get_option( 'piotnetgrid_global_settings' );
		$global_settings_data = !empty($global_settings) ? json_decode(stripslashes($global_settings), true) : [];

		$footer_script = '<script type="text/javascript">';
		if (!empty($global_settings_data['global-general']['fields']['custom_js'])) {
			$footer_script .= $global_settings_data['global-general']['fields']['custom_js'];
		}
		$footer_script .= '</script>';
		echo $footer_script;
	}

	public function admin_footer() {
		echo '<div data-piotnetgrid-admin-url="' . admin_url() . '"></div>';
		echo '<div data-piotnetgrid-plugin-url="' . plugins_url() . '"></div>';

		$demos = [
			'piotnetgrid-card' => [
				[	
					'type' => 'card',
					'title' => 'Card 1',
					'folder' => 'card-1',
				],
				[	
					'type' => 'card',
					'title' => 'Card 2',
					'folder' => 'card-2',
				],
				[	
					'type' => 'card',
					'title' => 'Card 3',
					'folder' => 'card-3',
				],
				[	
					'type' => 'card',
					'title' => 'Card 4',
					'folder' => 'card-4',
				],
				[	
					'type' => 'card',
					'title' => 'Card 5',
					'folder' => 'card-5',
				],
				[	
					'type' => 'card',
					'title' => 'Card 6',
					'folder' => 'card-6',
				],
				[	
					'type' => 'card',
					'title' => 'Card 7 - Term',
					'folder' => 'card-7',
				],
			],
			'piotnetgrid' => [
				[	
					'type' => 'grid',
					'title' => 'Grid 1',
					'folder' => [
						'card' => ['card-1-grid-1','card-2-grid-1','card-3-grid-1'],
						'grid' => ['grid-1'],
					],
				],
				[	
					'type' => 'grid',
					'title' => 'Grid 2',
					'folder' => [
						'card' => ['card-3','card-4'],
						'grid' => ['grid-2'],
					],
				],
				[	
					'type' => 'grid',
					'title' => 'Grid 3',
					'folder' => [
						'card' => ['card-2'],
						'grid' => ['grid-3'],
					],
				],
				[	
					'type' => 'grid',
					'title' => 'Grid 4',
					'folder' => [
						'card' => ['card-5'],
						'grid' => ['grid-4'],
					],
				],
				[	
					'type' => 'grid',
					'title' => 'Grid 5',
					'folder' => [
						'card' => ['card-6'],
						'grid' => ['grid-5'],
					],
				],
				[	
					'type' => 'grid',
					'title' => 'Grid 6 - Terms',
					'folder' => [
						'card' => ['card-7'],
						'grid' => ['grid-6'],
					],
				],
			],
			'piotnetgrid-facet' => [
				[	
					'type' => 'facet',
					'title' => 'Facet 1',
					'folder' => 'facet-1',
				],
				[	
					'type' => 'facet',
					'title' => 'Facet 2',
					'folder' => 'facet-2',
				],
				[	
					'type' => 'facet',
					'title' => 'Facet 3',
					'folder' => 'facet-3',
				],
				[	
					'type' => 'facet',
					'title' => 'Facet 4',
					'folder' => 'facet-4',
				],
				[	
					'type' => 'facet',
					'title' => 'Facet 5',
					'folder' => 'facet-5',
				],
				[	
					'type' => 'facet',
					'title' => 'Facet 6',
					'folder' => 'facet-6',
				],
			],
			'piotnetgrid-template' => [
				[	
					'type' => 'template',
					'title' => 'Template 1',
					'folder' => [
						'card' => ['card-1-grid-1','card-2-grid-1','card-3-grid-1'],
						'grid' => ['grid-1'],
						'facet' => ['facet-2'],
						'template' => ['template-1'],
					],
				],
				[	
					'type' => 'template',
					'title' => 'Template 2',
					'folder' => [
						'card' => ['card-4','card-3'],
						'grid' => ['grid-2'],
						'facet' => ['facet-1'],
						'template' => ['template-2'],
					],
				],
				[	
					'type' => 'template',
					'title' => 'Template 3',
					'folder' => [
						'card' => ['card-3'],
						'grid' => ['grid-1-template-3'],
						'facet' => ['facet-1-template-3','facet-2'],
						'template' => ['template-3'],
					],
				],
				[	
					'type' => 'template',
					'title' => 'Template 4',
					'folder' => [
						'card' => ['card-1-template-4','card-2-template-4'],
						'grid' => ['grid-1-template-4'],
						'facet' => ['facet-6'],
						'template' => ['template-4'],
					],
				],
				[	
					'type' => 'template',
					'title' => 'Template 5',
					'folder' => [
						'template' => ['template-5'],
					],
				],
				[	
					'type' => 'template',
					'title' => 'Template 6',
					'folder' => [
						'card' => ['card-2'],
						'grid' => ['grid-1-template-6'],
						'facet' => ['facet-1-template-6'],
						'template' => ['template-6'],
					],
				],
			],
			'piotnetgrid-archive' => [
				[	
					'type' => 'template',
					'title' => 'Archive 1',
					'folder' => [
						'card' => ['card-2'],
						'grid' => ['grid-1-archive-1'],
						'facet' => ['facet-3','facet-4','facet-5'],
						'template' => ['archive-1'],
					],
				],
			],
		];

		global $pagenow;
		global $typenow;
		if ( in_array($typenow, ['piotnetgrid','piotnetgrid-card','piotnetgrid-facet','piotnetgrid-archive','piotnetgrid-template']) ) :
		?>
			<script type="text/javascript">
				jQuery(document).ready(function( $ ) {
					var $toolbar = $(document).find('#titlediv');
					$toolbar.append('<button class="button button-primary button-large" data-edit-with-piotnetgrid>Edit With Piotnet Grid</button>');

					$(document).on('click', '[data-edit-with-piotnetgrid]', function(e){
						e.preventDefault();
						var post_id = $('#post_ID').val(),
							post_title = $('[name="post_title"]').val(),
							status = $('#original_post_status').val(),
							admin_url = $('[data-piotnetgrid-admin-url]').attr('data-piotnetgrid-admin-url');
                        $('#submitpost [type="submit"]').trigger('click');

						if (status === 'auto-draft') {
							var data = {
								post_id: post_id,
								post_title: post_title,
								action: 'piotnetgrid_save_draft',
							};

							$.post(ajaxurl, data, function (response) {
								window.location.href = admin_url + 'admin.php?page=piotnetgrid&post=' + post_id;
							});
						} else {
							window.location.href = admin_url + 'admin.php?page=piotnetgrid&post=' + post_id;
						}

					});
				});
			</script>
			<?php if(!empty($demos[$typenow])) : ?>
				<div data-piotnetgrid-demos='<?php echo json_encode($demos[$typenow]); ?>'></div>
			<?php endif; ?>
		<?php
		endif;

		if (( $pagenow == 'edit.php' ) && !empty($_GET['post_type'])) {
			$post_type = $_GET['post_type'];
			if ( $post_type == 'piotnetgrid' || $post_type == 'piotnetgrid-card' || $post_type == 'piotnetgrid-facet' ) {
				if ( get_option( 'piotnetgrid_do_flush', false ) ) {
					delete_option( 'piotnetgrid_do_flush' );
					flush_rewrite_rules();
				}
			}
		}
	}

	public function tab_widget_template() {
		?>
		<script type="text/html" data-piotnetgrid-template id="piotnetgrid-tab-widget-template">
			<div class="piotnetgrid-widget-controls" data-piotnetgrid-widget-controls="<%= data['widget_id'] %>">
				<div class="piotnetgrid-tabs" data-piotnetgrid-tabs="">
					<% for ( var key in data['tabs'] ) { var tab = data['tabs'][key]; %>
					<div class="piotnetgrid-tabs__item <%= tab.active ? 'active' : '' %>" data-piotnetgrid-tabs-item="<%= tab['name'] %>"><%= tab['label'] %></div>
					<% } %>
				</div>
				<% for ( var key in data['tabs'] ) {
					var tab = data['tabs'][key];
					var sections = tab['sections'];
				%>
				<div class="piotnetgrid-tabs-content <%= tab.active ? 'active' : '' %>" data-piotnetgrid-tabs-content="<%= tab['name'] %>">
					<%
					for ( var key in sections ) {
						var section = sections[key];
						const field_group_attributes = [];
						if ( section.conditions ) {
						field_group_attributes.push("data-piotnetgrid-control-conditions='" + JSON.stringify(section.conditions) + "'");
						}
					%>
					<div class="piotnetgrid-controls-section <%= section.active ? 'active' : '' %>" data-piotnetgrid-controls-section="<%= section['name'] %>" <% _.each(field_group_attributes, function(field_group_attribute) { %><%= " " + field_group_attribute %><% }); %>>
						<div class="piotnetgrid-controls-section__header" data-piotnetgrid-controls-section-header="">
							<div class="piotnetgrid-controls-section__header-label"><%= section['label'] %></div>
							<div class="piotnetgrid-controls-section__header-icon">
								<i class="fas fa-caret-down"></i>
								<i class="fas fa-caret-up"></i>
							</div>
						</div>
						<div class="piotnetgrid-controls-section__body" data-piotnetgrid-controls-section-body=""></div>
					</div>
					<% } %>
				</div>
				<% } %>
			</div>
		</script>
		<?php
	}

	public function output_template() {
		?>
		<script type="text/html" data-piotnetgrid-template id="piotnetgrid-output-template">
			<%
			view.add_attribute('widget_wrapper_editor', 'class', 'piotnetgrid-widget');
			view.add_attribute('widget_wrapper_editor', 'data-piotnetgrid-editor-widgets-item', JSON.stringify( data.widget_info ));
			view.add_attribute('widget_wrapper_editor', 'data-piotnetgrid-editor-widgets-item-id', data.widget_id);
			view.add_attribute('widget_wrapper_editor', 'draggable', 'true');
			%>
			<div <%= view.render_attributes('widget_wrapper_editor') %>>
				<div class="piotnetgrid-widget__controls">
					<div class="piotnetgrid-widget__controls-item piotnetgrid-widget__controls-item--edit" title="Edit" data-piotnetgrid-control-edit>
						<i class="fas fa-th"></i>
					</div>
					<div class="piotnetgrid-widget__controls-item piotnetgrid-widget__controls-item--duplicate" title="Duplicate" data-piotnetgrid-control-duplicate>
						<i class="far fa-clone"></i>
					</div>
					<div class="piotnetgrid-widget__controls-item piotnetgrid-widget__controls-item--remove" title="Delete" data-piotnetgrid-control-remove>
						<i class="fas fa-times"></i>
					</div>
				</div>
				<div class="piotnetgrid-widget__container"></div>
			</div>
		</script>
		<?php
	}

	private function division_output_template() {
		?>
		<script type="text/html" data-piotnetgrid-template id="piotnetgrid-division-output-template">
			<%
			const division_type = data.division_type;
			%>
			<div <%= view.render_attributes('widget_wrapper_editor') %>>
				<div class="<%= division_type %>__controls">
					<div class="<%= division_type %>__controls-item <%= division_type %>__controls-item--edit" title="Edit" data-piotnetgrid-control-edit>
						<i class="fas fa-th"></i>
					</div>
					<div class="<%= division_type %>__controls-item <%= division_type %>__controls-item--duplicate" title="Duplicate" data-piotnetgrid-control-duplicate>
						<i class="far fa-clone"></i>
					</div>
					<div class="<%= division_type %>__controls-item <%= division_type %>__controls-item--remove" title="Delete" data-piotnetgrid-control-remove>
						<i class="fas fa-times"></i>
					</div>
				</div>
				<div class="piotnetgrid-background-overlay"></div>
				<div <%= view.render_attributes('widget_wrapper_container') %>></div>
			</div>
		</script>
		<?php
	}

	public function register_post_type() {
		register_post_type(
			$this->slug,
			[
				'labels'       => [
					'name'          => __( 'Grids', 'piotnetgrid' ),
					'singular_name' => __( 'Grid', 'piotnetgrid' ),
				],
				'public'       => true,
				'has_archive'  => true,
				'show_in_menu' => false,
				'show_in_rest' => true,
				'supports'     => [
					'title',
					'custom-fields',
				],
			]
		);

		remove_post_type_support( $this->slug, 'editor' );

		register_post_type(
			$this->slug . '-card',
			[
				'labels'       => [
					'name'          => __( 'Cards', 'piotnetgrid' ),
					'singular_name' => __( 'Card', 'piotnetgrid' ),
				],
				'public'       => true,
				'has_archive'  => true,
				'show_in_menu' => false,
				'show_in_rest' => true,
				'supports'     => [
					'title',
					'custom-fields',
				],
			]
		);

		remove_post_type_support( $this->slug . '-card', 'editor' );

		register_post_type(
			$this->slug . '-facet',
			[
				'labels'       => [
					'name'          => __( 'Facets', 'piotnetgrid' ),
					'singular_name' => __( 'Facet', 'piotnetgrid' ),
				],
				'public'       => true,
				'has_archive'  => true,
				'show_in_menu' => false,
				'show_in_rest' => true,
				'supports'     => [
					'title',
					'custom-fields',
				],
			]
		);

		remove_post_type_support( $this->slug . '-facet', 'editor' );

		register_post_type(
			$this->slug . '-archive',
			[
				'labels'       => [
					'name'          => __( 'Archive Templates', 'piotnetgrid' ),
					'singular_name' => __( 'Archive Template', 'piotnetgrid' ),
				],
				'public'       => true,
				'has_archive'  => true,
				'show_in_menu' => false,
				'show_in_rest' => true,
				'supports'     => [
					'title',
					'custom-fields',
				],
			]
		);

		remove_post_type_support( $this->slug . '-archive', 'editor' );

		register_post_type(
			$this->slug . '-template',
			[
				'labels'       => [
					'name'          => __( 'Templates', 'piotnetgrid' ),
					'singular_name' => __( 'Template', 'piotnetgrid' ),
				],
				'public'       => true,
				'has_archive'  => true,
				'show_in_menu' => false,
				'show_in_rest' => true,
				'supports'     => [
					'title',
					'custom-fields',
				],
			]
		);

		remove_post_type_support( $this->slug . '-template', 'editor' );

		add_filter( 'wpseo_sitemap_exclude_post_type', function( $retval, $post_type ) {
			if ( 'piotnetgrid' === $post_type || 'piotnetgrid-card' === $post_type || 'piotnetgrid-facet' === $post_type ) {
				$retval = true;
			}

			return $retval;
		}, 10, 2 );
	}

	public function single_template($single) {
	    global $post;
	    $post_type = $post->post_type;
	    if ( in_array($post_type, ['piotnetgrid','piotnetgrid-card','piotnetgrid-facet','piotnetgrid-archive','piotnetgrid-template']) ) {
	        return plugin_dir_path( __FILE__ ) . 'inc/templates/single-template.php';
	    }
	    return $single;
	}

	public function admin_menu() {

		add_menu_page(
			$this->plugin_name,
			$this->plugin_name,
			'edit_others_posts',
			$this->slug,
			[ $this, 'settings_page' ],
			'dashicons-piotnetgrid-icon'
		);

		add_submenu_page( $this->slug, 'Settings', 'Settings', 'edit_others_posts', $this->slug, [ $this, 'settings_page' ] );

		add_submenu_page( $this->slug, 'Cards', 'Cards', 'edit_others_posts', 'edit.php?post_type=' . $this->slug . '-card' );

		add_submenu_page( $this->slug, 'Grids', 'Grids', 'edit_others_posts', 'edit.php?post_type=' . $this->slug );

		add_submenu_page( $this->slug, 'Facets', 'Facets', 'edit_others_posts', 'edit.php?post_type=' . $this->slug . '-facet' );

		add_submenu_page( $this->slug, 'Archive Templates', 'Archive Templates', 'edit_others_posts', 'edit.php?post_type=' . $this->slug . '-archive' );

		add_submenu_page( $this->slug, 'Templates', 'Templates', 'edit_others_posts', 'edit.php?post_type=' . $this->slug . '-template' );

		add_submenu_page( $this->slug, 'Import', 'Import', 'edit_others_posts', 'import-piotnetgrid', [ $this, 'import_page' ] );

		add_submenu_page( $this->slug, 'Analytics', 'Analytics', 'edit_others_posts', 'analytic-piotnetgrid', [ $this, 'analytic_page' ] );

		add_action( 'admin_init', [ $this, 'piotnet_base_settings' ] );

        add_action( 'admin_init',  [ $this, 'refresh_license' ] );
	}

    public function refresh_license() {
        Piotnetgrid_License_Service::refresh_license();
    }

    public function piotnet_base_settings() {
		register_setting( 'piotnetgrid-settings-group', 'piotnetgrid-username' );
		register_setting( 'piotnetgrid-settings-group', 'piotnetgrid-password' );
		register_setting( 'piotnetgrid-settings-group', 'piotnetgrid-column-gutter' );
		register_setting( 'piotnetgrid-settings-group', 'piotnetgrid-custom-css' );
		register_setting( 'piotnetgrid-settings-group', 'piotnetgrid-custom-js' );
	}

	public function settings_page() {

		require_once __DIR__ . '/inc/settings/settings-page.php';

	}

	public function import_page() {

		require_once __DIR__ . '/inc/settings/import-page.php';

	}

	public function analytic_page() {

		require_once __DIR__ . '/inc/settings/analytic-page.php';

	}

	public function admin_enqueue() {
		wp_enqueue_style( $this->slug . '-admin-css', plugin_dir_url( __FILE__ ) . 'assets/css/minify/admin.min.css', false, PIOTNETGRID_VERSION );
	}

	public function set_custom_edit_columns( $columns ) {
		$columns['piotnetgrid-widget-shortcode'] = __( 'Shortcode', 'piotnetgrid' );
		return $columns;
	}

	public function custom_column( $column, $post_id ) {
		switch ( $column ) {
			case 'piotnetgrid-widget-shortcode':
				$post_type = str_replace('piotnetgrid', '', get_post_type( $post_id ));
				$type = !empty($post_type) ? str_replace('-', '', $post_type) : 'grid';
				$args_additional = '';
				if ($type == 'facet') {
					$grid_query = new WP_Query( array(
						'post_type' => 'piotnetgrid',
				        'posts_per_page' => -1,
				    ));
				    $grid_id_first = 0;
					echo '<label>Apply for Grid</label>';
					echo '<select data-piotnetgrid-facet-apply-for-grid>';
						if ($grid_query->have_posts()) {
							$index = 0;
							while ($grid_query->have_posts()) {
								$index++;
								$grid_query->the_post();
								if ($index == 1) {
									$grid_id_first = get_the_ID();
								}
								echo '<option value="' . get_the_ID() . '">' . get_the_title() . '</option>';
							}
						}
					echo '</select>';

					$args_additional .= ' grid=' . $grid_id_first;
				}
				echo '<input data-piotnetgrid-facet-shortcode class="piotnetgrid-widget-shortcode-input" type="text" readonly="" onfocus="this.select()" value="[piotnetgrid id=' . $post_id  . ' type=' . $type . $args_additional . ']">';
				break;
		}
	}

	public function plugin_redirect() {

		if ( get_option( 'piotnetgrid_do_activation_redirect', false ) ) {
			delete_option( 'piotnetgrid_do_activation_redirect' );
			flush_rewrite_rules();
			wp_redirect( 'admin.php?page=piotnetgrid' );
		}

	}

	public function plugin_row_meta( $links, $file ) {

		if ( strpos( $file, 'piotnetgrid' ) !== false ) {
			$links[] = '<a href="https://piotnetgrid.com/docs/create-a-card/" target="_blank">' . esc_html__( 'Documents', 'piotnetgrid' ) . '</a>';
			$links[] = '<a href="https://my.piotnetgrid.com/change-log" target="_blank">' . esc_html__( 'Change Log', 'piotnetgrid' ) . '</a>';
		}
		return $links;

	}

	public function plugin_activate() {

		add_option( 'piotnetgrid_do_activation_redirect', true );
		add_option( 'piotnetgrid_do_flush', true );

		$this->create_analytics_table();
		$this->create_cache_table();

		global $wpdb;
		$wpdb->query("ALTER TABLE `wp_posts` ADD FULLTEXT(`post_content`, `post_title`)");
	}

    /**
     * @return piotnetgrid_Base_Control[]
     */
    private function new_widget( string $class_name ) {
        return new $class_name();
    }

	public function piotnetgrid_render_loop( $loop, $post_id, $preview_card_post_id = '' ) {

		ob_start();

		foreach ( $loop as $widget_item ) {
			if (!empty($widget_item['class_name'])) {
				$widget            = $this->new_widget( $widget_item['class_name'] );
				$widget->settings  = $widget_item['settings'];
				$widget_id         = $widget_item['id'];
				$widget->widget_id = $widget_id;
				$widget->post_id   = $post_id;

				if ( ! empty( $widget_item['fonts'] ) ) {
					$fonts = $widget_item['fonts'];
					if ( ! empty( $fonts ) ) {
						echo '<script>jQuery(document).ready(function( $ ) {';
						foreach ( $fonts as $font ) :
							?>
							$('head').append('<link href="<?php echo $font; ?>" rel="stylesheet">');
							<?php
						endforeach;
						echo '})</script>';
					}
				}

				$widget_type = $widget->get_type();
				if ( $widget_type === 'section' || $widget_type === 'column' ) {
					$visibility = @$widget->widget_visibility();
					if ($visibility) {
						echo @$widget->output_wrapper_start( $widget_id );
						$visibility = @$widget->widget_visibility();
						if ( isset( $widget_item['elements'] ) ) {
							echo @$this->piotnetgrid_render_loop( $widget_item['elements'], $post_id, $preview_card_post_id );
						}
					}
				} else {
					$output = @$widget->output( $widget_id );
					$output = @piotnetgrid_dynamic_tags( $output, $preview_card_post_id );
					echo @$output;
				}

				if ( $widget_type === 'section' || $widget_type === 'column' ) {
					echo @$widget->output_wrapper_end( $widget_id );
				}
			}
		}

		return ob_get_clean();
	}

	public function piotnetgrid_dynamic_tags( $output ) {

		if ( stripos( $output, '{{' ) !== false && stripos( $output, '}}' ) !== false ) {
			$pattern = '~\{\{\s*(.*?)\s*\}\}~';
			preg_match_all( $pattern, $output, $matches );
			$dynamic_tags = [];
			$post_id = get_the_ID();

			if ( ! empty( $matches[1] ) ) {
				$matches = array_unique( $matches[1] );

				foreach ( $matches as $key => $match ) {
					if ( stripos( $match, '|' ) !== false ) {
						$match_attr = explode( '|', $match );
						$attr_array = [];
						foreach ( $match_attr as $key_attr => $value_attr ) {
							if ( $key_attr != 0 ) {
								$attr                           = explode( ':', $value_attr, 2 );
								$attr_array[ trim( $attr[0] ) ] = trim( $attr[1] );
							}
						}

						$dynamic_tags[] = [
							'dynamic_tag' => '{{' . $match . '}}',
							'name'        => trim( $match_attr[0] ),
							'attr'        => $attr_array,
						];
					} else {
						$dynamic_tags[] = [
							'dynamic_tag' => '{{' . $match . '}}',
							'name'        => trim( $match ),
						];
					}
				}
			}

			if ( ! empty( $dynamic_tags ) ) {
				foreach ( $dynamic_tags as $tag ) {
					$tag_value = '';

					if ( $tag['name'] == 'current_date_time' ) {
						if ( empty( $tag['attr']['date_format'] ) ) {
							$tag_value = date( 'Y-m-d H:i:s' );
						} else {
							$tag_value = date( $tag['attr']['date_format'] );
						}
					}

					if ( $tag['name'] == 'request' ) {
						if ( !empty( $tag['attr']['parameter'] ) ) {
							$tag_value = $_REQUEST[ $tag['attr']['parameter'] ];
						}
					}

					if ( $tag['name'] == 'user_info' ) {
						if (is_user_logged_in()) {
							if ( !empty( $tag['attr']['meta'] ) ) {
								$meta = $tag['attr']['meta'];
								$current_user = wp_get_current_user();

								switch ( $meta ) {
									case 'ID':
									case 'user_login':
									case 'user_nicename':
									case 'user_email':
									case 'user_url':
									case 'user_registered':
									case 'user_status':
									case 'display_name':
										$tag_value = $current_user->$meta;
										break;
									default:
										$tag_value = get_user_meta( get_current_user_id(), $tag['attr']['meta'], true );
								}
							}
						}
					}
					if ( $tag['name'] == 'post_terms' ) {
						$terms = get_the_terms( $post_id, $tag['attr']['taxonomy'] );
						$post_terms = [];
						foreach($terms as $key => $val){
							$post_terms[$key] = $val->name;
						}
						$tag_value = implode($tag['attr']['separator'], $post_terms);
					}

                    if ( $tag['name'] == 'wc_product_short_description' ) {
                    	global $product;
                    	if ($product) {
                    		$length = $tag['attr']['length'];
                    		if(!empty($length) && is_numeric($length) && $length > 0) {
                    			$tag_value = wp_trim_words( $product->get_short_description(), $length, '...' );
                    		} else {
                        	    $tag_value = $product->get_short_description();
                        	}
                    	}
                    }

                    if ( $tag['name'] == 'wc_product_price' ) {
                    	global $product;
                    	if ($product) {
	                        //Full price
	                        if($tag['attr']['format'] == 'full'){
	                            $tag_value = $product->get_price_html();
	                            //Sale price
	                        }elseif($tag['attr']['format'] == 'sale'){
	                            if($product->is_type('simple')){
	                                if($product->is_on_sale()){
	                                    $tag_value = wc_price($product->get_sale_price());
	                                }else{
	                                    $tag_value = wc_price($product->get_price());
	                                }
	                            }elseif($product->is_type('variable')){
	                                if($product->is_on_sale()){
	                                    $tag_value = wc_price($product->get_variation_sale_price('min')).' - '.wc_price($product->get_variation_sale_price('max'));
	                                }else{
	                                    $tag_value = wc_price($product->get_variation_price('min')).' - '.wc_price($product->get_variation_price('max'));
	                                }
	                            }else{
	                            	$tag_value = $product->get_price_html();
	                            }
	                            //Regular price
	                        }else{
	                            if($product->is_type('simple')){
	                                $tag_value = wc_price($product->get_regular_price());
	                            }elseif($product->is_type('variable')){
	                                $tag_value = wc_price($product->get_variation_regular_price('min')).' - '.wc_price($product->get_variation_regular_price('max'));
	                            }else{
	                            	$tag_value = $product->get_price_html();
	                            }
	                        }
                        }
                    }

                    if ( $tag['name'] == 'wc_product_discount_percentage' ) {
						global $product;
                    	if ($product) {
							if( $product->is_type('variable')){
								$discount_percentages = array();

								// Get all variation prices
								$prices = $product->get_variation_prices();

								// Loop through variation prices
								foreach( $prices['price'] as $key => $price ){
									// Only on sale variations
									if( $prices['regular_price'][$key] !== $price ){
										// Calculate and set in the array the percentage for each variation on sale
										$discount_percentages[] = round( 100 - ( floatval($prices['sale_price'][$key]) / floatval($prices['regular_price'][$key]) * 100 ) );
									}
								}
								// We keep the highest value
								$discount_percentage = max($discount_percentages) . '%';

							} elseif( $product->is_type('grouped') ){
								$discount_percentages = array();

								// Get all variation prices
								$children_ids = $product->get_children();

								// Loop through variation prices
								foreach( $children_ids as $child_id ){
									$child_product = wc_get_product($child_id);

									$regular_price = (float) $child_product->get_regular_price();
									$sale_price    = (float) $child_product->get_sale_price();

									if ( $sale_price != 0 || ! empty($sale_price) ) {
										// Calculate and set in the array the percentage for each child on sale
										$discount_percentages[] = round(100 - ($sale_price / $regular_price * 100));
									}
								}
								// We keep the highest value
								$discount_percentage = max($discount_percentages) . '%';

							} else {
								$regular_price = (float) $product->get_regular_price();
								$sale_price    = (float) $product->get_sale_price();

								if ( $sale_price != 0 || ! empty($sale_price) ) {
									$discount_percentage = round(100 - ($sale_price / $regular_price * 100)) . '%';
								}
							}
							if ( $discount_percentage > 0 ) {
								$tag_value = $discount_percentage;
							}
							
						}
					}

					if ( $tag['name'] == 'wc_category_thumbnail' ) {
						global $term;
						if (!empty($term)) {
							$term_image_id = get_term_meta( $term->term_id, 'thumbnail_id', true );
	                    	if (!empty($term_image_id)) {
	                    		$tag_value = wp_get_attachment_image_src( $term_image_id, 'full' )[0];
	                    	}
						}
					}

					if ( $tag['name'] == 'author_info' ) {
						$tag_value = get_the_author_meta($tag['attr']['meta']);
					}

					if ( $tag['name'] == 'post_id' ) {
						$tag_value = $post_id;
					}

					if ( $tag['name'] == 'post_title' ) {
						if ( !empty($tag['attr']['length']) && is_numeric($tag['attr']['length']) && $tag['attr']['length'] > 0) {
							$tag_value = wp_trim_words( get_the_title(), $tag['attr']['length'], '...' );
						} else {
							$tag_value = get_the_title();
						}
					}

					if ( $tag['name'] == 'post_url' ) {
						$tag_value = get_permalink();
					}

					if ( $tag['name'] == 'post_time' ) {
						if (!empty( $tag['attr']['format'] ) ) {
							$tag_value = get_post_time($tag['attr']['format'],TRUE,$post_id,TRUE);
						}
					}

                    if ( $tag['name'] == 'post_modified_time' ) {
                        if (!empty( $tag['attr']['format'] ) ) {
                            $tag_value = get_the_modified_date($tag['attr']['format'],$post_id);
                        }
                    }

					if ( $tag['name'] == 'post_content' ) {
						if ( !empty($tag['attr']['length']) && is_numeric($tag['attr']['length']) && $tag['attr']['length'] > 0) {
							$tag_value = wp_trim_words( get_the_content(), $tag['attr']['length'], '...' );
						} else {
							$tag_value = get_the_content();
						}
					}

					if ( $tag['name'] == 'post_excerpt' ) {
						if (!empty($tag['attr']['length']) && is_numeric($tag['attr']['length']) && $tag['attr']['length'] > 0 ) {
							$tag_value = wp_trim_words( get_the_excerpt(), $tag['attr']['length'], '...' );
						} else {
                            $tag_value = get_the_excerpt();
                        }
					}

                    if ( $tag['name'] == 'post_comments_number' ) {
                        $num_comments = get_comments_number();
                    	if ($num_comments >= 1) {
                    	   $tag_value = $num_comments;
                    	}
					}

					if ( $tag['name'] == 'post_featured_image' ) {
                        $post_featured_image_size = !empty( $tag['attr']['size'] ) ? $tag['attr']['size'] : 'full';
                    	$tag_value = get_the_post_thumbnail( $post_id, $post_featured_image_size );
					}

					if ( $tag['name'] == 'wc_product_title') {
						$tag_value = get_the_title();
					}

					if ( $tag['name'] == 'wc_product_rating') {
						global $product;
						if ($product) {
							$rating_count = $product->get_rating_count();
							$average_rating  = $product->get_average_rating();
	                        if ($rating_count >= 1) {
	                        	$star_rating = wc_get_star_rating_html($average_rating);
	                        	$tag_value = strip_tags($star_rating);
	                        }
						}
					}

					if ( $tag['name'] == 'shortcode' ) {
						if ( !empty( $tag['attr']['shortcode'] ) ) {
							$tag_value = do_shortcode( $tag['attr']['shortcode'] );
						}
					}

                    if ( $tag['name'] == 'post_custom_field' ) {
                        $post_id = $post_id;
                        if ( !empty( $tag['attr']['name'] ) ) {
                            $tag_value = get_post_meta($post_id, $tag['attr']['name'], true);
                        }
                    }

                    if ( function_exists( 'get_field' ) && $tag['name'] == 'acf_field' ) {
                        $post_id = $post_id;
                        if ( !empty( $tag['attr']['name'] ) ) {
                            $tag_value = get_field($tag['attr']['name'], $post_id);
                            if ( $tag['attr']['name'] == 'date' ) {
                                $tag_value = get_post_meta( $post_id, $tag['attr']['name'], true );
                                $time  = strtotime( $tag_value );
                                $tag_value = date( get_option( 'date_format' ), $time );
                            }
                        }
                    }

                    if ( function_exists( 'rwmb_get_value' ) && $tag['name'] == 'metabox_field' ) {
                        $post_id = $post_id;
                        if ( !empty( $tag['attr']['name'] ) ) {
                            $tag_value = rwmb_get_value($tag['attr']['name'], array(), $post_id);
                            if ( $tag['attr']['name'] == 'date' ) {
                                $tag_value = get_post_meta( $post_id, $tag['attr']['name'], true );
                                $time  = strtotime( $tag_value );
                                $tag_value = date( get_option( 'date_format' ), $time );
                            }
                        }
                    }

                    if ( function_exists( 'pods_field' ) && $tag['name'] == 'pods_field' ) {
                        $post_id = $post_id;
                        $sp_post_type = get_post_type();
                        if ( !empty( $tag['attr']['name'] ) ) {
                            $tag_value = pods_field($sp_post_type, $post_id, $tag['attr']['name'], true);
                        }
                    }

                    if ( $tag['name'] == 'toolset_field' ) {
                        $post_id = $post_id;
                        if ( !empty( $tag['attr']['name'] ) ) {
                            $meta_key = 'wpcf-' . $tag['attr']['name'];
                            $tag_value = get_post_meta( $post_id, $meta_key, true );
                        }
                    }

                    if ( $tag['name'] == 'jetengine_field' ) {
                        $post_id = $post_id;
                        if ( !empty( $tag['attr']['name'] ) ) {
                            $tag_value = get_post_meta( $post_id, $tag['attr']['name'], true );
                        }
                    }

	                if ( $tag['name'] == 'archive_title' ) {
                        if ( is_post_type_archive() ) {
					        $tag_value = post_type_archive_title( '', false );
					    } elseif ( is_tax() ) {
					    	$tag_value = single_term_title( '', false );
					    } elseif ( is_category ()) {
							$tag_value = single_cat_title( '', false);
						}
                    }

                    if ( $tag['name'] == 'archive_description' ) {
                    	if (!empty($tag['attr']['length']) && is_numeric($tag['attr']['length']) && $tag['attr']['length'] > 0) {
                    		$tag_value = wp_trim_words( get_the_archive_description(), $tag['attr']['length'], '...' );
                    	} else {
                    		$tag_value = get_the_archive_description();
                    	}

                    }

                    if ( $tag['name'] == 'archive_meta' ) {
                        $tag_value = get_term_meta( $tag['attr']['term_id'], $tag['attr']['meta_key'], true );
                    }

                    global $term;

                    if (!empty($term)) {
                    	if ( $tag['name'] == 'term_name' ) {
	                        $tag_value = $term->name;
	                    }

	                    if ( $tag['name'] == 'term_description' ) {
	                        $tag_value = $term->description;
	                    }

	                    if ( $tag['name'] == 'term_url' ) {
	                        $tag_value = get_term_link( $term );
	                    }

	                    if ( $tag['name'] == 'term_count' ) {
	                        $tag_value = $term->count;
	                    }

	                    if ( $tag['name'] == 'term_color' ) {
	                        $tag_value = '#' . get_term_meta( $term->term_id, 'piotnetgrid_term_color', true );
	                    }

	                    if ( $tag['name'] == 'term_image' ) {
	                    	$term_image_id = get_term_meta( $term->term_id, 'piotnetgrid_term_image', true );
	                    	if (!empty($term_image_id)) {
	                    		$tag_value = wp_get_attachment_image_src( $term_image_id, 'full' )[0];
	                    	}
	                    }

	                    if ( $tag['name'] == 'term_meta' ) {
	                        $tag_value = get_term_meta( $term->term_id, $tag['attr']['meta_key'], true );
	                    }
                    }

					$output = str_replace( $tag['dynamic_tag'], $tag_value, $output );
				}
			}
		}

		return $output;
	}

	public function piotnetgrid_filter(){
	    if (isset($_GET['post_type'])) {
	        $type = $_GET['post_type'];
		    if ( $type == 'piotnetgrid-data' || $type == 'piotnetgrid-aban' ){
		        $form_id = array();
		        $submissions = new WP_Query( array(
		            'post_type' => $type,
		            'posts_per_page' => -1,
	            ) );

	            if ($submissions->have_posts()) : while ( $submissions->have_posts()) : $submissions->the_post();
	                $form_id[get_post_meta(get_the_ID(),'form_id',true)] = get_post_meta(get_the_ID(),'form_id',true);
	            endwhile; endif; wp_reset_postdata();
		        ?>
		        <select name="form_id">
		        <option value=""><?php _e('All Form ID', 'piotnetgrid'); ?></option>
		        <?php
		            $current_v = isset($_GET['form_id'])? $_GET['form_id']:'';
		            foreach ($form_id as $label => $value) {
		                printf
		                    (
		                        '<option value="%s"%s>%s</option>',
		                        $value,
		                        $value == $current_v? ' selected="selected"':'',
		                        $label
		                    );
		                }
		        ?>
		        </select>
		        <?php
		    }
	    }
	}

	public function piotnetgrid_filter_posts( $query ){
	    global $pagenow;
	    if (isset($_GET['post_type'])) {
	        $type = $_GET['post_type'];
	        if ( $type == 'piotnetgrid-data' || $type == 'piotnetgrid-aban' ){
			    if ( is_admin() && $pagenow=='edit.php' && isset($_GET['form_id']) && $_GET['form_id'] != '' && $query->is_main_query()) {
			        $query->query_vars['meta_key'] = 'form_id';
			        $query->query_vars['meta_value'] = $_GET['form_id'];
			    }
		    }
	    }
	}

	public function piotnetgrid_filter_column($defaults) {
	    $defaults['form_id'] = 'Form ID';
	    $defaults['status'] = 'Status';
	    return $defaults;
	}

	public function piotnetgrid_filter_column_content($column_name, $post_ID) {
	    if ($column_name == 'form_id') {
	        echo get_post_meta($post_ID,'form_id',true);
	    }

	    if ($column_name == 'status') {
	    	$status = !empty( get_post_meta($post_ID,'status',true) ) ? get_post_meta($post_ID,'status',true) : 'Success';
	        echo $status;
	    }
	}

	public function piotnetgrid_filter_export_btn() {
	    if (isset($_GET['post_type'])) {
	        $type = $_GET['post_type'];
	        if ( $type == 'piotnetgrid-data' || $type == 'piotnetgrid-aban' ){
	    ?>
		    <script type="text/javascript">
		        jQuery(document).ready( function($) {
		        	<?php if ( !empty($_GET['form_id']) ) : ?>
		            	$('.tablenav.top .clear, .tablenav.bottom .clear').before('<a class="button button-primary user_export_button" style="margin-top:3px;" href="<?php echo esc_url( get_admin_url( null, 'admin-ajax.php?action=piotnetgrid_export_form_submission' . '&post_status=' . $_REQUEST['post_status'] . '&post_type=' . $_REQUEST['post_type'] . '&m=' .$_REQUEST['m'] . '&form_id=' . $_REQUEST['form_id'] ) ); ?>"><?php esc_attr_e('Click on Filter and then click here to export as csv', 'piotnetgrid');?></a>');
	            	<?php else : ?>
	            		$('.tablenav.top .clear, .tablenav.bottom .clear').before('<input class="button button-primary user_export_button" style="margin-top:3px;" type="submit" value="<?php esc_attr_e('Select Form ID and click on Filter to export as csv', 'piotnetgrid');?>" />');
            		<?php endif; ?>
		        });
		    </script>
	    <?php
			}
		}
	}

	public function add_import_button() {
	    global $current_screen;

	    if ( !in_array($current_screen->post_type, ['piotnetgrid', 'piotnetgrid-card', 'piotnetgrid-facet', 'piotnetgrid-template', 'piotnetgrid-archive']) ) {
	        return;
	    }
	    ?>
	        <script type="text/javascript">
	            jQuery(document).ready( function($) {
	                jQuery(jQuery(".wrap .page-title-action")[0]).after("<a data-piotnetgrid-demos-import class='add-new-h2'>Import Demos</a>");
	            });
	        </script>
	    <?php
	}

	public function add_term_meta_boxes() {
		global $pagenow;

		if ( 'term.php' !== $pagenow && 'edit-tags.php' !== $pagenow ) {
			return;
		}

		$taxonomy = get_current_screen()->taxonomy;

		add_action( $taxonomy . '_add_form_fields', [ $this, 'colorpicker_field_add_new_category' ] );
		add_action( $taxonomy . '_edit_form_fields', [ $this, 'colorpicker_field_edit_category' ] );
	}

	public function colorpicker_field_add_new_category( $taxonomy ) {
	?>
	    <div class="form-field term-colorpicker-wrap">
	        <label for="term-colorpicker">Piotnetgrid Term Color</label>
	        <input name="piotnetgrid_term_color" value="" class="piotnetgrid-term-colorpicker" id="term-colorpicker" />
	    </div>
	    <div class="form-field term-group">
			<label for="piotnetgrid_term_image">Piotnetgrid Term Image</label>
			<input type="hidden" id="piotnetgrid_term_image" name="piotnetgrid_term_image" class="custom_media_url" value="">
			<div id="category-image-wrapper"></div>
			<p>
				<input type="button" class="button button-secondary ct_tax_media_button" id="ct_tax_media_button" name="ct_tax_media_button" value="<?php _e( 'Add Image', 'piotnetgrid' ); ?>" />
				<input type="button" class="button button-secondary ct_tax_media_remove" id="ct_tax_media_remove" name="ct_tax_media_remove" value="<?php _e( 'Remove Image', 'piotnetgrid' ); ?>" />
			</p>
		</div>
	<?php
	}

	public function colorpicker_field_edit_category( $term ) {
	    $color = get_term_meta( $term->term_id, 'piotnetgrid_term_color', true );
	    $color = ( ! empty( $color ) ) ? "#{$color}" : '';
	  ?>
	    <tr class="form-field term-colorpicker-wrap">
	        <th scope="row"><label for="term-colorpicker">Piotnetgrid Term Color</label></th>
	        <td>
	            <input name="piotnetgrid_term_color" value="<?php echo $color; ?>" class="piotnetgrid-term-colorpicker" id="term-colorpicker" />
	        </td>
	    </tr>

	    <tr class="form-field term-group-wrap">
			<th scope="row">
				<label for="piotnetgrid_term_image"><?php _e( 'Piotnetgrid Term Image', 'piotnetgrid' ); ?></label>
			</th>
			<td>
				<?php $image_id = get_term_meta ( $term->term_id, 'piotnetgrid_term_image', true ); ?>
				<input type="hidden" id="piotnetgrid_term_image" name="piotnetgrid_term_image" value="<?php echo $image_id; ?>">
				<div id="category-image-wrapper">
					<?php if ( $image_id ) { ?>
						<?php echo wp_get_attachment_image ( $image_id, 'thumbnail' ); ?>
					<?php } ?>
				</div>
				<p>
					<input type="button" class="button button-secondary ct_tax_media_button" id="ct_tax_media_button" name="ct_tax_media_button" value="<?php _e( 'Add Image', 'piotnetgrid' ); ?>" />
					<input type="button" class="button button-secondary ct_tax_media_remove" id="ct_tax_media_remove" name="ct_tax_media_remove" value="<?php _e( 'Remove Image', 'piotnetgrid' ); ?>" />
				</p>
			</td>
		</tr>
	  <?php
	}

	public function save_termmeta( $term_id ) {
	    if( isset( $_POST['piotnetgrid_term_color'] ) && ! empty( $_POST['piotnetgrid_term_color'] ) ) {
	        update_term_meta( $term_id, 'piotnetgrid_term_color', sanitize_hex_color_no_hash( $_POST['piotnetgrid_term_color'] ) );
	    } else {
	        delete_term_meta( $term_id, 'piotnetgrid_term_color' );
	    }

	    if( isset( $_POST['piotnetgrid_term_image'] ) && '' !== $_POST['piotnetgrid_term_image'] ){
			$image = $_POST['piotnetgrid_term_image'];
			update_term_meta( $term_id, 'piotnetgrid_term_image', $image );
		} else {
			update_term_meta( $term_id, 'piotnetgrid_term_image', '' );
		}
	}

	public function category_colorpicker_enqueue( $taxonomy ) {
	    wp_enqueue_script( 'wp-color-picker' );
	    wp_enqueue_style( 'wp-color-picker' );
	}

	public function colorpicker_init_inline() {
		wp_enqueue_media();
		?>
	    <script>
	        jQuery( document ).ready( function( $ ) {
	            $( '.piotnetgrid-term-colorpicker' ).wpColorPicker();
	        } );
	    </script>

	    <script>
	    jQuery(document).ready( function($) {
	       function ct_media_upload(button_class) {
	         var _custom_media = true,
	         _orig_send_attachment = wp.media.editor.send.attachment;
	         $('body').on('click', button_class, function(e) {
	           var button_id = '#'+$(this).attr('id');
	           var send_attachment_bkp = wp.media.editor.send.attachment;
	           var button = $(button_id);
	           _custom_media = true;
	           wp.media.editor.send.attachment = function(props, attachment){
	             if ( _custom_media ) {
	               $('#piotnetgrid_term_image').val(attachment.id);
	               $('#category-image-wrapper').html('<img class="custom_media_image" src="" style="margin:0;padding:0;max-height:100px;float:none;" />');
	               $('#category-image-wrapper .custom_media_image').attr('src',attachment.url).css('display','block');
	             } else {
	               return _orig_send_attachment.apply( button_id, [props, attachment] );
	             }
	            }
	         wp.media.editor.open(button);
	         return false;
	       });
	     }
	     ct_media_upload('.ct_tax_media_button.button'); 
	     $('body').on('click','.ct_tax_media_remove',function(){
	       $('#piotnetgrid_term_image').val('');
	       $('#category-image-wrapper').html('<img class="custom_media_image" src="" style="margin:0;padding:0;max-height:100px;float:none;" />');
	     });
	     // Thanks: http://stackoverflow.com/questions/15281995/wordpress-create-category-ajax-response
	     $(document).ajaxComplete(function(event, xhr, settings) {
	        if (settings.data !== undefined) {
	           var queryStringArr = settings.data.split('&');
		       if ( $.inArray('action=add-tag', queryStringArr) !== -1 ) {
		         var xml = xhr.responseXML;
		         $response = $(xml).find('term_id').text();
		         if($response!=""){
		           // Clear the thumb image
		           $('#category-image-wrapper').html('');
		         }
		       }
		    }
	     });
   		});
	 	</script>
		<?php
	}

	public function add_wc_attribute_piotnetgrid_attribute_type() {
	    $id = isset( $_GET['edit'] ) ? absint( $_GET['edit'] ) : 0;
	    $value = $id ? get_option( "wc_attribute_piotnetgrid_attribute_type_$id" ) : '';
	    ?>
	        <div class="form-field">
	            <label for="piotnetgrid_attribute_type">Piotnetgrid Attribute Type</label>
	            <select name="piotnetgrid_attribute_type" id="piotnetgrid_attribute_type">
					<option value="select"<?php if ($value == 'select') { echo ' selected'; } ?>>Select</option>
					<option value="color"<?php if ($value == 'color') { echo ' selected'; } ?>>Color</option>
					<option value="image"<?php if ($value == 'image') { echo ' selected'; } ?>>Image</option>
					<option value="button"<?php if ($value == 'button') { echo ' selected'; } ?>>Button</option>
				</select>
	        </div>
	    <?php
	}

	public function edit_wc_attribute_piotnetgrid_attribute_type() {
	    $id = isset( $_GET['edit'] ) ? absint( $_GET['edit'] ) : 0;
	    $value = $id ? get_option( "wc_attribute_piotnetgrid_attribute_type_$id" ) : '';
	    ?>
	        <tr class="form-field">
	            <th scope="row" valign="top">
	                <label for="piotnetgrid_attribute_type">Piotnetgrid Attribute Type</label>
	            </th>
	            <td>
	                <select name="piotnetgrid_attribute_type" id="piotnetgrid_attribute_type">
						<option value="select"<?php if ($value == 'select') { echo ' selected'; } ?>>Select</option>
						<option value="color"<?php if ($value == 'color') { echo ' selected'; } ?>>Color</option>
						<option value="image"<?php if ($value == 'image') { echo ' selected'; } ?>>Image</option>
						<option value="button"<?php if ($value == 'button') { echo ' selected'; } ?>>Button</option>
					</select>
	            </td>
	        </tr>
	    <?php
	}

	public function save_wc_attribute_piotnetgrid_attribute_type( $id ) {
	    if ( is_admin() && isset( $_POST['piotnetgrid_attribute_type'] ) ) {
	        $option = "wc_attribute_piotnetgrid_attribute_type_$id";
	        update_option( $option, sanitize_text_field( $_POST['piotnetgrid_attribute_type'] ) );
	    }
	}
}

$piotnetgrid = new piotnetgrid();